@@ -16,17 +16,16 @@
 #undef HAVE_KRB4
 #undef HAVE_KRB5
 #undef MOVEMAIL_PATH
-#undef HAVE_LIBGPGME
-#undef PGP_PROGRAM
-#undef GPG_PATH
-#undef PGP5_PATH
-#undef PGP_PATH
 #undef USING_GNOME_PRINT_0_20
 #undef USE_DOT
 #undef USE_FCNTL
 #undef USE_FLOCK
 #undef NEED_KRB_SENDAUTH_PROTO
 #undef HAVE_ISBLANK
+#undef ENABLE_NNTP
+
+/* Sub-version identification string.  */
+#undef SUB_VERSION
 
 /* Define this if you want to build against the development gtk */
 #undef HAVE_DEVGTK
