@@ -59,6 +59,13 @@ gboolean  e_book_load_uri                 (EBook                 *book,
 void      e_book_unload_uri               (EBook                 *book);
 char     *e_book_get_static_capabilities  (EBook                 *book);
 
+/* User authentication. */
+void      e_book_authenticate_user        (EBook                 *book,
+					   const char            *user,
+					   const char            *passwd,
+					   EBookCallback         cb,
+					   gpointer              closure);
+
 /* Fetching cards. */
 ECard    *e_book_get_card                 (EBook                 *book,
 					   const char            *id);
