@@ -411,8 +411,12 @@ char
 	if (card->fburl)
 		addPropValue(vobj, "FBURL", card->fburl);
 	
-	if (card->note)
-		addPropValue(vobj, VCNoteProp, card->note);
+	if (card->note) {
+		VObject *noteprop;
+
+		noteprop = addPropValue(vobj, VCNoteProp, card->note);
+		addProp(noteprop, VCQuotedPrintableProp);
+	}
 
 	if (card->categories) {
 		EIterator *iterator;
@@ -1436,6 +1440,9 @@ e_card_set_arg (GtkObject *object, GtkArg *arg, guint arg_id)
 	case ARG_FULL_NAME:
 		g_free(card->fname);
 		card->fname = g_strdup(GTK_VALUE_STRING(*arg));
+		if (card->name)
+			e_card_name_free (card->name);
+		card->name = e_card_name_from_string (card->fname);
 		break;
 	case ARG_NAME:
 		if ( card->name )
