@@ -8,6 +8,7 @@
 
 #include "config.h"  
 #include <gtk/gtksignal.h>
+#include <gnome.h>
 #include <unistd.h>
 #include <fcntl.h>
 #include <time.h>
@@ -75,7 +76,6 @@ struct _PasBackendFileChangeContext {
 	GList *add_ids;
 	GList *mod_cards;
 	GList *mod_ids;
-	GList *del_cards;
 	GList *del_ids;
 };
 
@@ -104,7 +104,6 @@ pas_backend_file_book_view_copy(const PASBackendFileBookView *book_view, void *c
 		new_book_view->change_context->add_ids = book_view->change_context->add_ids;
 		new_book_view->change_context->mod_cards = book_view->change_context->mod_cards;
 		new_book_view->change_context->mod_ids = book_view->change_context->mod_ids;
-		new_book_view->change_context->del_cards = book_view->change_context->del_cards;
 		new_book_view->change_context->del_ids = book_view->change_context->del_ids;
 	} else
 		new_book_view->change_context = NULL;
@@ -117,7 +116,7 @@ pas_backend_file_book_view_free(PASBackendFileBookView *book_view, void *closure
 {
 	g_free(book_view->search);
 	if (book_view->search_sexp)
-		gtk_object_unref(GTK_OBJECT(book_view->search_sexp));
+		e_sexp_unref (book_view->search_sexp);
 	g_free(book_view->search_context);
 
 	g_free(book_view->change_id);
@@ -126,13 +125,11 @@ pas_backend_file_book_view_free(PASBackendFileBookView *book_view, void *closure
 		g_list_foreach (book_view->change_context->add_ids, (GFunc)g_free, NULL);
 		g_list_foreach (book_view->change_context->mod_cards, (GFunc)g_free, NULL);
 		g_list_foreach (book_view->change_context->mod_ids, (GFunc)g_free, NULL);
-		g_list_foreach (book_view->change_context->del_cards, (GFunc)g_free, NULL);
 		g_list_foreach (book_view->change_context->del_ids, (GFunc)g_free, NULL);
 		g_list_free (book_view->change_context->add_cards);
 		g_list_free (book_view->change_context->add_ids);
 		g_list_free (book_view->change_context->mod_cards);
 		g_list_free (book_view->change_context->mod_ids);
-		g_list_free (book_view->change_context->del_cards);
 		g_list_free (book_view->change_context->del_ids);
 	}
 	g_free(book_view->change_context);
@@ -362,7 +359,7 @@ entry_compare(PASBackendFileSearchContext *ctx, struct _ESExp *f,
 		}
 		
 	}
-	r = e_sexp_result_new(ESEXP_RES_BOOL);
+	r = e_sexp_result_new(f, ESEXP_RES_BOOL);
 	r->value.bool = truth;
 
 	return r;
@@ -466,7 +463,7 @@ vcard_matches_search (const PASBackendFileBookView *view, char *vcard_string)
 
 	gtk_object_unref(GTK_OBJECT(view->search_context->card));
 
-	e_sexp_result_free(r);
+	e_sexp_result_free(view->search_sexp, r);
 
 	return retval;
 }
@@ -487,7 +484,7 @@ pas_backend_file_search (PASBackendFile  	      *bf,
 		return;
 
 	if (view->search_sexp)
-		gtk_object_unref(GTK_OBJECT(view->search_sexp));
+		e_sexp_unref(view->search_sexp);
 	view->search_sexp = e_sexp_new();
 
 	for(i=0;i<sizeof(symbols)/sizeof(symbols[0]);i++) {
@@ -514,7 +511,7 @@ pas_backend_file_search (PASBackendFile  	      *bf,
 
 			/* check if the vcard matches the search sexp */
 			if (vcard_matches_search (view, vcard_string)) {
-				cards = g_list_append (cards, strdup(vcard_string));
+				cards = g_list_append (cards, g_strdup (vcard_string));
 			}
 		}
 		
@@ -548,13 +545,8 @@ pas_backend_file_changes_foreach_key (const char *key, gpointer user_data)
 	db_error = db->get (db, &id_dbt, &vcard_dbt, 0);
 	
 	if (db_error == 1) {
-		ECard *ecard;
 		char *id = id_dbt.data;
 		
-		ecard = e_card_new ("");
-		e_card_set_id (ecard, id);
-		
-		ctx->del_cards = g_list_append (ctx->del_cards, e_card_get_vcard (ecard));
 		ctx->del_ids = g_list_append (ctx->del_ids, strdup(id));
 	}
 }
@@ -576,7 +568,7 @@ pas_backend_file_changes (PASBackendFile  	      *bf,
 	if (!bf->priv->loaded)
 		return;
 
-	/* Find the changed ids - FIX ME, patch should not be hard coded */
+	/* Find the changed ids - FIX ME, path should not be hard coded */
 	filename = g_strdup_printf ("%s/evolution/local/Contacts/%s.db", g_get_home_dir (), view->change_id);
 	ehash = e_dbhash_new (filename);
 	g_free (filename);
@@ -640,9 +632,9 @@ pas_backend_file_changes (PASBackendFile  	      *bf,
 		if (ctx->mod_cards != NULL)
 			pas_book_view_notify_change (view->book_view, ctx->mod_cards);
 
-		for (v = ctx->del_cards; v != NULL; v = v->next){
-			char *vcard = v->data;
-			pas_book_view_notify_remove (view->book_view, vcard);
+		for (v = ctx->del_ids; v != NULL; v = v->next){
+			char *id = v->data;
+			pas_book_view_notify_remove (view->book_view, id);
 		}
 		
 		pas_book_view_notify_complete (view->book_view);
@@ -1059,7 +1051,6 @@ pas_backend_file_process_get_changes (PASBackend *backend,
 	ctx.add_ids = NULL;
 	ctx.mod_cards = NULL;
 	ctx.mod_ids = NULL;
-	ctx.del_cards = NULL;
 	ctx.del_ids = NULL;
 	view.search = NULL;
 	view.search_sexp = NULL;
@@ -1093,6 +1084,15 @@ pas_backend_file_extract_path_from_uri (const char *uri)
 	return g_strdup (uri + 5);
 }
 
+static void
+pas_backend_file_process_authenticate_user (PASBackend *backend,
+					    PASBook    *book,
+					    PASRequest *req)
+{
+	pas_book_respond_authenticate_user (book,
+					    GNOME_Evolution_Addressbook_BookListener_Success);
+}
+
 static gboolean
 can_write (PASBackend *backend)
 {
@@ -1164,6 +1164,10 @@ pas_backend_file_process_client_requests (PASBook *book)
 	case GetChanges:
 		pas_backend_file_process_get_changes (backend, book, req);
 		break;
+
+	case AuthenticateUser:
+		pas_backend_file_process_authenticate_user (backend, book, req);
+		break;
 	}
 
 	g_free (req);
@@ -1262,14 +1266,14 @@ pas_backend_file_maybe_upgrade_db (PASBackendFile *bf)
 }
 
 #define INITIAL_VCARD "BEGIN:VCARD\n\
-X-EVOLUTION-FILE-AS:Helix Code, Inc.\n\
+X-EVOLUTION-FILE-AS:Ximian, Inc.\n\
 LABEL;WORK;QUOTED-PRINTABLE:101 Rogers St. Ste. 214=0ACambridge, MA 02142=0AUSA\n\
 TEL;WORK;VOICE:(617) 679-1984\n\
 TEL;WORK;FAX:(617) 679-1949\n\
-EMAIL;INTERNET:hello@helixcode.com\n\
-URL:http://www.helixcode.com/\n\
-ORG:Helix Code, Inc.;\n\
-NOTE:Welcome to the Helix Code Addressbook.\n\
+EMAIL;INTERNET:hello@ximian.com\n\
+URL:http://www.ximian.com/\n\
+ORG:Ximian, Inc.;\n\
+NOTE:Welcome to the Ximian Addressbook.\n\
 END:VCARD"
 
 static gboolean
@@ -1288,9 +1292,20 @@ pas_backend_file_load_uri (PASBackend             *backend,
 		bf->priv->file_db = dbopen (filename, O_RDWR | O_CREAT, 0666, DB_HASH, NULL);
 
 		if (bf->priv->file_db) {
-			char *id;
-			id = do_create(backend, INITIAL_VCARD, NULL);
-			g_free (id);
+			char *create_initial_file;
+			char *dir;
+
+			dir = g_dirname(filename);
+			create_initial_file = g_concat_dir_and_file(dir, "create-initial");
+
+			if (g_file_exists(create_initial_file)) {
+				char *id;
+				id = do_create(backend, INITIAL_VCARD, NULL);
+				g_free (id);
+			}
+
+			g_free(create_initial_file);
+			g_free(dir);
 		}
 	}
 
