@@ -22,6 +22,16 @@
 #undef LDAP_DEBUG
 #endif
 
+#if LDAP_VENDOR_VERSION > 20000
+#define OPENLDAP2
+#else
+#define OPENLDAP1
+#endif
+
+#ifdef OPENLDAP1
+#define LDAP_NAME_ERROR(x) NAME_ERROR(x)
+#endif
+
 #include "pas-backend-ldap.h"
 #include "pas-book.h"
 #include "pas-card-cursor.h"
@@ -32,7 +42,7 @@
 #define LDAP_MAX_SEARCH_RESPONSES 100
 
 /* this really needs addressing */
-#define OBJECT_CLASS "person"
+#define OBJECT_CLASS "inetOrgPerson"
 
 
 static gchar *query_prop_to_ldap(gchar *query_prop);
@@ -105,7 +115,7 @@ struct prop_info {
 	ECardSimpleField field_id;
 	char *query_prop;
 	char *ldap_attr;
-#define PROP_TYPE_NORMAL   0x01
+#define PROP_TYPE_STRING   0x01
 #define PROP_TYPE_LIST     0x02
 #define PROP_DN            0x04
 	int prop_type;
@@ -121,20 +131,53 @@ struct prop_info {
 
 } prop_info[] = {
 
-#define DN_NORMAL_PROP(fid,q,a) {fid, q, a, PROP_TYPE_NORMAL | PROP_DN, NULL}
-#define DN_LIST_PROP(fid,q,a,ctor,ber,cmp) {fid, q, a, PROP_TYPE_LIST | PROP_DN, ctor, ber, cmp}
-#define NORMAL_PROP(fid,q,a) {fid, q, a, PROP_TYPE_NORMAL, NULL}
+#define LIST_PROP(fid,q,a,ctor,ber,cmp) {fid, q, a, PROP_TYPE_LIST, ctor, ber, cmp}
+#define STRING_PROP(fid,q,a) {fid, q, a, PROP_TYPE_STRING}
 
-	DN_NORMAL_PROP (E_CARD_SIMPLE_FIELD_FULL_NAME, "full_name", "cn" ),
-	DN_NORMAL_PROP (E_CARD_SIMPLE_FIELD_FAMILY_NAME, "family_name", "sn" ),
-	NORMAL_PROP    (E_CARD_SIMPLE_FIELD_TITLE,     "title", "title"),
-	DN_NORMAL_PROP (E_CARD_SIMPLE_FIELD_ORG,       "org", "o"),
-	NORMAL_PROP    (E_CARD_SIMPLE_FIELD_PHONE_PRIMARY, "phone", "telephonenumber"),
-	DN_LIST_PROP   (E_CARD_SIMPLE_FIELD_EMAIL, "email", "mail", email_populate_func, email_ber_func, email_compare_func)
+/*  	E_CARD_SIMPLE_FIELD_FILE_AS, */
+	STRING_PROP (E_CARD_SIMPLE_FIELD_FULL_NAME,     "full_name", "cn" ),
+	STRING_PROP (E_CARD_SIMPLE_FIELD_FAMILY_NAME,   "family_name", "sn" ),
+	LIST_PROP   (E_CARD_SIMPLE_FIELD_EMAIL,         "email", "mail", email_populate_func, email_ber_func, email_compare_func),
+/*      don't need E_CARD_SIMPLE_FIELD_EMAIL_2, */
+/*      don't need E_CARD_SIMPLE_FIELD_EMAIL_3, */
 
-#undef DN_NORMAL_PROP
-#undef DN_LIST_PROP
-#undef NORMAL_PROP
+	STRING_PROP (E_CARD_SIMPLE_FIELD_PHONE_PRIMARY, "phone", "telephoneNumber"),
+/*  	E_CARD_SIMPLE_FIELD_PHONE_BUSINESS, */
+	STRING_PROP (E_CARD_SIMPLE_FIELD_PHONE_HOME,    "home_phone", "homePhone"),
+	STRING_PROP (E_CARD_SIMPLE_FIELD_ORG,           "org", "o"),
+	STRING_PROP (E_CARD_SIMPLE_FIELD_ADDRESS_BUSINESS, "business_address", "postalAddress"),
+	STRING_PROP (E_CARD_SIMPLE_FIELD_ADDRESS_HOME, "home_address", "homePostalAddress"),
+	STRING_PROP (E_CARD_SIMPLE_FIELD_PHONE_MOBILE,  "mobile", "mobile"),
+/*      E_CARD_SIMPLE_FIELD_PHONE_CAR, */
+	STRING_PROP (E_CARD_SIMPLE_FIELD_PHONE_BUSINESS_FAX, "business_fax", "facsimileTelephoneNumber"), 
+/*      E_CARD_SIMPLE_FIELD_PHONE_HOME_FAX, */
+/*      E_CARD_SIMPLE_FIELD_PHONE_BUSINESS_2, */
+/*      E_CARD_SIMPLE_FIELD_PHONE_HOME_2, */
+	STRING_PROP (E_CARD_SIMPLE_FIELD_PHONE_ISDN, "isdn", "internationalISDNNumber"), 
+/*      E_CARD_SIMPLE_FIELD_PHONE_OTHER, */
+	STRING_PROP (E_CARD_SIMPLE_FIELD_PHONE_PAGER,   "pager", "pager"),
+/*      E_CARD_SIMPLE_FIELD_ADDRESS_OTHER, */
+	STRING_PROP (E_CARD_SIMPLE_FIELD_URL,           "uri", "labeledURI"),
+	STRING_PROP (E_CARD_SIMPLE_FIELD_ORG_UNIT,      "org_unit",  "ou"),
+	STRING_PROP (E_CARD_SIMPLE_FIELD_OFFICE,        "office",  "roomNumber"),
+	STRING_PROP (E_CARD_SIMPLE_FIELD_TITLE,         "title", "title"),
+/*      E_CARD_SIMPLE_FIELD_ROLE, */
+	STRING_PROP (E_CARD_SIMPLE_FIELD_MANAGER,       "manager",  "manager"),
+	STRING_PROP (E_CARD_SIMPLE_FIELD_ASSISTANT,     "assistant", "secretary"),
+	/* map nickname to displayName */
+	STRING_PROP (E_CARD_SIMPLE_FIELD_NICKNAME,      "nickname",  "displayName"),
+/*      E_CARD_SIMPLE_FIELD_SPOUSE, */
+/*      E_CARD_SIMPLE_FIELD_NOTE, */
+/*      E_CARD_SIMPLE_FIELD_FBURL, */
+/*      E_CARD_SIMPLE_FIELD_ANNIVERSARY, */
+/*      E_CARD_SIMPLE_FIELD_BIRTH_DATE, */
+/*  	E_CARD_SIMPLE_FIELD_MAILER, */
+/*  	E_CARD_SIMPLE_FIELD_NAME_OR_ORG, */
+
+
+#undef STRING_PROP
+#undef LIST_PROP
+#undef STRUCT_PROP
 };
 
 static int num_prop_infos = sizeof(prop_info) / sizeof(prop_info[0]);
@@ -200,18 +243,18 @@ pas_backend_ldap_connect (PASBackendLDAP *bl)
 {
 	PASBackendLDAPPrivate *blpriv = bl->priv;
 
-#ifdef DEBUG
-	{
-		extern int ldap_debug;
-		ldap_debug = LDAP_DEBUG_ANY;
-	}
-#endif
-
 	/* close connection first if it's open first */
 	if (blpriv->ldap)
 		ldap_unbind (blpriv->ldap);
 
 	blpriv->ldap = ldap_open (blpriv->ldap_host, blpriv->ldap_port);
+#ifdef DEBUG
+	{
+		int debug_level = ~0;
+		ldap_set_option (blpriv->ldap, LDAP_OPT_DEBUG_LEVEL, &debug_level);
+	}
+#endif
+
 	if (NULL != blpriv->ldap) {
 		ldap_simple_bind_s(blpriv->ldap,
 				   NULL /*binddn*/, NULL /*passwd*/);
@@ -330,11 +373,7 @@ ldap_error_to_response (int ldap_error)
 {
 	if (ldap_error == LDAP_SUCCESS)
 		return GNOME_Evolution_Addressbook_BookListener_Success;
-#ifdef LDAP_NAME_ERROR
 	else if (LDAP_NAME_ERROR (ldap_error))
-#else
-	else if (NAME_ERROR (ldap_error))
-#endif
 		return GNOME_Evolution_Addressbook_BookListener_CardNotFound;
 	else if (ldap_error == LDAP_INSUFFICIENT_ACCESS)
 		return GNOME_Evolution_Addressbook_BookListener_PermissionDenied;
@@ -346,47 +385,25 @@ ldap_error_to_response (int ldap_error)
 
 
 static char *
-create_dn_from_ecard (ECardSimple *card)
+create_dn_from_ecard (ECardSimple *card, const char *root_dn)
 {
-	char *o, *o_part = NULL;
-	const char *mail;
-	char *mail_part = NULL;
 	char *cn, *cn_part = NULL;
 	char *dn;
 	gboolean need_comma = FALSE;
 
-	o = e_card_simple_get (card, E_CARD_SIMPLE_FIELD_ORG);
-	if (o) {
-		o_part = g_strdup_printf ("o=%s", o);
-		need_comma = TRUE;
-	}
-	else {
-		o_part = g_strdup ("");
-		need_comma = FALSE;
-	}
-
-	mail = e_card_simple_get_email (card, E_CARD_SIMPLE_EMAIL_ID_EMAIL);
-	if (mail) {
-		mail_part = g_strdup_printf ("mail=%s%s", mail, need_comma ? "," : "");
-		need_comma = TRUE;
-	}
-	else {
-		mail_part = g_strdup ("");
-	}
-
 	cn = e_card_simple_get (card, E_CARD_SIMPLE_FIELD_FULL_NAME);
 	if (cn) {
-		cn_part = g_strdup_printf ("cn=\"%s\"%s", cn, need_comma ? "," : "");
+		cn_part = g_strdup_printf ("cn=%s%s", cn, need_comma ? "," : "");
 	}
 	else {
 		cn_part = g_strdup ("");
 	}
 
-	dn = g_strdup_printf ("%s%s%s", cn_part, mail_part, o_part);
+	dn = g_strdup_printf ("%s%s%s", cn_part,
+			      (root_dn && strlen(root_dn)) ? "," : "",
+			      (root_dn && strlen(root_dn)) ? root_dn: "");
 
 	g_free (cn_part);
-	g_free (mail_part);
-	g_free (o_part);
 
 	g_print ("generated dn: %s\n", dn);
 
@@ -461,17 +478,19 @@ build_mods_from_ecards (ECardSimple *current, ECardSimple *new, gboolean *new_dn
 			
 			mod->mod_type = g_strdup (prop_info[i].ldap_attr);
 
-			if (prop_info[i].prop_type & PROP_TYPE_NORMAL) {
+			if (prop_info[i].prop_type & PROP_TYPE_STRING) {
 				mod->mod_values = g_new (char*, 2);
-				mod->mod_values[0] = e_card_simple_get (new, prop_info[i].field_id);
+				mod->mod_values[0] = new_prop;
 				mod->mod_values[1] = NULL;
 			}
-			else {
+			else { /* PROP_TYPE_LIST */
+				mod->mod_op |= LDAP_MOD_BVALUES;
 				mod->mod_bvalues = prop_info[i].ber_func (new);
 			}
 
 			g_ptr_array_add (result, mod);
 		}
+		
 	}
 
 	/* NULL terminate the list of modifications */
@@ -516,31 +535,44 @@ create_card_handler (PASBackend *backend, LDAPOp *op)
 	LDAPMod *objectclass_mod;
 	LDAP *ldap;
 
+	printf ("vcard = %s\n", create_op->vcard);
+
 	new_ecard = e_card_new (create_op->vcard);
 	new_card = e_card_simple_new (new_ecard);
 
-	dn = create_dn_from_ecard (new_card);
+	dn = create_dn_from_ecard (new_card, bl->priv->ldap_rootdn);
 
 	ldap = bl->priv->ldap;
 
 	/* build our mods */
 	mod_array = build_mods_from_ecards (NULL, new_card, NULL);
-	objectclass_mod = g_new (LDAPMod, 1);
 
+	/* remove the NULL at the end */
+	g_ptr_array_remove (mod_array, NULL);
+
+	objectclass_mod = g_new (LDAPMod, 1);
 	objectclass_mod->mod_op = LDAP_MOD_ADD;
-	objectclass_mod->mod_type = g_strdup ("objectclass");
+	objectclass_mod->mod_type = g_strdup ("objectClass");
 	objectclass_mod->mod_values = g_new (char*, 1);
 	objectclass_mod->mod_values[0] = g_strdup (OBJECT_CLASS);
 	objectclass_mod->mod_values[1] = NULL;
-
 	g_ptr_array_add (mod_array, objectclass_mod);
 
+	g_ptr_array_add (mod_array, NULL);
+
 	ldap_mods = (LDAPMod**)mod_array->pdata;
 
+	if (op->view)
+		pas_book_view_notify_status_message (op->view, "Adding card to LDAP server");
+
 	/* actually perform the ldap add */
 	ldap_error = ldap_add_s (ldap, dn, ldap_mods);
 
-	g_print ("ldap_add_s returned 0x%x (%s) status\n", ldap_error, ldap_err2string(ldap_error));
+	if (op->view)
+		pas_book_view_notify_status_message (op->view, "");
+
+	if (ldap_error != LDAP_SUCCESS)
+		ldap_perror (ldap, "ldap_add_s");
 
 	/* and clean up */
 	free_mods (mod_array);
@@ -573,8 +605,17 @@ pas_backend_ldap_process_create_card (PASBackend *backend,
 				      PASRequest *req)
 {
 	LDAPCreateOp *create_op = g_new (LDAPCreateOp, 1);
+	PASBackendLDAP *bl = PAS_BACKEND_LDAP (backend);
+	PASBookView *book_view = NULL;
 
-	ldap_op_init ((LDAPOp*)create_op, backend, book, NULL, create_card_handler, create_card_dtor);
+	if (bl->priv->book_views) {
+		PASBackendLDAPBookView *v = bl->priv->book_views->data;
+		book_view = v->book_view;
+	}
+
+	ldap_op_init ((LDAPOp*)create_op, backend, book,
+		      book_view,
+		      create_card_handler, create_card_dtor);
 
 	create_op->vcard = req->vcard;
 
@@ -596,6 +637,8 @@ remove_card_handler (PASBackend *backend, LDAPOp *op)
 	int ldap_error;
 
 	ldap_error = ldap_delete_s (bl->priv->ldap, remove_op->id);
+	if (ldap_error != LDAP_SUCCESS)
+		ldap_perror (bl->priv->ldap, "ldap_delete_s");
 
 	response = ldap_error_to_response (ldap_error);
 
@@ -621,8 +664,17 @@ pas_backend_ldap_process_remove_card (PASBackend *backend,
 				      PASRequest *req)
 {
 	LDAPRemoveOp *remove_op = g_new (LDAPRemoveOp, 1);
+	PASBackendLDAP *bl = PAS_BACKEND_LDAP (backend);
+	PASBookView *book_view = NULL;
 
-	ldap_op_init ((LDAPOp*)remove_op, backend, book, NULL, remove_card_handler, remove_card_dtor);
+	if (bl->priv->book_views) {
+		PASBackendLDAPBookView *v = bl->priv->book_views->data;
+		book_view = v->book_view;
+	}
+
+	ldap_op_init ((LDAPOp*)remove_op, backend, book,
+		      book_view,
+		      remove_card_handler, remove_card_dtor);
 
 	remove_op->id = req->id;
 
@@ -635,6 +687,50 @@ typedef struct {
 	char *vcard;
 } LDAPModifyOp;
 
+static ECardSimple *
+search_for_dn (PASBackendLDAP *bl, const char *dn)
+{
+#if 0
+	char **attrs;
+#endif
+	char *query;
+	LDAP *ldap = bl->priv->ldap;
+	LDAPMessage    *res, *e;
+	ECardSimple *result = NULL;
+
+#if 0
+	/* this is broken because if we (say) modify the cn and can't
+           modify the dn (because it overlaps), we lose the ability to
+           search by that component of the dn. */
+	attrs = ldap_explode_dn (dn, 0);
+	query = g_strdup_printf ("(%s)", attrs[0]);
+	printf ("searching for %s\n", query);
+#else
+	query = g_strdup ("(objectclass=*)");
+#endif
+
+	if (ldap_search_s (ldap,
+			   bl->priv->ldap_rootdn,
+			   bl->priv->ldap_scope,
+			   query,
+			   NULL, 0, &res) != -1) {
+		e = ldap_first_entry (ldap, res);
+		while (NULL != e) {
+			if (!strcmp (ldap_get_dn (ldap, e), dn)) {
+				printf ("found it\n");
+				result = build_card_from_entry (ldap, e);
+				break;
+			}
+			e = ldap_next_entry (ldap, e);
+		}
+
+		ldap_msgfree(res);
+	}
+
+	g_free (query);
+	return result;
+}
+
 static gboolean
 modify_card_handler (PASBackend *backend, LDAPOp *op)
 {
@@ -643,63 +739,44 @@ modify_card_handler (PASBackend *backend, LDAPOp *op)
 	ECard *new_ecard;
 	char *id;
 	int response;
-	int            ldap_error;
-	LDAPMessage    *res, *e;
-	char *query;
+	int            ldap_error = LDAP_SUCCESS;
 	GPtrArray *mod_array;
 	LDAPMod **ldap_mods;
 	LDAP *ldap;
+	ECardSimple *current_card;
 
 	new_ecard = e_card_new (modify_op->vcard);
 	id = e_card_get_id(new_ecard);
 
 	ldap = bl->priv->ldap;
 
-	/* we don't get sent the original vcard along with the new one
-           in this call, so we have to query the ldap server for the
-           original record so we can compute our delta. */
-	query = g_strdup_printf ("(dn=%s)", id);
-	ldap_error = ldap_search_s (ldap,
-				    bl->priv->ldap_rootdn,
-				    bl->priv->ldap_scope,
-				    query, NULL, 0, &res);
+	current_card = search_for_dn (bl, id);
 
-	if (ldap_error == LDAP_SUCCESS) {
-		/* get the single card from the list (we're guaranteed
-                   either 1 or 0 since we looked up by dn, which is
-                   unique) */
-		e = ldap_first_entry (ldap, res);
-		if (e)	{
-			ECardSimple *new_card = e_card_simple_new (new_ecard);
-			ECardSimple *current_card = build_card_from_entry (ldap, e);
-			gboolean need_new_dn;
+	if (current_card) {
+		ECardSimple *new_card = e_card_simple_new (new_ecard);
+		gboolean need_new_dn;
 
-			/* build our mods */
-			mod_array = build_mods_from_ecards (current_card, new_card, &need_new_dn);
-			if (mod_array->len > 0) {
-				ldap_mods = (LDAPMod**)mod_array->pdata;
+		/* build our mods */
+		mod_array = build_mods_from_ecards (current_card, new_card, &need_new_dn);
+		if (mod_array->len > 0) {
+			ldap_mods = (LDAPMod**)mod_array->pdata;
 
-				/* actually perform the ldap modify */
-				ldap_error = ldap_modify_s (ldap, id, ldap_mods);
-				g_print ("ldap_modify_s returned 0x%x (%s) status\n", ldap_error, ldap_err2string(ldap_error));
-			}
-			else {
-				g_print ("modify list empty.  on modification sent\n");
-			}
-
-			/* and clean up */
-			free_mods (mod_array);
-			gtk_object_unref (GTK_OBJECT(new_card));
-			gtk_object_unref (GTK_OBJECT(current_card));
+			/* actually perform the ldap modify */
+			ldap_error = ldap_modify_s (ldap, id, ldap_mods);
+			if (ldap_error != LDAP_SUCCESS)
+				ldap_perror (ldap, "ldap_modify_s");
 		}
 		else {
-			g_print ("didn't find original card\n");
+			g_print ("modify list empty.  no modification sent\n");
 		}
 
-		ldap_msgfree(res);
+		/* and clean up */
+		free_mods (mod_array);
+		gtk_object_unref (GTK_OBJECT(new_card));
+		gtk_object_unref (GTK_OBJECT(current_card));
 	}
 	else {
-		g_print ("ldap_search_s returned 0x%x (%s) status\n", ldap_error, ldap_err2string(ldap_error));
+		g_print ("didn't find original card\n");
 	}
 
 	response = ldap_error_to_response (ldap_error);
@@ -725,8 +802,17 @@ pas_backend_ldap_process_modify_card (PASBackend *backend,
 				      PASRequest *req)
 {
 	LDAPModifyOp *modify_op = g_new (LDAPModifyOp, 1);
+	PASBackendLDAP *bl = PAS_BACKEND_LDAP (backend);
+	PASBookView *book_view = NULL;
 
-	ldap_op_init ((LDAPOp*)modify_op, backend, book, NULL, modify_card_handler, modify_card_dtor);
+	if (bl->priv->book_views) {
+		PASBackendLDAPBookView *v = bl->priv->book_views->data;
+		book_view = v->book_view;
+	}
+
+	ldap_op_init ((LDAPOp*)modify_op, backend, book,
+		      book_view,
+		      modify_card_handler, modify_card_dtor);
 
 	modify_op->vcard = req->vcard;
 
@@ -922,8 +1008,10 @@ email_ber_func(ECardSimple *card)
 
 	j = 0;
 	for (i = 0; i < 3; i ++) {
-		if (emails[i])
-			result[j++]->bv_val = g_strdup (emails[i]);
+		if (emails[i]) {
+			result[j]->bv_val = g_strdup (emails[i]);
+			result[j++]->bv_len = strlen (emails[i]) + 1;
+		}
 	}
 
 	result[num] = NULL;
@@ -984,7 +1072,7 @@ func_and(struct _ESExp *f, int argc, struct _ESExpResult **argv, void *data)
 		g_free (strings);
 	}
 
-	r = e_sexp_result_new(ESEXP_RES_BOOL);
+	r = e_sexp_result_new(f, ESEXP_RES_BOOL);
 	r->value.bool = FALSE;
 
 	return r;
@@ -1019,7 +1107,7 @@ func_or(struct _ESExp *f, int argc, struct _ESExpResult **argv, void *data)
 		g_free (strings);
 	}
 
-	r = e_sexp_result_new(ESEXP_RES_BOOL);
+	r = e_sexp_result_new(f, ESEXP_RES_BOOL);
 	r->value.bool = FALSE;
 
 	return r;
@@ -1038,7 +1126,7 @@ func_not(struct _ESExp *f, int argc, struct _ESExpResult **argv, void *data)
 		g_free (term);
 	}
 
-	r = e_sexp_result_new(ESEXP_RES_BOOL);
+	r = e_sexp_result_new(f, ESEXP_RES_BOOL);
 	r->value.bool = FALSE;
 
 	return r;
@@ -1110,7 +1198,7 @@ func_contains(struct _ESExp *f, int argc, struct _ESExpResult **argv, void *data
 		}
 	}
 
-	r = e_sexp_result_new(ESEXP_RES_BOOL);
+	r = e_sexp_result_new(f, ESEXP_RES_BOOL);
 	r->value.bool = FALSE;
 
 	return r;
@@ -1135,7 +1223,7 @@ func_is(struct _ESExp *f, int argc, struct _ESExpResult **argv, void *data)
 							       ldap_attr, str));
 	}
 
-	r = e_sexp_result_new(ESEXP_RES_BOOL);
+	r = e_sexp_result_new(f, ESEXP_RES_BOOL);
 	r->value.bool = FALSE;
 
 	return r;
@@ -1165,7 +1253,7 @@ func_beginswith(struct _ESExp *f, int argc, struct _ESExpResult **argv, void *da
 							       str));
 	}
 
-	r = e_sexp_result_new(ESEXP_RES_BOOL);
+	r = e_sexp_result_new(f, ESEXP_RES_BOOL);
 	r->value.bool = FALSE;
 
 	return r;
@@ -1195,7 +1283,7 @@ func_endswith(struct _ESExp *f, int argc, struct _ESExpResult **argv, void *data
 							       str));
 	}
 
-	r = e_sexp_result_new(ESEXP_RES_BOOL);
+	r = e_sexp_result_new(f, ESEXP_RES_BOOL);
 	r->value.bool = FALSE;
 
 	return r;
@@ -1243,8 +1331,8 @@ pas_backend_ldap_build_query (gchar *query)
 
 	r = e_sexp_eval(sexp);
 
-	gtk_object_unref(GTK_OBJECT(sexp));
-	e_sexp_result_free(r);
+	e_sexp_result_free(sexp, r);
+	e_sexp_unref (sexp);
 
 	if (list->next) {
 		g_warning ("conversion to ldap query string failed");
@@ -1304,27 +1392,31 @@ build_card_from_entry (LDAP *ldap, LDAPMessage *e)
 			char **values;
 			values = ldap_get_values (ldap, e, attr);
 
-			if (info->prop_type & PROP_TYPE_NORMAL) {
+			if (values) {
+				if (info->prop_type & PROP_TYPE_STRING) {
 				/* if it's a normal property just set the string */
-				e_card_simple_set (card, info->field_id, values[0]);
+					e_card_simple_set (card, info->field_id, values[0]);
 
-			}
-			else if (info->prop_type & PROP_TYPE_LIST) {
+				}
+				else if (info->prop_type & PROP_TYPE_LIST) {
 				/* if it's a list call the ecard-populate function,
 				   which calls gtk_object_set to set the property */
-				info->populate_ecard_func(card,
-							  values);
-			}
+					info->populate_ecard_func(card,
+								  values);
+				}
 
-			ldap_value_free (values);
+				ldap_value_free (values);
+			}
 		}
 	}
 
+#ifndef OPENLDAP2
 	/* if ldap->ld_errno == LDAP_DECODING_ERROR there was an
 	   error decoding an attribute, and we shouldn't free ber,
 	   since the ldap library already did it. */
 	if (ldap->ld_errno != LDAP_DECODING_ERROR && ber)
 		ber_free (ber, 0);
+#endif
 
 	e_card_simple_sync_card (card);
 
@@ -1408,16 +1500,34 @@ ldap_search_handler (PASBackend *backend, LDAPOp *op)
 		PASBackendLDAP *bl = PAS_BACKEND_LDAP (backend);
 		PASBackendLDAPBookView *view = search_op->view;
 		LDAP *ldap = bl->priv->ldap;
+		int ldap_err;
 
+#ifdef OPENLDAP2
+		ldap_err = ldap_search_ext (ldap, bl->priv->ldap_rootdn,
+					    bl->priv->ldap_scope,
+					    search_op->ldap_query,
+					    NULL, 0,
+					    NULL, /* XXX */
+					    NULL, /* XXX */
+					    NULL,
+					    LDAP_MAX_SEARCH_RESPONSES, &view->search_msgid);
+
+		if (ldap_err != LDAP_SUCCESS) {
+			pas_book_view_notify_status_message (view->book_view, ldap_err2string(ldap_err));
+			return TRUE; /* act synchronous in this case */
+		}
+#else
 		ldap->ld_sizelimit = LDAP_MAX_SEARCH_RESPONSES;
 		ldap->ld_deref = LDAP_DEREF_ALWAYS;
+		view->search_msgid = ldap_search (ldap, bl->priv->ldap_rootdn,
+						  bl->priv->ldap_scope,
+						  search_op->ldap_query,
+						  NULL, 0);
+		ldap_err = ldap->ld_errno;
+#endif
 
-		if ((view->search_msgid = ldap_search (ldap,
-						       bl->priv->ldap_rootdn,
-						       bl->priv->ldap_scope,
-						       search_op->ldap_query,
-						       NULL, 0)) == -1) {
-			pas_book_view_notify_status_message (view->book_view, ldap_err2string(ldap->ld_errno));
+		if (view->search_msgid == -1) {
+			pas_book_view_notify_status_message (view->book_view, ldap_err2string(ldap_err));
 			return TRUE; /* act synchronous in this case */
 		}
 		else {
@@ -1522,6 +1632,20 @@ pas_backend_ldap_process_check_connection (PASBackend *backend,
 	pas_book_report_connection (book, bl->priv->connected);
 }
 
+static void
+pas_backend_ldap_process_authenticate_user (PASBackend *backend,
+					    PASBook    *book,
+					    PASRequest *req)
+{
+	PASBackendLDAP *bl = PAS_BACKEND_LDAP (backend);
+	int ldap_error = ldap_simple_bind_s(bl->priv->ldap,
+					    req->user,
+					    req->passwd);
+
+	pas_book_respond_authenticate_user (book,
+				    ldap_error_to_response (ldap_error));
+}
+
 static gboolean
 pas_backend_ldap_can_write (PASBook *book)
 {
@@ -1575,6 +1699,10 @@ pas_backend_ldap_process_client_requests (PASBook *book)
 	case GetChanges:
 		/* FIXME: Code this. */
 		break;
+
+	case AuthenticateUser:
+		pas_backend_ldap_process_authenticate_user (backend, book, req);
+		break;
 	}
 
 	g_free (req);
