@@ -22,7 +22,7 @@
 
 #include <config.h>
 #include <gnome.h>
-#include <e-util/e-unicode.h>
+#include <gal/widgets/e-unicode.h>
 #include <e-contact-editor-fullname.h>
 
 static void e_contact_editor_fullname_init		(EContactEditorFullname		 *card);
@@ -205,8 +205,11 @@ static void
 extract_info(EContactEditorFullname *editor)
 {
 	ECardName *name = editor->name;
-	if (!name)
+	if (!name) {
 		name = e_card_name_new();
+		editor->name = name;
+	}
+
 	name->prefix     = extract_field(editor, "entry-title" );
 	name->given      = extract_field(editor, "entry-first" );
 	name->additional = extract_field(editor, "entry-middle");
