@@ -25,10 +25,10 @@
 #include "e-contact-editor.h"
 #include <e-contact-editor-fullname.h>
 #include <e-contact-editor-address.h>
-#include <e-contact-editor-categories.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gdk-pixbuf/gnome-canvas-pixbuf.h>
 #include "e-util/e-gui-utils.h"
+#include <gal/widgets/e-categories.h>
 #include <gal/widgets/e-gui-utils.h>
 #include <gal/widgets/e-unicode.h>
 #include <e-contact-save-as.h>
@@ -569,7 +569,7 @@ categories_clicked(GtkWidget *button, EContactEditor *editor)
 		gtk_object_get(GTK_OBJECT(editor->card),
 			       "categories", &categories,
 			       NULL);
-	dialog = GNOME_DIALOG(e_contact_editor_categories_new(categories));
+	dialog = GNOME_DIALOG(e_categories_new(categories));
 	gtk_widget_show(GTK_WIDGET(dialog));
 	result = gnome_dialog_run (dialog);
 	g_free (categories);
@@ -716,18 +716,26 @@ static void
 delete_cb (GtkWidget *widget, gpointer data)
 {
 	EContactEditor *ce = E_CONTACT_EDITOR (data);
+	ECard *card = ce->card;
+	ECardSimple *simple = ce->simple;
+
+	gtk_object_ref(GTK_OBJECT(card));
+	gtk_object_ref(GTK_OBJECT(simple));
 
 	if (e_contact_editor_confirm_delete(GTK_WINDOW(ce->app))) {
 
 		extract_info (ce);
-		e_card_simple_sync_card (ce->simple);
+		e_card_simple_sync_card (simple);
 		
 		if (!ce->is_new_card)
 			gtk_signal_emit (GTK_OBJECT (ce), contact_editor_signals[DELETE_CARD],
-					 ce->card);
+					 card);
 		
 		file_close_cb(widget, data);
 	}
+
+	gtk_object_unref(GTK_OBJECT(card));
+	gtk_object_unref(GTK_OBJECT(simple));
 }
 
 /* Emits the signal to request printing a card */
