@@ -24,8 +24,8 @@
 #include <gnome.h>
 #include <glade/glade.h>
 #include <bonobo.h>
-#include <ebook/e-card.h>
-#include <ebook/e-card-simple.h>
+#include "addressbook/backend/ebook/e-card.h"
+#include "addressbook/backend/ebook/e-card-simple.h"
 
 #ifdef __cplusplus
 extern "C" {
@@ -60,7 +60,7 @@ struct _EContactEditor
 	ECardSimple *simple;
 
 	/* UI handler */
-	BonoboUIHandler *uih;
+	BonoboUIComponent *uic;
 	
 	GladeXML *gui;
 	GtkWidget *app;
@@ -104,7 +104,7 @@ EContactEditor *e_contact_editor_new (ECard *card, gboolean is_new_card);
 GtkType    e_contact_editor_get_type (void);
 
 
-gboolean e_contact_editor_confirm_delete(void);
+gboolean e_contact_editor_confirm_delete(GtkWindow *parent);
 
 #ifdef __cplusplus
 }
