@@ -30,18 +30,22 @@
 #include "evolution-shell-component.h"
 #include "evolution-storage.h"
 
+#include "addressbook-storage.h"
 #include "addressbook-component.h"
 #include "addressbook.h"
-#include "e-ldap-storage.h"
 
 
 
 #define GNOME_EVOLUTION_ADDRESSBOOK_COMPONENT_FACTORY_ID "OAFIID:GNOME_Evolution_Addressbook_ShellComponentFactory"
 
-/* Nasty hack for filters to be able to find folders */
-/* Not even used with addressbook??? */
 EvolutionShellClient *global_shell_client;
 
+EvolutionShellClient *
+addressbook_component_get_shell_client  (void)
+{
+	return global_shell_client;
+}
+
 static BonoboGenericFactory *factory = NULL;
 
 static const EvolutionShellComponentFolderType folder_types[] = {
@@ -72,6 +76,26 @@ create_view (EvolutionShellComponent *shell_component,
 	return EVOLUTION_SHELL_COMPONENT_OK;
 }
 
+static void
+create_folder (EvolutionShellComponent *shell_component,
+	       const char *physical_uri,
+	       const char *type,
+	       const GNOME_Evolution_ShellComponentListener listener,
+	       void *closure)
+{
+	CORBA_Environment ev;
+	GNOME_Evolution_ShellComponentListener_Result result;
+
+	if (g_strcasecmp (type, "contacts") != 0)
+		result = GNOME_Evolution_ShellComponentListener_UNSUPPORTED_TYPE;
+	else 
+		result = GNOME_Evolution_ShellComponentListener_OK;
+
+	CORBA_exception_init(&ev);
+	GNOME_Evolution_ShellComponentListener_notifyResult(listener, result, &ev);
+	CORBA_exception_free(&ev);
+}
+
 static int owner_count = 0;
 
 static void
@@ -84,8 +108,8 @@ owner_set_cb (EvolutionShellComponent *shell_component,
 
 	if (global_shell_client == NULL)
 		global_shell_client = shell_client;
-	
-	setup_ldap_storage (shell_component, evolution_homedir);
+
+	addressbook_storage_setup (shell_component, evolution_homedir);
 }
 
 static void
@@ -107,7 +131,7 @@ factory_fn (BonoboGenericFactory *factory,
 {
 	EvolutionShellComponent *shell_component;
 
-	shell_component = evolution_shell_component_new (folder_types, create_view, NULL, NULL, NULL, NULL);
+	shell_component = evolution_shell_component_new (folder_types, create_view, create_folder, NULL, NULL, NULL, NULL);
 
 	gtk_signal_connect (GTK_OBJECT (shell_component), "owner_set",
 			    GTK_SIGNAL_FUNC (owner_set_cb), NULL);
