@@ -22,14 +22,14 @@
 #include "addressbook/gui/search/e-addressbook-search-dialog.h"
 
 #include "addressbook/gui/widgets/e-addressbook-view.h"
-#include "filter/e-search-bar.h"
+#include <widgets/misc/e-search-bar.h>
 
 #include <select-names/e-select-names.h>
 #include <select-names/e-select-names-manager.h>
 
 #include "e-contact-editor.h"
 #include "e-contact-save-as.h"
-#include "e-ldap-server-dialog.h"
+#include "addressbook-config.h"
 
 #include <addressbook/printing/e-contact-print.h>
 
@@ -44,6 +44,7 @@ typedef struct {
 	BonoboControl *control;
 	BonoboPropertyBag *properties;
 	char *uri;
+	char *passwd;
 } AddressbookView;
 
 static void
@@ -127,45 +128,12 @@ new_contact_cb (BonoboUIComponent *uih, void *user_data, const char *path)
 	gtk_object_sink(GTK_OBJECT(card));
 }
 
-#ifdef HAVE_LDAP
 static void
-null_cb (EBook *book, EBookStatus status, gpointer closure)
+config_cb (BonoboUIComponent *uih, void *user_data, const char *path)
 {
+	addressbook_config (NULL /* XXX */);
 }
 
-static void
-new_server_cb (BonoboUIComponent *uih, void *user_data, const char *path)
-{
-	ELDAPServer *server = g_new (ELDAPServer, 1);
-	EBook *book;
-	AddressbookView *view = (AddressbookView *) user_data;
-
-	/* fill in the defaults */
-	server->name = g_strdup("");
-	server->host = g_strdup("");
-	server->port = g_strdup_printf("%d", 389);
-	server->description = g_strdup("");
-	server->rootdn = g_strdup("");
-	server->uri = g_strdup_printf ("ldap://%s:%s/%s", server->host, server->port, server->rootdn);
-	e_ldap_server_editor_show (server);
-
-	gtk_object_get(GTK_OBJECT(view->view),
-		       "book", &book,
-		       NULL);
-
-	g_assert (E_IS_BOOK (book));
-	
-	/* write out the new server info */
-	e_ldap_storage_add_server (server);
-
-	/* now update the view */
-	e_book_unload_uri (book);
-	if (! e_book_load_uri (book, server->uri, null_cb, NULL)) {
-		g_warning ("error calling load_uri!\n");
-	}
-}
-#endif
-
 static void
 search_cb (BonoboUIComponent *uih, void *user_data, const char *path)
 {
@@ -302,18 +270,47 @@ BonoboUIVerb verbs [] = {
 	BONOBO_UI_UNSAFE_VERB ("ViewNewContact", new_contact_cb),
 	BONOBO_UI_UNSAFE_VERB ("ToolSearch", search_cb),
 
+	BONOBO_UI_UNSAFE_VERB ("AddressbookConfig", config_cb),
+
 	BONOBO_UI_UNSAFE_VERB ("ContactNew", new_contact_cb),
 /*	BONOBO_UI_UNSAFE_VERB ("ContactFind", find_contact_cb),*/
 	BONOBO_UI_UNSAFE_VERB ("ContactDelete", delete_contact_cb),
 	BONOBO_UI_UNSAFE_VERB ("ContactViewAll", show_all_contacts_cb),
 	BONOBO_UI_UNSAFE_VERB ("ContactStop", stop_loading_cb),
-#ifdef HAVE_LDAP
-	BONOBO_UI_UNSAFE_VERB ("ContactNewServer", new_server_cb),
-#endif
 	
 	BONOBO_UI_VERB_END
 };
 
+static void
+set_pixmap (BonoboUIComponent *uic,
+	    const char        *xml_path,
+	    const char        *icon)
+{
+	char *path;
+	GdkPixbuf *pixbuf;
+
+	path = g_concat_dir_and_file (EVOLUTION_DATADIR "/images/evolution", icon);
+
+	pixbuf = gdk_pixbuf_new_from_file (path);
+	g_return_if_fail (pixbuf != NULL);
+
+	bonobo_ui_util_set_pixbuf (uic, xml_path, pixbuf);
+
+	gdk_pixbuf_unref (pixbuf);
+
+	g_free (path);
+}
+
+static void
+update_pixmaps (BonoboUIComponent *uic)
+{
+	set_pixmap (uic, "/menu/File/Print/Print", "16_print.xpm"); /* FIXME it doesn't seem to work */
+	set_pixmap (uic, "/menu/Settings/AddressbookConfig", "16_configure_addressbook.xpm");
+
+	set_pixmap (uic, "/Toolbar/View All", "24_all_contacts.xpm");
+	set_pixmap (uic, "/Toolbar/Find", "24_find_contact.xpm"); 
+}
+
 static void
 control_activate (BonoboControl     *control,
 		  BonoboUIComponent *uic,
@@ -333,16 +330,13 @@ control_activate (BonoboControl     *control,
 	bonobo_ui_util_set_ui (uic, EVOLUTION_DATADIR,
 			       "evolution-addressbook.xml",
 			       "evolution-addressbook");
-#ifdef HAVE_LDAP
-	bonobo_ui_util_set_ui (uic, EVOLUTION_DATADIR,
-			       "evolution-addressbook-ldap.xml",
-			       "evolution-addressbook");
-#endif
 
 	e_addressbook_view_setup_menus (view->view, uic);
 
 	update_view_type (view);
 
+	update_pixmaps (uic);
+
 	bonobo_ui_component_thaw (uic, NULL);
 }
 
@@ -367,19 +361,73 @@ addressbook_view_free(AddressbookView *view)
 {
 	if (view->properties)
 		bonobo_object_unref(BONOBO_OBJECT(view->properties));
+	g_free(view->passwd);
 	g_free(view->uri);
 	g_free(view);
 }
 
+static void
+book_auth_cb (EBook *book, EBookStatus status, gpointer closure)
+{
+	AddressbookView *view = closure;
+	if (status == E_BOOK_STATUS_SUCCESS) {
+		gtk_object_set(GTK_OBJECT(view->view),
+			       "book", book,
+			       NULL);
+	}
+	else {
+		/* pop up a nice dialog, or redo the authentication
+                   bit some number of times. */
+	}
+}
+
+static void
+passwd_cb (gchar *string, gpointer data)
+{
+	AddressbookView *view = (AddressbookView*)data;
+
+	view->passwd = g_strdup (string);
+}
+
 static void
 book_open_cb (EBook *book, EBookStatus status, gpointer closure)
 {
 	if (status == E_BOOK_STATUS_SUCCESS) {
 		AddressbookView *view = closure;
+		AddressbookSource *source;
 
+		/* check if the addressbook needs authentication */
+
+		source = addressbook_storage_get_source_by_uri (view->uri);
+		if (source &&
+		    source->type == ADDRESSBOOK_SOURCE_LDAP &&
+		    source->ldap.auth == ADDRESSBOOK_LDAP_AUTH_SIMPLE) {
+			int button;
+			char *msg = g_strdup_printf (_("Enter password for %s"), source->ldap.binddn);
+			/* give a password prompt for the binddn */
+			GtkWidget *dialog = gnome_request_dialog (TRUE, msg, NULL,
+								  0, passwd_cb, view, NULL); 
+			
+			button = gnome_dialog_run_and_close (GNOME_DIALOG (dialog));
+
+			if (button == 0 && *(view->passwd)) {
+				e_book_authenticate_user (book, source->ldap.binddn, view->passwd,
+							  book_auth_cb, closure);
+				memset (view->passwd, 0, strlen (view->passwd)); /* clear out the passwd */
+				g_free (view->passwd);
+				view->passwd = NULL;
+				return;
+			}
+		}
+
+
+		/* if they either didn't configure the source to use
+                   authentication, or they canceled the dialog,
+                   proceed without authenticating */
 		gtk_object_set(GTK_OBJECT(view->view),
 			       "book", book,
 			       NULL);
+
 	} else {
 		GtkWidget *warning_dialog, *label, *href;
         	warning_dialog = gnome_dialog_new (
@@ -425,6 +473,7 @@ static void
 get_prop (BonoboPropertyBag *bag,
 	  BonoboArg         *arg,
 	  guint              arg_id,
+	  CORBA_Environment *ev,
 	  gpointer           user_data)
 {
 	AddressbookView *view = user_data;
@@ -447,6 +496,7 @@ static void
 set_prop (BonoboPropertyBag *bag,
 	  const BonoboArg   *arg,
 	  guint              arg_id,
+	  CORBA_Environment *ev,
 	  gpointer           user_data)
 {
 	AddressbookView *view = user_data;
@@ -471,9 +521,20 @@ set_prop (BonoboPropertyBag *bag,
 		view->uri = g_strdup(BONOBO_ARG_GET_STRING (arg));
 		
 		if (!strncmp (view->uri, "file:", 5)) {
-			char *file_name = g_concat_dir_and_file(view->uri + 7, "addressbook.db");
-			uri_data = g_strdup_printf("file://%s", file_name);
-			g_free(file_name);
+
+			if (strlen (view->uri + 7) > 3
+			    && !strcmp (view->uri + strlen(view->uri) - 3, ".db")) {
+				/* it's a .db file */
+				uri_data = g_strdup (view->uri);
+			}
+			else {
+				char *file_name;
+				/* we assume it's a dir and glom addressbook.db onto the end. */
+				file_name = g_concat_dir_and_file(view->uri + 7, "addressbook.db");
+				uri_data = g_strdup_printf("file://%s", file_name);
+				g_free(file_name);
+			}
+
 		}
 		else {
 			uri_data = g_strdup (view->uri);
@@ -576,6 +637,63 @@ addressbook_query_changed (ESearchBar *esb, AddressbookView *view)
 	g_free (search_word);
 }
 
+static GNOME_Evolution_ShellView
+retrieve_shell_view_interface_from_control (BonoboControl *control)
+{
+	Bonobo_ControlFrame control_frame;
+	GNOME_Evolution_ShellView shell_view_interface;
+	CORBA_Environment ev;
+
+	shell_view_interface = gtk_object_get_data (GTK_OBJECT (control),
+						    "shell_view_interface");
+
+	if (shell_view_interface)
+		return shell_view_interface;
+
+	control_frame = bonobo_control_get_control_frame (control);
+
+	if (control_frame == NULL)
+		return CORBA_OBJECT_NIL;
+
+	CORBA_exception_init (&ev);
+	shell_view_interface = Bonobo_Unknown_queryInterface (control_frame,
+							       "IDL:GNOME/Evolution/ShellView:1.0",
+							       &ev);
+	CORBA_exception_free (&ev);
+
+	if (shell_view_interface != CORBA_OBJECT_NIL)
+		gtk_object_set_data (GTK_OBJECT (control),
+				     "shell_view_interface",
+				     shell_view_interface);
+	else
+		g_warning ("Control frame doesn't have Evolution/ShellView.");
+
+	return shell_view_interface;
+}
+
+static void
+set_status_message (EAddressbookView *eav, const char *message, AddressbookView *view)
+{
+	CORBA_Environment ev;
+	GNOME_Evolution_ShellView shell_view_interface;
+
+	CORBA_exception_init (&ev);
+
+	shell_view_interface = retrieve_shell_view_interface_from_control (view->control);
+
+	if (message == NULL || message[0] == 0) {
+		printf ("clearing message\n");
+		GNOME_Evolution_ShellView_unsetMessage (shell_view_interface, &ev);
+	}
+	else {
+		printf ("setting message %s\n", message);
+		GNOME_Evolution_ShellView_setMessage (shell_view_interface,
+						      message, 0 /* XXX */, &ev);
+	}
+
+	CORBA_exception_free (&ev);
+}
+
 BonoboControl *
 addressbook_factory_new_control (void)
 {
@@ -623,6 +741,11 @@ addressbook_factory_new_control (void)
 	bonobo_control_set_properties (view->control,
 					 view->properties);
 
+	gtk_signal_connect (GTK_OBJECT (view->view),
+			    "status_message",
+			    GTK_SIGNAL_FUNC(set_status_message),
+			    view);
+	
 	view->uri = NULL;
 
 	gtk_signal_connect (GTK_OBJECT (view->control), "activate",
