@@ -30,7 +30,12 @@
 #include <gal/widgets/e-scroll-frame.h>
 #include <gal/widgets/e-popup-menu.h>
 #include "widgets/menus/gal-view-menus.h"
+
 #include <gal/menus/gal-view-factory-etable.h>
+#include <gal/menus/gal-view-etable.h>
+
+#include "gal-view-factory-minicard.h"
+#include "gal-view-minicard.h"
 
 #include "e-addressbook-model.h"
 
@@ -68,6 +73,13 @@ enum {
 	ARG_TYPE,
 };
 
+enum {
+	STATUS_MESSAGE,
+	LAST_SIGNAL
+};
+
+static guint e_addressbook_view_signals [LAST_SIGNAL] = {0, };
+
 GtkType
 e_addressbook_view_get_type (void)
 {
@@ -111,6 +123,16 @@ e_addressbook_view_class_init (EAddressbookViewClass *klass)
 				 GTK_ARG_READWRITE, ARG_QUERY);
 	gtk_object_add_arg_type ("EAddressbookView::type", GTK_TYPE_ENUM,
 				 GTK_ARG_READWRITE, ARG_TYPE);
+
+	e_addressbook_view_signals [STATUS_MESSAGE] =
+		gtk_signal_new ("status_message",
+				GTK_RUN_LAST,
+				object_class->type,
+				GTK_SIGNAL_OFFSET (EAddressbookViewClass, status_message),
+				gtk_marshal_NONE__POINTER,
+				GTK_TYPE_NONE, 1, GTK_TYPE_POINTER);
+
+	gtk_object_class_add_signals (object_class, e_addressbook_view_signals, LAST_SIGNAL);
 }
 
 static void
@@ -152,8 +174,9 @@ e_addressbook_view_set_arg (GtkObject *object, GtkArg *arg, guint arg_id)
 
 	switch (arg_id){
 	case ARG_BOOK:
-		if (eav->book)
+		if (eav->book) {
 			gtk_object_unref(GTK_OBJECT(eav->book));
+		}
 		if (GTK_VALUE_OBJECT(*arg)) {
 			eav->book = E_BOOK(GTK_VALUE_OBJECT(*arg));
 			gtk_object_ref(GTK_OBJECT(eav->book));
@@ -401,7 +424,9 @@ table_double_click(ETableScrolled *table, gint row, EAddressbookView *view)
 typedef struct {
 	EBook *book;
 	ECard *card;
+	EAddressbookView *view;
 	GtkWidget *widget;
+	gpointer closure;
 } CardAndBook;
 
 static void
@@ -409,6 +434,38 @@ card_and_book_free (CardAndBook *card_and_book)
 {
 	gtk_object_unref(GTK_OBJECT(card_and_book->card));
 	gtk_object_unref(GTK_OBJECT(card_and_book->book));
+	gtk_object_unref(GTK_OBJECT(card_and_book->view));
+}
+
+static void
+get_card_list_1(gint model_row,
+	      gpointer closure)
+{
+	CardAndBook *card_and_book;
+	GList **list;
+	EAddressbookView *view;
+	ECard *card;
+
+	card_and_book = closure;
+	list = card_and_book->closure;
+	view = card_and_book->view;
+
+	card = e_addressbook_model_get_card(E_ADDRESSBOOK_MODEL(view->object), model_row);
+	*list = g_list_prepend(*list, card);
+}
+
+static GList *
+get_card_list (CardAndBook *card_and_book)
+{
+	GList *list = NULL;
+	ETable *table;
+
+	table = E_TABLE(card_and_book->widget);
+	card_and_book->closure = &list;
+	e_table_selected_row_foreach(table,
+				     get_card_list_1,
+				     card_and_book);
+	return list;
 }
 
 static void
@@ -427,6 +484,7 @@ send_as (GtkWidget *widget, CardAndBook *card_and_book)
 
 static void
 send_to (GtkWidget *widget, CardAndBook *card_and_book)
+
 {
 	e_card_send(card_and_book->card, E_CARD_DISPOSITION_AS_TO);
 	card_and_book_free(card_and_book);
@@ -450,11 +508,17 @@ static void
 delete (GtkWidget *widget, CardAndBook *card_and_book)
 {
 	if (e_contact_editor_confirm_delete(GTK_WINDOW(gtk_widget_get_toplevel(card_and_book->widget)))) {
-		/* Add the card in the contact editor to our ebook */
-		e_book_remove_card (card_and_book->book,
-				    card_and_book->card,
-				    NULL,
-				    NULL);
+		GList *list = get_card_list(card_and_book);
+		GList *iterator;
+		for (iterator = list; iterator; iterator = iterator->next) {
+			ECard *card = iterator->data;
+			/* Add the card in the contact editor to our ebook */
+			e_book_remove_card (card_and_book->book,
+					    card,
+					    NULL,
+					    NULL);
+		}
+		e_free_object_list(list);
 	}
 	card_and_book_free(card_and_book);
 }
@@ -465,7 +529,7 @@ table_right_click(ETableScrolled *table, gint row, gint col, GdkEvent *event, EA
 	if (E_IS_ADDRESSBOOK_MODEL(view->object)) {
 		EAddressbookModel *model = E_ADDRESSBOOK_MODEL(view->object);
 		CardAndBook *card_and_book;
-		
+
 		EPopupMenu menu[] = {
 			{"Save as VCard", NULL, GTK_SIGNAL_FUNC(save_as), NULL, 0}, 
 			{"Send contact to other", NULL, GTK_SIGNAL_FUNC(send_as), NULL, 0},
@@ -475,21 +539,65 @@ table_right_click(ETableScrolled *table, gint row, gint col, GdkEvent *event, EA
 			{"Delete", NULL, GTK_SIGNAL_FUNC(delete), NULL, 0},
 			{NULL, NULL, NULL, NULL, 0}
 		};
-		
+
 		card_and_book = g_new(CardAndBook, 1);
 		card_and_book->card = e_addressbook_model_get_card(model, row);
 		card_and_book->widget = GTK_WIDGET(table);
+		card_and_book->view = view;
 		gtk_object_get(GTK_OBJECT(model),
 			       "book", &(card_and_book->book),
 			       NULL);
-		
+
 		gtk_object_ref(GTK_OBJECT(card_and_book->book));
-		
-		e_popup_menu_run (menu, (GdkEventButton *)event, 0, 0, card_and_book);
+		gtk_object_ref(GTK_OBJECT(card_and_book->view));
+
+		e_popup_menu_run (menu, event, 0, 0, card_and_book);
 		return TRUE;
 	} else
 		return FALSE;
 }
+
+static void
+status_message (GtkObject *object, const gchar *status, EAddressbookView *eav)
+{
+	gtk_signal_emit (GTK_OBJECT (eav),
+			 e_addressbook_view_signals [STATUS_MESSAGE],
+			 status);
+}
+
+#ifdef JUST_FOR_TRANSLATORS
+static char *list [] = {
+	N_("Name"),
+	N_("Email"),
+	N_("Primary"),
+	N_("Business"),
+	N_("Home"),
+	N_("Organization"),
+	N_("Mobile"),
+	N_("Car"),
+	N_("Business Fax"),
+	N_("Home Fax"),
+	N_("Business 2"),
+	N_("Home 2"),
+	N_("ISDN"),
+	N_("Other"),
+	N_("Pager"),
+	N_("Email 2"),
+	N_("Email 3"),
+	N_("Web Site"),
+	N_("Department"),
+	N_("Office"),
+	N_("Title"),
+	N_("Profession"),
+	N_("Manager"),
+	N_("Assistant"),
+	N_("Nickname"),
+	N_("Spouse"),
+	N_("Note"),
+	N_("Free-busy URL"),
+};
+#endif
+
 #define SPEC "<?xml version=\"1.0\"?>      \
 <ETableSpecification click-to-add=\"true\" draw-grid=\"true\" _click-to-add-message=\"* Click here to add a contact *\">   \
   <ETableColumn model_col= \"0\" _title=\"Name\"          expansion=\"1.0\" minimum_width=\"20\" resizable=\"true\" cell=\"string\"       compare=\"string\"/> \
@@ -597,6 +705,11 @@ change_view_type (EAddressbookView *view, EAddressbookViewType view_type)
 
 	view->view_type = view_type;
 
+	gtk_signal_connect (view->object,
+			    "status_message",
+			    GTK_SIGNAL_FUNC (status_message),
+			    view);
+
 	gtk_object_set(view->object,
 		       "query", view->query,
 		       "book", view->book,
@@ -674,6 +787,20 @@ e_contact_print_button(GnomeDialog *dialog, gint button, gpointer data)
 	}
 }
 
+static void
+display_view(GalViewCollection *collection,
+	     GalView *view,
+	     gpointer data)
+{
+	EAddressbookView *address_view = data;
+	if (GAL_IS_VIEW_ETABLE(view)) {
+		change_view_type (address_view, E_ADDRESSBOOK_VIEW_TABLE);
+		e_table_set_state_object(e_table_scrolled_get_table(E_TABLE_SCROLLED(address_view->widget)), GAL_VIEW_ETABLE(view)->state);
+	} else if (GAL_IS_VIEW_MINICARD(view)) {
+		change_view_type (address_view, E_ADDRESSBOOK_VIEW_MINICARD);
+	}
+}
+
 void
 e_addressbook_view_setup_menus (EAddressbookView *view,
 				BonoboUIComponent *uic)
@@ -684,6 +811,10 @@ e_addressbook_view_setup_menus (EAddressbookView *view,
 	ETableSpecification *spec;
 
 	collection = gal_view_collection_new();
+	/* FIXME: Memory leak. */
+	gal_view_collection_set_storage_directories(collection,
+						    gnome_util_prepend_user_home("/evolution/system/"),
+						    gnome_util_prepend_user_home("/evolution/galview/"));
 
 	spec = e_table_specification_new();
 	e_table_specification_load_from_string(spec, SPEC);
@@ -692,15 +823,17 @@ e_addressbook_view_setup_menus (EAddressbookView *view,
 	gal_view_collection_add_factory(collection, factory);
 	gtk_object_sink(GTK_OBJECT(factory));
 
-#if 0
-	factory = e_minicard_view_factory_new();
+	factory = gal_view_factory_minicard_new();
 	gal_view_collection_add_factory(collection, factory);
 	gtk_object_sink(GTK_OBJECT(factory));
-#endif
+
+	gal_view_collection_load(collection);
 
 	views = gal_view_menus_new(collection);
-	gal_view_menus_apply(views, uic, NULL);
-	gtk_object_sink(GTK_OBJECT(views));
+	gal_view_menus_apply(views, uic, NULL); /* This function probably needs to sink the views object. */
+	gtk_signal_connect(GTK_OBJECT(collection), "display_view",
+			   display_view, view);
+	/*	gtk_object_sink(GTK_OBJECT(views)); */
 
 	gtk_object_sink(GTK_OBJECT(collection));
 }
