@@ -1,8 +1,9 @@
 /* Evolution calendar - iCalendar component object
  *
  * Copyright (C) 2000 Helix Code, Inc.
+ * Copyright (C) 2000 Ximian, Inc.
  *
- * Author: Federico Mena-Quintero <federico@helixcode.com>
+ * Author: Federico Mena-Quintero <federico@ximian.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -42,10 +43,7 @@ struct _CalComponentPrivate {
 
 	icalproperty *status;
 
-	struct categories {
-		icalproperty *prop;
-	};
-	GSList *categories_list; /* list of struct categories */
+	icalproperty *categories;
 
 	icalproperty *classification;
 
@@ -118,6 +116,9 @@ struct _CalComponentAlarm {
 	/* Alarm icalcomponent we wrap */
 	icalcomponent *icalcomp;
 
+	/* Our extension UID property */
+	icalproperty *uid;
+
 	/* Properties */
 
 	icalproperty *action;
@@ -241,7 +242,7 @@ free_icalcomponent (CalComponent *comp)
 
 	priv->status = NULL;
 
-	priv->categories_list = free_slist (priv->categories_list);
+	priv->categories = NULL;
 
 	priv->classification = NULL;
 	priv->comment_list = NULL;
@@ -402,21 +403,6 @@ cal_component_clone (CalComponent *comp)
 	return new_comp;
 }
 
-/* Scans the categories property */
-static void
-scan_categories (CalComponent *comp, icalproperty *prop)
-{
-	CalComponentPrivate *priv;
-	struct categories *categ;
-
-	priv = comp->priv;
-
-	categ = g_new (struct categories, 1);
-	categ->prop = prop;
-
-	priv->categories_list = g_slist_append (priv->categories_list, categ);
-}
-
 /* Scans a date/time and timezone pair property */
 static void
 scan_datetime (CalComponent *comp, struct datetime *datetime, icalproperty *prop)
@@ -507,7 +493,7 @@ scan_property (CalComponent *comp, icalproperty *prop)
 		break;
 
 	case ICAL_CATEGORIES_PROPERTY:
-		scan_categories (comp, prop);
+		priv->categories = prop;
 		break;
 
 	case ICAL_CLASS_PROPERTY:
@@ -702,6 +688,17 @@ add_alarm (CalComponent *comp, icalcomponent *alarm, const char *auid)
 	return auid;
 }
 
+static void
+remove_alarm (CalComponent *comp, const char *auid) 
+{
+	CalComponentPrivate *priv;
+	
+	priv = comp->priv;
+	
+	g_hash_table_remove (priv->alarm_uid_hash, auid);
+}
+
+
 /* Scans an alarm subcomponent, adds an UID extension property to it (so that we
  * can reference alarms by unique IDs), and adds its mapping to the component.  */
 static void
@@ -798,7 +795,7 @@ ensure_mandatory_properties (CalComponent *comp)
 		struct icaltimetype t;
 
 		tim = time (NULL);
-		t = icaltime_from_timet (tim, FALSE, TRUE);
+		t = icaltime_from_timet (tim, FALSE);
 
 		priv->dtstamp = icalproperty_new_dtstamp (t);
 		icalcomponent_add_property (priv->icalcomp, priv->dtstamp);
@@ -1127,43 +1124,39 @@ cal_component_set_uid (CalComponent *comp, const char *uid)
 }
 
 /**
- * cal_component_get_status:
- * @comp: A calendar component object.
- * @status: Return value for the status value.  It is set to #ICAL_STATUS_NONE
- * if the component has no status property.
- *
- * Queries the status property of a calendar component object.
+ * cal_component_get_categories:
+ * @comp: A calendar component object. 
+ * @categories: 
+ * 
+ * 
  **/
-void
-cal_component_get_status (CalComponent *comp, icalproperty_status *status)
+void 
+cal_component_get_categories (CalComponent *comp, const char **categories)
 {
 	CalComponentPrivate *priv;
 
 	g_return_if_fail (comp != NULL);
 	g_return_if_fail (IS_CAL_COMPONENT (comp));
-	g_return_if_fail (status != NULL);
+	g_return_if_fail (categories != NULL);
 
 	priv = comp->priv;
 	g_return_if_fail (priv->icalcomp != NULL);
 
-	if (!priv->status) {
-		*status = ICAL_STATUS_NONE;
-		return;
-	}
-
-	*status = icalproperty_get_status (priv->status);
+	if (priv->categories)
+		*categories = icalproperty_get_categories (priv->categories);
+	else
+		*categories = NULL;
 }
 
 /**
- * cal_component_set_status:
+ * cal_component_set_categories:
  * @comp: A calendar component object.
- * @status: Status value.  You should use #ICAL_STATUS_NONE if you want to unset
- * this property.
- *
- * Sets the status property of a calendar component object.
+ * @categories: 
+ * 
+ * 
  **/
-void
-cal_component_set_status (CalComponent *comp, icalproperty_status status)
+void 
+cal_component_set_categories (CalComponent *comp, const char *categories)
 {
 	CalComponentPrivate *priv;
 
@@ -1173,26 +1166,25 @@ cal_component_set_status (CalComponent *comp, icalproperty_status status)
 	priv = comp->priv;
 	g_return_if_fail (priv->icalcomp != NULL);
 
-	priv->need_sequence_inc = TRUE;
-
-	if (status == ICAL_STATUS_NONE) {
-		if (priv->status) {
-			icalcomponent_remove_property (priv->icalcomp, priv->status);
-			icalproperty_free (priv->status);
-			priv->status = NULL;
+	if (!categories || !(*categories)) {
+		if (priv->categories) {
+			icalcomponent_remove_property (priv->icalcomp, priv->categories);
+			icalproperty_free (priv->categories);
+			priv->url = NULL;
 		}
 
 		return;
 	}
 
-	if (priv->status) {
-		icalproperty_set_status (priv->status, status);
-	} else {
-		priv->status = icalproperty_new_status (status);
-		icalcomponent_add_property (priv->icalcomp, priv->status);
+	if (priv->categories)
+		icalproperty_set_categories (priv->categories, (char *) categories);
+	else {
+		priv->categories = icalproperty_new_categories ((char *) categories);
+		icalcomponent_add_property (priv->icalcomp, priv->categories);
 	}
 }
 
+
 /**
  * cal_component_get_categories_list:
  * @comp: A calendar component object.
@@ -1218,16 +1210,15 @@ cal_component_get_categories_list (CalComponent *comp, GSList **categ_list)
 	priv = comp->priv;
 	g_return_if_fail (priv->icalcomp != NULL);
 
-	if (!priv->categories_list) {
+	if (!priv->categories) {
 		*categ_list = NULL;
 		return;
 	}
 
-	categories = icalproperty_get_categories (priv->categories_list);
+	categories = icalproperty_get_categories (priv->categories);
 	g_assert (categories != NULL);
 
 	cat_start = categories;
-
 	*categ_list = NULL;
 
 	for (p = categories; *p; p++)
@@ -1278,7 +1269,6 @@ void
 cal_component_set_categories_list (CalComponent *comp, GSList *categ_list)
 {
 	CalComponentPrivate *priv;
-	struct categories *cat;
 	char *categories_str;
 
 	g_return_if_fail (comp != NULL);
@@ -1287,40 +1277,23 @@ cal_component_set_categories_list (CalComponent *comp, GSList *categ_list)
 	priv = comp->priv;
 	g_return_if_fail (priv->icalcomp != NULL);
 
-	/* Free the old list */
-
 	if (!categ_list) {
-		if (priv->categories_list) {
-			GSList *l;
-
-			for (l = priv->categories_list; l; l = l->next) {
-				struct categories *c;
-
-				c = l->data;
-				icalcomponent_remove_property (priv->icalcomp, c->prop);
-				icalproperty_free (c->prop);
-
-				g_free (c);
-			}
-
-			g_slist_free (priv->categories_list);
-			priv->categories_list = NULL;
+		if (priv->categories) {
+			icalcomponent_remove_property (priv->icalcomp, priv->categories);
+			icalproperty_free (priv->categories);
 		}
-
+		
 		return;
 	}
 
 	/* Create a single string of categories */
-
 	categories_str = stringify_categories (categ_list);
 
 	/* Set the categories */
-
-	cat = g_new (struct categories, 1);
-	cat->prop = icalproperty_new_categories (categories_str);
+	priv->categories = icalproperty_new_categories (categories_str);
 	g_free (categories_str);
 
-	icalcomponent_add_property (priv->icalcomp, cat->prop);
+	icalcomponent_add_property (priv->icalcomp, priv->categories);
 }
 
 /**
@@ -2018,7 +1991,7 @@ cal_component_set_due (CalComponent *comp, CalComponentDateTime *dt)
 /* Builds a list of CalComponentPeriod structures based on a list of icalproperties */
 static void
 get_period_list (GSList *period_list,
-		 struct icalperiodtype (* get_prop_func) (icalproperty *prop),
+		 struct icaldatetimeperiodtype (* get_prop_func) (icalproperty *prop),
 		 GSList **list)
 {
 	GSList *l;
@@ -2031,7 +2004,7 @@ get_period_list (GSList *period_list,
 	for (l = period_list; l; l = l->next) {
 		struct period *period;
 		CalComponentPeriod *p;
-		struct icalperiodtype ip;
+		struct icaldatetimeperiodtype ip;
 
 		period = l->data;
 		g_assert (period->prop != NULL);
@@ -2061,12 +2034,12 @@ get_period_list (GSList *period_list,
 
 		ip = (* get_prop_func) (period->prop);
 
-		p->start = ip.start;
+		p->start = ip.period.start;
 
 		if (p->type == CAL_COMPONENT_PERIOD_DATETIME)
-			p->u.end = ip.end;
+			p->u.end = ip.period.end;
 		else if (p->type == CAL_COMPONENT_PERIOD_DURATION)
-			p->u.duration = ip.duration;
+			p->u.duration = ip.period.duration;
 		else
 			g_assert_not_reached ();
 
@@ -2081,7 +2054,7 @@ get_period_list (GSList *period_list,
 /* Sets a period list value */
 static void
 set_period_list (CalComponent *comp,
-		 icalproperty *(* new_prop_func) (struct icalperiodtype period),
+		 icalproperty *(* new_prop_func) (struct icaldatetimeperiodtype period),
 		 GSList **period_list,
 		 GSList *pl)
 {
@@ -2111,7 +2084,7 @@ set_period_list (CalComponent *comp,
 	for (l = pl; l; l = l->next) {
 		CalComponentPeriod *p;
 		struct period *period;
-		struct icalperiodtype ip;
+		struct icaldatetimeperiodtype ip;
 		icalparameter_value value_type;
 
 		g_assert (l->data != NULL);
@@ -2119,14 +2092,14 @@ set_period_list (CalComponent *comp,
 
 		/* Create libical value */
 
-		ip.start = p->start;
+		ip.period.start = p->start;
 
 		if (p->type == CAL_COMPONENT_PERIOD_DATETIME) {
 			value_type = ICAL_VALUE_DATETIME;
-			ip.end = p->u.end;
+			ip.period.end = p->u.end;
 		} else if (p->type == CAL_COMPONENT_PERIOD_DURATION) {
 			value_type = ICAL_VALUE_DURATION;
-			ip.duration = p->u.duration;
+			ip.period.duration = p->u.duration;
 		} else {
 			g_assert_not_reached ();
 			return;
@@ -2378,8 +2351,7 @@ cal_component_get_exrule_list (CalComponent *comp, GSList **recur_list)
  * @comp: A calendar component object.
  * @recur_list: Returns a list of exception rule properties.
  *
- * Returns a list of exception rule properties of a calendar component
- * object.
+ * Queries the list of exception rule properties of a calendar component object.
  **/
 void
 cal_component_get_exrule_property_list (CalComponent *comp, GSList **recur_list)
@@ -2799,8 +2771,7 @@ cal_component_get_rrule_list (CalComponent *comp, GSList **recur_list)
  * @comp: A calendar component object.
  * @recur_list: Returns a list of recurrence rule properties.
  *
- * Returns a list of recurrence rule properties of a calendar component
- * object.
+ * Queries a list of recurrence rule properties of a calendar component object.
  **/
 void
 cal_component_get_rrule_property_list (CalComponent *comp, GSList **recur_list)
@@ -2947,6 +2918,73 @@ cal_component_set_sequence (CalComponent *comp, int *sequence)
 	}
 }
 
+/**
+ * cal_component_get_status:
+ * @comp: A calendar component object.
+ * @status: Return value for the status value.  It is set to #ICAL_STATUS_NONE
+ * if the component has no status property.
+ *
+ * Queries the status property of a calendar component object.
+ **/
+void
+cal_component_get_status (CalComponent *comp, icalproperty_status *status)
+{
+	CalComponentPrivate *priv;
+
+	g_return_if_fail (comp != NULL);
+	g_return_if_fail (IS_CAL_COMPONENT (comp));
+	g_return_if_fail (status != NULL);
+
+	priv = comp->priv;
+	g_return_if_fail (priv->icalcomp != NULL);
+
+	if (!priv->status) {
+		*status = ICAL_STATUS_NONE;
+		return;
+	}
+
+	*status = icalproperty_get_status (priv->status);
+}
+
+/**
+ * cal_component_set_status:
+ * @comp: A calendar component object.
+ * @status: Status value.  You should use #ICAL_STATUS_NONE if you want to unset
+ * this property.
+ *
+ * Sets the status property of a calendar component object.
+ **/
+void
+cal_component_set_status (CalComponent *comp, icalproperty_status status)
+{
+	CalComponentPrivate *priv;
+
+	g_return_if_fail (comp != NULL);
+	g_return_if_fail (IS_CAL_COMPONENT (comp));
+
+	priv = comp->priv;
+	g_return_if_fail (priv->icalcomp != NULL);
+
+	priv->need_sequence_inc = TRUE;
+
+	if (status == ICAL_STATUS_NONE) {
+		if (priv->status) {
+			icalcomponent_remove_property (priv->icalcomp, priv->status);
+			icalproperty_free (priv->status);
+			priv->status = NULL;
+		}
+
+		return;
+	}
+
+	if (priv->status) {
+		icalproperty_set_status (priv->status, status);
+	} else {
+		priv->status = icalproperty_new_status (status);
+		icalcomponent_add_property (priv->icalcomp, priv->status);
+	}
+}
+
 /**
  * cal_component_get_summary:
  * @comp: A calendar component object.
@@ -3166,7 +3204,7 @@ cal_component_set_url (CalComponent *comp, const char *url)
 	priv = comp->priv;
 	g_return_if_fail (priv->icalcomp != NULL);
 
-	if (!url) {
+	if (!url || !(*url)) {
 		if (priv->url) {
 			icalcomponent_remove_property (priv->icalcomp, priv->url);
 			icalproperty_free (priv->url);
@@ -3365,34 +3403,6 @@ cal_component_free_sequence (int *sequence)
 	g_free (sequence);
 }
 
-/**
- * cal_component_free_pilot_id:
- * @sequence: Sequence number value.
- *
- * Frees a sequence number value.
- **/
-void
-cal_component_free_pilot_id (unsigned long *pilot_id)
-{
-	g_return_if_fail (pilot_id != NULL);
-
-	g_free (pilot_id);
-}
-
-/**
- * cal_component_free_pilot_status:
- * @sequence: Sequence number value.
- *
- * Frees a sequence number value.
- **/
-void
-cal_component_free_pilot_status (unsigned long *pilot_status)
-{
-	g_return_if_fail (pilot_status != NULL);
-
-	g_free (pilot_status);
-}
-
 /**
  * cal_component_free_text_list:
  * @text_list: List of #CalComponentText structures.
@@ -3433,7 +3443,6 @@ gboolean
 cal_component_has_alarms (CalComponent *comp)
 {
 	CalComponentPrivate *priv;
-	icalcomponent *subcomp;
 
 	g_return_val_if_fail (comp != NULL, FALSE);
 	g_return_val_if_fail (IS_CAL_COMPONENT (comp), FALSE);
@@ -3441,11 +3450,69 @@ cal_component_has_alarms (CalComponent *comp)
 	priv = comp->priv;
 	g_return_val_if_fail (priv->icalcomp != NULL, FALSE);
 
-	subcomp = icalcomponent_get_first_component (priv->icalcomp, ICAL_VALARM_COMPONENT);
-
-	return subcomp != NULL ? TRUE : FALSE;
+	return g_hash_table_size (priv->alarm_uid_hash) != 0;
 }
 
+void 
+cal_component_add_alarm (CalComponent *comp, CalComponentAlarm *alarm)
+{
+	CalComponentPrivate *priv;
+	
+	g_return_if_fail (comp != NULL);
+	g_return_if_fail (IS_CAL_COMPONENT (comp));
+	g_return_if_fail (alarm != NULL);
+	
+	priv = comp->priv;
+	
+	alarm->parent = comp;
+	add_alarm (comp, alarm->icalcomp, icalproperty_get_x (alarm->uid));
+	icalcomponent_add_component (priv->icalcomp, alarm->icalcomp);
+}
+
+void 
+cal_component_remove_alarm (CalComponent *comp, const char *auid)
+{
+	CalComponentPrivate *priv;
+	icalcompiter iter;
+
+	g_return_if_fail (comp != NULL);
+	g_return_if_fail (IS_CAL_COMPONENT (comp));
+	g_return_if_fail (auid != NULL);
+
+	priv = comp->priv;
+
+	for (iter = icalcomponent_begin_component (priv->icalcomp, ICAL_VALARM_COMPONENT);
+	     icalcompiter_deref (&iter) != NULL;
+	     icalcompiter_next (&iter)) {
+		icalproperty *prop;
+		icalcomponent *subcomp;
+
+		subcomp = icalcompiter_deref (&iter);
+		for (prop = icalcomponent_get_first_property (subcomp, ICAL_X_PROPERTY);
+		     prop;
+		     prop = icalcomponent_get_next_property (subcomp, ICAL_X_PROPERTY)) {
+			const char *xname;
+			const char *alarm_uid;
+			
+			xname = icalproperty_get_x_name (prop);
+			g_assert (xname != NULL);
+
+			if (strcmp (xname, EVOLUTION_ALARM_UID_PROPERTY) == 0) {
+				alarm_uid = alarm_uid_from_prop (prop);
+				if (strcmp (alarm_uid, auid) == 0) {
+					remove_alarm (comp, auid);
+					icalcomponent_remove_component (priv->icalcomp, subcomp);
+					icalcomponent_free (subcomp);
+					break;
+				}
+				
+				return;
+			}
+		}
+	}
+}
+
+
 /* Scans an icalproperty from a calendar component and adds its mapping to our
  * own alarm structure.
  */
@@ -3453,6 +3520,7 @@ static void
 scan_alarm_property (CalComponentAlarm *alarm, icalproperty *prop)
 {
 	icalproperty_kind kind;
+	const char *xname;
 
 	kind = icalproperty_isa (prop);
 
@@ -3465,6 +3533,15 @@ scan_alarm_property (CalComponentAlarm *alarm, icalproperty *prop)
 		alarm->trigger = prop;
 		break;
 
+	case ICAL_X_PROPERTY:
+		xname = icalproperty_get_x_name (prop);
+		g_assert (xname != NULL);
+
+		if (strcmp (xname, EVOLUTION_ALARM_UID_PROPERTY) == 0)
+			alarm->uid = prop;
+
+		break;
+
 	default:
 		break;
 	}
@@ -3481,12 +3558,15 @@ make_alarm (CalComponent *comp, icalcomponent *subcomp)
 
 	alarm->parent = comp;
 	alarm->icalcomp = subcomp;
+	alarm->uid = NULL;
 
 	for (prop = icalcomponent_get_first_property (subcomp, ICAL_ANY_PROPERTY);
 	     prop;
 	     prop = icalcomponent_get_next_property (subcomp, ICAL_ANY_PROPERTY))
 		scan_alarm_property (alarm, prop);
 
+	g_assert (alarm->uid != NULL);
+
 	return alarm;
 }
 
@@ -3564,6 +3644,334 @@ cal_component_get_alarm (CalComponent *comp, const char *auid)
 		return NULL;
 }
 
+void 
+cal_component_free_alarm_uids (GList *alarm_uids)
+{
+	g_list_foreach (alarm_uids, (GFunc)g_free, NULL);
+}
+
+/**
+ * cal_component_alarms_free:
+ * @alarms: Component alarms structure.
+ * 
+ * Frees a #CalComponentAlarms structure.
+ **/
+void
+cal_component_alarms_free (CalComponentAlarms *alarms)
+{
+	GSList *l;
+
+	g_return_if_fail (alarms != NULL);
+
+	g_assert (alarms->comp != NULL);
+	gtk_object_unref (GTK_OBJECT (alarms->comp));
+
+	for (l = alarms->alarms; l; l = l->next) {
+		CalAlarmInstance *instance;
+
+		instance = l->data;
+		g_assert (instance != NULL);
+		g_free (instance);
+	}
+
+	g_slist_free (alarms->alarms);
+	g_free (alarms);
+}
+
+/**
+ * cal_component_alarm_new:
+  * 
+ * 
+ * 
+ * Return value: a new alarm component
+ **/
+CalComponentAlarm *
+cal_component_alarm_new (void)
+{
+	CalComponentAlarm *alarm = g_new0 (CalComponentAlarm, 1);
+	char *new_auid ;
+
+	alarm->icalcomp = icalcomponent_new (ICAL_VALARM_COMPONENT);
+
+	new_auid = cal_component_gen_uid ();
+	alarm->uid = icalproperty_new_x (new_auid);
+	icalproperty_set_x_name (alarm->uid, EVOLUTION_ALARM_UID_PROPERTY);
+	icalcomponent_add_property (alarm->icalcomp, alarm->uid);
+	g_free (new_auid);
+	
+	return alarm;
+}
+
+/**
+ * cal_component_alarm_get_uid:
+ * @alarm: An alarm subcomponent.
+ * 
+ * Queries the unique identifier of an alarm subcomponent.
+ * 
+ * Return value: UID of the alarm.
+ **/
+const char *
+cal_component_alarm_get_uid (CalComponentAlarm *alarm)
+{
+	g_return_val_if_fail (alarm != NULL, NULL);
+
+	return alarm_uid_from_prop (alarm->uid);
+}
+
+/**
+ * cal_component_alarm_get_action:
+ * @alarm: An alarm.
+ * @action: Return value for the alarm's action type.
+ *
+ * Queries the action type of an alarm.
+ **/
+void
+cal_component_alarm_get_action (CalComponentAlarm *alarm, CalAlarmAction *action)
+{
+	const char *str;
+
+	g_return_if_fail (alarm != NULL);
+	g_return_if_fail (action != NULL);
+
+	g_assert (alarm->icalcomp != NULL);
+
+	if (!alarm->action) {
+		*action = CAL_ALARM_NONE;
+		return;
+	}
+
+	str = icalproperty_get_action (alarm->action);
+
+	if (strcasecmp (str, "AUDIO") == 0)
+		*action = CAL_ALARM_AUDIO;
+	else if (strcasecmp (str, "DISPLAY") == 0)
+		*action = CAL_ALARM_DISPLAY;
+	else if (strcasecmp (str, "EMAIL") == 0)
+		*action = CAL_ALARM_EMAIL;
+	else if (strcasecmp (str, "PROCEDURE") == 0)
+		*action = CAL_ALARM_PROCEDURE;
+	else
+		*action = CAL_ALARM_UNKNOWN;
+}
+
+/**
+ * cal_component_alarm_set_action:
+ * @alarm: An alarm.
+ * @action: Action type.
+ *
+ * Sets the action type for an alarm.
+ **/
+void
+cal_component_alarm_set_action (CalComponentAlarm *alarm, CalAlarmAction action)
+{
+	char *str;
+
+	g_return_if_fail (alarm != NULL);
+	g_return_if_fail (action != CAL_ALARM_NONE);
+	g_return_if_fail (action != CAL_ALARM_UNKNOWN);
+
+	g_assert (alarm->icalcomp != NULL);
+
+	switch (action) {
+	case CAL_ALARM_AUDIO:
+		str = "AUDIO";
+		break;
+
+	case CAL_ALARM_DISPLAY:
+		str = "DISPLAY";
+		break;
+
+	case CAL_ALARM_EMAIL:
+		str = "EMAIL";
+		break;
+
+	case CAL_ALARM_PROCEDURE:
+		str = "PROCEDURE";
+		break;
+
+	default:
+		g_assert_not_reached ();
+		str = NULL;
+	}
+
+	if (alarm->action)
+		icalproperty_set_action (alarm->action, str);
+	else {
+		alarm->action = icalproperty_new_action (str);
+		icalcomponent_add_property (alarm->icalcomp, alarm->action);
+	}
+}
+
+/**
+ * cal_component_alarm_get_trigger:
+ * @alarm: An alarm.
+ * @trigger: Return value for the trigger time.
+ *
+ * Queries the trigger time for an alarm.
+ **/
+void
+cal_component_alarm_get_trigger (CalComponentAlarm *alarm, CalAlarmTrigger *trigger)
+{
+	icalparameter *param;
+	struct icaltriggertype t;
+	gboolean relative;
+
+	g_return_if_fail (alarm != NULL);
+	g_return_if_fail (trigger != NULL);
+
+	g_assert (alarm->icalcomp != NULL);
+
+	if (!alarm->trigger) {
+		trigger->type = CAL_ALARM_TRIGGER_NONE;
+		return;
+	}
+
+	/* Get trigger type */
+
+	param = icalproperty_get_first_parameter (alarm->trigger, ICAL_VALUE_PARAMETER);
+	if (param) {
+		icalparameter_value value;
+
+		value = icalparameter_get_value (param);
+
+		switch (value) {
+		case ICAL_VALUE_DURATION:
+			relative = TRUE;
+			break;
+
+		case ICAL_VALUE_DATETIME:
+			relative = FALSE;
+			break;
+
+		default:
+			g_message ("cal_component_alarm_get_trigger(): Unknown value for trigger "
+				   "value %d; using RELATIVE", value);
+
+			relative = TRUE;
+			break;
+		}
+	} else
+		relative = TRUE;
+
+	/* Get trigger value and the RELATED parameter */
+
+	t = icalproperty_get_trigger (alarm->trigger);
+
+	if (relative) {
+		trigger->u.rel_duration = t.duration;
+
+		param = icalproperty_get_first_parameter (alarm->trigger, ICAL_RELATED_PARAMETER);
+		if (param) {
+			icalparameter_related rel;
+
+			rel = icalparameter_get_related (param);
+
+			switch (rel) {
+			case ICAL_RELATED_START:
+				trigger->type = CAL_ALARM_TRIGGER_RELATIVE_START;
+				break;
+
+			case ICAL_RELATED_END:
+				trigger->type = CAL_ALARM_TRIGGER_RELATIVE_END;
+				break;
+
+			default:
+				g_assert_not_reached ();
+			}
+		} else
+			trigger->type = CAL_ALARM_TRIGGER_RELATIVE_START;
+	} else {
+		trigger->u.abs_time = t.time;
+		trigger->type = CAL_ALARM_TRIGGER_ABSOLUTE;
+	}
+}
+
+/**
+ * cal_component_alarm_set_trigger:
+ * @alarm: An alarm.
+ * @trigger: Trigger time structure.
+ *
+ * Sets the trigger time of an alarm.
+ **/
+void
+cal_component_alarm_set_trigger (CalComponentAlarm *alarm, CalAlarmTrigger trigger)
+{
+	struct icaltriggertype t;
+	icalparameter *param;
+	icalparameter_value value_type;
+	icalparameter_related related;
+
+	g_return_if_fail (alarm != NULL);
+	g_return_if_fail (trigger.type != CAL_ALARM_TRIGGER_NONE);
+
+	g_assert (alarm->icalcomp != NULL);
+
+	/* Delete old trigger */
+
+	if (alarm->trigger) {
+		icalcomponent_remove_property (alarm->icalcomp, alarm->trigger);
+		icalproperty_free (alarm->trigger);
+		alarm->trigger = NULL;
+	}
+
+	/* Set the value */
+
+	related = ICAL_RELATED_START; /* Keep GCC happy */
+
+	t.time = icaltime_null_time ();
+	t.duration = icaldurationtype_null_duration ();
+	switch (trigger.type) {
+	case CAL_ALARM_TRIGGER_RELATIVE_START:
+		t.duration = trigger.u.rel_duration;
+		t.time.is_date = -1;
+		value_type = ICAL_DURATION_VALUE;
+		related = ICAL_RELATED_START;
+		break;
+
+	case CAL_ALARM_TRIGGER_RELATIVE_END:
+		t.duration = trigger.u.rel_duration;
+		t.time.is_date = -1;
+		value_type = ICAL_DURATION_VALUE;
+		related = ICAL_RELATED_END;
+		break;
+
+	case CAL_ALARM_TRIGGER_ABSOLUTE:
+		t.time = trigger.u.abs_time;
+		value_type = ICAL_DATETIME_VALUE;
+		break;
+
+	default:
+		g_assert_not_reached ();
+		return;
+	}
+
+	alarm->trigger = icalproperty_new_trigger (t);
+	icalcomponent_add_property (alarm->icalcomp, alarm->trigger);
+
+	/* Value parameters */
+
+	param = icalproperty_get_first_parameter (alarm->trigger, ICAL_VALUE_PARAMETER);
+	if (param)
+		icalparameter_set_value (param, value_type);
+	else {
+		param = icalparameter_new_value (value_type);
+		icalproperty_add_parameter (alarm->trigger, param);
+	}
+
+	/* Related parameter */
+
+	if (trigger.type != CAL_ALARM_TRIGGER_ABSOLUTE) {
+		param = icalproperty_get_first_parameter (alarm->trigger, ICAL_RELATED_PARAMETER);
+
+		if (param)
+			icalparameter_set_related (param, related);
+		else {
+			param = icalparameter_new_related (related);
+			icalproperty_add_parameter (alarm->trigger, param);
+		}
+	}
+}
+
 /**
  * cal_component_alarm_free:
  * @alarm: A calendar alarm.
@@ -3588,287 +3996,76 @@ cal_component_alarm_free (CalComponentAlarm *alarm)
 	g_free (alarm);
 }
 
-/**
- * cal_component_alarm_get_action:
- * @alarm: An alarm.
- * @action: Return value for the alarm's action type.
- *
- * Queries the action type of an alarm.
- **/
-void
-cal_component_alarm_get_action (CalComponentAlarm *alarm, CalComponentAlarmAction *action)
+
+/* Returns TRUE if both strings match, i.e. they are both NULL or the
+   strings are equal. */
+static gboolean
+cal_component_strings_match	(const gchar	*string1,
+				 const gchar	*string2)
 {
-	const char *str;
+	if (string1 == NULL || string2 == NULL)
+		return (string1 == string2) ? TRUE : FALSE;
 
-	g_return_if_fail (alarm != NULL);
-	g_return_if_fail (action != NULL);
+	if (!strcmp (string1, string2))
+		return TRUE;
 
-	g_assert (alarm->icalcomp != NULL);
-
-	if (!alarm->action) {
-		*action = CAL_COMPONENT_ALARM_NONE;
-		return;
-	}
-
-	str = icalproperty_get_action (alarm->action);
-
-	if (strcasecmp (str, "AUDIO") == 0)
-		*action = CAL_COMPONENT_ALARM_AUDIO;
-	else if (strcasecmp (str, "DISPLAY") == 0)
-		*action = CAL_COMPONENT_ALARM_DISPLAY;
-	else if (strcasecmp (str, "EMAIL") == 0)
-		*action = CAL_COMPONENT_ALARM_EMAIL;
-	else if (strcasecmp (str, "PROCEDURE") == 0)
-		*action = CAL_COMPONENT_ALARM_PROCEDURE;
-	else
-		*action = CAL_COMPONENT_ALARM_UNKNOWN;
+	return FALSE;
 }
 
-/**
- * cal_component_alarm_set_action:
- * @alarm: An alarm.
- * @action: Action type.
- *
- * Sets the action type for an alarm.
- **/
-void
-cal_component_alarm_set_action (CalComponentAlarm *alarm, CalComponentAlarmAction action)
-{
-	char *str;
-
-	g_return_if_fail (alarm != NULL);
-	g_return_if_fail (action != CAL_COMPONENT_ALARM_NONE);
-	g_return_if_fail (action != CAL_COMPONENT_ALARM_UNKNOWN);
-
-	g_assert (alarm->icalcomp != NULL);
-
-	switch (action) {
-	case CAL_COMPONENT_ALARM_AUDIO:
-		str = "AUDIO";
-		break;
-
-	case CAL_COMPONENT_ALARM_DISPLAY:
-		str = "DISPLAY";
-		break;
-
-	case CAL_COMPONENT_ALARM_EMAIL:
-		str = "EMAIL";
-		break;
-
-	case CAL_COMPONENT_ALARM_PROCEDURE:
-		str = "PROCEDURE";
-		break;
-
-	default:
-		g_assert_not_reached ();
-		str = NULL;
-	}
-
-	if (alarm->action)
-		icalproperty_set_action (alarm->action, str);
-	else {
-		alarm->action = icalproperty_new_action (str);
-		icalcomponent_add_property (alarm->icalcomp, alarm->action);
-	}
-}
 
 /**
- * cal_component_alarm_get_trigger:
- * @alarm: An alarm.
- * @trigger: Return value for the trigger time.  This should be freed using the
- * cal_component_alarm_free_trigger() function.
+ * cal_component_event_dates_match:
+ * @comp1: A calendar component object.
+ * @comp2: A calendar component object.
  *
- * Queries the trigger time for an alarm.
+ * Checks if the DTSTART and DTEND properties of the 2 components match.
+ * Note that the events may have different recurrence properties which are not
+ * taken into account here.
+ *
+ * Returns: TRUE if the DTSTART and DTEND properties of the 2 components match.
  **/
-void
-cal_component_alarm_get_trigger (CalComponentAlarm *alarm, CalComponentAlarmTrigger **trigger)
+gboolean
+cal_component_event_dates_match	(CalComponent *comp1,
+				 CalComponent *comp2)
 {
-	icalparameter *param;
-	union icaltriggertype t;
+	CalComponentDateTime comp1_dtstart, comp1_dtend;
+	CalComponentDateTime comp2_dtstart, comp2_dtend;
 
-	g_return_if_fail (alarm != NULL);
-	g_return_if_fail (trigger != NULL);
+	cal_component_get_dtstart (comp1, &comp1_dtstart);
+	cal_component_get_dtend   (comp1, &comp1_dtend);
+	cal_component_get_dtstart (comp2, &comp2_dtstart);
+	cal_component_get_dtend   (comp2, &comp2_dtend);
 
-	g_assert (alarm->icalcomp != NULL);
-
-	if (!alarm->trigger) {
-		*trigger = NULL;
-		return;
+	/* If either value is NULL they must both be NULL to match. */
+	if (comp1_dtstart.value == NULL || comp2_dtstart.value == NULL) {
+		if (comp1_dtstart.value != comp2_dtstart.value)
+			return FALSE;
+	} else {
+		if (icaltime_compare (*comp1_dtstart.value,
+				      *comp2_dtstart.value))
+			return FALSE;
 	}
 
-	*trigger = g_new (CalComponentAlarmTrigger, 1);
-
-	/* Get trigger type */
-
-	param = icalproperty_get_first_parameter (alarm->trigger, ICAL_VALUE_PARAMETER);
-
-	if (param) {
-		icalparameter_value value;
-
-		value = icalparameter_get_value (param);
-
-		switch (value) {
-		case ICAL_VALUE_DURATION:
-			(*trigger)->type = CAL_COMPONENT_ALARM_TRIGGER_RELATIVE;
-			break;
-
-		case ICAL_VALUE_DATETIME:
-			(*trigger)->type = CAL_COMPONENT_ALARM_TRIGGER_ABSOLUTE;
-			break;
-
-		default:
-			g_message ("cal_component_alarm_get_trigger(): Unknown value for trigger "
-				   "value %d; using RELATIVE", value);
-
-			(*trigger)->type = CAL_COMPONENT_ALARM_TRIGGER_RELATIVE;
-			break;
-		}
-	} else
-		(*trigger)->type = CAL_COMPONENT_ALARM_TRIGGER_RELATIVE;
-
-	/* Get trigger value and the RELATED parameter */
-
-	t = icalproperty_get_trigger (alarm->trigger);
-
-	switch ((*trigger)->type) {
-	case CAL_COMPONENT_ALARM_TRIGGER_RELATIVE:
-		(*trigger)->u.relative.duration = t.duration;
-
-		param = icalproperty_get_first_parameter (alarm->trigger, ICAL_RELATED_PARAMETER);
-		if (param) {
-			icalparameter_related rel;
-
-			rel = icalparameter_get_related (param);
-
-			switch (rel) {
-			case ICAL_RELATED_START:
-				(*trigger)->u.relative.related =
-					CAL_COMPONENT_ALARM_TRIGGER_RELATED_START;
-				break;
-
-			case ICAL_RELATED_END:
-				(*trigger)->u.relative.related =
-					CAL_COMPONENT_ALARM_TRIGGER_RELATED_END;
-				break;
-
-			default:
-				g_assert_not_reached ();
-			}
-		} else
-			(*trigger)->u.relative.related = CAL_COMPONENT_ALARM_TRIGGER_RELATED_START;
-
-		break;
-
-	case CAL_COMPONENT_ALARM_TRIGGER_ABSOLUTE:
-		(*trigger)->u.absolute = t.time;
-		break;
-
-	default:
-		g_assert_not_reached ();
+	if (comp1_dtend.value == NULL || comp2_dtend.value == NULL) {
+		if (comp1_dtend.value != comp2_dtend.value)
+			return FALSE;
+	} else {
+		if (icaltime_compare (*comp1_dtend.value,
+				      *comp2_dtend.value))
+			return FALSE;
 	}
+
+	/* Now check the timezones. */
+	if (!cal_component_strings_match (comp1_dtstart.tzid,
+					  comp2_dtstart.tzid))
+		return FALSE;
+
+	if (!cal_component_strings_match (comp1_dtend.tzid,
+					  comp2_dtend.tzid))
+		return FALSE;
+
+	return TRUE;
 }
 
-/**
- * cal_component_alarm_set_trigger:
- * @alarm: An alarm.
- * @trigger: Trigger time structure.
- *
- * Sets the trigger time of an alarm.
- **/
-void
-cal_component_alarm_set_trigger (CalComponentAlarm *alarm, CalComponentAlarmTrigger *trigger)
-{
-	union icaltriggertype t;
-	icalparameter *param;
-	icalparameter_value value_type;
-	icalparameter_related related;
 
-	g_return_if_fail (alarm != NULL);
-	g_return_if_fail (trigger != NULL);
 
-	g_assert (alarm->icalcomp != NULL);
-
-	/* Delete old trigger */
-
-	if (alarm->trigger) {
-		icalcomponent_remove_property (alarm->icalcomp, alarm->trigger);
-		icalproperty_free (alarm->trigger);
-		alarm->trigger = NULL;
-	}
-
-	/* Set the value */
-
-	value_type = ICAL_DURATION_VALUE; /* Keep GCC happy */
-	related = ICAL_RELATED_START; /* Ditto */
-
-	switch (trigger->type) {
-	case CAL_COMPONENT_ALARM_TRIGGER_RELATIVE:
-		t.duration = trigger->u.relative.duration;
-		value_type = ICAL_DURATION_VALUE;
-
-		switch (trigger->u.relative.related) {
-		case CAL_COMPONENT_ALARM_TRIGGER_RELATED_START:
-			related = ICAL_RELATED_START;
-			break;
-
-		case CAL_COMPONENT_ALARM_TRIGGER_RELATED_END:
-			related = ICAL_RELATED_END;
-			break;
-
-		default:
-			g_assert_not_reached ();
-			return;
-		}
-
-		break;
-
-	case CAL_COMPONENT_ALARM_TRIGGER_ABSOLUTE:
-		t.time = trigger->u.absolute;
-		value_type = ICAL_DATETIME_VALUE;
-		break;
-
-	default:
-		g_assert_not_reached ();
-		return;
-	}
-
-	alarm->trigger = icalproperty_new_trigger (t);
-	icalcomponent_add_property (alarm->icalcomp, alarm->trigger);
-
-	/* Value parameters */
-
-	param = icalproperty_get_first_parameter (alarm->trigger, ICAL_VALUE_PARAMETER);
-	if (param)
-		icalparameter_set_value (param, value_type);
-	else {
-		param = icalparameter_new_value (value_type);
-		icalproperty_add_parameter (alarm->trigger, param);
-	}
-
-	/* Related parameter */
-
-	if (trigger->type == CAL_COMPONENT_ALARM_TRIGGER_RELATIVE) {
-		param = icalproperty_get_first_parameter (alarm->trigger, ICAL_RELATED_PARAMETER);
-
-		if (param)
-			icalparameter_set_related (param, related);
-		else {
-			param = icalparameter_new_related (related);
-			icalproperty_add_parameter (alarm->trigger, param);
-		}
-	}
-}
-
-/**
- * cal_component_alarm_free_trigger:
- * @trigger: A #CalComponentAlarmTrigger structure.
- *
- * Frees a #CalComponentAlarmTrigger structure.
- **/
-void
-cal_component_alarm_free_trigger (CalComponentAlarmTrigger *trigger)
-{
-	g_return_if_fail (trigger != NULL);
-
-	g_free (trigger);
-}
