@@ -1,8 +1,9 @@
 /* Evolution calendar - iCalendar component object
  *
  * Copyright (C) 2000 Helix Code, Inc.
+ * Copyright (C) 2000 Ximian, Inc.
  *
- * Author: Federico Mena-Quintero <federico@helixcode.com>
+ * Author: Federico Mena-Quintero <federico@ximian.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -51,7 +52,9 @@ typedef enum {
 	CAL_COMPONENT_TIMEZONE
 } CalComponentVType;
 
-/* Field identifiers for a calendar component */
+/* Field identifiers for a calendar component; these are used by the data model
+ * for ETable.
+ */
 typedef enum {
 	CAL_COMPONENT_FIELD_CATEGORIES,		/* concatenation of the categories list */
 	CAL_COMPONENT_FIELD_CLASSIFICATION,
@@ -74,8 +77,9 @@ typedef enum {
 	CAL_COMPONENT_FIELD_NUM_FIELDS
 } CalComponentField;
 
-/* Structures to return properties and their parameters */
+/* Structures and enumerations to return properties and their parameters */
 
+/* CLASSIFICATION property */
 typedef enum {
 	CAL_COMPONENT_CLASS_NONE,
 	CAL_COMPONENT_CLASS_PUBLIC,
@@ -84,6 +88,7 @@ typedef enum {
 	CAL_COMPONENT_CLASS_UNKNOWN
 } CalComponentClassification;
 
+/* Properties that have time and timezone information */
 typedef struct {
 	/* Actual date/time value */
 	struct icaltimetype *value;
@@ -92,11 +97,13 @@ typedef struct {
 	const char *tzid;
 } CalComponentDateTime;
 
+/* Way in which a period of time is specified */
 typedef enum {
 	CAL_COMPONENT_PERIOD_DATETIME,
 	CAL_COMPONENT_PERIOD_DURATION
 } CalComponentPeriodType;
 
+/* Period of time, can have explicit start/end times or start/duration instead */
 typedef struct {
 	CalComponentPeriodType type;
 
@@ -108,6 +115,7 @@ typedef struct {
 	} u;
 } CalComponentPeriod;
 
+/* Text properties */
 typedef struct {
 	/* Description string */
 	const char *value;
@@ -116,6 +124,7 @@ typedef struct {
 	const char *altrep;
 } CalComponentText;
 
+/* Time transparency */
 typedef enum {
 	CAL_COMPONENT_TRANSP_NONE,
 	CAL_COMPONENT_TRANSP_TRANSPARENT,
@@ -123,7 +132,7 @@ typedef enum {
 	CAL_COMPONENT_TRANSP_UNKNOWN
 } CalComponentTransparency;
 
-typedef struct _CalComponentAlarm CalComponentAlarm;
+/* Main calendar component object */
 
 typedef struct _CalComponent CalComponent;
 typedef struct _CalComponentClass CalComponentClass;
@@ -165,9 +174,8 @@ void cal_component_commit_sequence (CalComponent *comp);
 void cal_component_get_uid (CalComponent *comp, const char **uid);
 void cal_component_set_uid (CalComponent *comp, const char *uid);
 
-void cal_component_get_status (CalComponent *comp, icalproperty_status *status);
-void cal_component_set_status (CalComponent *comp, icalproperty_status status);
-
+void cal_component_get_categories (CalComponent *comp, const char **categories);
+void cal_component_set_categories (CalComponent *comp, const char *categories);
 void cal_component_get_categories_list (CalComponent *comp, GSList **categ_list);
 void cal_component_set_categories_list (CalComponent *comp, GSList *categ_list);
 
@@ -235,6 +243,9 @@ gboolean cal_component_has_recurrences (CalComponent *comp);
 void cal_component_get_sequence (CalComponent *comp, int **sequence);
 void cal_component_set_sequence (CalComponent *comp, int *sequence);
 
+void cal_component_get_status (CalComponent *comp, icalproperty_status *status);
+void cal_component_set_status (CalComponent *comp, icalproperty_status status);
+
 void cal_component_get_summary (CalComponent *comp, CalComponentText *summary);
 void cal_component_set_summary (CalComponent *comp, CalComponentText *summary);
 
@@ -244,6 +255,8 @@ void cal_component_set_transparency (CalComponent *comp, CalComponentTransparenc
 void cal_component_get_url (CalComponent *comp, const char **url);
 void cal_component_set_url (CalComponent *comp, const char *url);
 
+gboolean cal_component_event_dates_match (CalComponent *comp1, CalComponent *comp2);
+
 /* Functions to free returned values */
 
 void cal_component_free_categories_list (GSList *categ_list);
@@ -256,56 +269,86 @@ void cal_component_free_priority (int *priority);
 void cal_component_free_period_list (GSList *period_list);
 void cal_component_free_recur_list (GSList *recur_list);
 void cal_component_free_sequence (int *sequence);
-void cal_component_free_pilot_id (unsigned long *pilot_status);
-void cal_component_free_pilot_status (unsigned long *pilot_status);
 void cal_component_free_text_list (GSList *text_list);
 
 /* Alarms */
 
-typedef enum {
-	CAL_COMPONENT_ALARM_NONE,
-	CAL_COMPONENT_ALARM_AUDIO,
-	CAL_COMPONENT_ALARM_DISPLAY,
-	CAL_COMPONENT_ALARM_EMAIL,
-	CAL_COMPONENT_ALARM_PROCEDURE,
-	CAL_COMPONENT_ALARM_UNKNOWN
-} CalComponentAlarmAction;
+/* Opaque structure used to represent alarm subcomponents */
+typedef struct _CalComponentAlarm CalComponentAlarm;
 
-typedef enum {
-	CAL_COMPONENT_ALARM_TRIGGER_RELATIVE,
-	CAL_COMPONENT_ALARM_TRIGGER_ABSOLUTE
-} CalComponentAlarmTriggerType;
+/* An alarm occurrence, i.e. a trigger instance */
+typedef struct {
+	/* UID of the alarm that triggered */
+	const char *auid;
 
+	/* Trigger time, i.e. "5 minutes before the appointment" */
+	time_t trigger;
+
+	/* Actual event occurrence to which this trigger corresponds */
+	time_t occur;
+} CalAlarmInstance;
+
+/* Alarm trigger instances for a particular component */
+typedef struct {
+	/* The actual component */
+	CalComponent *comp;
+
+	/* List of CalAlarmInstance structures */
+	GSList *alarms;
+} CalComponentAlarms;
+
+/* Alarm types */
 typedef enum {
-	CAL_COMPONENT_ALARM_TRIGGER_RELATED_START,
-	CAL_COMPONENT_ALARM_TRIGGER_RELATED_END
-} CalComponentAlarmTriggerRelated;
+	CAL_ALARM_NONE,
+	CAL_ALARM_AUDIO,
+	CAL_ALARM_DISPLAY,
+	CAL_ALARM_EMAIL,
+	CAL_ALARM_PROCEDURE,
+	CAL_ALARM_UNKNOWN
+} CalAlarmAction;
+
+/* Whether a trigger is relative to the start or end of an event occurrence, or
+ * whether it is specified to occur at an absolute time.
+ */
+typedef enum {
+	CAL_ALARM_TRIGGER_NONE,
+	CAL_ALARM_TRIGGER_RELATIVE_START,
+	CAL_ALARM_TRIGGER_RELATIVE_END,
+	CAL_ALARM_TRIGGER_ABSOLUTE
+} CalAlarmTriggerType;
 
 typedef struct {
-	CalComponentAlarmTriggerType type;
+	CalAlarmTriggerType type;
 
 	union {
-		struct {
-			struct icaldurationtype duration;
-			CalComponentAlarmTriggerRelated related;
-		} relative;
-
-		struct icaltimetype absolute;
+		struct icaldurationtype rel_duration;
+		struct icaltimetype abs_time;
 	} u;
-} CalComponentAlarmTrigger;
+} CalAlarmTrigger;
 
 gboolean cal_component_has_alarms (CalComponent *comp);
+void cal_component_add_alarm (CalComponent *comp, CalComponentAlarm *alarm);
+void cal_component_remove_alarm (CalComponent *comp, const char *auid);
+
 GList *cal_component_get_alarm_uids (CalComponent *comp);
 CalComponentAlarm *cal_component_get_alarm (CalComponent *comp, const char *auid);
+void cal_component_free_alarm_uids (GList *alarm_uids);
+
+void cal_component_alarms_free (CalComponentAlarms *alarms);
+
+/* CalComponentAlarms */
+CalComponentAlarm *cal_component_alarm_new (void);
+
+const char *cal_component_alarm_get_uid (CalComponentAlarm *alarm);
+
+void cal_component_alarm_get_action (CalComponentAlarm *alarm, CalAlarmAction *action);
+void cal_component_alarm_set_action (CalComponentAlarm *alarm, CalAlarmAction action);
+
+void cal_component_alarm_get_trigger (CalComponentAlarm *alarm, CalAlarmTrigger *trigger);
+void cal_component_alarm_set_trigger (CalComponentAlarm *alarm, CalAlarmTrigger trigger);
 
 void cal_component_alarm_free (CalComponentAlarm *alarm);
 
-void cal_component_alarm_get_action (CalComponentAlarm *alarm, CalComponentAlarmAction *action);
-void cal_component_alarm_set_action (CalComponentAlarm *alarm, CalComponentAlarmAction action);
-
-void cal_component_alarm_get_trigger (CalComponentAlarm *alarm, CalComponentAlarmTrigger **trigger);
-void cal_component_alarm_set_trigger (CalComponentAlarm *alarm, CalComponentAlarmTrigger *trigger);
-void cal_component_alarm_free_trigger (CalComponentAlarmTrigger *trigger);
 
 
 
