@@ -2,9 +2,10 @@
 
 /*
  * Author :
- *  Damon Chaplin <damon@helixcode.com>
+ *  Damon Chaplin <damon@ximian.com>
  *
  * Copyright 2000, Helix Code, Inc.
+ * Copyright 2000, Ximian, Inc.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -29,6 +30,9 @@
 #ifndef _CALENDAR_CONFIG_H_
 #define _CALENDAR_CONFIG_H_
 
+#include <widgets/misc/e-calendar.h>
+#include <widgets/misc/e-dateedit.h>
+
 
 /* These are used to get/set the working days in the week. The bit-flags are
    combined together. The bits must be from 0 (Sun) to 6 (Sat) to match the
@@ -114,4 +118,9 @@ gfloat    calendar_config_get_month_vpane_pos	(void);
 void	  calendar_config_set_month_vpane_pos	(gfloat	      vpane_pos);
 
 
+/* Convenience functions to configure common properties of ECalendar and
+   EDateEdit widgets. */
+void	  calendar_config_configure_e_calendar	(ECalendar   *cal);
+void	  calendar_config_configure_e_date_edit	(EDateEdit   *dedit);
+
 #endif /* _CALENDAR_CONFIG_H_ */
