@@ -3,8 +3,9 @@
 /* Evolution calendar - Data model for ETable
  *
  * Copyright (C) 2000 Helix Code, Inc.
+ * Copyright (C) 2000 Ximian, Inc.
  *
- * Authors: Federico Mena-Quintero <federico@helixcode.com>
+ * Authors: Federico Mena-Quintero <federico@ximian.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,20 +24,28 @@
 
 
 #include <config.h>
+
 #include <math.h>
-#include <ctype.h>
-#include <time.h>
-#include <gnome.h>
-#include <cal-util/timeutil.h>
-#include "calendar-model.h"
-#include "calendar-commands.h"
+#include <sys/types.h>
 
 /* We need this for strptime. */
 #define _XOPEN_SOURCE 500
 #define __USE_XOPEN
+#include <time.h>
 #include <sys/time.h>
 #undef _XOPEN_SOURCE
 #undef __USE_XOPEN
+
+#include <ctype.h>
+#include <glib.h>
+#include <libgnome/gnome-defs.h>
+#include <libgnome/gnome-i18n.h>
+#include <libgnomeui/gnome-messagebox.h>
+#include <cal-util/timeutil.h>
+#include "calendar-model.h"
+#include "calendar-commands.h"
+
+
 
 
 /* Private part of the ECalendarModel structure */
@@ -53,13 +62,31 @@ struct _CalendarModelPrivate {
 	/* UID -> array index hash */
 	GHashTable *uid_index_hash;
 
+	/* Type of components to create when using click-to-add in the table */
+	CalComponentVType new_comp_vtype;
+
 	/* Whether we display dates in 24-hour format. */
 	gboolean use_24_hour_format;
 
 	/* HACK: so that ETable can do its stupid append_row() thing */
 	guint appending_row : 1;
+
+	/* The default category to use when creating new tasks, e.g. when the
+	   filter is set to a certain category we use that category when
+	   creating a new task. */
+	gchar *default_category;
+
+	/* A balanced tree of the categories used by all the tasks/events. */
+	GTree *categories;
 };
 
+enum {
+	CATEGORIES_CHANGED,
+	LAST_SIGNAL
+};
+
+static gint calendar_model_signals [LAST_SIGNAL] = { 0 };
+
 
 
 static void calendar_model_class_init (CalendarModelClass *class);
@@ -84,6 +111,9 @@ static int remove_object (CalendarModel *model, const char *uid);
 static void ensure_task_complete (CalComponent *comp,
 				  time_t completed_date);
 static void ensure_task_not_complete (CalComponent *comp);
+static void calendar_model_collect_all_categories (CalendarModel *model);
+static gboolean calendar_model_collect_categories	(CalendarModel	*model,
+							 CalComponent	*comp);
 
 static ETableModelClass *parent_class;
 
@@ -133,6 +163,17 @@ calendar_model_class_init (CalendarModelClass *class)
 
 	parent_class = gtk_type_class (E_TABLE_MODEL_TYPE);
 
+	calendar_model_signals [CATEGORIES_CHANGED] =
+		gtk_signal_new ("categories-changed",
+				GTK_RUN_LAST, object_class->type,
+				GTK_SIGNAL_OFFSET (CalendarModelClass,
+						   categories_changed),
+				gtk_signal_default_marshaller,
+				GTK_TYPE_NONE, 0);
+	
+	gtk_object_class_add_signals (object_class, calendar_model_signals,
+				      LAST_SIGNAL);
+
 	object_class->destroy = calendar_model_destroy;
 
 	etm_class->column_count = calendar_model_column_count;
@@ -148,6 +189,8 @@ calendar_model_class_init (CalendarModelClass *class)
 #if 0
 	etm_class->value_to_string = calendar_model_value_to_string;
 #endif
+
+	class->categories_changed = NULL;
 }
 
 /* Object initialization function for the calendar table model */
@@ -161,7 +204,10 @@ calendar_model_init (CalendarModel *model)
 
 	priv->objects = g_array_new (FALSE, TRUE, sizeof (CalComponent *));
 	priv->uid_index_hash = g_hash_table_new (g_str_hash, g_str_equal);
+	priv->new_comp_vtype = CAL_COMPONENT_EVENT;
 	priv->use_24_hour_format = TRUE;
+
+	priv->categories = g_tree_new ((GCompareFunc)strcmp);
 }
 
 /* Called from g_hash_table_foreach_remove(), frees a stored UID->index
@@ -231,6 +277,14 @@ calendar_model_destroy (GtkObject *object)
 	g_array_free (priv->objects, TRUE);
 	priv->objects = NULL;
 
+	g_free (priv->default_category);
+
+	/* We only need to free the first argument, the key, so g_free will do.
+	 */
+	g_tree_traverse (priv->categories, (GTraverseFunc) g_free,
+			 G_PRE_ORDER, NULL);
+	g_tree_destroy (priv->categories);
+
 	/* Free the private structure */
 
 	g_free (priv);
@@ -302,31 +356,11 @@ get_time_t (CalendarModel *model, time_t *t, gboolean skip_midnight)
 static char *
 get_categories (CalComponent *comp)
 {
-	GSList *categories;
-	GString *str;
-	char *s;
-	GSList *l;
+	const char *categories;
 
-	cal_component_get_categories_list (comp, &categories);
+	cal_component_get_categories (comp, &categories);
 
-	str = g_string_new (NULL);
-
-	for (l = categories; l; l = l->next) {
-		const char *category;
-
-		category = l->data;
-		g_string_append (str, category);
-
-		if (l->next != NULL)
-			g_string_append (str, ", ");
-	}
-
-	s = str->str;
-
-	g_string_free (str, FALSE);
-	cal_component_free_categories_list (categories);
-
-	return s;
+	return categories ? (char*) categories : "";
 }
 
 /* Returns a string based on the CLASSIFICATION property of a calendar component */
@@ -355,7 +389,7 @@ get_classification (CalComponent *comp)
 
 	default:
 		g_assert_not_reached ();
-		return NULL;
+		return "";
 	}
 }
 
@@ -975,7 +1009,7 @@ set_datetime (CalendarModel *model, CalComponent *comp, const char *value,
 		CalComponentDateTime dt;
 		struct icaltimetype itt;
 
-		itt = icaltime_from_timet (t, FALSE, TRUE);
+		itt = icaltime_from_timet (t, FALSE);
 		dt.value = &itt;
 		dt.tzid = NULL;
 
@@ -1161,6 +1195,10 @@ calendar_model_set_value_at (ETableModel *etm, int col, int row, const void *val
 	switch (col) {
 	case CAL_COMPONENT_FIELD_CATEGORIES:
 		set_categories (comp, value);
+		if (calendar_model_collect_categories (model, comp)) {
+			gtk_signal_emit (GTK_OBJECT (model),
+					 calendar_model_signals [CATEGORIES_CHANGED]);
+		}
 		break;
 
 	/* FIXME: CLASSIFICATION requires an option menu cell renderer */
@@ -1278,7 +1316,7 @@ calendar_model_append_row (ETableModel *etm, ETableModel *source, gint row)
 	 * is only used for the task list.
 	 */
 	comp = cal_component_new ();
-	cal_component_set_new_vtype (comp, CAL_COMPONENT_TODO);
+	cal_component_set_new_vtype (comp, priv->new_comp_vtype);
 
 	cal_component_get_uid (comp, &uid);
 
@@ -1332,11 +1370,7 @@ calendar_model_duplicate_value (ETableModel *etm, int col, const void *value)
 
 	switch (col) {
 	case CAL_COMPONENT_FIELD_CATEGORIES:
-		return dup_string (value);
-
 	case CAL_COMPONENT_FIELD_CLASSIFICATION:
-		return (void *) value;
-
 	case CAL_COMPONENT_FIELD_COMPLETED:
 	case CAL_COMPONENT_FIELD_DTEND:
 	case CAL_COMPONENT_FIELD_DTSTART:
@@ -1345,11 +1379,7 @@ calendar_model_duplicate_value (ETableModel *etm, int col, const void *value)
 	case CAL_COMPONENT_FIELD_PERCENT:
 	case CAL_COMPONENT_FIELD_PRIORITY:
 	case CAL_COMPONENT_FIELD_SUMMARY:
-		return dup_string (value);
-
 	case CAL_COMPONENT_FIELD_TRANSPARENCY:
-		return (void *) value;
-
 	case CAL_COMPONENT_FIELD_URL:
 		return dup_string (value);
 
@@ -1421,15 +1451,17 @@ init_string (void)
 static void *
 calendar_model_initialize_value (ETableModel *etm, int col)
 {
+	CalendarModel *model;
+
 	g_return_val_if_fail (col >= 0 && col < CAL_COMPONENT_FIELD_NUM_FIELDS, NULL);
 
+	model = CALENDAR_MODEL (etm);
+
 	switch (col) {
 	case CAL_COMPONENT_FIELD_CATEGORIES:
-		return init_string ();
+		return g_strdup (model->priv->default_category ? model->priv->default_category : "");
 
 	case CAL_COMPONENT_FIELD_CLASSIFICATION:
-		return NULL;
-
 	case CAL_COMPONENT_FIELD_COMPLETED:
 	case CAL_COMPONENT_FIELD_DTEND:
 	case CAL_COMPONENT_FIELD_DTSTART:
@@ -1438,11 +1470,7 @@ calendar_model_initialize_value (ETableModel *etm, int col)
 	case CAL_COMPONENT_FIELD_PERCENT:
 	case CAL_COMPONENT_FIELD_PRIORITY:
 	case CAL_COMPONENT_FIELD_SUMMARY:
-		return init_string ();
-
 	case CAL_COMPONENT_FIELD_TRANSPARENCY:
-		return NULL;
-
 	case CAL_COMPONENT_FIELD_URL:
 		return init_string ();
 
@@ -1515,18 +1543,20 @@ calendar_model_new (void)
 }
 
 
-/* Callback used when a calendar is loaded into the server */
+/* Callback used when a calendar is opened into the server */
 static void
-cal_loaded_cb (CalClient *client,
-	       CalClientLoadStatus status,
-	       CalendarModel *model)
+cal_opened_cb (CalClient *client, CalClientOpenStatus status, gpointer data)
 {
-	g_return_if_fail (IS_CALENDAR_MODEL (model));
+	CalendarModel *model;
+
+	model = CALENDAR_MODEL (data);
 
 	e_table_model_pre_change (E_TABLE_MODEL (model));
 
-	if (status == CAL_CLIENT_LOAD_SUCCESS)
+	if (status == CAL_CLIENT_OPEN_SUCCESS) {
 		load_objects (model);
+		calendar_model_collect_all_categories (model);
+	}
 
 	e_table_model_changed (E_TABLE_MODEL (model));
 }
@@ -1673,6 +1703,14 @@ obj_updated_cb (CalClient *client, const char *uid, gpointer data)
 			e_table_model_row_changed (E_TABLE_MODEL (model), *new_idx);
 		}
 
+		/* See if we need to add any categories. Note that old
+		   categories won't be removed, but I don't think that matters
+		   too much here. */
+		if (calendar_model_collect_categories (model, new_comp)) {
+			gtk_signal_emit (GTK_OBJECT (model),
+					 calendar_model_signals [CATEGORIES_CHANGED]);
+		}
+
 		break;
 
 	case CAL_CLIENT_GET_NOT_FOUND:
@@ -1723,8 +1761,7 @@ load_objects (CalendarModel *model)
 
 	priv = model->priv;
 
-	if (!cal_client_is_loaded (priv->client))
-		return;
+	g_assert (cal_client_get_load_state (priv->client) == CAL_CLIENT_LOAD_LOADED);
 
 	uids = cal_client_get_uids (priv->client, priv->type);
 
@@ -1840,20 +1877,65 @@ calendar_model_set_cal_client (CalendarModel *model, CalClient *client, CalObjTy
 	priv->type = type;
 
 	if (priv->client) {
-		gtk_signal_connect (GTK_OBJECT (priv->client), "cal_loaded",
-				    GTK_SIGNAL_FUNC (cal_loaded_cb), model);
 		gtk_signal_connect (GTK_OBJECT (priv->client), "obj_updated",
 				    GTK_SIGNAL_FUNC (obj_updated_cb), model);
 		gtk_signal_connect (GTK_OBJECT (priv->client), "obj_removed",
 				    GTK_SIGNAL_FUNC (obj_removed_cb), model);
 
-		load_objects (model);
+		if (cal_client_get_load_state (priv->client) != CAL_CLIENT_LOAD_LOADED)
+			gtk_signal_connect (GTK_OBJECT (priv->client), "cal_opened",
+					    GTK_SIGNAL_FUNC (cal_opened_cb), model);
+		else
+			load_objects (model);
 	}
 
 	e_table_model_changed (E_TABLE_MODEL (model));
 }
 
 
+/**
+ * calendar_model_set_new_comp_vtype:
+ * @model: A calendar model.
+ * @vtype: Type of calendar components to create.
+ * 
+ * Sets the type of calendar components that will be created by a calendar table
+ * model when the click-to-add functionality of the table is used.
+ **/
+void
+calendar_model_set_new_comp_vtype (CalendarModel *model, CalComponentVType vtype)
+{
+	CalendarModelPrivate *priv;
+
+	g_return_if_fail (model != NULL);
+	g_return_if_fail (IS_CALENDAR_MODEL (model));
+	g_return_if_fail (vtype != CAL_COMPONENT_NO_TYPE);
+
+	priv = model->priv;
+	priv->new_comp_vtype = vtype;
+}
+
+/**
+ * calendar_model_get_new_comp_vtype:
+ * @model: A calendar model.
+ * 
+ * Queries the type of calendar components that are created by a calendar table
+ * model when using the click-to-add functionality in a table.
+ * 
+ * Return value: Type of components that are created.
+ **/
+CalComponentVType
+calendar_model_get_new_comp_vtype (CalendarModel *model)
+{
+	CalendarModelPrivate *priv;
+
+	g_return_val_if_fail (model != NULL, CAL_COMPONENT_NO_TYPE);
+	g_return_val_if_fail (IS_CALENDAR_MODEL (model), CAL_COMPONENT_NO_TYPE);
+
+	priv = model->priv;
+	return priv->new_comp_vtype;
+}
+
+
 void
 calendar_model_mark_task_complete (CalendarModel *model,
 				   gint row)
@@ -1917,6 +1999,7 @@ ensure_task_complete (CalComponent *comp,
 	struct icaltimetype *old_completed = NULL;
 	struct icaltimetype new_completed;
 	int *old_percent, new_percent;
+	icalproperty_status status;
 	gboolean set_completed = TRUE;
 
 	/* Date Completed. */
@@ -1932,7 +2015,7 @@ ensure_task_complete (CalComponent *comp,
 	}
 
 	if (set_completed) {
-		new_completed = icaltime_from_timet (completed_date, FALSE, TRUE);
+		new_completed = icaltime_from_timet (completed_date, FALSE);
 		cal_component_set_completed (comp, &new_completed);
 	}
 
@@ -1944,6 +2027,12 @@ ensure_task_complete (CalComponent *comp,
 	}
 	if (old_percent)
 		cal_component_free_percent (old_percent);
+
+	/* Status. */
+	cal_component_get_status (comp, &status);
+	if (status != ICAL_STATUS_COMPLETED) {
+		cal_component_set_status (comp, ICAL_STATUS_COMPLETED);
+	}
 }
 
 
@@ -1997,3 +2086,79 @@ calendar_model_set_use_24_hour_format (CalendarModel *model,
 	}
 }
 
+
+void
+calendar_model_set_default_category	(CalendarModel	*model,
+					 gchar		*default_category)
+{
+	g_return_if_fail (IS_CALENDAR_MODEL (model));
+
+	g_free (model->priv->default_category);
+	model->priv->default_category = g_strdup (default_category);
+}
+
+
+static void
+calendar_model_collect_all_categories	(CalendarModel	*model)
+{
+	CalendarModelPrivate *priv;
+	CalComponent *comp;
+	int i;
+
+	priv = model->priv;
+
+	/* Destroy the current tree and start from scratch. */
+	g_tree_traverse (priv->categories, (GTraverseFunc) g_free,
+			 G_PRE_ORDER, NULL);
+	g_tree_destroy (priv->categories);
+
+	priv->categories = g_tree_new ((GCompareFunc)strcmp);
+
+	for (i = 0; i < priv->objects->len; i++) {
+		comp = g_array_index (priv->objects, CalComponent *, i);
+		calendar_model_collect_categories (model, comp);
+	}
+
+	gtk_signal_emit (GTK_OBJECT (model),
+			 calendar_model_signals [CATEGORIES_CHANGED]);
+}
+
+
+static gboolean
+calendar_model_collect_categories	(CalendarModel	*model,
+					 CalComponent	*comp)
+{
+	CalendarModelPrivate *priv;
+	GSList *categories_list, *elem;
+	gboolean changed = FALSE;
+
+	priv = model->priv;
+
+	cal_component_get_categories_list (comp, &categories_list);
+
+	for (elem = categories_list; elem; elem = elem->next) {
+		if (!g_tree_lookup (priv->categories, elem->data)) {
+			/* We store a '1' as the data, just so we can use
+			   g_tree_lookup() on it. Note that we don't free
+			   the string since it is now part of the tree. */
+			g_tree_insert (priv->categories, elem->data,
+				       GINT_TO_POINTER (1));
+			changed = TRUE;
+		} else {
+			g_free (elem->data);
+		}
+	}
+
+	g_slist_free (categories_list);
+
+	return changed;
+}
+
+
+GTree*
+calendar_model_get_categories		(CalendarModel	*model)
+{
+	g_return_val_if_fail (IS_CALENDAR_MODEL (model), NULL);
+
+	return model->priv->categories;
+}
