@@ -2,9 +2,10 @@
 
 /*
  * Author :
- *  Damon Chaplin <damon@helixcode.com>
+ *  Damon Chaplin <damon@ximian.com>
  *
  * Copyright 2000, Helix Code, Inc.
+ * Copyright 2000, Ximian, Inc.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -28,14 +29,17 @@
  */
 
 #include <config.h>
-#include <gnome.h>
 #include <glade/glade.h>
+#include <libgnome/gnome-defs.h>
+#include <libgnome/gnome-i18n.h>
 #include <gal/util/e-util.h>
 #include <gal/widgets/e-unicode.h>
+#include <gal/widgets/e-categories.h>
 #include <e-util/e-dialog-widgets.h>
 #include <widgets/misc/e-dateedit.h>
 #include <cal-util/timeutil.h>
-#include <cal-client/cal-client.h>
+#include "delete-comp.h"
+#include "save-comp.h"
 #include "task-editor.h"
 #include "../calendar-config.h"
 #include "../widget-util.h"
@@ -45,12 +49,9 @@ typedef struct {
 	/* Glade XML data */
 	GladeXML *xml;
 
-	/* UI handler */
-	BonoboUIComponent *uic;
-
 	/* Client to use */
 	CalClient *client;
-	
+
 	/* Calendar component we are editing; this is an internal copy and is
 	 * not one of the read-only objects from the parent calendar.
 	 */
@@ -79,6 +80,7 @@ typedef struct {
 	GtkWidget *description;
 
 	GtkWidget *contacts;
+	GtkWidget *categories_btn;
 	GtkWidget *categories;
 
 	GtkWidget *completed_date;
@@ -126,9 +128,9 @@ static const int classification_map[] = {
 
 static void task_editor_class_init (TaskEditorClass *class);
 static void task_editor_init (TaskEditor *tedit);
-static gint app_delete_event_cb (GtkWidget *widget,
-				 GdkEvent *event,
-				 gpointer data);
+static void tedit_apply_event_cb (GtkWidget *widget, gint page_num, gpointer data);
+static gint tedit_close_event_cb (GtkWidget *widget, gpointer data);
+static gint tedit_delete_event_cb (GtkWidget *widget, GdkEvent *event, gpointer data);
 static void close_dialog (TaskEditor *tedit);
 static gboolean get_widgets (TaskEditor *tedit);
 static void init_widgets (TaskEditor *tedit);
@@ -138,13 +140,6 @@ static void set_title_from_comp (TaskEditor *tedit, CalComponent *comp);
 static void clear_widgets (TaskEditor *tedit);
 static void fill_widgets (TaskEditor *tedit);
 
-static void file_save_cb (GtkWidget *widget, gpointer data);
-static void file_save_and_close_cb (GtkWidget *widget, gpointer data);
-static void file_delete_cb (GtkWidget *widget, gpointer data);
-static void file_close_cb (GtkWidget *widget, gpointer data);
-
-static void debug_xml_cb (GtkWidget *widget, gpointer data);
-
 static void save_todo_object (TaskEditor *tedit);
 static void dialog_to_comp_object (TaskEditor *tedit);
 
@@ -166,6 +161,8 @@ static void field_changed		(GtkWidget	*widget,
 static void task_editor_set_changed	(TaskEditor	*tedit,
 					 gboolean	 changed);
 static gboolean prompt_to_save_changes	(TaskEditor	*tedit);
+static void categories_clicked          (GtkWidget      *button,
+					 TaskEditor     *editor);
 
 /* The function libglade calls to create the EDateEdit widgets in the GUI. */
 GtkWidget * task_editor_create_date_edit (void);
@@ -218,24 +215,12 @@ task_editor_new (void)
 	return task_editor_construct (tedit);
 }
 
-static BonoboUIVerb verbs [] = {
-
-	BONOBO_UI_UNSAFE_VERB ("FileSave", file_save_cb),
-	BONOBO_UI_UNSAFE_VERB ("FileDelete", file_delete_cb),
-	BONOBO_UI_UNSAFE_VERB ("FileClose", file_close_cb),
-	BONOBO_UI_UNSAFE_VERB ("FileSaveAndClose", file_save_and_close_cb),
-
-	BONOBO_UI_UNSAFE_VERB ("DebugDumpXml", debug_xml_cb),
-
-	BONOBO_UI_VERB_END
-};
-
 /**
  * task_editor_construct:
  * @tedit: A #TaskEditor.
- * 
+ *
  * Constructs a task editor by loading its Glade XML file.
- * 
+ *
  * Return value: The same object as @tedit, or NULL if the widgets could not be
  * created.  In the latter case, the task editor will automatically be
  * destroyed.
@@ -244,7 +229,6 @@ TaskEditor *
 task_editor_construct (TaskEditor *tedit)
 {
 	TaskEditorPrivate *priv;
-	GtkWidget         *bonobo_win;
 
 	g_return_val_if_fail (tedit != NULL, NULL);
 	g_return_val_if_fail (IS_TASK_EDITOR (tedit), NULL);
@@ -266,53 +250,17 @@ task_editor_construct (TaskEditor *tedit)
 
 	init_widgets (tedit);
 
-	/* Construct the app */
-
-	priv->uic = bonobo_ui_component_new ("task-editor-dialog");
-	if (!priv->uic) {
-		g_message ("task_editor_construct(): Could not create the UI component");
-		goto error;
-	}
-
-	bonobo_win = bonobo_window_new ("event-editor-dialog", "Event Editor");
-
-	/* FIXME: The sucking bit */
-	{
-		GtkWidget *contents;
-
-		contents = gnome_dock_get_client_area (
-			GNOME_DOCK (GNOME_APP (priv->app)->dock));
-		if (!contents) {
-			g_message ("event_editor_construct(): Could not get contents");
-			goto error;
-		}
-		gtk_widget_ref (contents);
-		gtk_container_remove (GTK_CONTAINER (contents->parent), contents);
-		bonobo_window_set_contents (BONOBO_WINDOW (bonobo_win), contents);
-		gtk_widget_destroy (priv->app);
-		priv->app = GTK_WIDGET (bonobo_win);
-	}
-
-	{
-		BonoboUIContainer *container = bonobo_ui_container_new ();
-		bonobo_ui_container_set_win (container, BONOBO_WINDOW (priv->app));
-		bonobo_ui_component_set_container (
-			priv->uic, bonobo_object_corba_objref (BONOBO_OBJECT (container)));
-	}
-
-	bonobo_ui_component_add_verb_list_with_data (
-		priv->uic, verbs, tedit);
-
-	bonobo_ui_util_set_ui (priv->uic, EVOLUTION_DATADIR,
-			       "evolution-task-editor-dialog.xml",
-			       "evolution-task-editor");
-
 	/* Hook to destruction of the dialog */
+	gtk_signal_connect (GTK_OBJECT (priv->app), "apply",
+			    GTK_SIGNAL_FUNC (tedit_apply_event_cb), tedit); 
+	gtk_signal_connect (GTK_OBJECT (priv->app), "close",
+			    GTK_SIGNAL_FUNC (tedit_close_event_cb), tedit);
 	gtk_signal_connect (GTK_OBJECT (priv->app), "delete_event",
-			    GTK_SIGNAL_FUNC (app_delete_event_cb), tedit);
+			    GTK_SIGNAL_FUNC (tedit_delete_event_cb), tedit);
+
+	/* Add focus to the summary entry */
+	gtk_widget_grab_focus (GTK_WIDGET (priv->summary));
 
-	/* Show the dialog */
-	gtk_widget_show (priv->app);
 
 	return tedit;
 
@@ -335,10 +283,41 @@ task_editor_create_date_edit (void)
 	return dedit;
 }
 
+/* Callback used when the dialog box is destroyed */
+static void
+tedit_apply_event_cb (GtkWidget *widget, gint page_num, gpointer data)
+{
+	TaskEditor *tedit;
+
+	g_return_if_fail (IS_TASK_EDITOR (data));
+
+	tedit = TASK_EDITOR (data);
+
+	if (page_num != -1)
+		return;
+	
+	save_todo_object (tedit);
+}
 
 /* Callback used when the dialog box is destroyed */
 static gint
-app_delete_event_cb (GtkWidget *widget, GdkEvent *event, gpointer data)
+tedit_close_event_cb (GtkWidget *widget, gpointer data)
+{
+	TaskEditor *tedit;
+
+	g_return_val_if_fail (IS_TASK_EDITOR (data), TRUE);
+
+	tedit = TASK_EDITOR (data);
+
+	if (prompt_to_save_changes (tedit))
+		close_dialog (tedit);
+
+	return TRUE;
+}
+
+/* Callback used when the dialog box is destroyed */
+static gint
+tedit_delete_event_cb (GtkWidget *widget, GdkEvent *event, gpointer data)
 {
 	TaskEditor *tedit;
 
@@ -396,6 +375,7 @@ get_widgets (TaskEditor *tedit)
 	priv->description = GW ("description");
 
 	priv->contacts = GW ("contacts");
+	priv->categories_btn = GW ("categories-button");
 	priv->categories = GW ("categories");
 
 	priv->completed_date = GW ("completed-date");
@@ -413,6 +393,7 @@ get_widgets (TaskEditor *tedit)
 		&& priv->classification
 		&& priv->description
 		&& priv->contacts
+		&& priv->categories_btn
 		&& priv->categories
 		&& priv->completed_date
 		&& priv->url);
@@ -464,6 +445,9 @@ init_widgets (TaskEditor *tedit)
 	gtk_signal_connect (GTK_OBJECT (priv->url), "changed",
 			    GTK_SIGNAL_FUNC (field_changed), tedit);
 
+	/* Button clicks */
+	gtk_signal_connect (GTK_OBJECT (priv->categories_btn), "clicked",
+			    GTK_SIGNAL_FUNC (categories_clicked), tedit);
 }
 
 static void
@@ -478,11 +462,6 @@ task_editor_destroy (GtkObject *object)
 	tedit = TASK_EDITOR (object);
 	priv = tedit->priv;
 
-	if (priv->uic) {
-		bonobo_object_unref (BONOBO_OBJECT (priv->uic));
-		priv->uic = NULL;
-	}
-
 	if (priv->app) {
 		gtk_widget_destroy (priv->app);
 		priv->app = NULL;
@@ -513,7 +492,7 @@ task_editor_destroy (GtkObject *object)
 }
 
 
-void 
+void
 task_editor_set_cal_client (TaskEditor *tedit,
 			    CalClient *client)
 {
@@ -531,8 +510,8 @@ task_editor_set_cal_client (TaskEditor *tedit,
 		g_return_if_fail (IS_CAL_CLIENT (client));
 
 	if (client)
-		g_return_if_fail (cal_client_is_loaded (client));	
-	
+		g_return_if_fail (cal_client_get_load_state (client) == CAL_CLIENT_LOAD_LOADED);
+
 	if (client)
 		gtk_object_ref (GTK_OBJECT (client));
 
@@ -568,7 +547,7 @@ obj_updated_cb (CalClient *client, const char *uid, gpointer data)
 	g_return_if_fail (IS_TASK_EDITOR (tedit));
 
 	priv = tedit->priv;
-	
+
 	/* If we aren't showing the object which has been updated, return. */
 	if (!priv->comp)
 	  return;
@@ -642,7 +621,7 @@ raise_and_focus (GtkWidget *widget)
  * task_editor_set_todo_object:
  * @tedit: A #TaskEditor.
  * @comp: A todo object.
- * 
+ *
  * Sets the todo object that a task editor dialog will manipulate.
  **/
 void
@@ -668,6 +647,18 @@ task_editor_set_todo_object	(TaskEditor	*tedit,
 	fill_widgets (tedit);
 }
 
+void
+task_editor_focus (TaskEditor *tedit)
+{
+	TaskEditorPrivate *priv;
+
+	g_return_if_fail (tedit != NULL);
+	g_return_if_fail (IS_TASK_EDITOR (tedit));
+
+	priv = tedit->priv;
+	gtk_widget_show_now (priv->app);
+	raise_and_focus (priv->app);
+}
 
 /* Creates an appropriate title for the task editor dialog */
 static char *
@@ -676,7 +667,7 @@ make_title_from_comp (CalComponent *comp)
 	const char *summary;
 	CalComponentVType type;
 	CalComponentText text;
-	
+
 	if (!comp)
 		return g_strdup (_("Edit Task"));
 
@@ -686,7 +677,7 @@ make_title_from_comp (CalComponent *comp)
 	else
 		summary =  _("No summary");
 
-	
+
 	type = cal_component_get_vtype (comp);
 	switch (type) {
 	case CAL_COMPONENT_EVENT:
@@ -752,6 +743,7 @@ fill_widgets (TaskEditor *tedit)
 	icalproperty_status status;
 	TaskEditorPriority priority;
 	const char *url;
+	const char *categories;
 
 	priv = tedit->priv;
 
@@ -777,7 +769,7 @@ fill_widgets (TaskEditor *tedit)
 		e_dialog_editable_set (priv->description, NULL);
 	}
 	cal_component_free_text_list (l);
-	
+
 	/* Due Date. */
 	cal_component_get_due (priv->comp, &d);
 	if (d.value) {
@@ -848,6 +840,9 @@ fill_widgets (TaskEditor *tedit)
 	e_dialog_option_menu_set (priv->classification, classification,
 				  classification_map);
 
+	/* Categories */
+	cal_component_get_categories (priv->comp, &categories);
+	e_dialog_editable_set (priv->categories, categories);
 
 	/* URL. */
 	cal_component_get_url (priv->comp, &url);
@@ -891,9 +886,9 @@ dialog_to_comp_object (TaskEditor *tedit)
 	TaskEditorPriority priority;
 	int priority_value, percent;
 	CalComponentClassification classification;
-	char *url;
+	char *url, *cat;
 	char *str;
-	
+
 	priv = tedit->priv;
 	comp = priv->comp;
 
@@ -930,7 +925,7 @@ dialog_to_comp_object (TaskEditor *tedit)
 
 		cal_component_set_description_list (comp, &l);
 	}
-	
+
 	if (!str)
 		g_free (str);
 
@@ -942,7 +937,7 @@ dialog_to_comp_object (TaskEditor *tedit)
 	/* Due Date. */
 	t = e_date_edit_get_time (E_DATE_EDIT (priv->due_date));
 	if (t != -1) {
-		*date.value = icaltime_from_timet (t, FALSE, TRUE);
+		*date.value = icaltime_from_timet (t, FALSE);
 		cal_component_set_due (comp, &date);
 	} else {
 		cal_component_set_due (comp, NULL);
@@ -951,7 +946,7 @@ dialog_to_comp_object (TaskEditor *tedit)
 	/* Start Date. */
 	t = e_date_edit_get_time (E_DATE_EDIT (priv->start_date));
 	if (t != -1) {
-		*date.value = icaltime_from_timet (t, FALSE, TRUE);
+		*date.value = icaltime_from_timet (t, FALSE);
 		cal_component_set_dtstart (comp, &date);
 	} else {
 		cal_component_set_dtstart (comp, NULL);
@@ -960,7 +955,7 @@ dialog_to_comp_object (TaskEditor *tedit)
 	/* Completed Date. */
 	t = e_date_edit_get_time (E_DATE_EDIT (priv->completed_date));
 	if (t != -1) {
-		*date.value = icaltime_from_timet (t, FALSE, TRUE);
+		*date.value = icaltime_from_timet (t, FALSE);
 		cal_component_set_completed (comp, date.value);
 	} else {
 		cal_component_set_completed (comp, NULL);
@@ -986,6 +981,12 @@ dialog_to_comp_object (TaskEditor *tedit)
 						   classification_map);
 	cal_component_set_classification (comp, classification);
 
+	/* Categories */
+	cat = e_dialog_editable_get (priv->categories);
+	cal_component_set_categories (comp, cat);
+
+	if (cat)
+		g_free (cat);
 
 	/* URL. */
 	url = e_dialog_editable_get (priv->url);
@@ -994,81 +995,9 @@ dialog_to_comp_object (TaskEditor *tedit)
 	if (url)
 		g_free (url);
 
-
 	cal_component_commit_sequence (comp);
 }
 
-static void
-debug_xml_cb (GtkWidget *widget, gpointer data)
-{
-	TaskEditor *tedit = TASK_EDITOR (data);
-	TaskEditorPrivate *priv = tedit->priv;
-	
-	bonobo_window_dump (BONOBO_WINDOW (priv->app), "on demand");
-}
-
-/* File/Save callback */
-static void
-file_save_cb (GtkWidget *widget, gpointer data)
-{
-	TaskEditor *tedit;
-
-	tedit = TASK_EDITOR (data);
-	save_todo_object (tedit);
-}
-
-/* File/Save and Close callback */
-static void
-file_save_and_close_cb (GtkWidget *widget, gpointer data)
-{
-	TaskEditor *tedit;
-
-	tedit = TASK_EDITOR (data);
-	save_todo_object (tedit);
-	close_dialog (tedit);
-}
-
-/* File/Delete callback */
-static void
-file_delete_cb (GtkWidget *widget, gpointer data)
-{
-	TaskEditor *tedit;
-	TaskEditorPrivate *priv;
-	const char *uid;
-	
-	tedit = TASK_EDITOR (data);
-
-	g_return_if_fail (IS_TASK_EDITOR (tedit));
-
-	priv = tedit->priv;
-	
-	g_return_if_fail (priv->comp);
-
-	cal_component_get_uid (priv->comp, &uid);
-
-	/* We don't check the return value; FALSE can mean the object was not in
-	 * the server anyways.
-	 */
-	cal_client_remove_object (priv->client, uid);
-
-	close_dialog (tedit);
-}
-
-/* File/Close callback */
-static void
-file_close_cb (GtkWidget *widget, gpointer data)
-{
-	TaskEditor *tedit;
-
-	g_return_if_fail (IS_TASK_EDITOR (data));
-
-	tedit = TASK_EDITOR (data);
-
-	if (prompt_to_save_changes (tedit))
-		close_dialog (tedit);
-}
-
-
 static TaskEditorPriority
 priority_value_to_index (int priority_value)
 {
@@ -1080,7 +1009,7 @@ priority_value_to_index (int priority_value)
 		retval = PRIORITY_HIGH;
 	else if (priority_value == 5)
 		retval = PRIORITY_NORMAL;
-	else 
+	else
 		retval = PRIORITY_LOW;
 
 	return retval;
@@ -1255,6 +1184,9 @@ task_editor_set_changed		(TaskEditor	*tedit,
 #endif
 
 	priv->changed = changed;
+
+	if (priv->app)
+		gnome_property_box_set_state (GNOME_PROPERTY_BOX (priv->app), changed);
 }
 
 
@@ -1265,24 +1197,13 @@ static gboolean
 prompt_to_save_changes		(TaskEditor	*tedit)
 {
 	TaskEditorPrivate *priv;
-	GtkWidget *dialog;
 
 	priv = tedit->priv;
 
 	if (!priv->changed)
 		return TRUE;
 
-	dialog = gnome_message_box_new (_("Do you want to save changes?"),
-					GNOME_MESSAGE_BOX_QUESTION,
-					GNOME_STOCK_BUTTON_YES,
-					GNOME_STOCK_BUTTON_NO,
-					GNOME_STOCK_BUTTON_CANCEL,
-					NULL);
-
-	gnome_dialog_set_parent (GNOME_DIALOG (dialog),
-				 GTK_WINDOW (priv->app));
-		
-	switch (gnome_dialog_run_and_close (GNOME_DIALOG (dialog))) {
+	switch (save_component_dialog (GTK_WINDOW (priv->app))) {
 	case 0: /* Save */
 		/* FIXME: If an error occurs here, we should popup a dialog
 		   and then return FALSE. */
@@ -1297,3 +1218,28 @@ prompt_to_save_changes		(TaskEditor	*tedit)
 	}
 
 }
+
+static void
+categories_clicked(GtkWidget *button, TaskEditor *tedit)
+{
+	char *categories;
+	GnomeDialog *dialog;
+	int result;
+	GtkWidget *entry;
+
+	entry = ((TaskEditorPrivate *)tedit->priv)->categories;
+	categories = e_utf8_gtk_entry_get_text (GTK_ENTRY (entry));
+
+	dialog = GNOME_DIALOG (e_categories_new (categories));
+	result = gnome_dialog_run (dialog);
+	g_free (categories);
+
+	if (result == 0) {
+		gtk_object_get (GTK_OBJECT (dialog),
+				"categories", &categories,
+				NULL);
+		e_utf8_gtk_entry_set_text (GTK_ENTRY (entry), categories);
+		g_free (categories);
+	}
+	gtk_object_destroy (GTK_OBJECT (dialog));
+}
