@@ -2,9 +2,10 @@
 
 /*
  * Author :
- *  Damon Chaplin <damon@helixcode.com>
+ *  Damon Chaplin <damon@ximian.com>
  *
  * Copyright 2000, Helix Code, Inc.
+ * Copyright 2000, Ximian, Inc.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -49,34 +50,45 @@
 #include "check-filled.xpm"
 
 
-static void e_calendar_table_class_init (ECalendarTableClass *class);
-static void e_calendar_table_init (ECalendarTable *cal_table);
-static void e_calendar_table_destroy (GtkObject *object);
+static void e_calendar_table_class_init		(ECalendarTableClass *class);
+static void e_calendar_table_init		(ECalendarTable	*cal_table);
+static void e_calendar_table_destroy		(GtkObject	*object);
 
-static void e_calendar_table_on_double_click (ETable *table,
-					      gint row,
-					      gint col,
-					      GdkEvent *event,
-					      ECalendarTable *cal_table);
-static gint e_calendar_table_on_right_click (ETable *table,
-					     gint row,
-					     gint col,
-					     GdkEventButton *event,
-					     ECalendarTable *cal_table);
-static void e_calendar_table_on_open_task (GtkWidget *menuitem,
-					   gpointer	  data);
-static void e_calendar_table_on_mark_task_complete (GtkWidget *menuitem,
-						    gpointer   data);
-static void e_calendar_table_on_delete_task (GtkWidget *menuitem,
-					     gpointer   data);
-static gint e_calendar_table_on_key_press (ETable *table,
-					   gint row,
-					   gint col,
-					   GdkEventKey *event,
-					   ECalendarTable *cal_table);
+static void e_calendar_table_on_double_click	(ETable		*table,
+						 gint		 row,
+						 gint		 col,
+						 GdkEvent	*event,
+						 ECalendarTable *cal_table);
+static gint e_calendar_table_on_right_click	(ETable		*table,
+						 gint		 row,
+						 gint		 col,
+						 GdkEventButton *event,
+						 ECalendarTable *cal_table);
+static void e_calendar_table_on_open_task	(GtkWidget	*menuitem,
+						 gpointer	 data);
+static void e_calendar_table_on_mark_task_complete (GtkWidget	*menuitem,
+						    gpointer	 data);
+static void e_calendar_table_on_delete_task	(GtkWidget	*menuitem,
+						 gpointer	 data);
+static gint e_calendar_table_on_key_press	(ETable		*table,
+						 gint		 row,
+						 gint		 col,
+						 GdkEventKey	*event,
+						 ECalendarTable *cal_table);
+
+static void e_calendar_table_open_task		(ECalendarTable *cal_table,
+						 gint		 row);
+
+static void e_calendar_table_apply_filter	(ECalendarTable	*cal_table);
+static void e_calendar_table_on_model_changed	(ETableModel	*model,
+						 ECalendarTable	*cal_table);
+static void e_calendar_table_on_row_inserted	(ETableModel	*model,
+						 gint		 row,
+						 ECalendarTable	*cal_table);
+static void e_calendar_table_on_row_deleted	(ETableModel	*model,
+						 gint		 row,
+						 ECalendarTable	*cal_table);
 
-static void e_calendar_table_open_task (ECalendarTable *cal_table,
-					gint row);
 
 /* The icons to represent the task. */
 #define E_CALENDAR_MODEL_NUM_ICONS	4
@@ -137,62 +149,80 @@ e_calendar_table_class_init (ECalendarTableClass *class)
 #endif
 }
 
+#ifdef JUST_FOR_TRANSLATORS
+static char *list [] = {
+	N_("Categories"),
+	N_("Classification"),
+	N_("Completion Date"),
+	N_("End Date"),
+	N_("Start Date"),
+	N_("Due Date"),
+	N_("Geographical Position"),
+	N_("Precent complete"),
+	N_("Priority"),
+	N_("Summary"),
+	N_("Transparency"),
+	N_("URL"),
+	N_("Alarms"),
+	N_("Click here to add a task")
+};
+#endif
 
-#define E_CALENDAR_TABLE_SPEC							\
-	"<ETableSpecification click-to-add=\"true\" "				\
-	" _click-to-add-message=\"Click to add a task\" "			\
-	" draw-grid=\"true\">"							\
-        "  <ETableColumn model_col= \"0\" _title=\"Categories\" "		\
-	"   expansion=\"1.0\" minimum_width=\"10\" resizable=\"true\" "		\
-	"   cell=\"string\"   compare=\"string\"/>"				\
-        "  <ETableColumn model_col= \"1\" _title=\"Classification\" "		\
-	"   expansion=\"1.0\" minimum_width=\"10\" resizable=\"true\" "		\
-	"   cell=\"string\"   compare=\"string\"/>"				\
-        "  <ETableColumn model_col= \"2\" _title=\"Completion Date\" "		\
-	"   expansion=\"1.0\" minimum_width=\"10\" resizable=\"true\" "		\
-	"   cell=\"string\"   compare=\"string\"/>"				\
-        "  <ETableColumn model_col= \"3\" _title=\"End Date\" "			\
-	"   expansion=\"1.0\" minimum_width=\"10\" resizable=\"true\" "		\
-	"   cell=\"string\"   compare=\"string\"/>"				\
-        "  <ETableColumn model_col= \"4\" _title=\"Start Date\" "		\
-	"   expansion=\"1.0\" minimum_width=\"10\" resizable=\"true\" "		\
-	"   cell=\"string\"   compare=\"string\"/>"				\
-        "  <ETableColumn model_col= \"5\" _title=\"Due Date\" "			\
-	"   expansion=\"1.0\" minimum_width=\"10\" resizable=\"true\" "		\
-	"   cell=\"string\"   compare=\"string\"/>"				\
-        "  <ETableColumn model_col= \"6\" _title=\"Geographical Position\" "	\
-	"   expansion=\"1.0\" minimum_width=\"10\" resizable=\"true\" "		\
-	"   cell=\"string\"   compare=\"string\"/>"				\
-        "  <ETableColumn model_col= \"7\" _title=\"Percent complete\" "		\
-	"   expansion=\"1.0\" minimum_width=\"10\" resizable=\"true\" "		\
-	"   cell=\"string\"   compare=\"string\"/>"				\
-        "  <ETableColumn model_col= \"8\" _title=\"Priority\" "			\
-	"   expansion=\"1.0\" minimum_width=\"10\" resizable=\"true\" "		\
-	"   cell=\"string\"   compare=\"string\"/>"				\
-        "  <ETableColumn model_col= \"9\" _title=\"Summary\" "			\
-	"   expansion=\"1.0\" minimum_width=\"10\" resizable=\"true\" "		\
-	"   cell=\"summary\"  compare=\"string\"/>"				\
-        "  <ETableColumn model_col=\"10\" _title=\"Transparency\" "		\
-	"   expansion=\"1.0\" minimum_width=\"10\" resizable=\"true\" "		\
-	"   cell=\"string\"   compare=\"string\"/>"				\
-        "  <ETableColumn model_col=\"11\" _title=\"URL\" "			\
-	"   expansion=\"1.0\" minimum_width=\"10\" resizable=\"true\" "		\
-	"   cell=\"string\"   compare=\"string\"/>"				\
-        "  <ETableColumn model_col=\"12\" _title=\"Alarms\" "			\
-	"   expansion=\"1.0\" minimum_width=\"10\" resizable=\"true\" "		\
-	"   cell=\"string\"   compare=\"string\"/>"				\
-        "  <ETableColumn model_col=\"13\" pixbuf=\"icon\" "			\
-	"   expansion=\"1.0\" minimum_width=\"16\" resizable=\"false\" "	\
-	"   cell=\"icon\"     compare=\"integer\"/>"				\
-        "  <ETableColumn model_col=\"14\" pixbuf=\"complete\" "			\
-	"   expansion=\"1.0\" minimum_width=\"16\" resizable=\"false\" "	\
-	"   cell=\"checkbox\" compare=\"integer\"/>"				\
-	"  <ETableState>"							\
-	"    <column source=\"13\"/>"						\
-	"    <column source=\"14\"/>"						\
-	"    <column source= \"9\"/>"						\
-	"    <grouping></grouping>"						\
-	"  </ETableState>"							\
+#define E_CALENDAR_TABLE_SPEC						\
+	"<ETableSpecification click-to-add=\"true\" "			\
+	" _click-to-add-message=\"Click here to add a task\" "		\
+	" draw-grid=\"true\">"						\
+        "  <ETableColumn model_col= \"0\" _title=\"Categories\" "	\
+	"   expansion=\"1.0\" minimum_width=\"10\" resizable=\"true\" "	\
+	"   cell=\"calstring\"   compare=\"string\"/>"			\
+        "  <ETableColumn model_col= \"1\" _title=\"Classification\" "	\
+	"   expansion=\"1.0\" minimum_width=\"10\" resizable=\"true\" "	\
+	"   cell=\"calstring\"   compare=\"string\"/>"			\
+        "  <ETableColumn model_col= \"2\" _title=\"Completion Date\" "	\
+	"   expansion=\"2.0\" minimum_width=\"10\" resizable=\"true\" "	\
+	"   cell=\"calstring\"   compare=\"string\"/>"			\
+        "  <ETableColumn model_col= \"3\" _title=\"End Date\" "		\
+	"   expansion=\"2.0\" minimum_width=\"10\" resizable=\"true\" "	\
+	"   cell=\"calstring\"   compare=\"string\"/>"			\
+        "  <ETableColumn model_col= \"4\" _title=\"Start Date\" "	\
+	"   expansion=\"2.0\" minimum_width=\"10\" resizable=\"true\" "	\
+	"   cell=\"calstring\"   compare=\"string\"/>"			\
+        "  <ETableColumn model_col= \"5\" _title=\"Due Date\" "		\
+	"   expansion=\"2.0\" minimum_width=\"10\" resizable=\"true\" "	\
+	"   cell=\"calstring\"   compare=\"string\"/>"			\
+        "  <ETableColumn model_col= \"6\" _title=\"Geographical Position\" " \
+	"   expansion=\"1.0\" minimum_width=\"10\" resizable=\"true\" "	\
+	"   cell=\"calstring\"   compare=\"string\"/>"			\
+        "  <ETableColumn model_col= \"7\" _title=\"% Complete\" "	\
+	"   expansion=\"1.0\" minimum_width=\"10\" resizable=\"true\" "	\
+	"   cell=\"calstring\"   compare=\"string\"/>"			\
+        "  <ETableColumn model_col= \"8\" _title=\"Priority\" "		\
+	"   expansion=\"1.0\" minimum_width=\"10\" resizable=\"true\" "	\
+	"   cell=\"calstring\"   compare=\"string\"/>"			\
+        "  <ETableColumn model_col= \"9\" _title=\"Summary\" "		\
+	"   expansion=\"3.0\" minimum_width=\"10\" resizable=\"true\" "	\
+	"   cell=\"calstring\"  compare=\"string\"/>"			\
+        "  <ETableColumn model_col=\"10\" _title=\"Transparency\" "	\
+	"   expansion=\"1.0\" minimum_width=\"10\" resizable=\"true\" "	\
+	"   cell=\"calstring\"   compare=\"string\"/>"			\
+        "  <ETableColumn model_col=\"11\" _title=\"URL\" "		\
+	"   expansion=\"2.0\" minimum_width=\"10\" resizable=\"true\" "	\
+	"   cell=\"calstring\"   compare=\"string\"/>"			\
+        "  <ETableColumn model_col=\"12\" _title=\"Alarms\" "		\
+	"   expansion=\"1.0\" minimum_width=\"10\" resizable=\"true\" "	\
+	"   cell=\"calstring\"   compare=\"string\"/>"			\
+        "  <ETableColumn model_col=\"13\" pixbuf=\"icon\" "		\
+	"   expansion=\"1.0\" minimum_width=\"16\" resizable=\"false\" "\
+	"   cell=\"icon\"     compare=\"integer\"/>"			\
+        "  <ETableColumn model_col=\"14\" pixbuf=\"complete\" "		\
+	"   expansion=\"1.0\" minimum_width=\"16\" resizable=\"false\" "\
+	"   cell=\"checkbox\" compare=\"integer\"/>"			\
+	"  <ETableState>"						\
+	"    <column source=\"13\"/>"					\
+	"    <column source=\"14\"/>"					\
+	"    <column source= \"9\"/>"					\
+	"    <grouping></grouping>"					\
+	"  </ETableState>"						\
 	"</ETableSpecification>"
 
 static void
@@ -200,7 +230,6 @@ e_calendar_table_init (ECalendarTable *cal_table)
 {
 	GtkWidget *table;
 	ETable *e_table;
-	ETableModel *model;
 	ECell *cell;
 	ETableExtras *extras;
 	gint i;
@@ -226,7 +255,17 @@ e_calendar_table_init (ECalendarTable *cal_table)
 	/* Create the model */
 
 	cal_table->model = calendar_model_new ();
-	model = E_TABLE_MODEL (cal_table->model);
+	cal_table->subset_model = e_table_subset_variable_new (E_TABLE_MODEL (cal_table->model));
+
+	gtk_signal_connect (GTK_OBJECT (cal_table->model), "model_changed",
+			    GTK_SIGNAL_FUNC (e_calendar_table_on_model_changed),
+			    cal_table);
+	gtk_signal_connect (GTK_OBJECT (cal_table->model), "model_row_inserted",
+			    GTK_SIGNAL_FUNC (e_calendar_table_on_row_inserted),
+			    cal_table);
+	gtk_signal_connect (GTK_OBJECT (cal_table->model), "model_row_deleted",
+			    GTK_SIGNAL_FUNC (e_calendar_table_on_row_deleted),
+			    cal_table);
 
 	/* Create the header columns */
 
@@ -238,7 +277,7 @@ e_calendar_table_init (ECalendarTable *cal_table)
 			"bold_column", CAL_COMPONENT_FIELD_OVERDUE,
 			"color_column", CAL_COMPONENT_FIELD_COLOR,
 			NULL);
-	e_table_extras_add_cell(extras, "summary", cell);
+	e_table_extras_add_cell (extras, "calstring", cell);
 
 	/* Create pixmaps */
 
@@ -258,8 +297,8 @@ e_calendar_table_init (ECalendarTable *cal_table)
 
 	/* Create the table */
 
-	table = e_table_scrolled_new (model, extras, E_CALENDAR_TABLE_SPEC,
-				      NULL);
+	table = e_table_scrolled_new (cal_table->subset_model, extras,
+				      E_CALENDAR_TABLE_SPEC, NULL);
 	gtk_object_unref (GTK_OBJECT (extras));
 
 	cal_table->etable = table;
@@ -298,6 +337,24 @@ e_calendar_table_new (void)
 }
 
 
+/**
+ * e_calendar_table_get_model:
+ * @cal_table: A calendar table.
+ * 
+ * Queries the calendar data model that a calendar table is using.
+ * 
+ * Return value: A calendar model.
+ **/
+CalendarModel *
+e_calendar_table_get_model (ECalendarTable *cal_table)
+{
+	g_return_val_if_fail (cal_table != NULL, NULL);
+	g_return_val_if_fail (E_IS_CALENDAR_TABLE (cal_table), NULL);
+
+	return cal_table->model;
+}
+
+
 static void
 e_calendar_table_destroy (GtkObject *object)
 {
@@ -308,6 +365,9 @@ e_calendar_table_destroy (GtkObject *object)
 	gtk_object_unref (GTK_OBJECT (cal_table->model));
 	cal_table->model = NULL;
 
+	gtk_object_unref (GTK_OBJECT (cal_table->subset_model));
+	cal_table->subset_model = NULL;
+
 	GTK_OBJECT_CLASS (parent_class)->destroy (object);
 }
 
@@ -328,18 +388,23 @@ e_calendar_table_on_double_click (ETable *table,
 				  GdkEvent *event,
 				  ECalendarTable *cal_table)
 {
+	g_print ("In e_calendar_table_on_double_click\n");
+
 	e_calendar_table_open_task (cal_table, row);
 }
 
 
 static GnomeUIInfo e_calendar_table_popup_uiinfo[] = {
-	{ GNOME_APP_UI_ITEM, N_("Open..."),
-	  N_("Open the task"), e_calendar_table_on_open_task,
-	  NULL, NULL, 0, 0, 0, 0 },
 	{ GNOME_APP_UI_ITEM, N_("Mark Complete"),
 	  N_("Mark the task complete"), e_calendar_table_on_mark_task_complete,
 	  NULL, NULL, 0, 0, 0, 0 },
-	{ GNOME_APP_UI_ITEM, N_("Delete"),
+
+	GNOMEUIINFO_SEPARATOR,
+
+	{ GNOME_APP_UI_ITEM, N_("Edit this task..."),
+	  N_("Edit the task"), e_calendar_table_on_open_task,
+	  NULL, NULL, 0, 0, 0, 0 },
+	{ GNOME_APP_UI_ITEM, N_("Delete this task"),
 	  N_("Delete the task"), e_calendar_table_on_delete_task,
 	  NULL, NULL, 0, 0, 0, 0 },
 
@@ -400,13 +465,13 @@ e_calendar_table_on_mark_task_complete (GtkWidget *menuitem,
 
 /* Deletes a component from the table */
 static void
-delete_component (CalendarModel *model, int row)
+delete_component (CalendarModel *model, int row, GtkWidget *widget)
 {
 	CalComponent *comp;
 
 	comp = calendar_model_get_component (model, row);
 
-	if (delete_component_dialog (comp)) {
+	if (delete_component_dialog (comp, widget)) {
 		CalClient *client;
 		const char *uid;
 
@@ -426,7 +491,7 @@ e_calendar_table_on_delete_task (GtkWidget *menuitem,
 {
 	ECalendarMenuData *menu_data = (ECalendarMenuData*) data;
 
-	delete_component (menu_data->cal_table->model, menu_data->row);
+	delete_component (menu_data->cal_table->model, menu_data->row, menuitem);
 }
 
 
@@ -439,7 +504,7 @@ e_calendar_table_on_key_press (ETable *table,
 			       ECalendarTable *cal_table)
 {
 	if (event->keyval == GDK_Delete) {
-		delete_component (cal_table->model, row);
+		delete_component (cal_table->model, row, GTK_WIDGET (table));
 		return TRUE;
 	}
 
@@ -459,6 +524,8 @@ e_calendar_table_open_task (ECalendarTable *cal_table,
 
 	comp = calendar_model_get_component (cal_table->model, row);
 	task_editor_set_todo_object (tedit, comp);
+
+	task_editor_focus (tedit);
 }
 
 
@@ -483,9 +550,135 @@ void
 e_calendar_table_save_state (ECalendarTable	*cal_table,
 			     gchar		*filename)
 {
-
 	g_return_if_fail (E_IS_CALENDAR_TABLE (cal_table));
 
 	e_table_save_state (e_table_scrolled_get_table(E_TABLE_SCROLLED (cal_table->etable)),
 			    filename);
 }
+
+
+void
+e_calendar_table_set_filter_func	(ECalendarTable *cal_table,
+					 ECalendarTableFilterFunc filter_func,
+					 gpointer	 filter_data,
+					 GDestroyNotify  filter_data_destroy)
+{
+	g_return_if_fail (E_IS_CALENDAR_TABLE (cal_table));
+
+	if (cal_table->filter_func == filter_func
+	    && cal_table->filter_data == filter_data
+	    && cal_table->filter_data_destroy == filter_data_destroy)
+		return;
+
+	if (cal_table->filter_data_destroy)
+		(*cal_table->filter_data_destroy) (cal_table->filter_data);
+
+	cal_table->filter_func = filter_func;
+	cal_table->filter_data = filter_data;
+	cal_table->filter_data_destroy = filter_data_destroy;
+
+	e_calendar_table_apply_filter (cal_table);
+}
+
+
+static void
+e_calendar_table_apply_filter		(ECalendarTable	*cal_table)
+{
+	ETableSubsetVariable *etssv;
+	CalComponent *comp;
+	gint rows, row;
+
+	etssv = E_TABLE_SUBSET_VARIABLE (cal_table->subset_model);
+
+	/* Make sure that any edits get saved first. */
+	e_table_model_pre_change (cal_table->subset_model);
+
+	/* FIXME: A hack to remove all the existing rows quickly. */
+	E_TABLE_SUBSET (cal_table->subset_model)->n_map = 0;
+
+	if (cal_table->filter_func == NULL) {
+		e_table_subset_variable_add_all (etssv);
+	} else {
+		rows = e_table_model_row_count (E_TABLE_MODEL (cal_table->model));
+		for (row = 0; row < rows; row++) {
+			comp = calendar_model_get_component (cal_table->model,
+							     row);
+
+			if ((*cal_table->filter_func) (cal_table, comp,
+						       cal_table->filter_data))
+				e_table_subset_variable_add (etssv, row);
+		}
+	}
+
+	e_table_model_changed (cal_table->subset_model);
+}
+
+
+gboolean
+e_calendar_table_filter_by_category  (ECalendarTable	*cal_table,
+				      CalComponent	*comp,
+				      gpointer		 filter_data)
+{
+	GSList *categories_list, *elem;
+	gboolean retval = FALSE;
+
+	cal_component_get_categories_list (comp, &categories_list);
+
+	for (elem = categories_list; elem; elem = elem->next) {
+		if (retval == FALSE
+		    && !strcmp ((char*) elem->data, (char*) filter_data))
+			retval = TRUE;
+		g_free (elem->data);
+	}
+
+	g_slist_free (categories_list);
+
+	return retval;
+}
+
+
+static void
+e_calendar_table_on_model_changed	(ETableModel	*model,
+					 ECalendarTable	*cal_table)
+{
+	e_calendar_table_apply_filter (cal_table);
+}
+
+
+static void
+e_calendar_table_on_row_inserted	(ETableModel	*model,
+					 gint		 row,
+					 ECalendarTable	*cal_table)
+{
+	ETableSubsetVariable *etssv;
+	CalComponent *comp;
+	gboolean add_row = FALSE;
+
+	etssv = E_TABLE_SUBSET_VARIABLE (cal_table->subset_model);
+
+	if (cal_table->filter_func == NULL) {
+		add_row = TRUE;
+	} else {
+		comp = calendar_model_get_component (cal_table->model, row);
+
+		if ((*cal_table->filter_func) (cal_table, comp,
+					       cal_table->filter_data))
+			add_row = TRUE;
+	}
+
+	if (add_row) {
+		e_table_subset_variable_increment (etssv, row, 1);
+		e_table_subset_variable_add (etssv, row);
+	}
+}
+
+
+static void
+e_calendar_table_on_row_deleted		(ETableModel	*model,
+					 gint		 row,
+					 ECalendarTable	*cal_table)
+{
+	/* We just reapply the filter since we aren't too bothered about
+	   being efficient. It doesn't happen often. */
+	e_calendar_table_apply_filter (cal_table);
+}
