@@ -2,9 +2,10 @@
 
 /* 
  * Author : 
- *  Damon Chaplin <damon@helixcode.com>
+ *  Damon Chaplin <damon@ximian.com>
  *
  * Copyright 2000, Helix Code, Inc.
+ * Copyright 2000, Ximian, Inc.
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -53,16 +54,30 @@ typedef enum
 typedef struct _ECalendarTable       ECalendarTable;
 typedef struct _ECalendarTableClass  ECalendarTableClass;
 
+
+typedef gboolean (*ECalendarTableFilterFunc)	(ECalendarTable *cal_table,
+						 CalComponent   *comp,
+						 gpointer	 data);
+
 struct _ECalendarTable
 {
 	GtkTable table;
 
+	/* This is the underlying model which contains all the tasks/events. */
 	CalendarModel *model;
+
+	/* This is the model that we use when filtering the tasks/events. */
+	ETableModel *subset_model;
 	
 	GtkWidget *etable;
 	
 	/* Colors for drawing. */
 	GdkColor colors[E_CALENDAR_TABLE_COLOR_LAST];
+
+	/* Data for filtering the Tasks. */
+	ECalendarTableFilterFunc filter_func;
+	gpointer filter_data;
+	GDestroyNotify filter_data_destroy;
 };
 
 struct _ECalendarTableClass
@@ -74,6 +89,7 @@ struct _ECalendarTableClass
 GtkType	   e_calendar_table_get_type		(void);
 GtkWidget* e_calendar_table_new			(void);
 
+CalendarModel *e_calendar_table_get_model	(ECalendarTable *cal_table);
 
 void	   e_calendar_table_set_cal_client	(ECalendarTable *cal_table,
 						 CalClient	*client);
@@ -85,6 +101,15 @@ void	   e_calendar_table_load_state		(ECalendarTable *cal_table,
 void	   e_calendar_table_save_state		(ECalendarTable *cal_table,
 						 gchar		*filename);
 
+void	   e_calendar_table_set_filter_func	(ECalendarTable *cal_table,
+						 ECalendarTableFilterFunc filter_func,
+						 gpointer	 filter_data,
+						 GDestroyNotify  filter_data_destroy);
+gboolean   e_calendar_table_filter_by_category  (ECalendarTable *cal_table,
+						 CalComponent	*comp,
+						 gpointer	 filter_data);
+
+
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
