@@ -2,9 +2,10 @@
 
 /* 
  * Author : 
- *  Damon Chaplin <damon@helixcode.com>
+ *  Damon Chaplin <damon@ximian.com>
  *
  * Copyright 1999, Helix Code, Inc.
+ * Copyright 1999, Ximian, Inc.
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -28,7 +29,6 @@
  */
 
 #include <config.h>
-#include <gnome.h>
 #include <gal/widgets/e-gui-utils.h>
 #include "e-day-view-time-item.h"
 #include "calendar-config.h"
