@@ -1,19 +1,32 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
- * e-itip-control.c
+/* Evolution calendar - Control for displaying iTIP mail messages
  *
- * Authors:
- *    Jesse Pavel <jpavel@helixcode.com>
+ * Copyright (C) 2000 Helix Code, Inc.
+ * Copyright (C) 2000 Ximian, Inc.
  *
- * Copyright 2000, Helix Code, Inc.
+ * Author: Jesse Pavel <jpavel@ximian.com>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
  */
 
 #include <config.h>
+#include <time.h>
 #include <gnome.h>
 #include <bonobo.h>
 #include <glade/glade.h>
 #include <ical.h>
-#include <time.h>
 #include <Evolution-Composer.h>
 
 #include "e-itip-control.h"
@@ -101,9 +114,10 @@ itip_control_destroy_cb (GtkObject *object,
 	gtk_object_unref (GTK_OBJECT (priv->xml));
 	gtk_object_unref (GTK_OBJECT (priv->xml2));
 
-	icalcomponent_remove_component (priv->main_comp, priv->comp);
-
 	if (priv->main_comp != NULL) {
+		if (priv->comp != NULL)
+			icalcomponent_remove_component (priv->main_comp, priv->comp);
+	
 		icalcomponent_free (priv->main_comp);
 	}
 
@@ -126,32 +140,29 @@ itip_control_destroy_cb (GtkObject *object,
 	
 
 static void
-cal_loaded_cb (GtkObject *object, CalClientGetStatus status, gpointer data)
+cal_opened_cb (CalClient *client, CalClientOpenStatus status, gpointer data)
 {
-	CalClient *client = CAL_CLIENT (object);
 	EItipControlPrivate *priv = data;
 
 	gtk_widget_hide (priv->loading_progress);
 
-	if (status == CAL_CLIENT_GET_SUCCESS) {
+	if (status == CAL_CLIENT_OPEN_SUCCESS) {
 		if (cal_client_update_object (client, priv->cal_comp) == FALSE) {
 			GtkWidget *dialog;
 	
-			dialog = gnome_warning_dialog("I couldn't update your calendar file!\n");
+			dialog = gnome_warning_dialog(_("I couldn't update your calendar file!\n"));
 			gnome_dialog_run (GNOME_DIALOG(dialog));
-		}
-		else {
+		} else {
 			/* We have success. */
 			GtkWidget *dialog;
 	
-			dialog = gnome_ok_dialog("Component successfully updated.");
+			dialog = gnome_ok_dialog(_("Component successfully updated."));
 			gnome_dialog_run (GNOME_DIALOG(dialog));
 		}
-	}
-	else {
+	} else {
 		GtkWidget *dialog;
 
-		dialog = gnome_ok_dialog("There was an error loading the calendar file.");
+		dialog = gnome_ok_dialog(_("There was an error loading the calendar file."));
 		gnome_dialog_run (GNOME_DIALOG(dialog));
 	}
 
@@ -168,19 +179,20 @@ update_calendar (EItipControlPrivate *priv)
 	snprintf (cal_uri, 250, "%s/local/Calendar/calendar.ics", evolution_dir);
 	
 	client = cal_client_new ();
-	if (cal_client_load_calendar (client, cal_uri) == FALSE) {
+
+	gtk_signal_connect (GTK_OBJECT (client), "cal_opened",
+	    	   	    GTK_SIGNAL_FUNC (cal_opened_cb), priv);
+	
+	if (cal_client_open_calendar (client, cal_uri, FALSE) == FALSE) {
 		GtkWidget *dialog;
 
-		dialog = gnome_warning_dialog("I couldn't open your calendar file!\n");
+		dialog = gnome_warning_dialog(_("I couldn't open your calendar file!\n"));
 		gnome_dialog_run (GNOME_DIALOG(dialog));
 		gtk_object_unref (GTK_OBJECT (client));
 	
 		return;
 	}
 
-	gtk_signal_connect (GTK_OBJECT (client), "cal_loaded",
-	    	   	    GTK_SIGNAL_FUNC (cal_loaded_cb), priv);
-	
 	gtk_progress_bar_update (GTK_PROGRESS_BAR (priv->loading_progress), 0.5);
 	gtk_widget_show (priv->loading_progress);
 
@@ -406,11 +418,13 @@ load_calendar_store (char *cal_uri, CalClient **cal_client)
 	}
 
 	*cal_client = cal_client_new ();
-	if (cal_client_load_calendar (*cal_client, uri) == FALSE) {
+	if (cal_client_open_calendar (*cal_client, uri, FALSE) == FALSE) {
 		return FALSE;
 	}
 
-	while (!cal_client_is_loaded (*cal_client)) {
+	/* FIXME!!!!!!  This is fucking ugly. */
+
+	while (!cal_client_get_load_state (*cal_client) != CAL_CLIENT_LOAD_LOADED) {
 		gtk_main_iteration_do (FALSE);  /* Do a non-blocking iteration. */
 		usleep (200000L);   /* Pause for 1/5th of a second before checking again.*/
 	}
@@ -433,7 +447,7 @@ update_reply_cb (GtkWidget *widget, gpointer data)
 	if (load_calendar_store (NULL, &cal_client) == FALSE) {
 		GtkWidget *dialog;
 
-		dialog = gnome_warning_dialog("I couldn't load your calendar file!\n");
+		dialog = gnome_warning_dialog(_("I couldn't load your calendar file!\n"));
 		gnome_dialog_run (GNOME_DIALOG(dialog));
 		gtk_object_unref (GTK_OBJECT (cal_client));
 	
@@ -445,7 +459,7 @@ update_reply_cb (GtkWidget *widget, gpointer data)
 	if (cal_client_get_object (cal_client, uid, &cal_comp) != CAL_CLIENT_GET_SUCCESS) {
 		GtkWidget *dialog;
 
-		dialog = gnome_warning_dialog("I couldn't read your calendar file!\n");
+		dialog = gnome_warning_dialog(_("I couldn't read your calendar file!\n"));
 		gnome_dialog_run (GNOME_DIALOG(dialog));
 		gtk_object_unref (GTK_OBJECT (cal_client));
 	
@@ -458,7 +472,7 @@ update_reply_cb (GtkWidget *widget, gpointer data)
 	if (!prop) {
 		GtkWidget *dialog;
 
-		dialog = gnome_warning_dialog("This is a reply from someone who was uninvited!");
+		dialog = gnome_warning_dialog(_("This is a reply from someone who was uninvited!"));
 		gnome_dialog_run (GNOME_DIALOG(dialog));
 		gtk_object_unref (GTK_OBJECT (cal_client));
 		gtk_object_unref (GTK_OBJECT (cal_comp));
@@ -474,7 +488,7 @@ update_reply_cb (GtkWidget *widget, gpointer data)
 	if (!cal_client_update_object (cal_client, cal_comp)) {
 		GtkWidget *dialog;
 
-		dialog = gnome_warning_dialog("I couldn't update your calendar store.");
+		dialog = gnome_warning_dialog(_("I couldn't update your calendar store."));
 		gnome_dialog_run (GNOME_DIALOG(dialog));
 		gtk_object_unref (GTK_OBJECT (cal_client));
 		gtk_object_unref (GTK_OBJECT (cal_comp));
@@ -485,7 +499,7 @@ update_reply_cb (GtkWidget *widget, gpointer data)
 		/* We have success. */
 		GtkWidget *dialog;
 
-		dialog = gnome_ok_dialog("Component successfully updated.");
+		dialog = gnome_ok_dialog(_("Component successfully updated."));
 		gnome_dialog_run (GNOME_DIALOG(dialog));
 	}
 
@@ -504,7 +518,7 @@ cancel_meeting_cb (GtkWidget *widget, gpointer data)
 	if (load_calendar_store (NULL, &cal_client) == FALSE) {
 		GtkWidget *dialog;
 
-		dialog = gnome_warning_dialog("I couldn't load your calendar file!\n");
+		dialog = gnome_warning_dialog(_("I couldn't load your calendar file!\n"));
 		gnome_dialog_run (GNOME_DIALOG(dialog));
 		gtk_object_unref (GTK_OBJECT (cal_client));
 	
@@ -515,7 +529,7 @@ cancel_meeting_cb (GtkWidget *widget, gpointer data)
 	if (cal_client_remove_object (cal_client, uid) == FALSE) {
 		GtkWidget *dialog;
 
-		dialog = gnome_warning_dialog("I couldn't delete the calendar component!\n");
+		dialog = gnome_warning_dialog(_("I couldn't delete the calendar component!\n"));
 		gnome_dialog_run (GNOME_DIALOG(dialog));
 		gtk_object_unref (GTK_OBJECT (cal_client));
 	
@@ -525,7 +539,7 @@ cancel_meeting_cb (GtkWidget *widget, gpointer data)
 		/* We have success! */
 		GtkWidget *dialog;
 
-		dialog = gnome_ok_dialog("Component successfully deleted.");
+		dialog = gnome_ok_dialog(_("Component successfully deleted."));
 		gnome_dialog_run (GNOME_DIALOG(dialog));
 	}
 		
@@ -669,7 +683,7 @@ pstream_load (BonoboPersistStream *ps, const Bonobo_Stream stream,
 		{
 		GtkWidget *dialog;
 
-		dialog = gnome_warning_dialog("I don't recognize this type of calendar component.");
+		dialog = gnome_warning_dialog(_("I don't recognize this type of calendar component."));
 		gnome_dialog_run (GNOME_DIALOG(dialog));
 
 		g_free (priv->vcalendar);
@@ -755,7 +769,7 @@ pstream_load (BonoboPersistStream *ps, const Bonobo_Stream stream,
 				 	  "No reply is necessary.", 
 				 priv->from_address);
 			
-			button =  gtk_button_new_with_label ("Add to Calendar");
+			button =  gtk_button_new_with_label (_("Add to Calendar"));
 			gtk_box_pack_start (GTK_BOX (priv->button_box), button, FALSE, FALSE, 3);
 			gtk_widget_show (button);
 
@@ -789,9 +803,9 @@ pstream_load (BonoboPersistStream *ps, const Bonobo_Stream stream,
 			if (rsvp) {
 				GtkWidget *accept_button, *decline_button, *tentative_button;
 
-				accept_button = gtk_button_new_with_label ("Accept");
-				decline_button = gtk_button_new_with_label ("Decline");
-				tentative_button = gtk_button_new_with_label ("Tentative");
+				accept_button = gtk_button_new_with_label (_("Accept"));
+				decline_button = gtk_button_new_with_label (_("Decline"));
+				tentative_button = gtk_button_new_with_label (_("Tentative"));
 
 				gtk_box_pack_start (GTK_BOX (priv->button_box), accept_button, FALSE, FALSE, 3);
 				gtk_box_pack_start (GTK_BOX (priv->button_box), decline_button, FALSE, FALSE, 3);
@@ -835,7 +849,7 @@ pstream_load (BonoboPersistStream *ps, const Bonobo_Stream stream,
 			else {
 				GtkWidget *button;
 		
-				button =  gtk_button_new_with_label ("Update Calendar");
+				button =  gtk_button_new_with_label (_("Update Calendar"));
 				gtk_box_pack_start (GTK_BOX (priv->button_box), button, FALSE, FALSE, 3);
 				gtk_widget_show (button);
 	
@@ -858,7 +872,7 @@ pstream_load (BonoboPersistStream *ps, const Bonobo_Stream stream,
 			else {
 				GtkWidget *button;
 
-				button =  gtk_button_new_with_label ("Cancel Meeting");
+				button =  gtk_button_new_with_label (_("Cancel Meeting"));
 				gtk_box_pack_start (GTK_BOX (priv->button_box), button, FALSE, FALSE, 3);
 				gtk_widget_show (button);
 
@@ -931,8 +945,11 @@ pstream_get_content_types (BonoboPersistStream *ps, void *closure,
 }
 
 static void
-get_prop ( BonoboPropertyBag *bag, BonoboArg *arg,
-	   guint arg_id, gpointer user_data )
+get_prop ( BonoboPropertyBag *bag, 
+	   BonoboArg *arg,
+	   guint arg_id, 	      
+	   CORBA_Environment *ev,
+	   gpointer user_data)
 {
 	EItipControlPrivate *priv = user_data;
 
@@ -945,8 +962,11 @@ get_prop ( BonoboPropertyBag *bag, BonoboArg *arg,
 }
 
 static void
-set_prop ( BonoboPropertyBag *bag, const BonoboArg *arg,
-	   guint arg_id, gpointer user_data )
+set_prop ( BonoboPropertyBag *bag, 
+	   const BonoboArg *arg,
+	   guint arg_id, 
+	   CORBA_Environment *ev,
+	   gpointer user_data)
 {
 	EItipControlPrivate *priv = user_data;
 
