@@ -1,8 +1,9 @@
 /* Evolution calendar - Meeting editor dialog
  *
  * Copyright (C) 2000 Helix Code, Inc.
+ * Copyright (C) 2000 Ximian, Inc.
  *
- * Authors: Jesse Pavel <jpavel@helixcode.com>
+ * Authors: Jesse Pavel <jpavel@ximian.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,11 +24,9 @@
 #define __E_MEETING_EDIT_H__
 
 #include <config.h>
-#include <gnome.h>
 #include <glade/glade.h>
 #include <cal-util/cal-component.h>
 #include <cal-client/cal-client.h>
-#include "event-editor.h"
 
 typedef struct _EMeetingEditor EMeetingEditor;
 
@@ -36,8 +35,8 @@ struct _EMeetingEditor {
 };
 
 
-EMeetingEditor * e_meeting_editor_new (CalComponent *comp, CalClient *client, 
-				       EventEditor *ee);
+EMeetingEditor * e_meeting_editor_new (CalComponent *comp, CalClient *client);
+
 void e_meeting_edit (EMeetingEditor *editor);
 void e_meeting_editor_free (EMeetingEditor *editor);
 
