@@ -1,24 +1,38 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
- * GnomeCalendar widget
- * Copyright (C) 1998 the Free Software Foundation
+/* Evolution calendar - Main calendar view widget
  *
- * Authors: Miguel de Icaza (miguel@kernel.org)
- *          Federico Mena-Quintero <federico@helixcode.com>
+ * Copyright (C) 1998 The Free Software Foundation
+ * Copyright (C) 2000 Helix Code, Inc.
+ * Copyright (C) 2001 Ximian, Inc.
+ *
+ * Authors: Miguel de Icaza <miguel@ximian.com>
+ *          Federico Mena-Quintero <federico@ximian.com>
+ *          Seth Alves <alves@hungry.com>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
  */
 
 #include <config.h>
-#include <gnome.h>
 #include <unistd.h>
 #include <signal.h>
 #include <sys/wait.h>
 #include <fcntl.h>
-#include <gnome.h>
 #include <gal/e-paned/e-hpaned.h>
 #include <gal/e-paned/e-vpaned.h>
 #include <cal-util/timeutil.h>
 #include "dialogs/alarm-notify-dialog.h"
-#include "alarm.h"
 #include "e-calendar-table.h"
 #include "e-day-view.h"
 #include "e-week-view.h"
@@ -40,25 +54,25 @@ typedef enum {
 	VIEW_MONTH
 } ViewType;
 
-/* States for the calendar loading and creation state machine */
-typedef enum {
-	LOAD_STATE_NOT_LOADED,
-	LOAD_STATE_WAIT_LOAD,
-	LOAD_STATE_WAIT_LOAD_BEFORE_CREATE,
-	LOAD_STATE_WAIT_CREATE,
-	LOAD_STATE_LOADED
-} LoadState;
-
 /* Private part of the GnomeCalendar structure */
 struct _GnomeCalendarPrivate {
+	/*
+	 * The Calendar Folder.
+	 */
+
 	/* The calendar client object we monitor */
-	CalClient   *client;
+	CalClient *client;
 
-	/* Loading state; we can be loading or creating a calendar */
-	LoadState load_state;
+	/*
+	 * The TaskPad Folder.
+	 */
 
-	/* URI being loaded, NULL if we are not being loaded */
-	char *loading_uri;
+	/* The calendar client object we monitor */
+	CalClient   *task_pad_client;
+
+	/*
+	 * Fields for the calendar view
+	 */
 
 	/* Mapping of component UIDs to event editors */
 	GHashTable  *object_editor_hash;
@@ -106,12 +120,6 @@ struct _GnomeCalendarPrivate {
 	/* The signal handler id for our GtkCalendar "day_selected" handler. */
 	guint	     day_selected_id;
 
-	/* Alarm ID for the midnight refresh function */
-	gpointer midnight_alarm_refresh_id;
-
-	/* UID->alarms hash */
-	GHashTable *alarms;
-
 	/* Whether we are being destroyed and should not mess with the object
 	 * editor hash table.
 	 */
@@ -120,16 +128,6 @@ struct _GnomeCalendarPrivate {
 
 
 
-/* An entry in the UID->alarms hash table.  The UID key *is* the uid field in
- * this structure, so don't free it separately.
- */
-typedef struct {
-	char *uid;
-	GList *alarm_ids;
-} ObjectAlarms;
-
-
-
 static void gnome_calendar_class_init (GnomeCalendarClass *class);
 static void gnome_calendar_init (GnomeCalendar *gcal);
 static void gnome_calendar_destroy (GtkObject *object);
@@ -160,7 +158,6 @@ static gboolean gnome_calendar_get_days_shown	(GnomeCalendar	*gcal,
 
 static GtkVBoxClass *parent_class;
 
-static void setup_alarm (GnomeCalendar *cal, CalAlarmInstance *ai);
 
 
 
@@ -206,6 +203,7 @@ setup_widgets (GnomeCalendar *gcal)
 	GnomeCalendarPrivate *priv;
 	GtkWidget *w;
 	gchar *filename;
+	CalendarModel *model;
 
 	priv = gcal->priv;
 
@@ -230,7 +228,10 @@ setup_widgets (GnomeCalendar *gcal)
 	/* The ECalendar. */
 	w = e_calendar_new ();
 	priv->date_navigator = E_CALENDAR (w);
+	e_calendar_item_set_days_start_week_sel (priv->date_navigator->calitem, 9);
+	e_calendar_item_set_max_days_sel (priv->date_navigator->calitem, 42);
 	gtk_widget_show (w);
+
 	e_paned_pack1 (E_PANED (priv->vpane), w, FALSE, TRUE);
 	gtk_signal_connect (GTK_OBJECT (priv->date_navigator),
 			    "style_set",
@@ -251,6 +252,8 @@ setup_widgets (GnomeCalendar *gcal)
 
 	/* The ToDo list. */
 	priv->todo = e_calendar_table_new ();
+	model = e_calendar_table_get_model (E_CALENDAR_TABLE (priv->todo));
+	calendar_model_set_new_comp_vtype (model, CAL_COMPONENT_TODO);
 	e_paned_pack2 (E_PANED (priv->vpane), priv->todo, TRUE, TRUE);
 	gtk_widget_show (priv->todo);
 
@@ -302,10 +305,7 @@ gnome_calendar_init (GnomeCalendar *gcal)
 	priv = g_new0 (GnomeCalendarPrivate, 1);
 	gcal->priv = priv;
 
-	priv->load_state = LOAD_STATE_NOT_LOADED;
-
 	priv->object_editor_hash = g_hash_table_new (g_str_hash, g_str_equal);
-	priv->alarms = g_hash_table_new (g_str_hash, g_str_equal);
 
 	priv->current_view_type = VIEW_NOT_SET;
 	priv->range_selected = FALSE;
@@ -316,25 +316,6 @@ gnome_calendar_init (GnomeCalendar *gcal)
 	priv->selection_end_time = time_add_day (priv->selection_start_time, 1);
 }
 
-/* Used from g_hash_table_foreach(); frees an object alarms entry */
-static void
-free_object_alarms (gpointer key, gpointer value, gpointer data)
-{
-	ObjectAlarms *oa;
-
-	oa = value;
-
-	g_assert (oa->uid != NULL);
-	g_free (oa->uid);
-	oa->uid = NULL;
-
-	g_assert (oa->alarm_ids != NULL);
-	g_list_free (oa->alarm_ids);
-	oa->alarm_ids = NULL;
-
-	g_free (oa);
-}
-
 /* Used from g_hash_table_foreach(); frees an UID string */
 static void
 destroy_editor_cb (gpointer key, gpointer value, gpointer data)
@@ -363,21 +344,21 @@ gnome_calendar_destroy (GtkObject *object)
 	e_calendar_table_save_state (E_CALENDAR_TABLE (priv->todo), filename);
 	g_free (filename);
 
-	priv->load_state = LOAD_STATE_NOT_LOADED;
-
-	if (priv->loading_uri) {
-		g_free (priv->loading_uri);
-		priv->loading_uri = NULL;
-	}
-
 	if (priv->client) {
+#if 0
+		alarm_notify_remove_client (priv->client);
+#endif
 		gtk_object_unref (GTK_OBJECT (priv->client));
 		priv->client = NULL;
 	}
 
-	g_hash_table_foreach (priv->alarms, free_object_alarms, NULL);
-	g_hash_table_destroy (priv->alarms);
-	priv->alarms = NULL;
+	if (priv->task_pad_client) {
+#if 0
+		alarm_notify_remove_client (priv->task_pad_client);
+#endif
+		gtk_object_unref (GTK_OBJECT (priv->task_pad_client));
+		priv->task_pad_client = NULL;
+	}
 
 	priv->in_destroy = TRUE;
 	g_hash_table_foreach (priv->object_editor_hash, destroy_editor_cb, NULL);
@@ -691,389 +672,8 @@ gnome_calendar_set_pane_positions	(GnomeCalendar	*gcal)
 	gtk_widget_set_usize (GTK_WIDGET (priv->date_navigator), -2, top_pane_height + 1);
 }
 
-#if 0
-
-/* Sends a mail notification of an alarm trigger */
-static void
-mail_notification (char *mail_address, char *text, time_t app_time)
-{
-	pid_t pid;
-	int   p [2];
-	char *command;
-
-	pipe (p);
-	pid = fork ();
-	if (pid == 0){
-		int dev_null;
-
-		dev_null = open ("/dev/null", O_RDWR);
-		dup2 (p [0], 0);
-		dup2 (dev_null, 1);
-		dup2 (dev_null, 2);
-		execl ("/usr/lib/sendmail", "/usr/lib/sendmail",
-		       mail_address, NULL);
-		_exit (127);
-	}
-	command = g_strconcat ("To: ", mail_address, "\n",
-			       "Subject: ", _("Reminder of your appointment at "),
-			       ctime (&app_time), "\n\n", text, "\n", NULL);
-	write (p [1], command, strlen (command));
- 	close (p [1]);
-	close (p [0]);
-	g_free (command);
-}
-
-static int
-max_open_files (void)
-{
-        static int files;
-
-        if (files)
-                return files;
-
-        files = sysconf (_SC_OPEN_MAX);
-        if (files != -1)
-                return files;
-#ifdef OPEN_MAX
-        return files = OPEN_MAX;
-#else
-        return files = 256;
-#endif
-}
-
-/* Executes a program as a notification of an alarm trigger */
-static void
-program_notification (char *command, int close_standard)
-{
-	struct sigaction ignore, save_intr, save_quit;
-	int status = 0, i;
-	pid_t pid;
-
-	ignore.sa_handler = SIG_IGN;
-	sigemptyset (&ignore.sa_mask);
-	ignore.sa_flags = 0;
-
-	sigaction (SIGINT, &ignore, &save_intr);
-	sigaction (SIGQUIT, &ignore, &save_quit);
-
-	if ((pid = fork ()) < 0){
-		fprintf (stderr, "\n\nfork () = -1\n");
-		return;
-	}
-	if (pid == 0){
-		pid = fork ();
-		if (pid == 0){
-			const int top = max_open_files ();
-			sigaction (SIGINT,  &save_intr, NULL);
-			sigaction (SIGQUIT, &save_quit, NULL);
-
-			for (i = (close_standard ? 0 : 3); i < top; i++)
-				close (i);
-
-			/* FIXME: As an excercise to the reader, copy the
-			 * code from mc to setup shell properly instead of
-			 * /bin/sh.  Yes, this comment is larger than a cut and paste.
-			 */
-			execl ("/bin/sh", "/bin/sh", "-c", command, (char *) 0);
-
-			_exit (127);
-		} else {
-			_exit (127);
-		}
-	}
-	wait (&status);
-	sigaction (SIGINT,  &save_intr, NULL);
-	sigaction (SIGQUIT, &save_quit, NULL);
-}
-
-#endif
-
-/* Queues a snooze alarm */
-static void
-snooze (GnomeCalendar *gcal, CalComponent *comp, time_t occur, int snooze_mins, gboolean audio)
-{
-	time_t now, trigger;
-	struct tm tm;
-	CalAlarmInstance ai;
-
-	now = time (NULL);
-	tm = *localtime (&now);
-	tm.tm_min += snooze_mins;
-
-	trigger = mktime (&tm);
-	if (trigger == -1) {
-		g_message ("snooze(): produced invalid time_t; not queueing alarm!");
-		return;
-	}
-
-#if 0
-	cal_component_get_uid (comp, &ai.uid);
-	ai.type = audio ? ALARM_AUDIO : ALARM_DISPLAY;
-#endif
-	ai.trigger = trigger;
-	ai.occur = occur;
-
-	setup_alarm (gcal, &ai);
-}
-
-struct alarm_notify_closure {
-	GnomeCalendar *gcal;
-	CalComponent *comp;
-	time_t occur;
-};
-
-/* Callback used for the result of the alarm notification dialog */
-static void
-display_notification_cb (AlarmNotifyResult result, int snooze_mins, gpointer data)
-{
-	struct alarm_notify_closure *c;
-
-	c = data;
-
-	switch (result) {
-	case ALARM_NOTIFY_CLOSE:
-		break;
-
-	case ALARM_NOTIFY_SNOOZE:
-		snooze (c->gcal, c->comp, c->occur, snooze_mins, FALSE);
-		break;
-
-	case ALARM_NOTIFY_EDIT:
-		gnome_calendar_edit_object (c->gcal, c->comp);
-		break;
-
-	default:
-		g_assert_not_reached ();
-	}
-
-	gtk_object_unref (GTK_OBJECT (c->comp));
-	g_free (c);
-}
-
-/* Present a display notification of an alarm trigger */
-static void
-display_notification (time_t trigger, time_t occur, CalComponent *comp, GnomeCalendar *gcal)
-{
-	gboolean result;
-	struct alarm_notify_closure *c;
-
-	gtk_object_ref (GTK_OBJECT (comp));
-
-	c = g_new (struct alarm_notify_closure, 1);
-	c->gcal = gcal;
-	c->comp = comp;
-	c->occur = occur;
-
-	result = alarm_notify_dialog (trigger, occur, comp, display_notification_cb, c);
-	if (!result) {
-		g_message ("display_notification(): could not display the alarm notification dialog");
-		g_free (c);
-		gtk_object_unref (GTK_OBJECT (comp));
-	}
-}
-
-/* Present an audible notification of an alarm trigger */
-static void
-audio_notification (time_t trigger, time_t occur, CalComponent *comp, GnomeCalendar *gcal)
-{
-	g_message ("AUDIO NOTIFICATION!");
-	/* FIXME */
-}
-
-struct trigger_alarm_closure {
-	GnomeCalendar *gcal;
-	char *uid;
-	CalComponentAlarmAction type;
-	time_t occur;
-};
-
-/* Callback function used when an alarm is triggered */
-static void
-trigger_alarm_cb (gpointer alarm_id, time_t trigger, gpointer data)
-{
-	struct trigger_alarm_closure *c;
-	GnomeCalendarPrivate *priv;
-	CalComponent *comp;
-	CalClientGetStatus status;
-	const char *uid;
-	ObjectAlarms *oa;
-   	GList *l;
-
-	c = data;
-	priv = c->gcal->priv;
-
-	/* Fetch the object */
-
-	status = cal_client_get_object (priv->client, c->uid, &comp);
-
-	switch (status) {
-	case CAL_CLIENT_GET_SUCCESS:
-		/* Go on */
-		break;
-	case CAL_CLIENT_GET_SYNTAX_ERROR:
-	case CAL_CLIENT_GET_NOT_FOUND:
-		g_message ("trigger_alarm_cb(): syntax error in fetched object");
-		return;
-	}
-
-	g_assert (comp != NULL);
-
-	/* Present notification */
-
-	switch (c->type) {
-	case CAL_COMPONENT_ALARM_EMAIL:
-#if 0
-		g_assert (ico->malarm.enabled);
-		mail_notification (ico->malarm.data, ico->summary, c->occur);
-#endif
-		break;
-
-	case CAL_COMPONENT_ALARM_PROCEDURE:
-#if 0
-		g_assert (ico->palarm.enabled);
-		program_notification (ico->palarm.data, FALSE);
-#endif
-		break;
-
-	case CAL_COMPONENT_ALARM_DISPLAY:
-#if 0
-		g_assert (ico->dalarm.enabled);
-#endif
-		display_notification (trigger, c->occur, comp, c->gcal);
-		break;
-
-	case CAL_COMPONENT_ALARM_AUDIO:
-#if 0
-		g_assert (ico->aalarm.enabled);
-#endif
-		audio_notification (trigger, c->occur, comp, c->gcal);
-		break;
-
-	default:
-		break;
-	}
-
-	/* Remove the alarm from the hash table */
-	cal_component_get_uid (comp, &uid);
-	oa = g_hash_table_lookup (priv->alarms, uid);
-	g_assert (oa != NULL);
-
-	l = g_list_find (oa->alarm_ids, alarm_id);
-	g_assert (l != NULL);
-
-	oa->alarm_ids = g_list_remove_link (oa->alarm_ids, l);
-	g_list_free_1 (l);
-
-	if (!oa->alarm_ids) {
-		g_hash_table_remove (priv->alarms, uid);
-		g_free (oa->uid);
-		g_free (oa);
-	}
-
-	gtk_object_unref (GTK_OBJECT (comp));
-}
-
-/* Frees a struct trigger_alarm_closure */
-static void
-free_trigger_alarm_closure (gpointer data)
-{
-	struct trigger_alarm_closure *c;
-
-	c = data;
-	g_free (c->uid);
-	g_free (c);
-}
-
-/* Queues the specified alarm */
-static void
-setup_alarm (GnomeCalendar *gcal, CalAlarmInstance *ai)
-{
-	GnomeCalendarPrivate *priv;
-	struct trigger_alarm_closure *c;
-	gpointer alarm;
-	ObjectAlarms *oa;
-
-	priv = gcal->priv;
-
-	c = g_new (struct trigger_alarm_closure, 1);
-	c->gcal = gcal;
-	c->uid = g_strdup (ai->uid);
-#if 0
-	c->type = ai->type;
-#endif
-	c->occur = ai->occur;
-
-	alarm = alarm_add (ai->trigger, trigger_alarm_cb, c, free_trigger_alarm_closure);
-	if (!alarm) {
-		g_message ("setup_alarm(): Could not set up alarm");
-		g_free (c->uid);
-		g_free (c);
-		return;
-	}
-
-	oa = g_hash_table_lookup (priv->alarms, ai->uid);
-	if (oa)
-		oa->alarm_ids = g_list_prepend (oa->alarm_ids, alarm);
-	else {
-		oa = g_new (ObjectAlarms, 1);
-		oa->uid = g_strdup (ai->uid);
-		oa->alarm_ids = g_list_prepend (NULL, alarm);
-
-		g_hash_table_insert (priv->alarms, oa->uid, oa);
-	}
-}
-
-static void load_alarms (GnomeCalendar *cal);
-
-/* Called nightly to refresh the day's alarms */
-static void
-midnight_refresh_cb (gpointer alarm_id, time_t trigger, gpointer data)
-{
-	GnomeCalendar *cal;
-	GnomeCalendarPrivate *priv;
-
-	cal = GNOME_CALENDAR (data);
-	priv = cal->priv;
-
-	priv->midnight_alarm_refresh_id = NULL;
-
-	load_alarms (cal);
-}
-
-/* Loads and queues the alarms from the current time up to midnight. */
-static void
-load_alarms (GnomeCalendar *gcal)
-{
-	GnomeCalendarPrivate *priv;
-	time_t now;
-	time_t end_of_day;
-	GList *alarms, *l;
-
-	priv = gcal->priv;
-
-	now = time (NULL);
-	end_of_day = time_day_end (now);
-
-	/* Queue alarms */
-
-	alarms = cal_client_get_alarms_in_range (priv->client, now, end_of_day);
-
-	for (l = alarms; l; l = l->next)
-		setup_alarm (gcal, l->data);
-
-	cal_alarm_instance_list_free (alarms);
-
-	/* Queue the midnight alarm refresh */
-
-	priv->midnight_alarm_refresh_id = alarm_add (end_of_day, midnight_refresh_cb, gcal, NULL);
-	if (!priv->midnight_alarm_refresh_id) {
-		g_message ("load_alarms(): Could not set up the midnight refresh alarm!");
-		/* FIXME: what to do? */
-	}
-}
-
 /* Loads the initial data into the calendar; this should be called right after
- * the cal_loaded signal from the client is invoked.
+ * the cal_opened signal from the client is invoked.
  */
 static void
 initial_load (GnomeCalendar *gcal)
@@ -1082,85 +682,16 @@ initial_load (GnomeCalendar *gcal)
 
 	priv = gcal->priv;
 
-	load_alarms (gcal);
 	tag_calendar_by_client (priv->date_navigator, priv->client);
 }
 
-/* Removes any queued alarms for the specified UID */
+/* Displays an error to indicate that opening a calendar failed */
 static void
-remove_alarms_for_object (GnomeCalendar *gcal, const char *uid)
-{
-	GnomeCalendarPrivate *priv;
-	ObjectAlarms *oa;
-	GList *l;
-
-	priv = gcal->priv;
-
-	oa = g_hash_table_lookup (priv->alarms, uid);
-	if (!oa)
-		return;
-
-	for (l = oa->alarm_ids; l; l = l->next) {
-		gpointer alarm_id;
-
-		alarm_id = l->data;
-		alarm_remove (alarm_id);
-	}
-
-	g_hash_table_remove (priv->alarms, uid);
-
-	g_free (oa->uid);
-	g_list_free (oa->alarm_ids);
-	g_free (oa);
-}
-
-/* Adds today's alarms for the specified object */
-static void
-add_alarms_for_object (GnomeCalendar *gcal, const char *uid)
-{
-	GnomeCalendarPrivate *priv;
-	GList *alarms;
-	gboolean result;
-	time_t now, end_of_day;
-	GList *l;
-
-	priv = gcal->priv;
-
-	now = time (NULL);
-	end_of_day = time_day_end (now);
-
-	result = cal_client_get_alarms_for_object (priv->client, uid, now, end_of_day, &alarms);
-	if (!result) {
-		/* FIXME: should we warn here, or is it OK if the object
-		 * disappeared in the meantime?
-		 */
-		return;
-	}
-
-	for (l = alarms; l; l = l->next)
-		setup_alarm (gcal, l->data);
-
-	cal_alarm_instance_list_free (alarms);
-}
-
-/* Displays an error to indicate that loading a calendar failed */
-static void
-load_error (GnomeCalendar *gcal, const char *uri)
+open_error (GnomeCalendar *gcal, const char *uri)
 {
 	char *msg;
 
-	msg = g_strdup_printf (_("Could not load the calendar in `%s'"), uri);
-	gnome_error_dialog_parented (msg, GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (gcal))));
-	g_free (msg);
-}
-
-/* Displays an error to indicate that creating a calendar failed */
-static void
-create_error (GnomeCalendar *gcal, const char *uri)
-{
-	char *msg;
-
-	msg = g_strdup_printf (_("Could not create a calendar in `%s'"), uri);
+	msg = g_strdup_printf (_("Could not open the folder in `%s'"), uri);
 	gnome_error_dialog_parented (msg, GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (gcal))));
 	g_free (msg);
 }
@@ -1171,98 +702,44 @@ method_error (GnomeCalendar *gcal, const char *uri)
 {
 	char *msg;
 
-	msg = g_strdup_printf (_("The method required to load `%s' is not supported"), uri);
+	msg = g_strdup_printf (_("The method required to open `%s' is not supported"), uri);
 	gnome_error_dialog_parented (msg, GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (gcal))));
 	g_free (msg);
 }
 
 /* Callback from the calendar client when a calendar is loaded */
 static void
-cal_loaded_cb (CalClient *client, CalClientLoadStatus status, gpointer data)
+cal_opened_cb (CalClient *client, CalClientOpenStatus status, gpointer data)
 {
 	GnomeCalendar *gcal;
 	GnomeCalendarPrivate *priv;
-	gboolean free_uri;
 
 	gcal = GNOME_CALENDAR (data);
 	priv = gcal->priv;
 
-	g_assert (priv->load_state != LOAD_STATE_NOT_LOADED && priv->load_state != LOAD_STATE_LOADED);
-	g_assert (priv->loading_uri != NULL);
-
-	free_uri = TRUE;
-
-	switch (priv->load_state) {
-	case LOAD_STATE_WAIT_LOAD:
-		if (status == CAL_CLIENT_LOAD_SUCCESS) {
-			priv->load_state = LOAD_STATE_LOADED;
+	switch (status) {
+	case CAL_CLIENT_OPEN_SUCCESS:
+		if (client == priv->client)
 			initial_load (gcal);
-		} else if (status == CAL_CLIENT_LOAD_ERROR) {
-			priv->load_state = LOAD_STATE_NOT_LOADED;
-			load_error (gcal, priv->loading_uri);
-		} else if (status == CAL_CLIENT_LOAD_METHOD_NOT_SUPPORTED) {
-			priv->load_state = LOAD_STATE_NOT_LOADED;
-			method_error (gcal, priv->loading_uri);
-		} else
-			g_assert_not_reached ();
 
 		break;
 
-	case LOAD_STATE_WAIT_LOAD_BEFORE_CREATE:
-		if (status == CAL_CLIENT_LOAD_SUCCESS) {
-			priv->load_state = LOAD_STATE_LOADED;
-			initial_load (gcal);
-		} else if (status == CAL_CLIENT_LOAD_ERROR) {
-			priv->load_state = LOAD_STATE_WAIT_CREATE;
-			free_uri = FALSE;
-
-			if (!cal_client_create_calendar (priv->client, priv->loading_uri)) {
-				priv->load_state = LOAD_STATE_NOT_LOADED;
-				free_uri = TRUE;
-				g_message ("cal_loaded_cb(): Could not issue the create request");
-			}
-		} else if (status == CAL_CLIENT_LOAD_METHOD_NOT_SUPPORTED) {
-			priv->load_state = LOAD_STATE_NOT_LOADED;
-			method_error (gcal, priv->loading_uri);
-		} else
-			g_assert_not_reached ();
-
+	case CAL_CLIENT_OPEN_ERROR:
+		open_error (gcal, cal_client_get_uri (client));
 		break;
 
-	case LOAD_STATE_WAIT_CREATE:
-		if (status == CAL_CLIENT_LOAD_SUCCESS) {
-			priv->load_state = LOAD_STATE_LOADED;
-			initial_load (gcal);
-		} else if (status == CAL_CLIENT_LOAD_ERROR) {
-			priv->load_state = LOAD_STATE_NOT_LOADED;
-			create_error (gcal, priv->loading_uri);
-		} else if (status == CAL_CLIENT_LOAD_IN_USE) {
-			/* Someone created the URI while we were issuing the
-			 * create request, so we just try to reload.
-			 */
-			priv->load_state = LOAD_STATE_WAIT_LOAD;
-			free_uri = FALSE;
-
-			if (!cal_client_load_calendar (priv->client, priv->loading_uri)) {
-				priv->load_state = LOAD_STATE_NOT_LOADED;
-				free_uri = TRUE;
-				g_message ("cal_loaded_cb(): Could not issue the load request");
-			}
-		} else if (status == CAL_CLIENT_LOAD_METHOD_NOT_SUPPORTED) {
-			priv->load_state = LOAD_STATE_NOT_LOADED;
-			method_error (gcal, priv->loading_uri);
-		} else
-			g_assert_not_reached ();
+	case CAL_CLIENT_OPEN_NOT_FOUND:
+		/* bullshit; we did not specify only_if_exists */
+		g_assert_not_reached ();
+		return;
 
+	case CAL_CLIENT_OPEN_METHOD_NOT_SUPPORTED:
+		method_error (gcal, cal_client_get_uri (client));
 		break;
 
 	default:
 		g_assert_not_reached ();
-	}
-
-	if (free_uri) {
-		g_free (priv->loading_uri);
-		priv->loading_uri = NULL;
+		return;
 	}
 }
 
@@ -1276,9 +753,6 @@ obj_updated_cb (CalClient *client, const char *uid, gpointer data)
 	gcal = GNOME_CALENDAR (data);
 	priv = gcal->priv;
 
-	remove_alarms_for_object (gcal, uid);
-	add_alarms_for_object (gcal, uid);
-
 	tag_calendar_by_client (priv->date_navigator, priv->client);
 }
 
@@ -1292,12 +766,9 @@ obj_removed_cb (CalClient *client, const char *uid, gpointer data)
 	gcal = GNOME_CALENDAR (data);
 	priv = gcal->priv;
 
-	remove_alarms_for_object (gcal, uid);
-
 	tag_calendar_by_client (priv->date_navigator, priv->client);
 }
 
-
 GtkWidget *
 gnome_calendar_construct (GnomeCalendar *gcal)
 {
@@ -1310,24 +781,52 @@ gnome_calendar_construct (GnomeCalendar *gcal)
 
 	priv = gcal->priv;
 
+	/*
+	 * Calendar Folder Client.
+	 */
 	priv->client = cal_client_new ();
 	if (!priv->client)
 		return NULL;
 
-	gtk_signal_connect (GTK_OBJECT (priv->client), "cal_loaded",
-			    GTK_SIGNAL_FUNC (cal_loaded_cb), gcal);
+	gtk_signal_connect (GTK_OBJECT (priv->client), "cal_opened",
+			    GTK_SIGNAL_FUNC (cal_opened_cb), gcal);
 	gtk_signal_connect (GTK_OBJECT (priv->client), "obj_updated",
 			    GTK_SIGNAL_FUNC (obj_updated_cb), gcal);
 	gtk_signal_connect (GTK_OBJECT (priv->client), "obj_removed",
 			    GTK_SIGNAL_FUNC (obj_removed_cb), gcal);
 
-	e_calendar_table_set_cal_client (E_CALENDAR_TABLE (priv->todo), priv->client);
+#if 0
+	alarm_notify_add_client (priv->client);
+#endif
 
-	e_day_view_set_cal_client (E_DAY_VIEW (priv->day_view), priv->client);
-	e_day_view_set_cal_client (E_DAY_VIEW (priv->work_week_view), priv->client);
-	e_week_view_set_cal_client (E_WEEK_VIEW (priv->week_view), priv->client);
-	e_week_view_set_cal_client (E_WEEK_VIEW (priv->month_view), priv->client);
+	e_day_view_set_cal_client (E_DAY_VIEW (priv->day_view),
+				   priv->client);
+	e_day_view_set_cal_client (E_DAY_VIEW (priv->work_week_view),
+				   priv->client);
+	e_week_view_set_cal_client (E_WEEK_VIEW (priv->week_view),
+				    priv->client);
+	e_week_view_set_cal_client (E_WEEK_VIEW (priv->month_view),
+				    priv->client);
 
+	/*
+	 * TaskPad Folder Client.
+	 */
+	priv->task_pad_client = cal_client_new ();
+	if (!priv->task_pad_client)
+		return NULL;
+
+	gtk_signal_connect (GTK_OBJECT (priv->task_pad_client), "cal_opened",
+			    GTK_SIGNAL_FUNC (cal_opened_cb), gcal);
+
+#if 0
+	alarm_notify_add_client (priv->task_pad_client);
+#endif
+
+	e_calendar_table_set_cal_client (E_CALENDAR_TABLE (priv->todo),
+					 priv->task_pad_client);
+
+
+	/* Get the default view to show. */
 	view = calendar_config_get_default_view ();
 	switch (view) {
 	case 1:
@@ -1368,9 +867,9 @@ gnome_calendar_new (void)
 /**
  * gnome_calendar_get_cal_client:
  * @gcal: A calendar view.
- * 
+ *
  * Queries the calendar client interface object that a calendar view is using.
- * 
+ *
  * Return value: A calendar client interface object.
  **/
 CalClient *
@@ -1387,35 +886,40 @@ gnome_calendar_get_cal_client (GnomeCalendar *gcal)
 }
 
 gboolean
-gnome_calendar_open (GnomeCalendar *gcal, char *file, GnomeCalendarOpenMode gcom)
+gnome_calendar_open (GnomeCalendar *gcal, char *str_uri)
 {
 	GnomeCalendarPrivate *priv;
+	char *tasks_uri;
+	gboolean success;
 
 	g_return_val_if_fail (gcal != NULL, FALSE);
 	g_return_val_if_fail (GNOME_IS_CALENDAR (gcal), FALSE);
-	g_return_val_if_fail (file != NULL, FALSE);
+	g_return_val_if_fail (str_uri != NULL, FALSE);
 
 	priv = gcal->priv;
-	g_return_val_if_fail (priv->load_state == LOAD_STATE_NOT_LOADED, FALSE);
 
-	g_assert (priv->loading_uri == NULL);
+	g_return_val_if_fail (
+		cal_client_get_load_state (priv->client) == CAL_CLIENT_LOAD_NOT_LOADED,
+		FALSE);
 
-	priv->loading_uri = g_strdup (file);
+	g_return_val_if_fail (
+		cal_client_get_load_state (priv->task_pad_client) == CAL_CLIENT_LOAD_NOT_LOADED,
+		FALSE);
 
-	if (gcom == CALENDAR_OPEN)
-		priv->load_state = LOAD_STATE_WAIT_LOAD;
-	else if (gcom == CALENDAR_OPEN_OR_CREATE)
-		priv->load_state = LOAD_STATE_WAIT_LOAD_BEFORE_CREATE;
-	else {
-		g_assert_not_reached ();
+	if (!cal_client_open_calendar (priv->client, str_uri, FALSE)) {
+		g_message ("gnome_calendar_open(): Could not issue the request");
 		return FALSE;
 	}
 
-	if (!cal_client_load_calendar (priv->client, file)) {
-		priv->load_state = LOAD_STATE_NOT_LOADED;
-		g_free (priv->loading_uri);
-		priv->loading_uri = NULL;
+	/* Open the appropriate Tasks folder to show in the TaskPad.
+	   Currently we just show the folder named "Tasks", but it will be
+	   a per-calendar option in future. */
 
+	tasks_uri = g_strdup_printf ("%s/local/Tasks/tasks.ics", evolution_dir);
+	success = cal_client_open_calendar (priv->task_pad_client, tasks_uri, FALSE);
+	g_free (tasks_uri);
+
+	if (!success) {
 		g_message ("gnome_calendar_open(): Could not issue the request");
 		return FALSE;
 	}
@@ -1423,136 +927,6 @@ gnome_calendar_open (GnomeCalendar *gcal, char *file, GnomeCalendarOpenMode gcom
 	return TRUE;
 }
 
-#if 0
-
-static void
-stop_beeping (GtkObject* object, gpointer data)
-{
-	guint timer_tag, beep_tag;
-	timer_tag = GPOINTER_TO_INT (gtk_object_get_data (object, "timer_tag"));
-	beep_tag  = GPOINTER_TO_INT (gtk_object_get_data (object, "beep_tag"));
-
-	if (beep_tag > 0) {
-		gtk_timeout_remove (beep_tag);
-		gtk_object_set_data (object, "beep_tag", GINT_TO_POINTER (0));
-	}
-	if (timer_tag > 0) {
-		gtk_timeout_remove (timer_tag);
-		gtk_object_set_data (object, "timer_tag", GINT_TO_POINTER (0));
-	}
-}
-
-static gint
-start_beeping (gpointer data)
-{
-	gdk_beep ();
-
-	return TRUE;
-}
-
-static gint
-timeout_beep (gpointer data)
-{
-	stop_beeping (data, NULL);
-	return FALSE;
-}
-
-void
-calendar_notify (time_t activation_time, CalendarAlarm *which, void *data)
-{
-	iCalObject *ico = data;
-	guint beep_tag, timer_tag;
-	int ret;
-	gchar* snooze_button = (enable_snooze ? _("Snooze") : NULL);
-	time_t now, diff;
-
-	if (&ico->aalarm == which){
-		time_t app = ico->aalarm.trigger + ico->aalarm.offset;
-		GtkWidget *w;
-		char *msg;
-
-		msg = g_strconcat (_("Reminder of your appointment at "),
-					ctime (&app), "`",
-					ico->summary, "'", NULL);
-
-		/* Idea: we need Snooze option :-) */
-		w = gnome_message_box_new (msg, GNOME_MESSAGE_BOX_INFO, _("Ok"), snooze_button, NULL);
-		beep_tag = gtk_timeout_add (1000, start_beeping, NULL);
-		if (enable_aalarm_timeout)
-			timer_tag = gtk_timeout_add (audio_alarm_timeout*1000,
-						     timeout_beep, w);
-		else
-			timer_tag = 0;
-		gtk_object_set_data (GTK_OBJECT (w), "timer_tag",
-				     GINT_TO_POINTER (timer_tag));
-		gtk_object_set_data (GTK_OBJECT (w), "beep_tag",
-				     GINT_TO_POINTER (beep_tag));
-		gtk_widget_ref (w);
-		gtk_window_set_modal (GTK_WINDOW (w), FALSE);
-		ret = gnome_dialog_run (GNOME_DIALOG (w));
-		switch (ret) {
-		case 1:
-			stop_beeping (GTK_OBJECT (w), NULL);
-			now = time (NULL);
-			diff = now - which->trigger;
-			which->trigger = which->trigger + diff + snooze_secs;
-			which->offset  = which->offset - diff - snooze_secs;
-			alarm_add (which, &calendar_notify, data);
-			break;
-		default:
-			stop_beeping (GTK_OBJECT (w), NULL);
-			break;
-		}
-
-		gtk_widget_unref (w);
-		return;
-	}
-
-        if (&ico->palarm == which){
-		execute (ico->palarm.data, 0);
-		return;
-	}
-
-	if (&ico->malarm == which){
-		time_t app = ico->malarm.trigger + ico->malarm.offset;
-
-		mail_notify (ico->malarm.data, ico->summary, app);
-		return;
-	}
-
-	if (&ico->dalarm == which){
-		time_t app = ico->dalarm.trigger + ico->dalarm.offset;
-		GtkWidget *w;
-		char *msg;
-
-		if (beep_on_display)
-			gdk_beep ();
-		msg = g_strconcat (_("Reminder of your appointment at "),
-					ctime (&app), "`",
-					ico->summary, "'", NULL);
-		w = gnome_message_box_new (msg, GNOME_MESSAGE_BOX_INFO,
-					   _("Ok"), snooze_button, NULL);
-		gtk_window_set_modal (GTK_WINDOW (w), FALSE);
-		ret = gnome_dialog_run (GNOME_DIALOG (w));
-		switch (ret) {
-		case 1:
-			now = time (NULL);
-			diff = now - which->trigger;
-			which->trigger = which->trigger + diff + snooze_secs;
-			which->offset  = which->offset - diff - snooze_secs;
-			alarm_add (which, &calendar_notify, data);
-			break;
-		default:
-			break;
-		}
-
-		return;
-	}
-}
-
-#endif
-
-
 /* Tells the calendar to reload all config settings.
    If initializing is TRUE it sets the pane positions as well. (We don't
    want to reset the pane positions after the user clicks 'Apply' in the
@@ -1566,7 +940,6 @@ gnome_calendar_update_config_settings (GnomeCalendar *gcal,
 	gint week_start_day, time_divisions;
 	gint start_hour, start_minute, end_hour, end_minute;
 	gboolean use_24_hour, show_event_end, compress_weekend;
-	gboolean dnav_show_week_no;
 
 	g_return_if_fail (GNOME_IS_CALENDAR (gcal));
 
@@ -1594,9 +967,6 @@ gnome_calendar_update_config_settings (GnomeCalendar *gcal,
 					week_start_day);
 	e_week_view_set_week_start_day (E_WEEK_VIEW (priv->month_view),
 					week_start_day);
-	gnome_canvas_item_set (GNOME_CANVAS_ITEM (E_CALENDAR (priv->date_navigator)->calitem),
-			       "week_start_day", week_start_day,
-			       NULL);
 
 	start_hour = calendar_config_get_day_start_hour ();
 	start_minute = calendar_config_get_day_start_minute ();
@@ -1639,10 +1009,7 @@ gnome_calendar_update_config_settings (GnomeCalendar *gcal,
 	e_week_view_set_compress_weekend (E_WEEK_VIEW (priv->month_view),
 					  compress_weekend);
 
-	dnav_show_week_no = calendar_config_get_dnav_show_week_no ();
-	gnome_canvas_item_set (GNOME_CANVAS_ITEM (E_CALENDAR (priv->date_navigator)->calitem),
-			       "show_week_numbers", dnav_show_week_no,
-			       NULL);
+	calendar_config_configure_e_calendar (E_CALENDAR (priv->date_navigator));
 
 	if (initializing) {
 		priv->hpane_pos = calendar_config_get_hpane_pos ();
@@ -1679,7 +1046,7 @@ gnome_calendar_set_selected_time_range (GnomeCalendar *gcal,
  * @gcal: A calendar view.
  * @start_time: Return value for the start of the time selection.
  * @end_time: Return value for the end of the time selection.
- * 
+ *
  * Queries the time selection range on the calendar view.
  **/
 void
@@ -1804,10 +1171,10 @@ gnome_calendar_new_appointment (GnomeCalendar *gcal)
 	comp = cal_component_new ();
 	cal_component_set_new_vtype (comp, CAL_COMPONENT_EVENT);
 
-	itt = icaltime_from_timet (dtstart, FALSE, TRUE);
+	itt = icaltime_from_timet (dtstart, FALSE);
 	cal_component_set_dtstart (comp, &dt);
 
-	itt = icaltime_from_timet (dtend, FALSE, TRUE);
+	itt = icaltime_from_timet (dtend, FALSE);
 	cal_component_set_dtend (comp, &dt);
 
 	cal_component_commit_sequence (comp);
