@@ -25,30 +25,30 @@
 #include <config.h>
 #endif
 
-#include <gnome.h>
 #include <glade/glade.h>
 #include <bonobo.h>
-#include <bonobo/bonobo-control.h>
 #include <glade/glade.h>
 
 #include <liboaf/liboaf.h>
 
 #include <gal/widgets/e-cursors.h>
+
 #include "alarm.h"
 #include "calendar-commands.h"
 #include "calendar-config.h"
 #include "component-factory.h"
 #include "control-factory.h"
 #include "e-itip-control.h"
+#include "tasks-control-factory.h"
 
 static void
-init_bonobo (int *argc, char **argv)
+init_bonobo (int argc, char **argv)
 {
-	/* FIXME: VERSION instead of "0.0".  */
-	gnome_init_with_popt_table ("evolution-calendar", "0.0",
-				    *argc, argv, oaf_popt_options,
-				    0, NULL);
-	oaf_init (*argc, argv);
+	if (gnome_init_with_popt_table ("evolution-calendar", VERSION, argc, argv,
+					oaf_popt_options, 0, NULL) != 0)
+		g_error (_("Could not initialize GNOME"));
+
+	oaf_init (argc, argv);
 
 	if (bonobo_init (CORBA_OBJECT_NIL, CORBA_OBJECT_NIL, CORBA_OBJECT_NIL) == FALSE)
 		g_error (_("Could not initialize Bonobo"));
@@ -61,13 +61,11 @@ main (int argc, char **argv)
 	bindtextdomain(PACKAGE, EVOLUTION_LOCALEDIR);
 	textdomain(PACKAGE);
 
-	init_bonobo (&argc, argv);
+	init_bonobo (argc, argv);
 	glade_gnome_init ();
 	alarm_init ();
 	e_cursors_init ();
 
-	init_calendar ();
-
 #if 0
 	//g_log_set_always_fatal ((GLogLevelFlags) 0xFFFF);
 	g_log_set_always_fatal (G_LOG_LEVEL_ERROR |
@@ -78,9 +76,11 @@ main (int argc, char **argv)
 	control_factory_init ();
 	component_factory_init ();
 	e_itip_control_factory_init ();
+	tasks_control_factory_init ();
 
 	bonobo_main ();
 
+	alarm_done ();
 	calendar_config_write_on_exit ();
 
 	return 0;
