@@ -2,9 +2,10 @@
 /* Evolution calendar - generic backend class
  *
  * Copyright (C) 2000 Helix Code, Inc.
+ * Copyright (C) 2000 Ximian, Inc.
  *
- * Authors: Federico Mena-Quintero <federico@helixcode.com>
- *          JP Rosevear <jpr@helixcode.com>
+ * Authors: Federico Mena-Quintero <federico@ximian.com>
+ *          JP Rosevear <jpr@ximian.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -40,10 +41,6 @@ enum {
 };
 
 static void cal_backend_class_init (CalBackendClass *class);
-static void cal_backend_init (CalBackend *backend);
-static void cal_backend_destroy (GtkObject *object);
-
-static GtkObjectClass *parent_class;
 
 static guint cal_backend_signals[LAST_SIGNAL];
 
@@ -71,7 +68,7 @@ cal_backend_get_type (void)
 			sizeof (CalBackend),
 			sizeof (CalBackendClass),
 			(GtkClassInitFunc) cal_backend_class_init,
-			(GtkObjectInitFunc) cal_backend_init,
+			(GtkObjectInitFunc) NULL,
 			NULL, /* reserved_1 */
 			NULL, /* reserved_2 */
 			(GtkClassInitFunc) NULL
@@ -92,10 +89,6 @@ cal_backend_class_init (CalBackendClass *class)
 
 	object_class = (GtkObjectClass *) class;
 
-	parent_class = gtk_type_class (GTK_TYPE_OBJECT);
-
-	object_class->destroy = cal_backend_destroy;
-
 	cal_backend_signals[LAST_CLIENT_GONE] =
 		gtk_signal_new ("last_client_gone",
 				GTK_RUN_FIRST,
@@ -107,37 +100,13 @@ cal_backend_class_init (CalBackendClass *class)
 	gtk_object_class_add_signals (object_class, cal_backend_signals, LAST_SIGNAL);
 }
 
-/* Per instance initialization function */
-static void
-cal_backend_init (CalBackend *backend)
-{
-	backend->uri = NULL;
-}
-
-static void
-cal_backend_destroy (GtkObject *object)
-{
-	CalBackend *backend;
-	
-	g_return_if_fail (object != NULL);
-	g_return_if_fail (IS_CAL_BACKEND (object));
-
-	backend = CAL_BACKEND (object);
-
- 	if (backend->uri)
-		gnome_vfs_uri_unref (backend->uri);
-	
-	if (GTK_OBJECT_CLASS (parent_class)->destroy)
-		(* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
-}
-
 
 
 /**
  * cal_backend_get_uri:
  * @backend: A calendar backend.
  *
- * Queries the URI of a calendar backend, which must already have a loaded
+ * Queries the URI of a calendar backend, which must already have an open
  * calendar.
  *
  * Return value: The URI where the calendar is stored.
@@ -152,23 +121,13 @@ cal_backend_get_uri (CalBackend *backend)
 	return (* CLASS (backend)->get_uri) (backend);
 }
 
-static void
-cal_backend_set_uri (CalBackend *backend, GnomeVFSURI *uri)
-{
-	if (backend->uri)
-		gnome_vfs_uri_unref (backend->uri);
-
-	gnome_vfs_uri_ref (uri);
-	backend->uri = uri;
-}
-
 /**
  * cal_backend_add_cal:
  * @backend: A calendar backend.
  * @cal: A calendar client interface object.
  *
  * Adds a calendar client interface object to a calendar @backend.
- * The calendar backend must already have a loaded calendar.
+ * The calendar backend must already have an open calendar.
  **/
 void
 cal_backend_add_cal (CalBackend *backend, Cal *cal)
@@ -181,53 +140,33 @@ cal_backend_add_cal (CalBackend *backend, Cal *cal)
 }
 
 /**
- * cal_backend_load:
+ * cal_backend_open:
  * @backend: A calendar backend.
  * @uri: URI that contains the calendar data.
+ * @only_if_exists: Whether the calendar should be opened only if it already
+ * exists.  If FALSE, a new calendar will be created when the specified @uri
+ * does not exist.
  *
- * Loads a calendar backend with data from a calendar stored at the specified
+ * Opens a calendar backend with data from a calendar stored at the specified
  * URI.
  *
  * Return value: An operation status code.
  **/
-CalBackendLoadStatus
-cal_backend_load (CalBackend *backend, GnomeVFSURI *uri)
+CalBackendOpenStatus
+cal_backend_open (CalBackend *backend, GnomeVFSURI *uri, gboolean only_if_exists)
 {
-	CalBackendLoadStatus result;
+	CalBackendOpenStatus result;
 
-	g_return_val_if_fail (backend != NULL, CAL_BACKEND_LOAD_ERROR);
-	g_return_val_if_fail (IS_CAL_BACKEND (backend), CAL_BACKEND_LOAD_ERROR);
-	g_return_val_if_fail (uri != NULL, CAL_BACKEND_LOAD_ERROR);
+	g_return_val_if_fail (backend != NULL, CAL_BACKEND_OPEN_ERROR);
+	g_return_val_if_fail (IS_CAL_BACKEND (backend), CAL_BACKEND_OPEN_ERROR);
+	g_return_val_if_fail (uri != NULL, CAL_BACKEND_OPEN_ERROR);
 
-	g_assert (CLASS (backend)->load != NULL);
-	result =  (* CLASS (backend)->load) (backend, uri);
+	g_assert (CLASS (backend)->open != NULL);
+	result =  (* CLASS (backend)->open) (backend, uri, only_if_exists);
 
-	if (result == CAL_BACKEND_LOAD_SUCCESS)
-		cal_backend_set_uri (backend, uri);
-	
 	return result;
 }
 
-/**
- * cal_backend_create:
- * @backend: A calendar backend.
- * @uri: URI that will contain the calendar data.
- *
- * Creates a new empty calendar in a calendar backend.
- **/
-void
-cal_backend_create (CalBackend *backend, GnomeVFSURI *uri)
-{
-	g_return_if_fail (backend != NULL);
-	g_return_if_fail (IS_CAL_BACKEND (backend));
-	g_return_if_fail (uri != NULL);
-
-	g_assert (CLASS (backend)->create != NULL);
-	(* CLASS (backend)->create) (backend, uri);
-
-	cal_backend_set_uri (backend, uri);
-}
-
 /**
  * cal_backend_get_n_objects:
  * @backend: A calendar backend.
@@ -332,7 +271,10 @@ cal_backend_compute_changes (CalBackend *backend, CalObjType type, const char *c
 	int n;
 	
 	/* Find the changed ids - FIX ME, path should not be hard coded */
-	filename = g_strdup_printf ("%s/evolution/local/Calendar/%s.db", g_get_home_dir (), change_id);
+	if (type == GNOME_Evolution_Calendar_TYPE_TODO)
+		filename = g_strdup_printf ("%s/evolution/local/Tasks/%s.db", g_get_home_dir (), change_id);
+	else 
+		filename = g_strdup_printf ("%s/evolution/local/Calendar/%s.db", g_get_home_dir (), change_id);
 	ehash = e_dbhash_new (filename);
 	g_free (filename);
 	
@@ -464,21 +406,29 @@ cal_backend_get_objects_in_range (CalBackend *backend, CalObjType type,
  * @backend: A calendar backend.
  * @start: Start time for query.
  * @end: End time for query.
+ * @valid_range: Return value that says whether the range is valid or not.
  * 
  * Builds a sorted list of the alarms that trigger in the specified time range.
  * 
- * Return value: A list of #CalAlarmInstance structures, sorted by trigger time.
+ * Return value: A sequence of component alarm instances structures, or NULL
+ * if @valid_range returns FALSE.
  **/
-GList *
-cal_backend_get_alarms_in_range (CalBackend *backend, time_t start, time_t end)
+GNOME_Evolution_Calendar_CalComponentAlarmsSeq *
+cal_backend_get_alarms_in_range (CalBackend *backend, time_t start, time_t end, gboolean *valid_range)
 {
 	g_return_val_if_fail (backend != NULL, NULL);
 	g_return_val_if_fail (IS_CAL_BACKEND (backend), NULL);
-	g_return_val_if_fail (start != -1 && end != -1, NULL);
-	g_return_val_if_fail (start <= end, NULL);
+	g_return_val_if_fail (valid_range != NULL, NULL);
 
 	g_assert (CLASS (backend)->get_alarms_in_range != NULL);
-	return (* CLASS (backend)->get_alarms_in_range) (backend, start, end);
+
+	if (!(start != -1 && end != -1 && start <= end)) {
+		*valid_range = FALSE;
+		return NULL;
+	} else {
+		*valid_range = TRUE;
+		return (* CLASS (backend)->get_alarms_in_range) (backend, start, end);
+	}
 }
 
 /**
@@ -487,27 +437,43 @@ cal_backend_get_alarms_in_range (CalBackend *backend, time_t start, time_t end)
  * @uid: Unique identifier for a calendar object.
  * @start: Start time for query.
  * @end: End time for query.
- * @alarms: Return value for the list of alarm instances.
+ * @result: Return value for the result code for the operation.
  * 
  * Builds a sorted list of the alarms of the specified event that trigger in a
  * particular time range.
  * 
- * Return value: TRUE on success, FALSE if the object was not found.
+ * Return value: A structure of the component's alarm instances, or NULL if @result
+ * returns something other than #CAL_BACKEND_GET_ALARMS_SUCCESS.
  **/
-gboolean
+GNOME_Evolution_Calendar_CalComponentAlarms *
 cal_backend_get_alarms_for_object (CalBackend *backend, const char *uid,
 				   time_t start, time_t end,
-				   GList **alarms)
+				   CalBackendGetAlarmsForObjectResult *result)
 {
-	g_return_val_if_fail (backend != NULL, FALSE);
-	g_return_val_if_fail (IS_CAL_BACKEND (backend), FALSE);
-	g_return_val_if_fail (uid != NULL, FALSE);
-	g_return_val_if_fail (start != -1 && end != -1, FALSE);
-	g_return_val_if_fail (start <= end, FALSE);
-	g_return_val_if_fail (alarms != NULL, FALSE);
+	g_return_val_if_fail (backend != NULL, NULL);
+	g_return_val_if_fail (IS_CAL_BACKEND (backend), NULL);
+	g_return_val_if_fail (uid != NULL, NULL);
+	g_return_val_if_fail (result != NULL, NULL);
 
 	g_assert (CLASS (backend)->get_alarms_for_object != NULL);
-	return (* CLASS (backend)->get_alarms_for_object) (backend, uid, start, end, alarms);
+
+	if (!(start != -1 && end != -1 && start <= end)) {
+		*result = CAL_BACKEND_GET_ALARMS_INVALID_RANGE;
+		return NULL;
+	} else {
+		gboolean object_found;
+		GNOME_Evolution_Calendar_CalComponentAlarms *alarms;
+
+		alarms = (* CLASS (backend)->get_alarms_for_object) (backend, uid, start, end,
+								     &object_found);
+
+		if (object_found)
+			*result = CAL_BACKEND_GET_ALARMS_SUCCESS;
+		else
+			*result = CAL_BACKEND_GET_ALARMS_NOT_FOUND;
+
+		return alarms;
+	}
 }
 
 /**
