@@ -2,8 +2,9 @@
 /* Evolution calendar - generic backend class
  *
  * Copyright (C) 2000 Helix Code, Inc.
+ * Copyright (C) 2000 Ximian, Inc.
  *
- * Author: Federico Mena-Quintero <federico@helixcode.com>
+ * Author: Federico Mena-Quintero <federico@ximian.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,7 +28,7 @@
 #include <libgnomevfs/gnome-vfs.h>
 #include <cal-util/cal-util.h>
 #include <cal-util/cal-component.h>
-#include "calendar/pcs/evolution-calendar.h"
+#include "evolution-calendar.h"
 #include "cal-common.h"
 #include "cal.h"
 
@@ -44,14 +45,20 @@ BEGIN_GNOME_DECLS
 
 /* Load status values */
 typedef enum {
-	CAL_BACKEND_LOAD_SUCCESS,	/* Loading OK */
-	CAL_BACKEND_LOAD_ERROR		/* We need better error reporting in libversit */
-} CalBackendLoadStatus;
+	CAL_BACKEND_OPEN_SUCCESS,	/* Loading OK */
+	CAL_BACKEND_OPEN_ERROR,		/* We need better error reporting in libversit */
+	CAL_BACKEND_OPEN_NOT_FOUND
+} CalBackendOpenStatus;
+
+/* Result codes for ::get_alarms_in_range() */
+typedef enum {
+	CAL_BACKEND_GET_ALARMS_SUCCESS,
+	CAL_BACKEND_GET_ALARMS_NOT_FOUND,
+	CAL_BACKEND_GET_ALARMS_INVALID_RANGE
+} CalBackendGetAlarmsForObjectResult;
 
 struct _CalBackend {
 	GtkObject object;
-
-	GnomeVFSURI *uri;
 };
 
 struct _CalBackendClass {
@@ -63,19 +70,24 @@ struct _CalBackendClass {
 	/* Virtual methods */
 	GnomeVFSURI *(* get_uri) (CalBackend *backend);
 	void (* add_cal) (CalBackend *backend, Cal *cal);
-	CalBackendLoadStatus (* load) (CalBackend *backend, GnomeVFSURI *uri);
-	void (* create) (CalBackend *backend, GnomeVFSURI *uri);
+
+	CalBackendOpenStatus (* open) (CalBackend *backend, GnomeVFSURI *uri,
+				       gboolean only_if_exists);
 
 	int (* get_n_objects) (CalBackend *backend, CalObjType type);
 	char *(* get_object) (CalBackend *backend, const char *uid);
 	CalObjType(* get_type_by_uid) (CalBackend *backend, const char *uid);
 	GList *(* get_uids) (CalBackend *backend, CalObjType type);
+
 	GList *(* get_objects_in_range) (CalBackend *backend, CalObjType type,
 					 time_t start, time_t end);
-	GList *(* get_alarms_in_range) (CalBackend *backend, time_t start, time_t end);
-	gboolean (* get_alarms_for_object) (CalBackend *backend, const char *uid,
-					    time_t start, time_t end,
-					    GList **alarms);
+
+	GNOME_Evolution_Calendar_CalComponentAlarmsSeq *(* get_alarms_in_range) (
+		CalBackend *backend, time_t start, time_t end);
+	GNOME_Evolution_Calendar_CalComponentAlarms *(* get_alarms_for_object) (
+		CalBackend *backend, const char *uid,
+		time_t start, time_t end, gboolean *object_found);
+
 	gboolean (* update_object) (CalBackend *backend, const char *uid, const char *calobj);
 	gboolean (* remove_object) (CalBackend *backend, const char *uid);
 };
@@ -86,9 +98,8 @@ GnomeVFSURI *cal_backend_get_uri (CalBackend *backend);
 
 void cal_backend_add_cal (CalBackend *backend, Cal *cal);
 
-CalBackendLoadStatus cal_backend_load (CalBackend *backend, GnomeVFSURI *uri);
-
-void cal_backend_create (CalBackend *backend, GnomeVFSURI *uri);
+CalBackendOpenStatus cal_backend_open (CalBackend *backend, GnomeVFSURI *uri,
+				       gboolean only_if_exists);
 
 int cal_backend_get_n_objects (CalBackend *backend, CalObjType type);
 
@@ -101,11 +112,13 @@ GNOME_Evolution_Calendar_CalObjChangeSeq * cal_backend_get_changes (CalBackend *
 GList *cal_backend_get_objects_in_range (CalBackend *backend, CalObjType type,
 					 time_t start, time_t end);
 
-GList *cal_backend_get_alarms_in_range (CalBackend *backend, time_t start, time_t end);
+GNOME_Evolution_Calendar_CalComponentAlarmsSeq *cal_backend_get_alarms_in_range (
+	CalBackend *backend, time_t start, time_t end, gboolean *valid_range);
 
-gboolean cal_backend_get_alarms_for_object (CalBackend *backend, const char *uid,
-					    time_t start, time_t end,
-					    GList **alarms);
+GNOME_Evolution_Calendar_CalComponentAlarms *cal_backend_get_alarms_for_object (
+	CalBackend *backend, const char *uid,
+	time_t start, time_t end,
+	CalBackendGetAlarmsForObjectResult *result);
 
 
 gboolean cal_backend_update_object (CalBackend *backend, const char *uid, const char *calobj);
