@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000 Helix Code Inc.
+ *  Copyright (C) 2000,2001 Ximian Inc.
  *
  *  Authors: Michael Zucchi <notzed@helixcode.com>
  *
@@ -25,11 +25,11 @@
 
 #include <stdio.h>
 #include <string.h>
+#include <ctype.h>
 #include <glib.h>
 #include <sys/types.h>
 #include <regex.h>
 
-#include <gal/widgets/e-unicode.h>
 #include "camel-folder-search.h"
 #include "string-utils.h"
 
@@ -37,20 +37,25 @@
 #include "camel-medium.h"
 #include "camel-multipart.h"
 #include "camel-mime-message.h"
-#include "gmime-content-field.h"
 #include "camel-stream-mem.h"
 #include "e-util/e-memory.h"
+#include "camel-search-private.h"
 
 #define d(x)
 #define r(x)
 
 struct _CamelFolderSearchPrivate {
 	GHashTable *mempool_hash;
+	CamelException *ex;
 };
 
 #define _PRIVATE(o) (((CamelFolderSearch *)(o))->priv)
 
 static ESExpResult *search_header_contains(struct _ESExp *f, int argc, struct _ESExpResult **argv, CamelFolderSearch *search);
+static ESExpResult *search_header_matches(struct _ESExp *f, int argc, struct _ESExpResult **argv, CamelFolderSearch *search);
+static ESExpResult *search_header_starts_with(struct _ESExp *f, int argc, struct _ESExpResult **argv, CamelFolderSearch *search);
+static ESExpResult *search_header_ends_with(struct _ESExp *f, int argc, struct _ESExpResult **argv, CamelFolderSearch *search);
+static ESExpResult *search_header_exists(struct _ESExp *f, int argc, struct _ESExpResult **argv, CamelFolderSearch *search);
 static ESExpResult *search_match_all(struct _ESExp *f, int argc, struct _ESExpTerm **argv, CamelFolderSearch *search);
 static ESExpResult *search_body_contains(struct _ESExp *f, int argc, struct _ESExpResult **argv, CamelFolderSearch *search);
 static ESExpResult *search_user_flag(struct _ESExp *f, int argc, struct _ESExpResult **argv, CamelFolderSearch *s);
@@ -76,6 +81,10 @@ camel_folder_search_class_init (CamelFolderSearchClass *klass)
 	klass->match_all = search_match_all;
 	klass->body_contains = search_body_contains;
 	klass->header_contains = search_header_contains;
+	klass->header_matches = search_header_matches;
+	klass->header_starts_with = search_header_starts_with;
+	klass->header_ends_with = search_header_ends_with;
+	klass->header_exists = search_header_exists;
 	klass->user_tag = search_user_tag;
 	klass->user_flag = search_user_flag;
 	klass->system_flag = search_system_flag;
@@ -118,9 +127,8 @@ camel_folder_search_finalize (CamelObject *obj)
 	CamelFolderSearch *search = (CamelFolderSearch *)obj;
 	struct _CamelFolderSearchPrivate *p = _PRIVATE(obj);
 
-	/* yeah, this is a gtk object */
 	if (search->sexp)
-		gtk_object_unref((GtkObject *)search->sexp);
+		e_sexp_unref(search->sexp);
 
 	g_free(search->last_search);
 	g_hash_table_foreach(p->mempool_hash, free_mempool, obj);
@@ -170,6 +178,10 @@ struct {
 	{ "match-all", CAMEL_STRUCT_OFFSET(CamelFolderSearchClass, match_all), 3 },
 	{ "body-contains", CAMEL_STRUCT_OFFSET(CamelFolderSearchClass, body_contains), 1 },
 	{ "header-contains", CAMEL_STRUCT_OFFSET(CamelFolderSearchClass, header_contains), 1 },
+	{ "header-matches", CAMEL_STRUCT_OFFSET(CamelFolderSearchClass, header_matches), 1 },
+	{ "header-starts-with", CAMEL_STRUCT_OFFSET(CamelFolderSearchClass, header_starts_with), 1 },
+	{ "header-ends-with", CAMEL_STRUCT_OFFSET(CamelFolderSearchClass, header_ends_with), 1 },
+	{ "header-exists", CAMEL_STRUCT_OFFSET(CamelFolderSearchClass, header_exists), 1 },
 	{ "user-tag", CAMEL_STRUCT_OFFSET(CamelFolderSearchClass, user_tag), 1 },
 	{ "user-flag", CAMEL_STRUCT_OFFSET(CamelFolderSearchClass, user_flag), 1 },
 	{ "system-flag", CAMEL_STRUCT_OFFSET(CamelFolderSearchClass, system_flag), 1 },
@@ -292,21 +304,34 @@ GPtrArray *
 camel_folder_search_execute_expression(CamelFolderSearch *search, const char *expr, CamelException *ex)
 {
 	ESExpResult *r;
-	GPtrArray *matches = g_ptr_array_new ();
+	GPtrArray *matches;
 	int i;
 	GHashTable *results;
 	EMemPool *pool;
 	struct _CamelFolderSearchPrivate *p = _PRIVATE(search);
 
+	p->ex = ex;
+
 	/* only re-parse if the search has changed */
 	if (search->last_search == NULL
 	    || strcmp(search->last_search, expr)) {
 		e_sexp_input_text(search->sexp, expr, strlen(expr));
-		e_sexp_parse(search->sexp);
+		if (e_sexp_parse(search->sexp) == -1) {
+			camel_exception_setv(ex, 1, _("Cannot parse search expression: %s:\n%s"), e_sexp_error(search->sexp), expr);
+			return NULL;
+		}
+
 		g_free(search->last_search);
 		search->last_search = g_strdup(expr);
 	}
 	r = e_sexp_eval(search->sexp);
+	if (r == NULL) {
+		if (!camel_exception_is_set(ex))
+			camel_exception_setv(ex, 1, _("Error executing search expression: %s:\n%s"), e_sexp_error(search->sexp), expr);
+		return NULL;
+	}
+
+	matches = g_ptr_array_new();
 
 	/* now create a folder summary to return?? */
 	if (r
@@ -338,7 +363,7 @@ camel_folder_search_execute_expression(CamelFolderSearch *search, const char *ex
 				g_ptr_array_add(matches, e_mempool_strdup(pool, g_ptr_array_index(r->value.ptrarray, i)));
 			}
 		}
-		e_sexp_result_free(r);
+		e_sexp_result_free(search->sexp, r);
 		/* instead of putting the mempool_hash in the structure, we keep the api clean by
 		   putting a reference to it in a hashtable.  Lets us do some debugging and catch
 		   unfree'd results as well. */
@@ -404,6 +429,9 @@ void camel_folder_search_free_result(CamelFolderSearch *search, GPtrArray *resul
 	g_ptr_array_free(result, TRUE);
 }
 
+
+
+
 /* dummy function, returns false always, or an empty match array */
 static ESExpResult *
 search_dummy(struct _ESExp *f, int argc, struct _ESExpResult **argv, CamelFolderSearch *search)
@@ -411,10 +439,10 @@ search_dummy(struct _ESExp *f, int argc, struct _ESExpResult **argv, CamelFolder
 	ESExpResult *r;
 
 	if (search->current == NULL) {
-		r = e_sexp_result_new(ESEXP_RES_BOOL);
+		r = e_sexp_result_new(f, ESEXP_RES_BOOL);
 		r->value.bool = FALSE;
 	} else {
-		r = e_sexp_result_new(ESEXP_RES_ARRAY_PTR);
+		r = e_sexp_result_new(f, ESEXP_RES_ARRAY_PTR);
 		r->value.ptrarray = g_ptr_array_new();
 	}
 
@@ -430,7 +458,7 @@ search_match_all(struct _ESExp *f, int argc, struct _ESExpTerm **argv, CamelFold
 	if (argc>1) {
 		g_warning("match-all only takes a single argument, other arguments ignored");
 	}
-	r = e_sexp_result_new(ESEXP_RES_ARRAY_PTR);
+	r = e_sexp_result_new(f, ESEXP_RES_ARRAY_PTR);
 	r->value.ptrarray = g_ptr_array_new();
 
 	/* we are only matching a single message? */
@@ -444,8 +472,9 @@ search_match_all(struct _ESExp *f, int argc, struct _ESExpTerm **argv, CamelFold
 					g_ptr_array_add(r->value.ptrarray, (char *)camel_message_info_uid(search->current));
 			} else {
 				g_warning("invalid syntax, matches require a single bool result");
+				e_sexp_fatal_error(f, _("(match-all) requires a single bool result"));
 			}
-			e_sexp_result_free(r1);
+			e_sexp_result_free(f, r1);
 		} else {
 			g_ptr_array_add(r->value.ptrarray, (char *)camel_message_info_uid(search->current));
 		}
@@ -457,6 +486,7 @@ search_match_all(struct _ESExp *f, int argc, struct _ESExpTerm **argv, CamelFold
 	if (search->summary == NULL) {
 		/* TODO: make it work - e.g. use the folder and so forth for a slower search */
 		g_warning("No summary supplied, match-all doesn't work with no summary");
+		g_assert(0);
 		return r;
 	}
 
@@ -470,8 +500,9 @@ search_match_all(struct _ESExp *f, int argc, struct _ESExpTerm **argv, CamelFold
 					g_ptr_array_add(r->value.ptrarray, (char *)camel_message_info_uid(search->current));
 			} else {
 				g_warning("invalid syntax, matches require a single bool result");
+				e_sexp_fatal_error(f, _("(match-all) requires a single bool result"));
 			}
-			e_sexp_result_free(r1);
+			e_sexp_result_free(f, r1);
 		} else {
 			g_ptr_array_add(r->value.ptrarray, (char *)camel_message_info_uid(search->current));
 		}
@@ -482,12 +513,12 @@ search_match_all(struct _ESExp *f, int argc, struct _ESExpTerm **argv, CamelFold
 }
 
 static ESExpResult *
-search_header_contains(struct _ESExp *f, int argc, struct _ESExpResult **argv, CamelFolderSearch *search)
+check_header(struct _ESExp *f, int argc, struct _ESExpResult **argv, CamelFolderSearch *search, camel_search_match_t how)
 {
 	ESExpResult *r;
 	int truth = FALSE;
 
-	r(printf("executing header-contains\n"));
+	r(printf("executing check-header\n"));
 
 	/* are we inside a match-all? */
 	if (search->current && argc>1
@@ -512,29 +543,72 @@ search_header_contains(struct _ESExp *f, int argc, struct _ESExpResult **argv, C
 		} else if (!strcasecmp(headername, "cc")) {
 			header = camel_message_info_cc(search->current);
 		} else {
-			g_warning("Performing query on unknown header: %s", headername);
+			e_sexp_resultv_free(f, argc, argv);
+			e_sexp_fatal_error(f, _("Performing query on unknown header: %s"), headername);
 		}
 
 		if (header) {
 			/* performs an OR of all words */
 			for (i=1;i<argc && !truth;i++) {
 				if (argv[i]->type == ESEXP_RES_STRING
-				    && e_utf8_strstrcase (header, argv[i]->value.string)) {
-					r(printf("%s got a match with %s of %s\n", camel_message_info_uid(search->current), header, argv[i]->value.string));
+				    && camel_search_header_match(header, argv[i]->value.string, how)) {
 					truth = TRUE;
-					break;
 				}
 			}
 		}
 	}
 	/* TODO: else, find all matches */
 
-	r = e_sexp_result_new(ESEXP_RES_BOOL);
+	r = e_sexp_result_new(f, ESEXP_RES_BOOL);
 	r->value.bool = truth;
 
 	return r;
 }
 
+static ESExpResult *
+search_header_contains(struct _ESExp *f, int argc, struct _ESExpResult **argv, CamelFolderSearch *search)
+{
+	return check_header(f, argc, argv, search, CAMEL_SEARCH_MATCH_CONTAINS);
+}
+
+static ESExpResult *
+search_header_matches(struct _ESExp *f, int argc, struct _ESExpResult **argv, CamelFolderSearch *search)
+{
+	return check_header(f, argc, argv, search, CAMEL_SEARCH_MATCH_EXACT);
+}
+
+static ESExpResult *
+search_header_starts_with (struct _ESExp *f, int argc, struct _ESExpResult **argv, CamelFolderSearch *search)
+{
+	return check_header(f, argc, argv, search, CAMEL_SEARCH_MATCH_STARTS);
+}
+
+static ESExpResult *
+search_header_ends_with (struct _ESExp *f, int argc, struct _ESExpResult **argv, CamelFolderSearch *search)
+{
+	return check_header(f, argc, argv, search, CAMEL_SEARCH_MATCH_ENDS);
+}
+
+static ESExpResult *
+search_header_exists (struct _ESExp *f, int argc, struct _ESExpResult **argv, CamelFolderSearch *search)
+{
+	ESExpResult *r;
+	
+	r(printf ("executing header-exists\n"));
+	
+	if (search->current) {
+		r = e_sexp_result_new(f, ESEXP_RES_BOOL);
+		if (argc == 1 && argv[0]->type == ESEXP_RES_STRING)
+			r->value.bool = camel_medium_get_header(CAMEL_MEDIUM(search->current), argv[0]->value.string) != NULL;
+		
+	} else {
+		r = e_sexp_result_new(f, ESEXP_RES_ARRAY_PTR);
+		r->value.ptrarray = g_ptr_array_new();
+	}
+	
+	return r;
+}
+
 /* this is just to OR results together */
 struct _glib_sux_donkeys {
 	int count;
@@ -548,103 +622,17 @@ g_lib_sux_htor(char *key, int value, struct _glib_sux_donkeys *fuckup)
 	g_ptr_array_add(fuckup->uids, key);
 }
 
-/* performs a 'slow' content-based match */
-static gboolean
-message_body_contains(CamelDataWrapper *object, regex_t *pattern)
-{
-	CamelDataWrapper *containee;
-	int truth = FALSE;
-	int parts, i;
-
-	containee = camel_medium_get_content_object(CAMEL_MEDIUM(object));
-
-	if (containee == NULL)
-		return FALSE;
-
-	/* TODO: I find it odd that get_part and get_content_object do not
-	   add a reference, probably need fixing for multithreading */
-
-	/* using the object types is more accurate than using the mime/types */
-	if (CAMEL_IS_MULTIPART(containee)) {
-		parts = camel_multipart_get_number(CAMEL_MULTIPART(containee));
-		for (i=0;i<parts && truth==FALSE;i++) {
-			CamelDataWrapper *part = (CamelDataWrapper *)camel_multipart_get_part(CAMEL_MULTIPART(containee), i);
-			if (part) {
-				truth = message_body_contains(part, pattern);
-			}
-		}
-	} else if (CAMEL_IS_MIME_MESSAGE(containee)) {
-		/* for messages we only look at its contents */
-		truth = message_body_contains((CamelDataWrapper *)containee, pattern);
-	} else if (gmime_content_field_is_type(CAMEL_DATA_WRAPPER(containee)->mime_type, "text", "*")) {
-		/* for all other text parts, we look inside, otherwise we dont care */
-		CamelStreamMem *mem = (CamelStreamMem *)camel_stream_mem_new();
-
-		camel_data_wrapper_write_to_stream(containee, (CamelStream *)mem);
-		camel_stream_write((CamelStream *)mem, "", 1);
-		truth = regexec(pattern, mem->buffer->data, 0, NULL, 0) == 0;
-		camel_object_unref((CamelObject *)mem);
-	}
-	return truth;
-}
-
-/* builds the regex into pattern */
 static int
-build_match_regex(regex_t *pattern, int argc, struct _ESExpResult **argv)
-{
-	GString *match = g_string_new("");
-	int c, i, count=0, err;
-	char *word;
-
-	/* build a regex pattern we can use to match the words, we OR them together */
-	if (argc>1)
-		g_string_append_c(match, '(');
-	for (i=0;i<argc;i++) {
-		if (argv[i]->type == ESEXP_RES_STRING) {
-			if (count > 0)
-				g_string_append_c(match, '|');
-			/* escape any special chars (not sure if this list is complete) */
-			word = argv[i]->value.string;
-			while ((c = *word++)) {
-				if (strchr("*\\.()[]^$+", c) != NULL) {
-					g_string_append_c(match, '\\');
-				}
-				g_string_append_c(match, c);
-			}
-			count++;
-		} else {
-			g_warning("Invalid type passed to body-contains match function");
-		}
-	}
-	if (argc>1)
-		g_string_append_c(match, ')');
-	err = regcomp(pattern, match->str, REG_EXTENDED|REG_ICASE|REG_NOSUB);
-	if (err != 0) {
-		char buffer[1024]; /* dont really care if its longer than this ... */
-		
-		regerror(err, pattern, buffer, 1023);
-		g_warning("Internal error with search pattern: %s: %s", match->str, buffer);
-		regfree(pattern);
-	}
-	d(printf("Built regex: '%s'\n", match->str));
-	g_string_free(match, TRUE);
-	return err;
-}
-
-static int
-match_message(CamelFolder *folder, const char *uid, regex_t *pattern)
+match_message(CamelFolder *folder, const char *uid, regex_t *pattern, CamelException *ex)
 {
 	CamelMimeMessage *msg;
 	int truth = FALSE;
-	CamelException *ex;
 
-	ex = camel_exception_new();
 	msg = camel_folder_get_message(folder, uid, ex);
 	if (!camel_exception_is_set(ex) && msg!=NULL) {
-		truth = message_body_contains((CamelDataWrapper *)msg, pattern);
+		truth = camel_search_message_body_contains((CamelDataWrapper *)msg, pattern);
 		camel_object_unref((CamelObject *)msg);
 	}
-	camel_exception_free(ex);
 	return truth;
 }
 
@@ -658,28 +646,29 @@ search_body_contains(struct _ESExp *f, int argc, struct _ESExpResult **argv, Cam
 	if (search->current) {
 		int truth = FALSE;
 
-		r = e_sexp_result_new(ESEXP_RES_BOOL);
 		if (search->body_index) {
 			for (i=0;i<argc && !truth;i++) {
 				if (argv[i]->type == ESEXP_RES_STRING) {
 					truth = ibex_find_name(search->body_index, (char *)camel_message_info_uid(search->current),
 							       argv[i]->value.string);
 				} else {
-					g_warning("Invalid type passed to body-contains match function");
+					e_sexp_resultv_free(f, argc, argv);
+					e_sexp_fatal_error(f, _("Invalid type in body-contains, expecting string"));
 				}
 			}
 		} else if (search->folder) {
 			/* we do a 'slow' direct search */
-			if (build_match_regex(&pattern, argc, argv) == 0) {
-				truth = match_message(search->folder, camel_message_info_uid(search->current), &pattern);
+			if (camel_search_build_match_regex(&pattern, CAMEL_SEARCH_MATCH_REGEX|CAMEL_SEARCH_MATCH_ICASE, argc, argv, search->priv->ex) == 0) {
+				truth = match_message(search->folder, camel_message_info_uid(search->current), &pattern, search->priv->ex);
 				regfree(&pattern);
 			}
 		} else {
 			g_warning("Cannot perform indexed body query with no index or folder set");
 		}
+		r = e_sexp_result_new(f, ESEXP_RES_BOOL);
 		r->value.bool = truth;
 	} else {
-		r = e_sexp_result_new(ESEXP_RES_ARRAY_PTR);
+		r = e_sexp_result_new(f, ESEXP_RES_ARRAY_PTR);
 
 		if (search->body_index) {
 			if (argc==1) {
@@ -699,7 +688,9 @@ search_body_contains(struct _ESExp *f, int argc, struct _ESExpResult **argv, Cam
 						}
 						g_ptr_array_free(pa, FALSE);
 					} else {
-						g_warning("invalid type passed to body-contains");
+						e_sexp_result_free(f, r);
+						e_sexp_resultv_free(f, argc, argv);
+						e_sexp_fatal_error(f, _("Invalid type in body-contains, expecting string"));
 					}
 				}
 				lambdafoo.uids = g_ptr_array_new();
@@ -710,12 +701,12 @@ search_body_contains(struct _ESExp *f, int argc, struct _ESExpResult **argv, Cam
 		} else if (search->folder) {
 			/* do a slow search */
 			r->value.ptrarray = g_ptr_array_new();
-			if (build_match_regex(&pattern, argc, argv) == 0) {
+			if (camel_search_build_match_regex(&pattern, CAMEL_SEARCH_MATCH_REGEX|CAMEL_SEARCH_MATCH_ICASE, argc, argv, search->priv->ex) == 0) {
 				if (search->summary) {
 					for (i=0;i<search->summary->len;i++) {
 						CamelMessageInfo *info = g_ptr_array_index(search->summary, i);
 
-						if (match_message(search->folder, camel_message_info_uid(info), &pattern))
+						if (match_message(search->folder, camel_message_info_uid(info), &pattern, search->priv->ex))
 							g_ptr_array_add(r->value.ptrarray, (char *)camel_message_info_uid(info));
 					}
 				} /* else?  we could always get the summary from the folder, but then
@@ -750,10 +741,10 @@ search_user_flag(struct _ESExp *f, int argc, struct _ESExpResult **argv, CamelFo
 				break;
 			}
 		}
-		r = e_sexp_result_new(ESEXP_RES_BOOL);
+		r = e_sexp_result_new(f, ESEXP_RES_BOOL);
 		r->value.bool = truth;
 	} else {
-		r = e_sexp_result_new(ESEXP_RES_ARRAY_PTR);
+		r = e_sexp_result_new(f, ESEXP_RES_ARRAY_PTR);
 		r->value.ptrarray = g_ptr_array_new();
 	}
 
@@ -773,10 +764,10 @@ search_system_flag (struct _ESExp *f, int argc, struct _ESExpResult **argv, Came
 		if (argc == 1)
 			truth = camel_system_flag_get (search->current->flags, argv[0]->value.string);
 		
-		r = e_sexp_result_new (ESEXP_RES_BOOL);
+		r = e_sexp_result_new(f, ESEXP_RES_BOOL);
 		r->value.bool = truth;
 	} else {
-		r = e_sexp_result_new (ESEXP_RES_ARRAY_PTR);
+		r = e_sexp_result_new(f, ESEXP_RES_ARRAY_PTR);
 		r->value.ptrarray = g_ptr_array_new ();
 	}
 	
@@ -795,10 +786,10 @@ static ESExpResult *search_user_tag(struct _ESExp *f, int argc, struct _ESExpRes
 		if (argc == 1) {
 			value = camel_tag_get(&search->current->user_tags, argv[0]->value.string);
 		}
-		r = e_sexp_result_new(ESEXP_RES_STRING);
+		r = e_sexp_result_new(f, ESEXP_RES_STRING);
 		r->value.string = g_strdup(value?value:"");
 	} else {
-		r = e_sexp_result_new(ESEXP_RES_ARRAY_PTR);
+		r = e_sexp_result_new(f, ESEXP_RES_ARRAY_PTR);
 		r->value.ptrarray = g_ptr_array_new();
 	}
 
@@ -814,11 +805,11 @@ search_get_sent_date(struct _ESExp *f, int argc, struct _ESExpResult **argv, Cam
 
 	/* are we inside a match-all? */
 	if (s->current) {
-		r = e_sexp_result_new (ESEXP_RES_INT);
+		r = e_sexp_result_new(f, ESEXP_RES_INT);
 
 		r->value.number = s->current->date_sent;
 	} else {
-		r = e_sexp_result_new (ESEXP_RES_ARRAY_PTR);
+		r = e_sexp_result_new(f, ESEXP_RES_ARRAY_PTR);
 		r->value.ptrarray = g_ptr_array_new ();
 	}
 
@@ -834,11 +825,11 @@ search_get_received_date(struct _ESExp *f, int argc, struct _ESExpResult **argv,
 
 	/* are we inside a match-all? */
 	if (s->current) {
-		r = e_sexp_result_new (ESEXP_RES_INT);
+		r = e_sexp_result_new(f, ESEXP_RES_INT);
 
 		r->value.number = s->current->date_received;
 	} else {
-		r = e_sexp_result_new (ESEXP_RES_ARRAY_PTR);
+		r = e_sexp_result_new(f, ESEXP_RES_ARRAY_PTR);
 		r->value.ptrarray = g_ptr_array_new ();
 	}
 
@@ -852,7 +843,7 @@ search_get_current_date(struct _ESExp *f, int argc, struct _ESExpResult **argv,
 
 	r(printf("executing get-current-date\n"));
 
-	r = e_sexp_result_new (ESEXP_RES_INT);
+	r = e_sexp_result_new(f, ESEXP_RES_INT);
 	r->value.number = time (NULL);
 	return r;
 }
