@@ -43,6 +43,21 @@
 #include "e-util/md5-utils.h"
 #include "e-util/e-memory.h"
 
+#include "camel-private.h"
+
+#ifdef ENABLE_THREADS
+#include <pthread.h>
+
+static pthread_mutex_t info_lock = PTHREAD_MUTEX_INITIALIZER;
+
+/* this lock is ONLY for the standalone messageinfo stuff */
+#define GLOBAL_INFO_LOCK(i) pthread_mutex_lock(&info_lock)
+#define GLOBAL_INFO_UNLOCK(i) pthread_mutex_unlock(&info_lock)
+#else
+#define GLOBAL_INFO_LOCK(i) 
+#define GLOBAL_INFO_UNLOCK(i) 
+#endif
+
 /* this should probably be conditional on it existing */
 #define USE_BSEARCH
 
@@ -55,17 +70,6 @@ extern int strdup_count, malloc_count, free_count;
 
 #define CAMEL_FOLDER_SUMMARY_VERSION (11)
 
-struct _CamelFolderSummaryPrivate {
-	GHashTable *filter_charset;	/* CamelMimeFilterCharset's indexed by source charset */
-
-	CamelMimeFilterIndex *filter_index;
-	CamelMimeFilterBasic *filter_64;
-	CamelMimeFilterBasic *filter_qp;
-	CamelMimeFilterSave *filter_save;
-
-	ibex *index;
-};
-
 #define _PRIVATE(o) (((CamelFolderSummary *)(o))->priv)
 
 /* trivial lists, just because ... */
@@ -155,6 +159,14 @@ camel_folder_summary_init (CamelFolderSummary *s)
 
 	s->messages = g_ptr_array_new();
 	s->messages_uid = g_hash_table_new(g_str_hash, g_str_equal);
+
+#ifdef ENABLE_THREADS
+	p->summary_lock = g_mutex_new();
+	p->io_lock = g_mutex_new();
+	p->filter_lock = g_mutex_new();
+	p->alloc_lock = g_mutex_new();
+	p->ref_lock = g_mutex_new();
+#endif
 }
 
 static void free_o_name(void *key, void *value, void *data)
@@ -194,6 +206,14 @@ camel_folder_summary_finalize (CamelObject *obj)
 	if (p->filter_save)
 		camel_object_unref ((CamelObject *)p->filter_save);
 
+#ifdef ENABLE_THREADS
+	g_mutex_free(p->summary_lock);
+	g_mutex_free(p->io_lock);
+	g_mutex_free(p->filter_lock);
+	g_mutex_free(p->alloc_lock);
+	g_mutex_free(p->ref_lock);
+#endif
+
 	g_free(p);
 }
 
@@ -225,8 +245,7 @@ camel_folder_summary_get_type (void)
 CamelFolderSummary *
 camel_folder_summary_new (void)
 {
-	CamelFolderSummary *new = CAMEL_FOLDER_SUMMARY ( camel_object_new (camel_folder_summary_get_type ()));
-	return new;
+	CamelFolderSummary *new = CAMEL_FOLDER_SUMMARY ( camel_object_new (camel_folder_summary_get_type ()));	return new;
 }
 
 
@@ -239,8 +258,12 @@ camel_folder_summary_new (void)
  **/
 void camel_folder_summary_set_filename(CamelFolderSummary *s, const char *name)
 {
+	CAMEL_SUMMARY_LOCK(s, summary_lock);
+
 	g_free(s->summary_path);
 	s->summary_path = g_strdup(name);
+
+	CAMEL_SUMMARY_UNLOCK(s, summary_lock);
 }
 
 /**
@@ -294,16 +317,83 @@ camel_folder_summary_count(CamelFolderSummary *s)
  * @i: 
  * 
  * Retrieve a summary item by index number.
+ *
+ * A referenced to the summary item is returned, which may be
+ * ref'd or free'd as appropriate.
  * 
  * Return value: The summary item, or NULL if the index @i is out
  * of range.
+ * It must be freed using camel_folder_summary_info_free().
  **/
 CamelMessageInfo *
 camel_folder_summary_index(CamelFolderSummary *s, int i)
 {
+	CamelMessageInfo *info = NULL;
+
+	CAMEL_SUMMARY_LOCK(s, ref_lock);
+	CAMEL_SUMMARY_LOCK(s, summary_lock);
+
 	if (i<s->messages->len)
-		return g_ptr_array_index(s->messages, i);
-	return NULL;
+		info = g_ptr_array_index(s->messages, i);
+
+	CAMEL_SUMMARY_UNLOCK(s, summary_lock);
+
+	if (info)
+		info->refcount++;
+
+	CAMEL_SUMMARY_UNLOCK(s, ref_lock);
+
+	return info;
+}
+
+/**
+ * camel_folder_summary_index:
+ * @s: 
+ * @i: 
+ * 
+ * Obtain a copy of the summary array.  This is done atomically,
+ * so cannot contain empty entries.
+ *
+ * It must be freed using camel_folder_summary_array_free().
+ **/
+GPtrArray *
+camel_folder_summary_array(CamelFolderSummary *s)
+{
+	CamelMessageInfo *info;
+	GPtrArray *res = g_ptr_array_new();
+	int i;
+	
+	CAMEL_SUMMARY_LOCK(s, ref_lock);
+	CAMEL_SUMMARY_LOCK(s, summary_lock);
+
+	g_ptr_array_set_size(res, s->messages->len);
+	for (i=0;i<s->messages->len;i++) {
+		info = res->pdata[i] = g_ptr_array_index(s->messages, i);
+		info->refcount++;
+	}
+
+	CAMEL_SUMMARY_UNLOCK(s, summary_lock);
+	CAMEL_SUMMARY_UNLOCK(s, ref_lock);
+
+	return res;
+}
+
+/**
+ * camel_folder_summary_array_free:
+ * @s: 
+ * @array: 
+ * 
+ * Free the folder summary array.
+ **/
+void
+camel_folder_summary_array_free(CamelFolderSummary *s, GPtrArray *array)
+{
+	int i;
+
+	for (i=0;i<array->len;i++)
+		camel_folder_summary_info_free(s, array->pdata[i]);
+
+	g_ptr_array_free(array, TRUE);
 }
 
 /**
@@ -312,14 +402,32 @@ camel_folder_summary_index(CamelFolderSummary *s, int i)
  * @uid: 
  * 
  * Retrieve a summary item by uid.
+ *
+ * A referenced to the summary item is returned, which may be
+ * ref'd or free'd as appropriate.
  * 
  * Return value: The summary item, or NULL if the uid @uid
  * is not available.
+ * It must be freed using camel_folder_summary_info_free().
  **/
 CamelMessageInfo *
 camel_folder_summary_uid(CamelFolderSummary *s, const char *uid)
 {
-	return g_hash_table_lookup(s->messages_uid, uid);
+	CamelMessageInfo *info;
+
+	CAMEL_SUMMARY_LOCK(s, ref_lock);
+	CAMEL_SUMMARY_LOCK(s, summary_lock);
+
+	info = g_hash_table_lookup(s->messages_uid, uid);
+
+	CAMEL_SUMMARY_UNLOCK(s, summary_lock);
+
+	if (info)
+		info->refcount++;
+
+	CAMEL_SUMMARY_UNLOCK(s, ref_lock);
+
+	return info;
 }
 
 /**
@@ -333,7 +441,14 @@ camel_folder_summary_uid(CamelFolderSummary *s, const char *uid)
  **/
 guint32 camel_folder_summary_next_uid(CamelFolderSummary *s)
 {
-	guint32 uid = s->nextuid++;
+	guint32 uid;
+
+
+	CAMEL_SUMMARY_LOCK(s, summary_lock);
+
+	uid = s->nextuid++;
+
+	CAMEL_SUMMARY_UNLOCK(s, summary_lock);
 
 	/* FIXME: sync this to disk */
 /*	summary_header_save(s);*/
@@ -352,7 +467,11 @@ guint32 camel_folder_summary_next_uid(CamelFolderSummary *s)
 void camel_folder_summary_set_uid(CamelFolderSummary *s, guint32 uid)
 {
 	/* TODO: sync to disk? */
+	CAMEL_SUMMARY_LOCK(s, summary_lock);
+
 	s->nextuid = MAX(s->nextuid, uid);
+
+	CAMEL_SUMMARY_UNLOCK(s, summary_lock);
 }
 
 /**
@@ -379,7 +498,14 @@ perform_content_info_load(CamelFolderSummary *s, FILE *in)
 	CamelMessageContentInfo *ci, *part;
 
 	ci = ((CamelFolderSummaryClass *)(CAMEL_OBJECT_GET_CLASS(s)))->content_info_load(s, in);
-	camel_folder_summary_decode_uint32(in, &count);
+	if (ci == NULL)
+		return NULL;
+
+	if (camel_folder_summary_decode_uint32(in, &count) == -1 || count > 500) {
+		camel_folder_summary_content_info_free(s, ci);
+		return NULL;
+	}
+
 	for (i=0;i<count;i++) {
 		part = perform_content_info_load(s, in);
 		if (part) {
@@ -387,6 +513,8 @@ perform_content_info_load(CamelFolderSummary *s, FILE *in)
 			part->parent = ci;
 		} else {
 			g_warning("Summary file format messed up?");
+			camel_folder_summary_content_info_free(s, ci);
+			return NULL;
 		}
 	}
 	return ci;
@@ -402,26 +530,32 @@ camel_folder_summary_load(CamelFolderSummary *s)
 	g_assert(s->summary_path);
 
 	in = fopen(s->summary_path, "r");
-	if ( in == NULL ) {
+	if (in == NULL)
 		return -1;
-	}
 
-	if ( ((CamelFolderSummaryClass *)(CAMEL_OBJECT_GET_CLASS(s)))->summary_header_load(s, in) == -1) {
-		fclose(in);
-		return -1;
-	}
+	CAMEL_SUMMARY_LOCK(s, io_lock);
+	if ( ((CamelFolderSummaryClass *)(CAMEL_OBJECT_GET_CLASS(s)))->summary_header_load(s, in) == -1)
+		goto error;
 
 	/* now read in each message ... */
-	/* FIXME: check returns */
 	for (i=0;i<s->saved_count;i++) {
 		mi = ((CamelFolderSummaryClass *)(CAMEL_OBJECT_GET_CLASS(s)))->message_info_load(s, in);
 
+		if (mi == NULL)
+			goto error;
+
 		if (s->build_content) {
 			mi->content = perform_content_info_load(s, in);
+			if (mi->content == NULL) {
+				camel_folder_summary_info_free(s, mi);
+				goto error;
+			}
 		}
 
 		camel_folder_summary_add(s, mi);
 	}
+
+	CAMEL_SUMMARY_UNLOCK(s, io_lock);
 	
 	if (fclose(in) == -1)
 		return -1;
@@ -429,6 +563,14 @@ camel_folder_summary_load(CamelFolderSummary *s)
 	s->flags &= ~CAMEL_SUMMARY_DIRTY;
 
 	return 0;
+
+error:
+	g_warning("Cannot load summary file: %s", strerror(ferror(in)));
+	CAMEL_SUMMARY_UNLOCK(s, io_lock);
+	fclose(in);
+	s->flags |= ~CAMEL_SUMMARY_DIRTY;
+
+	return -1;
 }
 
 /* saves the content descriptions, recursively */
@@ -481,22 +623,29 @@ camel_folder_summary_save(CamelFolderSummary *s)
 
 	io(printf("saving header\n"));
 
+	CAMEL_SUMMARY_LOCK(s, io_lock);
+
 	if ( ((CamelFolderSummaryClass *)(CAMEL_OBJECT_GET_CLASS(s)))->summary_header_save(s, out) == -1) {
 		fclose(out);
+		CAMEL_SUMMARY_UNLOCK(s, io_lock);
 		return -1;
 	}
 
 	/* now write out each message ... */
 	/* FIXME: check returns */
-	count = camel_folder_summary_count(s);
+
+	count = s->messages->len;
 	for (i=0;i<count;i++) {
-		mi = camel_folder_summary_index(s, i);
+		mi = s->messages->pdata[i];
 		((CamelFolderSummaryClass *)(CAMEL_OBJECT_GET_CLASS(s)))->message_info_save(s, out, mi);
 
 		if (s->build_content) {
 			perform_content_info_save(s, out, mi->content);
 		}
 	}
+
+	CAMEL_SUMMARY_UNLOCK(s, io_lock);
+
 	if (fclose(out) == -1)
 		return -1;
 
@@ -515,12 +664,18 @@ summary_assign_uid(CamelFolderSummary *s, CamelMessageInfo *info)
 		uid = camel_message_info_uid(info);
 	}
 
+	CAMEL_SUMMARY_LOCK(s, summary_lock);
+
 	while (g_hash_table_lookup(s->messages_uid, uid)) {
 		g_warning("Trying to insert message with clashing uid (%s).  new uid re-assigned", camel_message_info_uid(info));
+		CAMEL_SUMMARY_UNLOCK(s, summary_lock);
 		camel_message_info_set_uid(info, camel_folder_summary_next_uid_string(s));
 		uid = camel_message_info_uid(info);
 		info->flags |= CAMEL_MESSAGE_FOLDER_FLAGGED;
+		CAMEL_SUMMARY_LOCK(s, summary_lock);
 	}
+
+	CAMEL_SUMMARY_UNLOCK(s, summary_lock);
 }
 
 /**
@@ -542,6 +697,8 @@ void camel_folder_summary_add(CamelFolderSummary *s, CamelMessageInfo *info)
 
 	summary_assign_uid(s, info);
 
+	CAMEL_SUMMARY_LOCK(s, summary_lock);
+
 #ifdef DOESTRV
 	/* this is vitally important, and also if this is ever modified, then
 	   the hash table needs to be resynced */
@@ -551,6 +708,8 @@ void camel_folder_summary_add(CamelFolderSummary *s, CamelMessageInfo *info)
 	g_ptr_array_add(s->messages, info);
 	g_hash_table_insert(s->messages_uid, (char *)camel_message_info_uid(info), info);
 	s->flags |= CAMEL_SUMMARY_DIRTY;
+
+	CAMEL_SUMMARY_UNLOCK(s, summary_lock);
 }
 
 /**
@@ -668,6 +827,8 @@ CamelMessageInfo *camel_folder_summary_info_new_from_parser(CamelFolderSummary *
 		 * know if we are going to store this in the summary, but no matter */
 		summary_assign_uid(s, info);
 
+		CAMEL_SUMMARY_LOCK(s, filter_lock);
+
 		if (p->index) {
 			if (p->filter_index == NULL)
 				p->filter_index = camel_mime_filter_index_new_ibex(p->index);
@@ -677,6 +838,9 @@ CamelMessageInfo *camel_folder_summary_info_new_from_parser(CamelFolderSummary *
 
 		/* always scan the content info, even if we dont save it */
 		info->content = summary_build_content_info(s, info, mp);
+
+		CAMEL_SUMMARY_UNLOCK(s, filter_lock);
+
 		info->size = camel_mime_parser_tell(mp) - start;
 	}
 	return info;
@@ -712,8 +876,15 @@ CamelMessageInfo *camel_folder_summary_info_new_from_message(CamelFolderSummary
 	return info;
 }
 
-static void
-perform_content_info_free(CamelFolderSummary *s, CamelMessageContentInfo *ci)
+/**
+ * camel_folder_summary_content_info_free:
+ * @s: 
+ * @ci: 
+ * 
+ * Free the content info @ci, and all associated memory.
+ **/
+void
+camel_folder_summary_content_info_free(CamelFolderSummary *s, CamelMessageContentInfo *ci)
 {
 	CamelMessageContentInfo *pw, *pn;
 
@@ -721,7 +892,7 @@ perform_content_info_free(CamelFolderSummary *s, CamelMessageContentInfo *ci)
 	((CamelFolderSummaryClass *)(CAMEL_OBJECT_GET_CLASS(s)))->content_info_free(s, ci);
 	while (pw) {
 		pn = pw->next;
-		perform_content_info_free(s, pw);
+		camel_folder_summary_content_info_free(s, pw);
 		pw = pn;
 	}
 }
@@ -731,7 +902,7 @@ perform_content_info_free(CamelFolderSummary *s, CamelMessageContentInfo *ci)
  * @s: 
  * @mi: 
  * 
- * Free the message info @mi, and all associated memory.
+ * Unref and potentially free the message info @mi, and all associated memory.
  **/
 void camel_folder_summary_info_free(CamelFolderSummary *s, CamelMessageInfo *mi)
 {
@@ -740,14 +911,44 @@ void camel_folder_summary_info_free(CamelFolderSummary *s, CamelMessageInfo *mi)
 	g_assert(mi);
 	g_assert(s);
 
+	CAMEL_SUMMARY_LOCK(s, ref_lock);
+
+	g_assert(mi->refcount >= 1);
+
+	mi->refcount--;
+	if (mi->refcount > 0) {
+		CAMEL_SUMMARY_UNLOCK(s, ref_lock);
+		return;
+	}
+
+	CAMEL_SUMMARY_UNLOCK(s, ref_lock);
+
 	ci = mi->content;
 
 	((CamelFolderSummaryClass *)(CAMEL_OBJECT_GET_CLASS(s)))->message_info_free(s, mi);		
 	if (s->build_content && ci) {
-		perform_content_info_free(s, ci);
+		camel_folder_summary_content_info_free(s, ci);
 	}
 }
 
+/**
+ * camel_folder_summary_info_ref:
+ * @s: 
+ * @mi: 
+ * 
+ * Add an extra reference to @mi.
+ **/
+void camel_folder_summary_info_ref(CamelFolderSummary *s, CamelMessageInfo *mi)
+{
+	g_assert(mi);
+	g_assert(s);
+
+	CAMEL_SUMMARY_LOCK(s, ref_lock);
+	g_assert(mi->refcount >= 1);
+	mi->refcount++;
+	CAMEL_SUMMARY_UNLOCK(s, ref_lock);
+}
+
 /**
  * camel_folder_summary_touch:
  * @s: 
@@ -757,7 +958,9 @@ void camel_folder_summary_info_free(CamelFolderSummary *s, CamelMessageInfo *mi)
 void
 camel_folder_summary_touch(CamelFolderSummary *s)
 {
+	CAMEL_SUMMARY_LOCK(s, summary_lock);
 	s->flags |= CAMEL_SUMMARY_DIRTY;
+	CAMEL_SUMMARY_UNLOCK(s, summary_lock);
 }
 
 /**
@@ -771,16 +974,20 @@ camel_folder_summary_clear(CamelFolderSummary *s)
 {
 	int i;
 
-	if (camel_folder_summary_count(s) == 0)
+	CAMEL_SUMMARY_LOCK(s, summary_lock);
+	if (camel_folder_summary_count(s) == 0) {
+		CAMEL_SUMMARY_UNLOCK(s, summary_lock);
 		return;
+	}
 
-	for (i=0;i<camel_folder_summary_count(s);i++)
-		camel_folder_summary_info_free(s, camel_folder_summary_index(s, i));
+	for (i=0;i<s->messages->len;i++)
+		camel_folder_summary_info_free(s, s->messages->pdata[i]);
 
 	g_ptr_array_set_size(s->messages, 0);
 	g_hash_table_destroy(s->messages_uid);
 	s->messages_uid = g_hash_table_new(g_str_hash, g_str_equal);
 	s->flags |= CAMEL_SUMMARY_DIRTY;
+	CAMEL_SUMMARY_UNLOCK(s, summary_lock);
 }
 
 /**
@@ -792,10 +999,13 @@ camel_folder_summary_clear(CamelFolderSummary *s)
  **/
 void camel_folder_summary_remove(CamelFolderSummary *s, CamelMessageInfo *info)
 {
+	CAMEL_SUMMARY_LOCK(s, summary_lock);
 	g_hash_table_remove(s->messages_uid, camel_message_info_uid(info));
 	g_ptr_array_remove(s->messages, info);
-	camel_folder_summary_info_free(s, info);
 	s->flags |= CAMEL_SUMMARY_DIRTY;
+	CAMEL_SUMMARY_UNLOCK(s, summary_lock);
+
+	camel_folder_summary_info_free(s, info);
 }
 
 /**
@@ -810,10 +1020,19 @@ void camel_folder_summary_remove_uid(CamelFolderSummary *s, const char *uid)
         CamelMessageInfo *oldinfo;
         char *olduid;
 
+	CAMEL_SUMMARY_LOCK(s, ref_lock);
+	CAMEL_SUMMARY_LOCK(s, summary_lock);
         if (g_hash_table_lookup_extended(s->messages_uid, uid, (void *)&olduid, (void *)&oldinfo)) {
+		/* make sure it doesn't vanish while we're removing it */
+		oldinfo->refcount++;
+		CAMEL_SUMMARY_UNLOCK(s, summary_lock);
+		CAMEL_SUMMARY_UNLOCK(s, ref_lock);
 		camel_folder_summary_remove(s, oldinfo);
-		g_free(olduid);
-        }
+		camel_folder_summary_info_free(s, oldinfo);
+        } else {
+		CAMEL_SUMMARY_UNLOCK(s, summary_lock);
+		CAMEL_SUMMARY_UNLOCK(s, ref_lock);
+	}
 }
 
 /**
@@ -825,11 +1044,20 @@ void camel_folder_summary_remove_uid(CamelFolderSummary *s, const char *uid)
  **/
 void camel_folder_summary_remove_index(CamelFolderSummary *s, int index)
 {
-        CamelMessageInfo *oldinfo;
-
-	oldinfo = camel_folder_summary_index (s, index);
-	if (oldinfo)
-		camel_folder_summary_remove(s, oldinfo);
+	CAMEL_SUMMARY_LOCK(s, ref_lock);
+	CAMEL_SUMMARY_LOCK(s, summary_lock);
+	if (index < s->messages->len) {
+		CamelMessageInfo *info = s->messages->pdata[index];
+		/* make sure it doesn't vanish while we're not looking */
+		info->refcount++;
+		CAMEL_SUMMARY_UNLOCK(s, summary_lock);
+		CAMEL_SUMMARY_UNLOCK(s, ref_lock);
+		camel_folder_summary_remove(s, info);
+		camel_folder_summary_info_free(s, info);
+	} else {
+		CAMEL_SUMMARY_UNLOCK(s, summary_lock);
+		CAMEL_SUMMARY_UNLOCK(s, ref_lock);
+	}
 }
 
 /**
@@ -1223,7 +1451,7 @@ camel_folder_summary_encode_string(FILE *out, const char *str)
 int
 camel_folder_summary_decode_string(FILE *in, char **str)
 {
-	gint32 len;
+	guint32 len;
 	register char *ret;
 
 	io(printf("Decode string ...\n", str));
@@ -1234,7 +1462,7 @@ camel_folder_summary_decode_string(FILE *in, char **str)
 	}
 
 	len--;
-	if (len < 0) {
+	if (len > 65536) {
 		*str = NULL;
 		io(printf("String = '%s'\n", *str));
 		return -1;
@@ -1430,24 +1658,40 @@ camel_folder_summary_info_new(CamelFolderSummary *s)
 {
 	CamelMessageInfo *mi;
 
+	CAMEL_SUMMARY_LOCK(s, alloc_lock);
 	if (s->message_info_chunks == NULL)
 		s->message_info_chunks = e_memchunk_new(32, s->message_info_size);
 	mi = e_memchunk_alloc(s->message_info_chunks);
+	CAMEL_SUMMARY_UNLOCK(s, alloc_lock);
+
 	memset(mi, 0, s->message_info_size);
 #ifdef DOESTRV
 	mi->strings = e_strv_new(s->message_info_strings);
 #endif
+	mi->refcount = 1;
 	return mi;
 }
 
-static CamelMessageContentInfo *
-content_info_alloc(CamelFolderSummary *s)
+/**
+ * camel_folder_summary_content_info_new:
+ * @s: 
+ * 
+ * Allocate a new camel message content info, suitable for adding
+ * to this summary.
+ * 
+ * Return value: 
+ **/
+CamelMessageContentInfo *
+camel_folder_summary_content_info_new(CamelFolderSummary *s)
 {
 	CamelMessageContentInfo *ci;
 
+	CAMEL_SUMMARY_LOCK(s, alloc_lock);
 	if (s->content_info_chunks == NULL)
 		s->content_info_chunks = e_memchunk_new(32, s->content_info_size);
 	ci = e_memchunk_alloc(s->content_info_chunks);
+	CAMEL_SUMMARY_UNLOCK(s, alloc_lock);
+
 	memset(ci, 0, s->content_info_size);
 	return ci;
 }
@@ -1565,7 +1809,9 @@ message_info_load(CamelFolderSummary *s, FILE *in)
 	camel_folder_summary_decode_fixed_int32(in, &mi->message_id.id.part.hi);
 	camel_folder_summary_decode_fixed_int32(in, &mi->message_id.id.part.lo);
 
-	camel_folder_summary_decode_uint32(in, &count);
+	if (camel_folder_summary_decode_uint32(in, &count) == -1 || count > 500)
+		goto error;
+
 	if (count > 0) {
 		mi->references = g_malloc(sizeof(*mi->references) + ((count-1) * sizeof(mi->references->references[0])));
 		mi->references->size = count;
@@ -1575,7 +1821,9 @@ message_info_load(CamelFolderSummary *s, FILE *in)
 		}
 	}
 
-	camel_folder_summary_decode_uint32(in, &count);
+	if (camel_folder_summary_decode_uint32(in, &count) == -1 || count > 500)
+		goto error;
+
 	for (i=0;i<count;i++) {
 		char *name;
 		camel_folder_summary_decode_string(in, &name);
@@ -1583,7 +1831,9 @@ message_info_load(CamelFolderSummary *s, FILE *in)
 		g_free(name);
 	}
 
-	camel_folder_summary_decode_uint32(in, &count);
+	if (camel_folder_summary_decode_uint32(in, &count) == -1 || count > 500)
+		goto error;
+
 	for (i=0;i<count;i++) {
 		char *name, *value;
 		camel_folder_summary_decode_string(in, &name);
@@ -1593,7 +1843,13 @@ message_info_load(CamelFolderSummary *s, FILE *in)
 		g_free(value);
 	}
 
-	return mi;
+	if (!ferror(in))
+		return mi;
+
+error:
+	camel_folder_summary_info_free(s, mi);
+
+	return NULL;
 }
 
 static int
@@ -1672,7 +1928,7 @@ content_info_new(CamelFolderSummary *s, struct _header_raw *h)
 {
 	CamelMessageContentInfo *ci;
 
-	ci = content_info_alloc(s);
+	ci = camel_folder_summary_content_info_new(s);
 
 	ci->id = header_msgid_decode(header_raw_find(&h, "content-id", NULL));
 	ci->description = header_decode_string(header_raw_find(&h, "content-description", NULL));
@@ -1691,14 +1947,16 @@ content_info_load(CamelFolderSummary *s, FILE *in)
 
 	io(printf("Loading content info\n"));
 
-	ci = content_info_alloc(s);
+	ci = camel_folder_summary_content_info_new(s);
 	
 	camel_folder_summary_decode_token(in, &type);
 	camel_folder_summary_decode_token(in, &subtype);
 	ct = header_content_type_new(type, subtype);
 	g_free(type);		/* can this be removed? */
 	g_free(subtype);
-	camel_folder_summary_decode_uint32(in, &count);
+	if (camel_folder_summary_decode_uint32(in, &count) == -1 || count > 500)
+		goto error;
+	    
 	for (i=0;i<count;i++) {
 		char *name, *value;
 		camel_folder_summary_decode_token(in, &name);
@@ -1714,8 +1972,16 @@ content_info_load(CamelFolderSummary *s, FILE *in)
 	camel_folder_summary_decode_token(in, &ci->description);
 	camel_folder_summary_decode_token(in, &ci->encoding);
 
+	camel_folder_summary_decode_uint32(in, &ci->size);
+
 	ci->childs = NULL;
-	return ci;
+
+	if (!ferror(in))
+		return ci;
+
+error:
+	camel_folder_summary_content_info_free(s, ci);
+	return NULL;
 }
 
 static int
@@ -1744,7 +2010,8 @@ content_info_save(CamelFolderSummary *s, FILE *out, CamelMessageContentInfo *ci)
 	}
 	camel_folder_summary_encode_token(out, ci->id);
 	camel_folder_summary_encode_token(out, ci->description);
-	return camel_folder_summary_encode_token(out, ci->encoding);
+	camel_folder_summary_encode_token(out, ci->encoding);
+	return camel_folder_summary_encode_uint32(out, ci->size);
 }
 
 static void
@@ -1769,6 +2036,7 @@ next_uid_string(CamelFolderSummary *s)
   and any indexing and what not is performed
 */
 
+/* must have filter_lock before calling this function */
 static CamelMessageContentInfo *
 summary_build_content_info(CamelFolderSummary *s, CamelMessageInfo *msginfo, CamelMimeParser *mp)
 {
@@ -1892,6 +2160,7 @@ summary_build_content_info(CamelFolderSummary *s, CamelMessageInfo *msginfo, Cam
 }
 
 /* build the content-info, from a message */
+/* this needs no lock, as we copy all data, and ibex is threadsafe */
 static CamelMessageContentInfo *
 summary_build_content_info_message(CamelFolderSummary *s, CamelMessageInfo *msginfo, CamelMimePart *object)
 {
@@ -1912,10 +2181,10 @@ summary_build_content_info_message(CamelFolderSummary *s, CamelMessageInfo *msgi
 	   add a reference, probably need fixing for multithreading */
 
 	/* check for attachments */
-	if (gmime_content_field_is_type(CAMEL_DATA_WRAPPER(containee)->mime_type, "multipart", "*")) {
-		if (gmime_content_field_is_type(CAMEL_DATA_WRAPPER(containee)->mime_type, "multipart", "mixed"))
+	if (header_content_type_is(CAMEL_DATA_WRAPPER(containee)->mime_type, "multipart", "*")) {
+		if (header_content_type_is(CAMEL_DATA_WRAPPER(containee)->mime_type, "multipart", "mixed"))
 			msginfo->flags |= CAMEL_MESSAGE_ATTACHMENTS;
-	} else if (!gmime_content_field_is_type(CAMEL_DATA_WRAPPER(containee)->mime_type, "text", "*"))
+	} else if (!header_content_type_is(CAMEL_DATA_WRAPPER(containee)->mime_type, "text", "*"))
 		msginfo->flags |= CAMEL_MESSAGE_ATTACHMENTS;
 
 	/* using the object types is more accurate than using the mime/types */
@@ -1938,7 +2207,7 @@ summary_build_content_info_message(CamelFolderSummary *s, CamelMessageInfo *msgi
 			my_list_append((struct _node **)&info->childs, (struct _node *)child);
 		}
 	} else if (p->index
-		   && gmime_content_field_is_type(CAMEL_DATA_WRAPPER(containee)->mime_type, "text", "*")) {
+		   && header_content_type_is(CAMEL_DATA_WRAPPER(containee)->mime_type, "text", "*")) {
 		/* index all text parts if we're indexing */
 		CamelStreamMem *mem = (CamelStreamMem *)camel_stream_mem_new();
 
@@ -2212,14 +2481,29 @@ camel_message_info_new (void)
 {
 	CamelMessageInfo *info;
 	
-	info = g_new0 (CamelMessageInfo, 1);
+	info = g_malloc0(sizeof(*info));
 #ifdef DOESTRV
 	info->strings = e_strv_new (CAMEL_MESSAGE_INFO_LAST);
 #endif
-	
+	info->refcount = 1;
+
 	return info;
 }
 
+/**
+ * camel_message_info_ref:
+ * @info: 
+ * 
+ * Reference an info.
+ *
+ * NOTE: This interface is not MT-SAFE, like the others.
+ **/
+void camel_message_info_ref(CamelMessageInfo *info)
+{
+	GLOBAL_INFO_LOCK(info);
+	info->refcount++;
+	GLOBAL_INFO_UNLOCK(info);
+}
 
 /**
  * camel_message_info_new_from_header:
@@ -2238,20 +2522,13 @@ camel_message_info_new_from_header (struct _header_raw *header)
 	to = camel_folder_summary_format_address (header, "to");
 	cc = camel_folder_summary_format_address (header, "cc");
 	
-	info = g_new0 (CamelMessageInfo, 1);
-#ifdef DOESTRV
-	info->strings = e_strv_new (CAMEL_MESSAGE_INFO_LAST);
+	info = camel_message_info_new();
+
 	camel_message_info_set_subject (info, subject);
 	camel_message_info_set_from (info, from);
 	camel_message_info_set_to (info, to);
 	camel_message_info_set_cc (info, cc);
-#else
-	info->subject = subject;
-	info->from = from;
-	info->to = to;
-	info->cc = cc;
-#endif
-	
+
 	return info;
 }
 
@@ -2276,6 +2553,7 @@ camel_message_info_dup_to(const CamelMessageInfo *from, CamelMessageInfo *to)
 	to->size = from->size;
 	to->date_sent = from->date_sent;
 	to->date_received = from->date_received;
+	to->refcount = 1;
 
 	/* Copy strings */
 #ifdef DOESTRV
@@ -2325,15 +2603,27 @@ camel_message_info_dup_to(const CamelMessageInfo *from, CamelMessageInfo *to)
  * camel_message_info_free:
  * @mi: the message info
  *
- * Frees a CamelMessageInfo and its contents.
+ * Unref's and potentially frees a CamelMessageInfo and its contents.
  *
  * Can only be used to free CamelMessageInfo's created with
  * camel_message_info_dup_to.
  *
+ * NOTE: This interface is not MT-SAFE, like the others.
+ *
  **/
 void
 camel_message_info_free(CamelMessageInfo *mi)
 {
+	g_return_if_fail(mi != NULL);
+
+	GLOBAL_INFO_LOCK(info);
+	mi->refcount--;
+	if (mi->refcount > 0) {
+		GLOBAL_INFO_UNLOCK(info);
+		return;
+	}
+	GLOBAL_INFO_UNLOCK(info);
+
 #ifdef DOESTRV
 	e_strv_destroy(mi->strings);
 #else
@@ -2381,10 +2671,10 @@ content_info_dump(CamelMessageContentInfo *ci, int depth)
 		return;
 	}
 
-	printf("%sconent-type: %s/%s\n", p, ci->type->type, ci->type->subtype);
-	printf("%sontent-transfer-encoding: %s\n", p, ci->encoding);
+	printf("%scontent-type: %s/%s\n", p, ci->type->type, ci->type->subtype);
+	printf("%scontent-transfer-encoding: %s\n", p, ci->encoding);
 	printf("%scontent-description: %s\n", p, ci->description);
-	printf("%sbytes: %d %d %d\n", p, (int)ci->pos, (int)ci->bodypos, (int)ci->endpos);
+	printf("%ssize: %lu\n", p, (unsigned long)ci->size);
 	ci = ci->childs;
 	while (ci) {
 		content_info_dump(ci, depth+1);
@@ -2392,7 +2682,7 @@ content_info_dump(CamelMessageContentInfo *ci, int depth)
 	}
 }
 
-static void
+void
 message_info_dump(CamelMessageInfo *mi)
 {
 	if (mi == NULL) {
@@ -2401,10 +2691,10 @@ message_info_dump(CamelMessageInfo *mi)
 	}
 
 	printf("Subject: %s\n", camel_message_info_subject(mi));
-	printf("To: %s\n", camel_message_info_to(to));
-	printf("Cc: %s\n", camel_message_info_cc(cc));
-	printf("From: %s\n", camel_message_info_from(from));
-	printf("UID: %s\n", camel_message_info_uid(uid));
+	printf("To: %s\n", camel_message_info_to(mi));
+	printf("Cc: %s\n", camel_message_info_cc(mi));
+	printf("From: %s\n", camel_message_info_from(mi));
+	printf("UID: %s\n", camel_message_info_uid(mi));
 	printf("Flags: %04x\n", mi->flags & 0xffff);
 	content_info_dump(mi->content, 0);
 }
@@ -2468,7 +2758,9 @@ int main(int argc, char **argv)
 
 	printf("Printing summary\n");
 	for (i=0;i<camel_folder_summary_count(s);i++) {
-		message_info_dump(camel_folder_summary_index(s, i));
+		CamelMessageInfo *info = camel_folder_summary_index(s, i);
+		message_info_dump(info);
+		camel_folder_summary_info_free(info);
 	}
 
 	printf("Saivng summary\n");
@@ -2486,7 +2778,9 @@ int main(int argc, char **argv)
 
 		printf("Printing summary\n");
 		for (i=0;i<camel_folder_summary_count(n);i++) {
-			message_info_dump(camel_folder_summary_index(n, i));
+			CamelMessageInfo *info = camel_folder_summary_index(s, i);
+			message_info_dump(info);
+			camel_folder_summary_info_free(info);
 		}
 		camel_object_unref(n);		
 	}
