@@ -37,7 +37,7 @@ typedef struct _CamelFolderSummaryClass CamelFolderSummaryClass;
 
 /* A tree of message content info structures
    describe the content structure of the message (if it has any) */
-typedef struct _CamelMessageContentInfo {
+struct _CamelMessageContentInfo {
 	struct _CamelMessageContentInfo *next;
 
 	struct _CamelMessageContentInfo *childs;
@@ -47,7 +47,8 @@ typedef struct _CamelMessageContentInfo {
 	char *id;
 	char *description;
 	char *encoding;
-} CamelMessageContentInfo;
+	guint32 size;
+};
 
 /* system flag bits */
 enum _CamelMessageFlags {
@@ -107,7 +108,7 @@ enum {
 #endif
 
 /* information about a given object */
-typedef struct {
+struct _CamelMessageInfo {
 	/* public fields */
 #ifdef DOESTRV
 	struct _EStrv *strings;		/* all strings packed into a single compact array */
@@ -121,6 +122,7 @@ typedef struct {
 #endif
 	guint32 flags;
 	guint32 size;
+	guint32 refcount;
 
 	time_t date_sent;
 	time_t date_received;
@@ -133,7 +135,7 @@ typedef struct {
 
 	/* tree of content description - NULL if it is not available */
 	CamelMessageContentInfo *content;
-} CamelMessageInfo;
+};
 
 enum _CamelFolderSummaryFlags {
 	CAMEL_SUMMARY_DIRTY = 1<<0,
@@ -228,8 +230,12 @@ CamelMessageInfo *camel_folder_summary_info_new_from_header(CamelFolderSummary *
 CamelMessageInfo *camel_folder_summary_info_new_from_parser(CamelFolderSummary *, CamelMimeParser *);
 CamelMessageInfo *camel_folder_summary_info_new_from_message(CamelFolderSummary *, CamelMimeMessage *);
 
+void camel_folder_summary_info_ref(CamelFolderSummary *, CamelMessageInfo *);
 void camel_folder_summary_info_free(CamelFolderSummary *, CamelMessageInfo *);
 
+CamelMessageContentInfo *camel_folder_summary_content_info_new(CamelFolderSummary *s);
+void camel_folder_summary_content_info_free(CamelFolderSummary *s, CamelMessageContentInfo *ci);
+
 /* removes a summary item, doesn't fix content offsets */
 void camel_folder_summary_remove(CamelFolderSummary *s, CamelMessageInfo *info);
 void camel_folder_summary_remove_uid(CamelFolderSummary *s, const char *uid);
@@ -241,6 +247,8 @@ void camel_folder_summary_clear(CamelFolderSummary *s);
 int camel_folder_summary_count(CamelFolderSummary *);
 CamelMessageInfo *camel_folder_summary_index(CamelFolderSummary *, int);
 CamelMessageInfo *camel_folder_summary_uid(CamelFolderSummary *, const char *uid);
+GPtrArray *camel_folder_summary_array(CamelFolderSummary *s);
+void camel_folder_summary_array_free(CamelFolderSummary *s, GPtrArray *array);
 
 /* summary formatting utils */
 char *camel_folder_summary_format_address(struct _header_raw *h, const char *name);
@@ -280,8 +288,9 @@ void		camel_tag_list_free(CamelTag **list);
 /* message info utils for working with pseudo-messageinfo structures
    NOTE: These cannot be added to a real summary object, but suffice for all
    other external interfaces that use message info's */
-CamelMessageInfo *camel_message_info_new (void);
-CamelMessageInfo *camel_message_info_new_from_header (struct _header_raw *header);
+CamelMessageInfo *camel_message_info_new(void);
+void camel_message_info_ref(CamelMessageInfo *info);
+CamelMessageInfo *camel_message_info_new_from_header(struct _header_raw *header);
 void camel_message_info_dup_to(const CamelMessageInfo *from, CamelMessageInfo *to);
 void camel_message_info_free(CamelMessageInfo *mi);
 
