@@ -2345,11 +2345,12 @@ header_param_list_format_append(GString *out, struct _header_param *p)
 	while (p) {
 		int here = out->len;
 		if (len+strlen(p->name)+strlen(p->value)>60) {
-			out = g_string_append(out, "\n\t");
+			out = g_string_append(out, ";\n\t");
 			len = 0;
-		}
-
-		g_string_sprintfa(out, "; %s=", p->name);
+		} else
+			out = g_string_append (out, "; ");
+		
+		g_string_sprintfa(out, "%s=", p->name);
 
 		for (ch = p->value; *ch; ch++) {
 			if (is_tspecial(*ch))
@@ -2463,6 +2464,12 @@ header_content_type_format(struct _header_content_type *ct)
 	return ret;
 }
 
+char *
+header_content_type_simple(struct _header_content_type *ct)
+{
+	return g_strdup_printf("%s/%s", ct->type, ct->subtype);
+}
+
 char *
 header_content_encoding_decode(const char *in)
 {
@@ -3125,7 +3132,7 @@ header_fold(const char *in, int headerlen)
 		if (len >= CAMEL_FOLD_SIZE)
 			break;
 		len = 0;
-		p = n;
+		p = n + 1;
 	}
 	if (len < CAMEL_FOLD_SIZE)
 		return g_strdup(in);
@@ -3143,9 +3150,12 @@ header_fold(const char *in, int headerlen)
 		} else {
 			len = strlen(inptr);
 		}
-		printf("next word '%.*s'\n", len, inptr);
+		d(printf("next word '%.*s'\n", len, inptr));
 		if (outlen + len > CAMEL_FOLD_SIZE) {
-			printf("outlen = %d wordlen = %d\n", outlen, len);
+			d(printf("outlen = %d wordlen = %d\n", outlen, len));
+			/* strip trailing space */
+			if (out->len > 0 && out->str[out->len-1] == ' ')
+				g_string_truncate(out, out->len-1);
 			g_string_append(out, "\n\t");
 			outlen = 1;
 			/* check for very long words, just cut them up */
