@@ -25,6 +25,7 @@
 
 #include <config.h>
 
+#include <sys/time.h>
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
@@ -47,6 +48,8 @@
 #include "camel-url.h"
 #include "string-utils.h"
 
+#include "camel-private.h"
+
 #define d(x) x
 #if d(!)0
 extern gboolean camel_verbose_debug;
@@ -60,8 +63,7 @@ static CamelStoreClass *store_class = NULL;
 
 static gboolean remote_connect         (CamelService *service, CamelException *ex);
 static gboolean remote_disconnect      (CamelService *service, gboolean clean, CamelException *ex);
-static GList   *remote_query_auth_types_generic   (CamelService *service, CamelException *ex);
-static GList   *remote_query_auth_types_connected (CamelService *service, CamelException *ex);
+static GList   *remote_query_auth_types(CamelService *service, gboolean connect, CamelException *ex);
 static void     remote_free_auth_types (CamelService *service, GList *authtypes);
 static char    *remote_get_name        (CamelService *service, gboolean brief);
 static char    *remote_get_folder_name (CamelStore *store, 
@@ -88,8 +90,7 @@ camel_remote_store_class_init (CamelRemoteStoreClass *camel_remote_store_class)
 	/* virtual method overload */
 	camel_service_class->connect = remote_connect;
 	camel_service_class->disconnect = remote_disconnect;
-	camel_service_class->query_auth_types_generic = remote_query_auth_types_generic;
-	camel_service_class->query_auth_types_connected = remote_query_auth_types_connected;
+	camel_service_class->query_auth_types = remote_query_auth_types;
 	camel_service_class->free_auth_types = remote_free_auth_types;
 	camel_service_class->get_name = remote_get_name;
 	
@@ -112,17 +113,24 @@ camel_remote_store_init (CamelObject *object)
 	remote_store->istream = NULL;
 	remote_store->ostream = NULL;
 	remote_store->timeout_id = 0;
+
+	remote_store->priv = g_malloc0(sizeof(*remote_store->priv));
+#ifdef ENABLE_THREADS
+	remote_store->priv->stream_lock = e_mutex_new(E_MUTEX_REC);
+#endif
+}
+
+static void
+camel_remote_store_finalise(CamelObject *object)
+{
+	CamelRemoteStore *remote_store = CAMEL_REMOTE_STORE (object);
+
+#ifdef ENABLE_THREADS
+	e_mutex_destroy(remote_store->priv->stream_lock);
+#endif
+	g_free(remote_store->priv);
 }
 
-/*
- *static void
- *camel_remote_store_finalize (CamelObject *object)
- *{
- *	CamelRemoteStore *remote_store = CAMEL_REMOTE_STORE (object);
- *
- *	g_free (remote_store->nice_name);
- *}
- */
 
 CamelType
 camel_remote_store_get_type (void)
@@ -137,7 +145,7 @@ camel_remote_store_get_type (void)
 					     (CamelObjectClassInitFunc) camel_remote_store_class_init,
 					     NULL,
 					     (CamelObjectInitFunc) camel_remote_store_init,
-					     (CamelObjectFinalizeFunc) NULL);
+					     (CamelObjectFinalizeFunc) camel_remote_store_finalise);
 	}
 	
 	return camel_remote_store_type;
@@ -158,13 +166,7 @@ static CamelServiceAuthType password_authtype = {
 */
 
 static GList *
-remote_query_auth_types_connected (CamelService *service, CamelException *ex)
-{
-	return NULL;
-}
-
-static GList *
-remote_query_auth_types_generic (CamelService *service, CamelException *ex)
+remote_query_auth_types (CamelService *service, gboolean connect, CamelException *ex)
 {
 	return NULL;
 }
@@ -193,10 +195,97 @@ remote_get_name (CamelService *service, gboolean brief)
 static gboolean
 timeout_cb (gpointer data)
 {
-	CRSC (data)->keepalive (CAMEL_REMOTE_STORE (data));
+	CamelRemoteStore *store = CAMEL_REMOTE_STORE(data);
+
+	CRSC (data)->keepalive(store);
+
 	return TRUE;
 }
 
+/* this is a 'cancellable' connect, cancellable from camel_cancel etc */
+/* returns -1 & errno == EINTR if the connection was cancelled */
+static int socket_connect(struct hostent *h, int port)
+{
+	struct sockaddr_in sin;
+	int fd;
+	int ret;
+	socklen_t len;
+	struct timeval tv;
+	int cancel_fd;
+
+	/* see if we're cancelled yet */
+	if (camel_cancel_check(NULL)) {
+		errno = EINTR;
+		return -1;
+	}
+
+	/* setup connect, we do it using a nonblocking socket so we can poll it */
+	sin.sin_port = htons(port);
+	sin.sin_family = h->h_addrtype;
+	memcpy (&sin.sin_addr, h->h_addr, sizeof (sin.sin_addr));
+
+	fd = socket (h->h_addrtype, SOCK_STREAM, 0);
+
+	cancel_fd = camel_cancel_fd(NULL);
+	if (cancel_fd == -1) {
+		ret = connect(fd, (struct sockaddr *)&sin, sizeof (sin));
+		if (ret == -1) {
+			close(fd);
+			return -1;
+		}
+		return fd;
+	} else {
+		fd_set rdset, wrset;
+		int flags, fdmax;
+
+		flags = fcntl(fd, F_GETFL);
+		fcntl(fd, F_SETFL, flags | O_NONBLOCK);
+
+		ret = connect(fd, (struct sockaddr *)&sin, sizeof (sin));
+		if (ret == 0) {
+			fcntl(fd, F_SETFL, flags);
+			return fd;
+		}
+
+		if (errno != EINPROGRESS) {
+			close(fd);
+			return -1;
+		}
+
+		FD_ZERO(&rdset);
+		FD_ZERO(&wrset);
+		FD_SET(fd, &wrset);
+		FD_SET(cancel_fd, &rdset);
+		fdmax = MAX(fd, cancel_fd)+1;
+		tv.tv_usec = 0;
+		tv.tv_sec = 60*4;
+		if (select(fdmax, &rdset, &wrset, 0, &tv) == 0) {
+			close(fd);
+			errno = ETIMEDOUT;
+			return -1;
+		}
+		if (cancel_fd != -1 && FD_ISSET(cancel_fd, &rdset)) {
+			close(fd);
+			errno = EINTR;
+			return -1;
+		} else {
+			len = sizeof(int);
+			if (getsockopt(fd, SOL_SOCKET, SO_ERROR, &ret, &len) == -1) {
+				close(fd);
+				return -1;
+			}
+			if (ret != 0) {
+				close(fd);
+				errno = ret;
+				return -1;
+			}
+		}
+		fcntl(fd, F_SETFL, flags);
+	}
+
+	return fd;
+}
+
 static gboolean
 remote_connect (CamelService *service, CamelException *ex)
 {
@@ -217,6 +306,20 @@ remote_connect (CamelService *service, CamelException *ex)
 		port = service->url->port;
 	else
 		port = store->default_port;	
+
+#if 1
+	fd = socket_connect(h, port);
+	if (fd == -1) {
+		if (errno == EINTR)
+			camel_exception_set(ex, CAMEL_EXCEPTION_USER_CANCEL, _("Connection cancelled"));
+		else
+			camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_UNAVAILABLE,
+					      _("Could not connect to %s (port %d): %s"),
+					      service->url->host ? service->url->host : _("(unknown host)"),
+					      port, strerror (errno));
+		return FALSE;
+	}
+#else
 	sin.sin_port = htons (port);
 	
 	memcpy (&sin.sin_addr, h->h_addr, sizeof (sin.sin_addr));
@@ -232,6 +335,7 @@ remote_connect (CamelService *service, CamelException *ex)
 		
 		return FALSE;
 	}
+#endif
 	
 	/* parent class connect initialization */
 	if (CAMEL_SERVICE_CLASS (store_class)->connect (service, ex) == FALSE)
@@ -296,13 +400,8 @@ remote_send_string (CamelRemoteStore *store, CamelException *ex, char *fmt, va_l
 	
 	/* Check for connectedness. Failed (or cancelled) operations will
 	 * close the connection. */
-	
-	if (store->ostream == NULL) {
-		d(g_message ("remote: (send) disconnected, reconnecting."));
-		
-		if (!camel_service_connect (CAMEL_SERVICE (store), ex))
-			return -1;
-	}
+	if (!camel_remote_store_connected (store, ex))
+		return -1;
 	
 	/* create the command */
 	cmdbuf = g_strdup_vprintf (fmt, ap);
@@ -319,9 +418,11 @@ remote_send_string (CamelRemoteStore *store, CamelException *ex, char *fmt, va_l
 #endif
 	
 	if (camel_stream_printf (store->ostream, "%s", cmdbuf) == -1) {
+		if (errno == EINTR)
+			camel_exception_set(ex, CAMEL_EXCEPTION_USER_CANCEL, _("Operation cancelled"));
+		else
+			camel_exception_set(ex, CAMEL_EXCEPTION_SERVICE_UNAVAILABLE, strerror(errno));
 		g_free (cmdbuf);
-		camel_exception_set (ex, CAMEL_EXCEPTION_SERVICE_UNAVAILABLE,
-				     g_strerror (errno));
 		
 		camel_service_disconnect (CAMEL_SERVICE (store), FALSE, NULL);
 		return -1;
@@ -331,6 +432,9 @@ remote_send_string (CamelRemoteStore *store, CamelException *ex, char *fmt, va_l
 	return 0;
 }
 
+/* FIXME: All of these functions need an api overhaul, they're not like
+   any other functions, anywhere in the world ... */
+
 /**
  * camel_remote_store_send_string: Writes a string to the server
  * @store: a CamelRemoteStore
@@ -353,7 +457,9 @@ camel_remote_store_send_string (CamelRemoteStore *store, CamelException *ex,
 	g_return_val_if_fail (fmt, -1);
 	
 	va_start (ap, fmt);
+	CAMEL_REMOTE_STORE_LOCK(store, stream_lock);
 	ret = CRSC (store)->send_string (store, ex, fmt, ap);
+	CAMEL_REMOTE_STORE_UNLOCK(store, stream_lock);
 	va_end (ap);
 	
 	return ret;
@@ -362,27 +468,27 @@ camel_remote_store_send_string (CamelRemoteStore *store, CamelException *ex,
 static gint
 remote_send_stream (CamelRemoteStore *store, CamelStream *stream, CamelException *ex)
 {
+	int ret;
+
 	/* Check for connectedness. Failed (or cancelled) operations will
 	 * close the connection. */
-	
-	if (store->ostream == NULL) {
-		d(g_message ("remote: (sendstream) disconnected, reconnecting."));
-		
-		if (!camel_service_connect (CAMEL_SERVICE (store), ex))
-			return -1;
-	}
+
+	if (!camel_remote_store_connected (store, ex))
+		return -1;
 	
 	d(fprintf (stderr, "(sending stream)\n"));
 	
-	if (camel_stream_write_to_stream (stream, store->ostream) < 0) {
-		camel_exception_set (ex, CAMEL_EXCEPTION_SERVICE_UNAVAILABLE,
-				     g_strerror (errno));
+	ret = camel_stream_write_to_stream (stream, store->ostream);
+	if (ret == -1) {
+		if (errno == EINTR)
+			camel_exception_set(ex, CAMEL_EXCEPTION_USER_CANCEL, _("Operation cancelled"));
+		else
+			camel_exception_set(ex, CAMEL_EXCEPTION_SERVICE_UNAVAILABLE, strerror(errno));
 		
 		camel_service_disconnect (CAMEL_SERVICE (store), FALSE, NULL);
-		return -1;
 	}
-	
-	return 0;
+
+	return ret;
 }
 
 /**
@@ -398,10 +504,18 @@ remote_send_stream (CamelRemoteStore *store, CamelStream *stream, CamelException
 gint 
 camel_remote_store_send_stream (CamelRemoteStore *store, CamelStream *stream, CamelException *ex)
 {
+	int ret;
+
 	g_return_val_if_fail (CAMEL_IS_REMOTE_STORE (store), -1);
 	g_return_val_if_fail (CAMEL_IS_STREAM (stream), -1);
+
+	CAMEL_REMOTE_STORE_LOCK(store, stream_lock);
 	
-	return CRSC (store)->send_stream (store, stream, ex);
+	ret = CRSC (store)->send_stream (store, stream, ex);
+
+	CAMEL_REMOTE_STORE_UNLOCK(store, stream_lock);
+
+	return ret;
 }
 
 static int
@@ -409,7 +523,7 @@ remote_recv_line (CamelRemoteStore *store, char **dest, CamelException *ex)
 {
 	CamelStreamBuffer *stream = CAMEL_STREAM_BUFFER (store->istream);
 	GByteArray *bytes;
-	gchar buf[1025], *ret;
+	gchar buf[1024], *ret;
 	gint nread;
 	
 	*dest = NULL;
@@ -419,42 +533,35 @@ remote_recv_line (CamelRemoteStore *store, char **dest, CamelException *ex)
 	 * meaning if we reconnect, so always set an exception.
 	 */
 	
-	if (store->istream == NULL) {
-		g_message ("remote: (recv) disconnected, reconnecting.");
-		
-		camel_service_connect (CAMEL_SERVICE (store), ex);
-		
-		if (!camel_exception_is_set (ex))
-			camel_exception_set (ex, CAMEL_EXCEPTION_SERVICE_NOT_CONNECTED,
-					     g_strerror (errno));
-		
+	if (!camel_remote_store_connected (store, ex)) {
+		camel_exception_set (ex, CAMEL_EXCEPTION_SERVICE_NOT_CONNECTED,
+				     g_strerror (errno));
 		return -1;
 	}
-	
+
 	bytes = g_byte_array_new ();
 	
 	do {
-		nread = camel_stream_buffer_gets (stream, buf, 1024);
+		nread = camel_stream_buffer_gets (stream, buf, sizeof (buf));
 		if (nread > 0)
 			g_byte_array_append (bytes, buf, nread);
-	} while (nread == 1024);
-	
+	} while (nread == sizeof (buf) - 1);
+
+	if (nread == -1) {
+		if (errno == EINTR)
+			camel_exception_set(ex, CAMEL_EXCEPTION_USER_CANCEL, _("Operation cancelled"));
+		else
+			camel_exception_set(ex, CAMEL_EXCEPTION_SERVICE_UNAVAILABLE, strerror(errno));
+		g_byte_array_free(bytes, TRUE);
+		camel_service_disconnect (CAMEL_SERVICE (store), FALSE, NULL);
+		return -1;
+	}
+
 	g_byte_array_append (bytes, "", 1);
 	ret = bytes->data;
 	nread = bytes->len - 1;
 	g_byte_array_free (bytes, FALSE);
 	
-	if (nread <= 0) {
-		g_free (ret);
-		ret = NULL;
-		camel_exception_set (ex, CAMEL_EXCEPTION_SERVICE_UNAVAILABLE,
-				     nread ? g_strerror (errno) :
-				     _("Server disconnected."));
-		
-		camel_service_disconnect (CAMEL_SERVICE (store), FALSE, NULL);
-		return -1;
-	}
-	
 	/* strip off the CRLF sequence */
 	while (nread > 0 && ret[nread] != '\r')
 		ret[nread--] = '\0';
@@ -476,7 +583,7 @@ remote_recv_line (CamelRemoteStore *store, char **dest, CamelException *ex)
  * @dest: a pointer that will be set to the location of a buffer
  *        holding the server's response
  * @ex: a CamelException
- * Return value: 0 on success, -1 on error
+ * Return value: -1 on error, otherwise the length read.
  *
  * Reads a line from the server (terminated by \n or \r\n).
  **/
@@ -485,10 +592,18 @@ gint
 camel_remote_store_recv_line (CamelRemoteStore *store, char **dest,
 			      CamelException *ex)
 {
+	int ret;
+
 	g_return_val_if_fail (CAMEL_IS_REMOTE_STORE (store), -1);
 	g_return_val_if_fail (dest, -1);
+
+	CAMEL_REMOTE_STORE_LOCK(store, stream_lock);
 	
-	return CRSC (store)->recv_line (store, dest, ex);
+	ret = CRSC (store)->recv_line (store, dest, ex);
+
+	CAMEL_REMOTE_STORE_UNLOCK(store, stream_lock);
+
+	return ret;
 }
 
 static void
@@ -507,9 +622,31 @@ refresh_folder_info (gpointer key, gpointer value, gpointer data)
  *
  * Refreshes the folders listed in the folders hashtable.
  **/
-
 void
 camel_remote_store_refresh_folders (CamelRemoteStore *store, CamelException *ex)
 {
+	CAMEL_STORE_LOCK(store, cache_lock);
+
 	g_hash_table_foreach (CAMEL_STORE (store)->folders, refresh_folder_info, ex);
+
+	CAMEL_STORE_UNLOCK(store, cache_lock);
 }	
+
+/**
+ * camel_remote_store_connected:
+ * @store: a CamelRemoteStore
+ * @ex: a CamelException
+ *
+ * Ensure that the remote store is connected.
+ *
+ * Return value: Whether or not it is connected
+ **/
+gboolean
+camel_remote_store_connected (CamelRemoteStore *store, CamelException *ex)
+{
+	if (store->istream == NULL) {
+		camel_service_connect (CAMEL_SERVICE (store), ex);
+		return !camel_exception_is_set (ex);
+	}
+	return TRUE;
+}
