@@ -40,7 +40,8 @@ extern "C" {
 
 typedef struct {
 	CamelStore parent_object;	
-	
+	struct _CamelRemoteStorePrivate *priv;
+
 	CamelStream *istream, *ostream;
 	guint timeout_id, default_port;
 } CamelRemoteStore;
@@ -62,6 +63,9 @@ typedef struct {
 /* Standard Camel function */
 CamelType camel_remote_store_get_type (void);
 
+/* FIXME: All of these i/o functions need an api overhaul, they're not like
+   any other functions, anywhere in the world ... */
+
 /* Extra public functions */
 gint camel_remote_store_send_string (CamelRemoteStore *store, CamelException *ex,
 				     char *fmt, ...);
@@ -71,6 +75,7 @@ gint camel_remote_store_recv_line (CamelRemoteStore *store, char **dest,
 				   CamelException *ex);
 void camel_remote_store_refresh_folders (CamelRemoteStore *store, 
 					 CamelException *ex);
+gboolean camel_remote_store_connected (CamelRemoteStore *store, CamelException *ex);
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
