@@ -41,6 +41,10 @@
 #include "camel-url.h"
 #include "hash-table-utils.h"
 
+#include "camel-private.h"
+
+#define d(x)
+
 static CamelObjectClass *parent_class;
 
 static void
@@ -48,6 +52,10 @@ camel_session_init (CamelSession *session)
 {
 	session->modules = camel_provider_init ();
 	session->providers = g_hash_table_new (g_strcase_hash, g_strcase_equal);
+	session->priv = g_malloc0(sizeof(*session->priv));
+#ifdef ENABLE_THREADS
+	session->priv->lock = g_mutex_new();
+#endif	
 }
 
 static gboolean
@@ -69,6 +77,12 @@ camel_session_finalise (CamelObject *o)
 	g_hash_table_foreach_remove (session->providers,
 				     camel_session_destroy_provider, NULL);
 	g_hash_table_destroy (session->providers);
+
+#ifdef ENABLE_THREADS
+	g_mutex_free(session->priv->lock);
+#endif	
+
+	g_free(session->priv);
 }
 
 static void
@@ -131,6 +145,9 @@ camel_session_new (const char *storage_path,
  * @provider: provider object
  *
  * Registers a protocol to provider mapping for the session.
+ *
+ * Assumes the session lock has already been obtained,
+ * which is the case for automatically loaded provider modules.
  **/
 void
 camel_session_register_provider (CamelSession *session,
@@ -194,12 +211,16 @@ camel_session_list_providers (CamelSession *session, gboolean load)
 
 	g_return_val_if_fail (CAMEL_IS_SESSION (session), NULL);
 
-	if (load) {
+	CAMEL_SESSION_LOCK(session, lock);
+
+	if (load)
 		g_hash_table_foreach (session->modules, ensure_loaded, session);
-	}
 
 	list = NULL;
 	g_hash_table_foreach (session->providers, add_to_list, &list);
+
+	CAMEL_SESSION_UNLOCK(session, lock);
+
 	return list;
 }
 
@@ -213,8 +234,12 @@ service_cache_remove (CamelService *service, gpointer event_data, gpointer user_
 	g_return_if_fail (service != NULL);
 	g_return_if_fail (service->url != NULL);
 	
+	CAMEL_SESSION_LOCK(session, lock);
+
 	provider = g_hash_table_lookup (session->providers, service->url->protocol);
 	g_hash_table_remove (provider->service_cache, service->url);
+
+	CAMEL_SESSION_UNLOCK(session, lock);
 }
 
 /**
@@ -250,6 +275,7 @@ camel_session_get_service (CamelSession *session, const char *url_string,
 
 	/* We need to look up the provider so we can then lookup
 	   the service in the provider's cache */
+	CAMEL_SESSION_LOCK(session, lock);
 	provider = g_hash_table_lookup (session->providers, url->protocol);
 	if (!provider) {
 		/* See if there's one we can load. */
@@ -261,6 +287,7 @@ camel_session_get_service (CamelSession *session, const char *url_string,
 			if (camel_exception_get_id (ex) !=
 			    CAMEL_EXCEPTION_NONE) {
 				camel_url_free (url);
+				CAMEL_SESSION_UNLOCK(session, lock);
 				return NULL;
 			}
 		}
@@ -272,6 +299,7 @@ camel_session_get_service (CamelSession *session, const char *url_string,
 				      _("No provider available for protocol `%s'"),
 				      url->protocol);
 		camel_url_free (url);
+		CAMEL_SESSION_UNLOCK(session, lock);
 		return NULL;
 	}
 	
@@ -280,6 +308,7 @@ camel_session_get_service (CamelSession *session, const char *url_string,
 	if (service != NULL) {
 		camel_url_free (url);
 		camel_object_ref (CAMEL_OBJECT (service));
+		CAMEL_SESSION_UNLOCK(session, lock);
 		return service;
 	}
 
@@ -288,6 +317,7 @@ camel_session_get_service (CamelSession *session, const char *url_string,
 		g_hash_table_insert (provider->service_cache, url, service);
 		camel_object_hook_event (CAMEL_OBJECT (service), "finalize", (CamelObjectEventHookFunc) service_cache_remove, session);
 	}
+	CAMEL_SESSION_UNLOCK(session, lock);
 
 	return service;
 }
@@ -408,6 +438,31 @@ camel_session_query_authenticator (CamelSession *session,
 				       service, item, ex);
 }
 
+#ifdef U_CANT_TOUCH_THIS
+/**
+ * camel_session_query_cert_authenticator:
+ * @session: session object
+ * @prompt: prompt to query user with
+ *
+ * This function is used by SSL to discuss certificate authentication
+ * information with the application and/or user. Allows the user to
+ * override the SSL certificate authenticator, which, at this point,
+ * must have failed to authenticate the server.
+ *
+ * UI should be a Yes/No prompt probably defaulting to No.
+ *
+ * Return value: TRUE if the user decides that the SSL connection
+ * should continue with the untrusted server or FALSE otherwise.
+ **/
+gboolean
+camel_session_query_cert_authenticator (CamelSession *session,
+					char *prompt)
+{
+	return session->cert_authenticator (prompt);
+}
+#endif /* U_CANT_TOUCH_THIS */
+
+
 /**
  * camel_session_register_timeout: Register a timeout to be called
  * periodically.
@@ -454,3 +509,256 @@ camel_session_remove_timeout (CamelSession *session, guint handle)
 {
 	return session->remover (handle);
 }
+
+/* ********************************************************************** */
+
+struct _CamelCancel {
+	pthread_t id;		/* id of running thread */
+	guint32 flags;		/* cancelled ? */
+	int blocked;		/* cancellation blocked depth */
+	int refcount;
+#ifdef ENABLE_THREADS
+	EMsgPort *cancel_port;
+	int cancel_fd;
+	pthread_mutex_t lock;
+#endif
+};
+
+#define CAMEL_CANCEL_CANCELLED (1<<0)
+
+#ifdef ENABLE_THREADS
+#define CAMEL_CANCEL_LOCK(cc) pthread_mutex_lock(&cc->lock)
+#define CAMEL_CANCEL_UNLOCK(cc) pthread_mutex_unlock(&cc->lock)
+#define CAMEL_ACTIVE_LOCK() pthread_mutex_lock(&cancel_active_lock)
+#define CAMEL_ACTIVE_UNLOCK() pthread_mutex_unlock(&cancel_active_lock)
+static pthread_mutex_t cancel_active_lock = PTHREAD_MUTEX_INITIALIZER;
+#else
+#define CAMEL_CANCEL_LOCK(cc)
+#define CAMEL_CANCEL_UNLOCK(cc)
+#define CAMEL_ACTIVE_LOCK()
+#define CAMEL_ACTIVE_UNLOCK()
+#endif
+
+static GHashTable *cancel_active;
+
+typedef struct _CamelCancelMsg {
+	EMsg msg;
+} CamelCancelMsg ;
+
+/* creates a new cancel handle */
+CamelCancel *camel_cancel_new(void)
+{
+	CamelCancel *cc;
+
+	cc = g_malloc0(sizeof(*cc));
+
+	cc->flags = 0;
+	cc->blocked = 0;
+	cc->refcount = 1;
+#ifdef ENABLE_THREADS
+	cc->id = ~0;
+	cc->cancel_port = e_msgport_new();
+	cc->cancel_fd = e_msgport_fd(cc->cancel_port);
+	pthread_mutex_init(&cc->lock, NULL);
+#endif
+
+	return cc;
+}
+
+void camel_cancel_reset(CamelCancel *cc)
+{
+#ifdef ENABLE_THREADS
+	CamelCancelMsg *msg;
+
+	while ((msg = (CamelCancelMsg *)e_msgport_get(cc->cancel_port)))
+		g_free(msg);
+#endif
+
+	cc->flags = 0;
+	cc->blocked = 0;
+}
+
+void camel_cancel_ref(CamelCancel *cc)
+{
+	CAMEL_CANCEL_LOCK(cc);
+	cc->refcount++;
+	CAMEL_CANCEL_UNLOCK(cc);
+}
+
+void camel_cancel_unref(CamelCancel *cc)
+{
+#ifdef ENABLE_THREADS
+	CamelCancelMsg *msg;
+
+	if (cc->refcount == 1) {
+		while ((msg = (CamelCancelMsg *)e_msgport_get(cc->cancel_port)))
+			g_free(msg);
+
+		e_msgport_destroy(cc->cancel_port);
+#endif
+		g_free(cc);
+	} else {
+		CAMEL_CANCEL_LOCK(cc);
+		cc->refcount--;
+		CAMEL_CANCEL_UNLOCK(cc);
+	}
+}
+
+/* block cancellation */
+void camel_cancel_block(CamelCancel *cc)
+{
+	CAMEL_CANCEL_LOCK(cc);
+
+	cc->blocked++;
+
+	CAMEL_CANCEL_UNLOCK(cc);
+}
+
+/* unblock cancellation */
+void camel_cancel_unblock(CamelCancel *cc)
+{
+	CAMEL_CANCEL_LOCK(cc);
+
+	cc->blocked--;
+
+	CAMEL_CANCEL_UNLOCK(cc);
+}
+
+static void
+cancel_thread(void *key, CamelCancel *cc, void *data)
+{
+	if (cc)
+		camel_cancel_cancel(cc);
+}
+
+/* cancels an operation */
+void camel_cancel_cancel(CamelCancel *cc)
+{
+	CamelCancelMsg *msg;
+
+	if (cc == NULL) {
+		if (cancel_active) {
+			CAMEL_ACTIVE_LOCK();
+			g_hash_table_foreach(cancel_active, (GHFunc)cancel_thread, NULL);
+			CAMEL_ACTIVE_UNLOCK();
+		}
+	} else if ((cc->flags & CAMEL_CANCEL_CANCELLED) == 0) {
+		d(printf("cancelling thread %d\n", cc->id));
+
+		CAMEL_CANCEL_LOCK(cc);
+		msg = g_malloc0(sizeof(*msg));
+		e_msgport_put(cc->cancel_port, (EMsg *)msg);
+		cc->flags |= CAMEL_CANCEL_CANCELLED;
+		CAMEL_CANCEL_UNLOCK(cc);
+	}
+}
+
+/* register a thread for cancellation */
+void camel_cancel_register(CamelCancel *cc)
+{
+	pthread_t id = pthread_self();
+
+	CAMEL_ACTIVE_LOCK();
+
+	if (cancel_active == NULL)
+		cancel_active = g_hash_table_new(NULL, NULL);
+
+	if (cc == NULL) {
+		cc = g_hash_table_lookup(cancel_active, (void *)id);
+		if (cc == NULL) {
+			cc = camel_cancel_new();
+		}
+	}
+
+	cc->id = id;
+	g_hash_table_insert(cancel_active, (void *)id, cc);
+
+	d(printf("registering thread %d for cancellation\n", id));
+
+	CAMEL_ACTIVE_UNLOCK();
+
+	camel_cancel_ref(cc);
+}
+
+/* remove a thread from being able to be cancelled */
+void camel_cancel_unregister(CamelCancel *cc)
+{
+	CAMEL_ACTIVE_LOCK();
+
+	if (cancel_active == NULL)
+		cancel_active = g_hash_table_new(NULL, NULL);
+
+	if (cc == NULL) {
+		cc = g_hash_table_lookup(cancel_active, (void *)pthread_self());
+		if (cc == NULL) {
+			g_warning("Trying to unregister a thread that was never registered for cancellation");
+		}
+	}
+
+	if (cc)
+		g_hash_table_remove(cancel_active, (void *)cc->id);
+
+	CAMEL_ACTIVE_UNLOCK();
+
+	d({if (cc) printf("unregistering thread %d for cancellation\n", cc->id)});
+
+	if (cc)
+		camel_cancel_unref(cc);
+}
+
+/* test for cancellation */
+gboolean camel_cancel_check(CamelCancel *cc)
+{
+	CamelCancelMsg *msg;
+
+	d(printf("checking for cancel in thread %d\n", pthread_self()));
+
+	if (cc == NULL) {
+		if (cancel_active) {
+			CAMEL_ACTIVE_LOCK();
+			cc = g_hash_table_lookup(cancel_active, (void *)pthread_self());
+			CAMEL_ACTIVE_UNLOCK();
+		}
+		if (cc == NULL)
+			return FALSE;
+	}
+
+	if (cc->blocked > 0) {
+		d(printf("ahah!  cancellation is blocked\n"));
+		return FALSE;
+	}
+
+	if (cc->flags & CAMEL_CANCEL_CANCELLED) {
+		d(printf("previously cancelled\n"));
+		return TRUE;
+	}
+
+	msg = (CamelCancelMsg *)e_msgport_get(cc->cancel_port);
+	if (msg) {
+		d(printf("Got cancellation message\n"));
+		CAMEL_CANCEL_LOCK(cc);
+		cc->flags |= CAMEL_CANCEL_CANCELLED;
+		CAMEL_CANCEL_UNLOCK(cc);
+		return TRUE;
+	}
+	return FALSE;
+}
+
+/* get the fd for cancellation waiting */
+int camel_cancel_fd(CamelCancel *cc)
+{
+	if (cc == NULL) {
+		if (cancel_active) {
+			CAMEL_ACTIVE_LOCK();
+			cc = g_hash_table_lookup(cancel_active, (void *)pthread_self());
+			CAMEL_ACTIVE_UNLOCK();
+		}
+		if (cc == NULL)
+			return -1;
+	}
+	if (cc->blocked)
+		return -1;
+
+	return cc->cancel_fd;
+}
+
