@@ -51,6 +51,12 @@ typedef char *(*CamelAuthCallback) (CamelAuthCallbackMode mode,
 				    char *data, gboolean secret,
 				    CamelService *service, char *item,
 				    CamelException *ex);
+
+#ifdef U_CANT_TOUCH_THIS
+/* this is just a guess as to what we'll actually need */
+typedef gboolean (*CamelBadCertCallback) (char *data);
+#endif
+
 typedef gboolean (*CamelTimeoutCallback) (gpointer data);
 typedef guint (*CamelTimeoutRegisterCallback) (guint32 interval,
 					       CamelTimeoutCallback cb,
@@ -60,9 +66,13 @@ typedef gboolean (*CamelTimeoutRemoveCallback) (guint id);
 struct _CamelSession
 {
 	CamelObject parent_object;
+	struct _CamelSessionPrivate *priv;
 
 	char *storage_path;
 	CamelAuthCallback authenticator;
+#ifdef U_CANT_TOUCH_THIS
+	CamelBadCertCallback cert_authenticator;
+#endif
 	CamelTimeoutRegisterCallback registrar;
 	CamelTimeoutRemoveCallback remover;
 
@@ -117,6 +127,11 @@ char *          camel_session_query_authenticator (CamelSession *session,
 						   char *item,
 						   CamelException *ex);
 
+#ifdef U_CANT_TOUCH_THIS
+gboolean        camel_session_query_cert_authenticator (CamelSession *session,
+							char *prompt);
+#endif
+
 guint           camel_session_register_timeout (CamelSession *session,
 						guint32 interval,
 						CamelTimeoutCallback callback,
@@ -125,6 +140,24 @@ guint           camel_session_register_timeout (CamelSession *session,
 gboolean        camel_session_remove_timeout (CamelSession *session,
 					      guint handle);
 
+
+/* cancellation helper stuff, not yet finalised */
+typedef struct _CamelCancel CamelCancel;
+/* main thread functions */
+CamelCancel *camel_cancel_new(void);
+void camel_cancel_ref(CamelCancel *cc);
+void camel_cancel_unref(CamelCancel *cc);
+void camel_cancel_reset(CamelCancel *cc);
+void camel_cancel_cancel(CamelCancel *cc);
+/* subthread functions */
+void camel_cancel_register(CamelCancel *cc);
+void camel_cancel_unregister(CamelCancel *cc);
+/* called internally by camel, for the current thread */
+void camel_cancel_block(CamelCancel *cc);
+void camel_cancel_unblock(CamelCancel *cc);
+gboolean camel_cancel_check(CamelCancel *cc);
+int camel_cancel_fd(CamelCancel *cc);
+
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
