@@ -31,6 +31,8 @@
 #include "camel-folder.h"
 #include "camel-exception.h"
 
+#include "camel-private.h"
+
 static CamelServiceClass *parent_class = NULL;
 
 /* Returns the class for a CamelStore */
@@ -48,6 +50,7 @@ static char *get_folder_name (CamelStore *store, const char *folder_name,
 static char *get_root_folder_name (CamelStore *store, CamelException *ex);
 static char *get_default_folder_name (CamelStore *store, CamelException *ex);
 
+static void store_sync (CamelStore *store, CamelException *ex);
 static CamelFolderInfo *get_folder_info (CamelStore *store, const char *top,
 					 gboolean fast, gboolean recursive,
 					 gboolean subscribed_only,
@@ -75,6 +78,7 @@ camel_store_class_init (CamelStoreClass *camel_store_class)
 	camel_store_class->get_folder_name = get_folder_name;
 	camel_store_class->get_root_folder_name = get_root_folder_name;
 	camel_store_class->get_default_folder_name = get_default_folder_name;
+	camel_store_class->sync = store_sync;
 	camel_store_class->get_folder_info = get_folder_info;
 	camel_store_class->free_folder_info = free_folder_info;
 	camel_store_class->lookup_folder = lookup_folder;
@@ -93,6 +97,12 @@ camel_store_init (void *o, void *k)
 	if (store->folders == NULL)
 		store->folders = g_hash_table_new (g_str_hash, g_str_equal);
 	store->flags = 0;
+
+	store->priv = g_malloc0(sizeof(*store->priv));
+#ifdef ENABLE_THREADS
+	store->priv->folder_lock = g_mutex_new();
+	store->priv->cache_lock = g_mutex_new();
+#endif
 }
 
 static void
@@ -108,6 +118,12 @@ camel_store_finalize (CamelObject *object)
 		}
 		g_hash_table_destroy (store->folders);
 	}
+
+#ifdef ENABLE_THREADS
+	g_mutex_free(store->priv->folder_lock);
+	g_mutex_free(store->priv->cache_lock);
+#endif
+	g_free(store->priv);
 }
 
 
@@ -185,13 +201,19 @@ get_default_folder_name (CamelStore *store, CamelException *ex)
 static CamelFolder *
 lookup_folder (CamelStore *store, const char *folder_name)
 {
+	CamelFolder *folder = NULL;
+
+	CAMEL_STORE_LOCK(store, cache_lock);
+
 	if (store->folders) {
-		CamelFolder *folder = g_hash_table_lookup (store->folders, folder_name);
+		folder = g_hash_table_lookup (store->folders, folder_name);
 		if (folder)
 			camel_object_ref(CAMEL_OBJECT(folder));
-		return folder;
 	}
-	return NULL;
+
+	CAMEL_STORE_UNLOCK(store, cache_lock);
+
+	return folder;
 }
 
 static void folder_finalize (CamelObject *folder, gpointer event_data, gpointer user_data)
@@ -202,16 +224,18 @@ static void folder_finalize (CamelObject *folder, gpointer event_data, gpointer
 static void
 cache_folder (CamelStore *store, const char *folder_name, CamelFolder *folder)
 {
-	if (!store->folders)
-		return;
+	CAMEL_STORE_LOCK(store, cache_lock);
 
-	if (g_hash_table_lookup (store->folders, folder_name)) {
-		g_warning ("Caching folder %s that already exists.",
-			   folder_name);
+	if (store->folders) {
+		if (g_hash_table_lookup (store->folders, folder_name)) {
+			g_warning ("Caching folder %s that already exists.", folder_name);
+		}
+		g_hash_table_insert (store->folders, g_strdup (folder_name), folder);
+
+		camel_object_hook_event (CAMEL_OBJECT (folder), "finalize", folder_finalize, store);
 	}
-	g_hash_table_insert (store->folders, g_strdup (folder_name), folder);
 
-	camel_object_hook_event (CAMEL_OBJECT (folder), "finalize", folder_finalize, store);
+	CAMEL_STORE_UNLOCK(store, cache_lock);
 
 	/*
 	 * gt_k so as not to get caught by my little gt_k cleanliness detector.
@@ -235,7 +259,11 @@ folder_matches (gpointer key, gpointer value, gpointer user_data)
 static void
 uncache_folder (CamelStore *store, CamelFolder *folder)
 {
+	CAMEL_STORE_LOCK(store, cache_lock);
+
 	g_hash_table_foreach_remove (store->folders, folder_matches, folder);
+
+	CAMEL_STORE_UNLOCK(store, cache_lock);
 }
 
 
@@ -244,6 +272,8 @@ get_folder_internal(CamelStore *store, const char *folder_name, guint32 flags, C
 {
 	CamelFolder *folder = NULL;
 
+	/* NB: we already have folder_lock */
+
 	/* Try cache first. */
 	folder = CS_CLASS(store)->lookup_folder(store, folder_name);
 
@@ -279,11 +309,16 @@ camel_store_get_folder(CamelStore *store, const char *folder_name, guint32 flags
 	char *name;
 	CamelFolder *folder = NULL;
 
+	CAMEL_STORE_LOCK(store, folder_lock);
+
 	name = CS_CLASS(store)->get_folder_name(store, folder_name, ex);
 	if (name) {
 		folder = get_folder_internal(store, name, flags, ex);
 		g_free (name);
 	}
+
+	CAMEL_STORE_UNLOCK(store, folder_lock);
+
 	return folder;
 }
 
@@ -302,11 +337,15 @@ camel_store_delete_folder (CamelStore *store, const char *folder_name,
 {
 	char *name;
 
+	CAMEL_STORE_LOCK(store, folder_lock);
+
 	name = CS_CLASS (store)->get_folder_name (store, folder_name, ex);
 	if (name) {
 		CS_CLASS (store)->delete_folder (store, name, ex);
 		g_free (name);
 	}
+
+	CAMEL_STORE_UNLOCK(store, folder_lock);
 }
 
 /**
@@ -325,6 +364,8 @@ void             camel_store_rename_folder      (CamelStore *store,
 {
 	char *old, *new;
 
+	CAMEL_STORE_LOCK(store, folder_lock);
+
 	old = CS_CLASS (store)->get_folder_name(store, old_name, ex);
 	if (old) {
 		new = CS_CLASS (store)->get_folder_name(store, new_name, ex);
@@ -334,6 +375,8 @@ void             camel_store_rename_folder      (CamelStore *store,
 		}
 		g_free(old);
 	}
+
+	CAMEL_STORE_UNLOCK(store, folder_lock);
 }
 
 
@@ -351,11 +394,16 @@ camel_store_get_root_folder (CamelStore *store, CamelException *ex)
 	char *name;
 	CamelFolder *folder = NULL;
 
+	CAMEL_STORE_LOCK(store, folder_lock);
+
 	name = CS_CLASS (store)->get_root_folder_name (store, ex);
 	if (name) {
 		folder = get_folder_internal (store, name, CAMEL_STORE_FOLDER_CREATE, ex);
 		g_free (name);
 	}
+
+	CAMEL_STORE_UNLOCK(store, folder_lock);
+
 	return folder;
 }
 
@@ -374,15 +422,52 @@ camel_store_get_default_folder (CamelStore *store, CamelException *ex)
 	char *name;
 	CamelFolder *folder = NULL;
 
+	CAMEL_STORE_LOCK(store, folder_lock);
+
 	name = CS_CLASS (store)->get_default_folder_name (store, ex);
 	if (name) {
 		folder = get_folder_internal (store, name, CAMEL_STORE_FOLDER_CREATE, ex);
 		g_free (name);
 	}
+
+	CAMEL_STORE_UNLOCK(store, folder_lock);
+
 	return folder;
 }
 
 
+static void
+sync_folder (gpointer key, gpointer folder, gpointer ex)
+{
+	if (!camel_exception_is_set (ex))
+		camel_folder_sync (folder, FALSE, ex);
+}
+
+static void
+store_sync (CamelStore *store, CamelException *ex)
+{
+	CAMEL_STORE_LOCK(store, cache_lock);
+	g_hash_table_foreach (store->folders, sync_folder, ex);
+	CAMEL_STORE_UNLOCK(store, cache_lock);
+}
+
+/**
+ * camel_store_sync:
+ * @store: a CamelStore
+ * @ex: a CamelException
+ *
+ * Syncs any changes that have been made to the store object and its
+ * folders with the real store.
+ **/
+void
+camel_store_sync (CamelStore *store, CamelException *ex)
+{
+	g_return_if_fail (CAMEL_IS_STORE (store));
+
+	CS_CLASS (store)->sync (store, ex);
+}
+
+
 static CamelFolderInfo *
 get_folder_info (CamelStore *store, const char *top,
 		 gboolean fast, gboolean recursive,
@@ -420,11 +505,17 @@ camel_store_get_folder_info (CamelStore *store, const char *top,
 			     gboolean subscribed_only,
 			     CamelException *ex)
 {
+	CamelFolderInfo *ret;
+
 	g_return_val_if_fail (CAMEL_IS_STORE (store), NULL);
 
-	return CS_CLASS (store)->get_folder_info (store, top, fast,
-						  recursive, subscribed_only,
-						  ex);
+	CAMEL_STORE_LOCK(store, folder_lock);
+
+	ret = CS_CLASS (store)->get_folder_info (store, top, fast, recursive, subscribed_only, ex);
+
+	CAMEL_STORE_UNLOCK(store, folder_lock);
+
+	return ret;
 }
 
 
@@ -501,48 +592,59 @@ camel_folder_info_free (CamelFolderInfo *fi)
 /**
  * camel_folder_info_build:
  * @folders: an array of CamelFolderInfo
- * @top: the top of the folder tree
+ * @namespace: an ignorable prefix on the folder names
  * @separator: the hieararchy separator character
  * @short_names: %TRUE if the (short) name of a folder is the part after
  * the last @separator in the full name. %FALSE if it is the full name.
  *
- * This takes an array of folders and attaches them together. @top points
- * to the (or at least, "a") top-level element of the tree: it may or may
- * not also be an element of @folders. If necessary, camel_folder_info_build
- * will create additional CamelFolderInfo with %NULL urls to fill in gaps
- * in the tree. The value of @short_names is used in constructing the
- * names of these intermediate folders.
+ * This takes an array of folders and attaches them together according
+ * to the hierarchy described by their full_names and @separator. If
+ * @namespace is non-%NULL, then it will be ignored as a full_name
+ * prefix, for purposes of comparison. If necessary,
+ * camel_folder_info_build will create additional CamelFolderInfo with
+ * %NULL urls to fill in gaps in the tree. The value of @short_names
+ * is used in constructing the names of these intermediate folders.
+ *
+ * Return value: the top level of the tree of linked folder info.
  **/
-void
-camel_folder_info_build (GPtrArray *folders, CamelFolderInfo *top,
+CamelFolderInfo *
+camel_folder_info_build (GPtrArray *folders, const char *namespace,
 			 char separator, gboolean short_names)
 {
-	CamelFolderInfo *fi, *pfi;
+	CamelFolderInfo *fi, *pfi, *top = NULL;
 	GHashTable *hash;
-	char *p, *pname;
-	int i;
+	char *name, *p, *pname;
+	int i, nlen;
+
+	if (!namespace)
+		namespace = "";
+	nlen = strlen (namespace);
 
 	/* Hash the folders. */
 	hash = g_hash_table_new (g_str_hash, g_str_equal);
-	pfi = top;
 	for (i = 0; i < folders->len; i++) {
 		fi = folders->pdata[i];
-		if (fi == top)
-			pfi = NULL;
-		g_hash_table_insert (hash, fi->full_name, fi);
+		if (!strncmp (namespace, fi->full_name, nlen))
+			name = fi->full_name + nlen;
+		else
+			name = fi->full_name;
+		if (*name == separator)
+			name++;
+		g_hash_table_insert (hash, name, fi);
 	}
-	if (pfi)
-		g_hash_table_insert (hash, pfi->full_name, pfi);
 
 	/* Now find parents. */
 	for (i = 0; i < folders->len; i++) {
 		fi = folders->pdata[i];
-		if (fi == top)
-			continue;
-
-		p = strrchr (fi->full_name, separator);
+		if (!strncmp (namespace, fi->full_name, nlen))
+			name = fi->full_name + nlen;
+		else
+			name = fi->full_name;
+		if (*name == separator)
+			name++;
+		p = strrchr (name, separator);
 		if (p) {
-			pname = g_strndup (fi->full_name, p - fi->full_name);
+			pname = g_strndup (name, p - name);
 			pfi = g_hash_table_lookup (hash, pname);
 			if (pfi) {
 				g_free (pname);
@@ -563,12 +665,22 @@ camel_folder_info_build (GPtrArray *folders, CamelFolderInfo *top,
 			fi->sibling = pfi->child;
 			fi->parent = pfi;
 			pfi->child = fi;
-		} else {
-			fi->sibling = top->child;
-			fi->parent = top;
-			top->child = fi;
-		}
+		} else if (!top)
+			top = fi;
 	}
+	g_hash_table_destroy (hash);
+
+	/* Link together the top-level folders */
+	for (i = 0; i < folders->len; i++) {
+		fi = folders->pdata[i];
+		if (fi->parent || fi == top)
+			continue;
+		if (top)
+			fi->sibling = top;
+		top = fi;
+	}
+
+	return top;			
 }
 
 gboolean
@@ -596,10 +708,18 @@ gboolean
 camel_store_folder_subscribed (CamelStore *store,
 			       const char *folder_name)
 {
+	gboolean ret;
+
 	g_return_val_if_fail (CAMEL_IS_STORE (store), FALSE);
 	g_return_val_if_fail (store->flags & CAMEL_STORE_SUBSCRIPTIONS, FALSE);
 
-	return CS_CLASS (store)->folder_subscribed (store, folder_name);
+	CAMEL_STORE_LOCK(store, folder_lock);
+
+	ret = CS_CLASS (store)->folder_subscribed (store, folder_name);
+
+	CAMEL_STORE_UNLOCK(store, folder_lock);
+
+	return ret;
 }
 
 static void
@@ -622,7 +742,11 @@ camel_store_subscribe_folder (CamelStore *store,
 	g_return_if_fail (CAMEL_IS_STORE (store));
 	g_return_if_fail (store->flags & CAMEL_STORE_SUBSCRIPTIONS);
 
+	CAMEL_STORE_LOCK(store, folder_lock);
+
 	CS_CLASS (store)->subscribe_folder (store, folder_name, ex);
+
+	CAMEL_STORE_UNLOCK(store, folder_lock);
 }
 
 static void
@@ -646,6 +770,9 @@ camel_store_unsubscribe_folder (CamelStore *store,
 	g_return_if_fail (CAMEL_IS_STORE (store));
 	g_return_if_fail (store->flags & CAMEL_STORE_SUBSCRIPTIONS);
 
-	CS_CLASS (store)->unsubscribe_folder (store, folder_name, ex);
-}
+	CAMEL_STORE_LOCK(store, folder_lock);
 
+	CS_CLASS (store)->unsubscribe_folder (store, folder_name, ex);
+
+	CAMEL_STORE_UNLOCK(store, folder_lock);
+}
