@@ -25,6 +25,7 @@
 
 #include <config.h>
 #include "camel-stream-fs.h"
+#include "camel-session.h"
 #include <unistd.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -32,6 +33,8 @@
 #include <errno.h>
 #include <string.h>
 
+#include "camel-session.h"	/* for camel_cancel_* */
+
 static CamelSeekableStreamClass *parent_class = NULL;
 
 /* Returns the class for a CamelStreamFS */
@@ -209,13 +212,40 @@ stream_read (CamelStream *stream, char *buffer, size_t n)
 	CamelStreamFs *stream_fs = CAMEL_STREAM_FS (stream);
 	CamelSeekableStream *seekable = CAMEL_SEEKABLE_STREAM (stream);
 	ssize_t nread;
+	int cancel_fd;
+
+	if (camel_cancel_check(NULL)) {
+		errno = EINTR;
+		return  -1;
+	}
 
 	if (seekable->bound_end != CAMEL_STREAM_UNBOUND)
 		n = MIN (seekable->bound_end - seekable->position, n);
 
-	do {
-		nread = read (stream_fs->fd, buffer, n);
-	} while (nread == -1 && errno == EINTR);
+	cancel_fd = camel_cancel_fd(NULL);
+	if (cancel_fd == -1) {
+		do {
+			nread = read (stream_fs->fd, buffer, n);
+		} while (nread == -1 && errno == EINTR);
+	} else {
+		fd_set rdset;
+		int flags, fdmax;
+
+		flags = fcntl(stream_fs->fd, F_GETFL);
+		fcntl(stream_fs->fd, F_SETFL, flags | O_NONBLOCK);
+		FD_ZERO(&rdset);
+		FD_SET(stream_fs->fd, &rdset);
+		FD_SET(cancel_fd, &rdset);
+		fdmax = MAX(stream_fs->fd, cancel_fd)+1;
+		select(fdmax, &rdset, 0, 0, NULL);
+		if (FD_ISSET(cancel_fd, &rdset)) {
+			fcntl(stream_fs->fd, F_SETFL, flags);
+			errno = EINTR;
+			return -1;
+		}
+		nread = read(stream_fs->fd, buffer, n);
+		fcntl(stream_fs->fd, F_SETFL, flags);
+	}
 
 	if (nread > 0)
 		seekable->position += nread;
@@ -231,15 +261,47 @@ stream_write (CamelStream *stream, const char *buffer, size_t n)
 	CamelStreamFs *stream_fs = CAMEL_STREAM_FS (stream);
 	CamelSeekableStream *seekable = CAMEL_SEEKABLE_STREAM (stream);
 	ssize_t v, written = 0;
+	int cancel_fd;
+
+	if (camel_cancel_check(NULL)) {
+		errno = EINTR;
+		return  -1;
+	}
 
 	if (seekable->bound_end != CAMEL_STREAM_UNBOUND)
 		n = MIN (seekable->bound_end - seekable->position, n);
 
-	do {
-		v = write (stream_fs->fd, buffer, n);
-		if (v > 0)
-			written += v;
-	} while (v == -1 && errno == EINTR);
+	cancel_fd = camel_cancel_fd(NULL);
+	if (cancel_fd == -1) {
+		do {
+			v = write (stream_fs->fd, buffer+written, n-written);
+			if (v > 0)
+				written += v;
+		} while (v == -1 && errno == EINTR);
+	} else {
+		fd_set rdset, wrset;
+		int flags, fdmax;
+
+		flags = fcntl(stream_fs->fd, F_GETFL);
+		fcntl(stream_fs->fd, F_SETFL, flags | O_NONBLOCK);
+		fdmax = MAX(stream_fs->fd, cancel_fd)+1;
+		do {
+			FD_ZERO(&rdset);
+			FD_ZERO(&wrset);
+			FD_SET(stream_fs->fd, &wrset);
+			FD_SET(cancel_fd, &rdset);
+			select(fdmax, &rdset, &wrset, 0, NULL);
+			if (FD_ISSET(cancel_fd, &rdset)) {
+				fcntl(stream_fs->fd, F_SETFL, flags);
+				errno = EINTR;
+				return -1;
+			}
+			v = write(stream_fs->fd, buffer+written, n-written);
+			if (v>0)
+				written += v;
+		} while (v != -1 && written < n);
+		fcntl(stream_fs->fd, F_SETFL, flags);
+	}
 
 	if (written > 0)
 		seekable->position += written;
@@ -258,7 +320,11 @@ stream_flush (CamelStream *stream)
 static int
 stream_close (CamelStream *stream)
 {
-	return close(((CamelStreamFs *)stream)->fd);
+	if (close (((CamelStreamFs *)stream)->fd) == -1)
+		return -1;
+	
+	((CamelStreamFs *)stream)->fd = -1;
+	return 0;
 }
 
 static off_t
