@@ -28,6 +28,7 @@ extern "C" {
 #endif /* __cplusplus }*/
 
 typedef struct _CamelAddress CamelAddress;
+typedef struct _header_content_type CamelContentType;
 typedef struct _CamelDataWrapper CamelDataWrapper;
 typedef struct _CamelException CamelException;
 typedef struct _CamelFolder CamelFolder;
@@ -35,6 +36,8 @@ typedef struct _CamelFolderSearch CamelFolderSearch;
 typedef struct _CamelFolderSummary CamelFolderSummary;
 typedef struct _CamelInternetAddress CamelInternetAddress;
 typedef struct _CamelMedium CamelMedium;
+typedef struct _CamelMessageContentInfo CamelMessageContentInfo;
+typedef struct _CamelMessageInfo CamelMessageInfo;
 typedef struct _CamelMimeFilter CamelMimeFilter;
 typedef struct _CamelMimeFilterBasic CamelMimeFilterBasic;
 typedef struct _CamelMimeFilterBestenc CamelMimeFilterBestenc;
@@ -68,6 +71,3 @@ typedef struct _CamelTransport CamelTransport;
 #endif /* __cplusplus */
 
 #endif /* CAMEL_TYPES_H */
-
-
-
