@@ -59,9 +59,6 @@
  *
  * The port, if present, must be numeric.
  * 
- * If nothing but the protocol (and the ":") is present, the "empty"
- * flag will be set on the returned URL.
- *
  * Return value: a CamelURL structure containing the URL items.
  **/
 CamelURL *
@@ -101,8 +98,7 @@ camel_url_new (const char *url_string, CamelException *ex)
 		if (*(colon + 1)) {
 			url->path = g_strdup (colon + 1);
 			camel_url_decode (url->path);
-		} else
-			url->empty = TRUE;
+		}
 		return url;
 	}
 
@@ -185,7 +181,7 @@ camel_url_to_string (CamelURL *url, gboolean show_passwd)
 	if (url->user)
 		user = camel_url_encode (url->user, TRUE, ":;@/");
 	
-	if (url->authmech)
+	if (url->authmech && *url->authmech)
 		authmech = camel_url_encode (url->authmech, TRUE, ":@/");
 	
 	if (show_passwd && url->passwd)
