@@ -33,6 +33,7 @@ extern "C" {
 #pragma }
 #endif /* __cplusplus }*/
 
+#include "camel-imap-types.h"
 #include "camel-folder.h"
 #include <camel/camel-folder-search.h>
 
@@ -41,12 +42,13 @@ extern "C" {
 #define CAMEL_IMAP_FOLDER_CLASS(k) (CAMEL_CHECK_CLASS_CAST ((k), CAMEL_IMAP_FOLDER_TYPE, CamelImapFolderClass))
 #define CAMEL_IS_IMAP_FOLDER(o)    (CAMEL_CHECK_TYPE((o), CAMEL_IMAP_FOLDER_TYPE))
 
-typedef struct {
+struct _CamelImapFolder {
 	CamelFolder parent_object;
 
-	CamelFolderSummary *summary;
-	int exists;
-} CamelImapFolder;
+	struct _CamelImapFolderPrivate *priv;
+
+	CamelFolderSearch *search;
+};
 
 
 typedef struct {
@@ -64,6 +66,10 @@ CamelFolder *camel_imap_folder_new (CamelStore *parent,
 				    const char *summary_file,
 				    CamelException *ex);
 
+void camel_imap_folder_selected (CamelFolder *folder,
+				 CamelImapResponse *response,
+				 CamelException *ex);
+
 void camel_imap_folder_changed (CamelFolder *folder, int exists,
 				GArray *expunged, CamelException *ex);
 
