@@ -36,16 +36,19 @@ static gint imap_url_equal (gconstpointer a, gconstpointer b);
 
 static CamelProvider imap_provider = {
 	"imap",
-	"IMAPv4",
+	N_("IMAPv4"),
 
-	"For reading and storing mail on IMAP servers.",
+	N_("For reading and storing mail on IMAP servers."),
 
 	"mail",
 
-	CAMEL_PROVIDER_IS_REMOTE | CAMEL_PROVIDER_IS_SOURCE | CAMEL_PROVIDER_IS_STORAGE,
+	CAMEL_PROVIDER_IS_REMOTE | CAMEL_PROVIDER_IS_SOURCE |
+	CAMEL_PROVIDER_IS_STORAGE,
+
+	CAMEL_URL_NEED_USER | CAMEL_URL_NEED_HOST |
+	CAMEL_URL_ALLOW_PATH | CAMEL_URL_ALLOW_AUTH,
 
 	{ 0, 0 },
-	{ 143, 0 },
 
 	NULL
 };
