@@ -300,22 +300,15 @@ static CamelServiceAuthType password_authtype = {
 };
 
 static GList *
-nntp_store_query_auth_types_generic (CamelService *service, CamelException *ex)
+nntp_store_query_auth_types (CamelService *service, gboolean connect, CamelException *ex)
 {
 	GList *prev;
 	
-	prev = CAMEL_SERVICE_CLASS (remote_store_class)->query_auth_types_generic (service, ex);
+	g_warning ("nntp::query_auth_types: not implemented. Defaulting.");
+	prev = CAMEL_SERVICE_CLASS (remote_store_class)->query_auth_types (service, connect, ex);
 	return g_list_prepend (prev, &password_authtype);
 }
 
-static GList *
-nntp_store_query_auth_types_connected (CamelService *service, CamelException *ex)
-{
-	g_warning ("nntp::query_auth_types_connected: not implemented. Defaulting.");
-	/* FIXME: use the classfunc instead of the local? */
-	return nntp_store_query_auth_types_generic (service, ex);
-}
-
 static CamelFolder *
 nntp_store_get_folder (CamelStore *store, const gchar *folder_name,
 		       guint32 flags, CamelException *ex)
@@ -366,7 +359,6 @@ build_folder_info (CamelNNTPStore *nntp_store, CamelFolderInfo **root,
 			node->name = g_strdup (node_name);
 			node->full_name = g_strdup (node_full_name);
 			node->url = NULL;
-			node->message_count = -1;
 			node->unread_message_count = -1;
 
 			if (parent) {
@@ -406,8 +398,7 @@ build_folder_info (CamelNNTPStore *nntp_store, CamelFolderInfo **root,
 						  url->user ? "@" : "",
 						  url->host, (char *)entry->group_name);
 
-		new_group->message_count = entry->high - entry->low;
-		new_group->unread_message_count = (new_group->message_count - 
+		new_group->unread_message_count = (entry->high - entry->low - 
 						   camel_nntp_newsrc_get_num_articles_read (nntp_store->newsrc, entry->group_name));
 
 		if (parent) {
@@ -461,8 +452,7 @@ build_folder_info_from_grouplist (CamelNNTPStore *nntp_store, const char *top)
 						   url->user ? "@" : "",
 						   url->host, (char *)entry->group_name);
 
-			fi->message_count = entry->high - entry->low;
-			fi->unread_message_count = (fi->message_count - 
+			fi->unread_message_count = (entry->high - entry->low - 
 						    camel_nntp_newsrc_get_num_articles_read (
 							     nntp_store->newsrc, entry->group_name));
 
@@ -527,7 +517,7 @@ nntp_store_get_folder_info (CamelStore *store, const char *top,
 						   url->user ? "@" : "",
 						   url->host, (char *)names->pdata[i]);
 			/* FIXME */
-			fi->message_count = fi->unread_message_count = -1;
+			fi->unread_message_count = -1;
 
 			if (last)
 				last->sibling = fi;
@@ -547,7 +537,7 @@ nntp_store_get_folder_info (CamelStore *store, const char *top,
 		fi->full_name = g_strdup (top);
 		fi->url = g_strdup_printf ("nntp://%s/%s", url->host, top);
 		/* FIXME */
-		fi->message_count = fi->unread_message_count = -1;
+		fi->unread_message_count = -1;
 
 		return fi;
 	}
@@ -607,8 +597,7 @@ camel_nntp_store_class_init (CamelNNTPStoreClass *camel_nntp_store_class)
 	/* virtual method overload */
 	camel_service_class->connect = nntp_store_connect;
 	camel_service_class->disconnect = nntp_store_disconnect;
-	camel_service_class->query_auth_types_generic = nntp_store_query_auth_types_generic;
-	camel_service_class->query_auth_types_connected = nntp_store_query_auth_types_connected;
+	camel_service_class->query_auth_types = nntp_store_query_auth_types;
 	camel_service_class->get_name = nntp_store_get_name;
 
 	camel_store_class->get_folder = nntp_store_get_folder;
