@@ -48,6 +48,7 @@
 #include "camel-stream-fs.h"
 #include "camel-session.h"
 #include "camel-exception.h"
+#include <gal/util/e-util.h>
 
 #define d(x) x
 
@@ -55,16 +56,15 @@
 #define SMTP_PORT 25
 
 /* camel smtp transport class prototypes */
-static gboolean _can_send (CamelTransport *transport, CamelMedium *message);
-static gboolean _send (CamelTransport *transport, CamelMedium *message, CamelException *ex);
-static gboolean _send_to (CamelTransport *transport, CamelMedium *message, GList *recipients, CamelException *ex);
+static gboolean smtp_can_send (CamelTransport *transport, CamelMedium *message);
+static gboolean smtp_send (CamelTransport *transport, CamelMedium *message, CamelException *ex);
+static gboolean smtp_send_to (CamelTransport *transport, CamelMedium *message, GList *recipients, CamelException *ex);
 
 /* support prototypes */
 static gboolean smtp_connect (CamelService *service, CamelException *ex);
 static gboolean smtp_disconnect (CamelService *service, gboolean clean, CamelException *ex);
-static GList *esmtp_get_authtypes(gchar *buffer);
-static GList *query_auth_types_connected (CamelService *service, CamelException *ex);
-static GList *query_auth_types_generic (CamelService *service, CamelException *ex);
+static GList *esmtp_get_authtypes (gchar *buffer);
+static GList *query_auth_types (CamelService *service, gboolean connect, CamelException *ex);
 static void free_auth_types (CamelService *service, GList *authtypes);
 static char *get_name (CamelService *service, gboolean brief);
 
@@ -93,14 +93,13 @@ camel_smtp_transport_class_init (CamelSmtpTransportClass *camel_smtp_transport_c
 	/* virtual method overload */
 	camel_service_class->connect = smtp_connect;
 	camel_service_class->disconnect = smtp_disconnect;
-	camel_service_class->query_auth_types_generic = query_auth_types_generic;
-	camel_service_class->query_auth_types_connected = query_auth_types_connected;
+	camel_service_class->query_auth_types = query_auth_types;
 	camel_service_class->free_auth_types = free_auth_types;
 	camel_service_class->get_name = get_name;
 
-	camel_transport_class->can_send = _can_send;
-	camel_transport_class->send = _send;
-	camel_transport_class->send_to = _send_to;
+	camel_transport_class->can_send = smtp_can_send;
+	camel_transport_class->send = smtp_send;
+	camel_transport_class->send_to = smtp_send_to;
 }
 
 static void
@@ -130,6 +129,59 @@ camel_smtp_transport_get_type (void)
 	return camel_smtp_transport_type;
 }
 
+static const char *
+get_smtp_error_string (int error)
+{
+	/* SMTP error codes grabbed from rfc821 */
+	switch (error) {
+	case 0:
+		/* looks like a read problem, check errno */
+		return g_strerror (errno);
+	case 500:
+		return _("Syntax error, command unrecognized");
+	case 501:
+		return _("Syntax error in parameters or arguments");
+	case 502:
+		return _("Command not implemented");
+	case 504:
+		return _("Command parameter not implemented");
+	case 211:
+		return _("System status, or system help reply");
+	case 214:
+		return _("Help message");
+	case 220:
+		return _("Service ready");
+	case 221:
+		return _("Service closing transmission channel");
+	case 421:
+		return _("Service not available, closing transmission channel");
+	case 250:
+		return _("Requested mail action okay, completed");
+	case 251:
+		return _("User not local; will forward to <forward-path>");
+	case 450:
+		return _("Requested mail action not taken: mailbox unavailable");
+	case 550:
+		return _("Requested action not taken: mailbox unavailable");
+	case 451:
+		return _("Requested action aborted: error in processing");
+	case 551:
+		return _("User not local; please try <forward-path>");
+	case 452:
+		return _("Requested action not taken: insufficient system storage");
+	case 552:
+		return _("Requested mail action aborted: exceeded storage allocation");
+	case 553:
+		return _("Requested action not taken: mailbox name not allowed");
+	case 354:
+		return _("Start mail input; end with <CRLF>.<CRLF>");
+	case 554:
+		return _("Transaction failed");
+	default:
+		return _("Unknown");
+	}
+}
+
 static gboolean
 smtp_connect (CamelService *service, CamelException *ex)
 {
@@ -183,9 +235,13 @@ smtp_connect (CamelService *service, CamelException *ex)
 		g_free (respbuf);
 		respbuf = camel_stream_buffer_read_line (CAMEL_STREAM_BUFFER (transport->istream));
 		if (!respbuf || strncmp (respbuf, "220", 3)) {
+			int error;
+			
+			error = respbuf ? atoi (respbuf) : 0;
+			g_free (respbuf);
 			camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
 					      _("Welcome response error: %s: possibly non-fatal"),
-					      g_strerror (errno));
+					      get_smtp_error_string (error));
 			return FALSE;
 		}
 		if (strstr (respbuf, "ESMTP"))
@@ -300,17 +356,7 @@ static CamelServiceAuthType cram_md5_authtype = {
 #endif
 
 static GList *
-query_auth_types_connected (CamelService *service, CamelException *ex)
-{
-	/* FIXME: Re-enable this when auth types are actually
-	 * implemented.
-	 */
-	
-	return NULL;
-}
-
-static GList *
-query_auth_types_generic (CamelService *service, CamelException *ex)
+query_auth_types (CamelService *service, gboolean connect, CamelException *ex)
 {
 	/* FIXME: Re-enable this when auth types are actually
 	 * implemented.
@@ -337,14 +383,14 @@ get_name (CamelService *service, gboolean brief)
 }
 
 static gboolean
-_can_send (CamelTransport *transport, CamelMedium *message)
+smtp_can_send (CamelTransport *transport, CamelMedium *message)
 {
 	return CAMEL_IS_MIME_MESSAGE (message);
 }
 
 static gboolean
-_send_to (CamelTransport *transport, CamelMedium *message,
-	  GList *recipients, CamelException *ex)
+smtp_send_to (CamelTransport *transport, CamelMedium *message,
+	      GList *recipients, CamelException *ex)
 {
 	CamelSmtpTransport *smtp_transport = CAMEL_SMTP_TRANSPORT (transport);
 	const CamelInternetAddress *cia;
@@ -404,7 +450,7 @@ _send_to (CamelTransport *transport, CamelMedium *message,
 }
 
 static gboolean
-_send (CamelTransport *transport, CamelMedium *message, CamelException *ex)
+smtp_send (CamelTransport *transport, CamelMedium *message, CamelException *ex)
 {
 	const CamelInternetAddress *to, *cc, *bcc;
 	GList *recipients = NULL;
@@ -441,7 +487,7 @@ _send (CamelTransport *transport, CamelMedium *message, CamelException *ex)
 			recipients = g_list_append (recipients, g_strdup (addr));
 	}
 	
-	return _send_to (transport, message, recipients, ex);
+	return smtp_send_to (transport, message, recipients, ex);
 }
 
 static gboolean
@@ -485,15 +531,18 @@ smtp_helo (CamelSmtpTransport *transport, CamelException *ex)
 		d(fprintf (stderr, "received: %s\n", respbuf ? respbuf : "(null)"));
 		
 		if (!respbuf || strncmp (respbuf, "250", 3)) {
+			int error;
+
+			error = respbuf ? atoi (respbuf) : 0;
 			g_free (respbuf);
 			camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
 					      _("HELO response error: %s: non-fatal"),
-					      g_strerror (errno));
+					      get_smtp_error_string (error));
 			return FALSE;
 		}
 		
-		if (strstr (respbuf, "8BITMIME")) {
-			d(fprintf (stderr, "This server supports 8bit\n"));
+		if (e_strstrcase (respbuf, "8BITMIME")) {
+			d(fprintf (stderr, "This server supports 8bit MIME\n"));
 			CAMEL_TRANSPORT (transport)->supports_8bit = TRUE;
 		}
 		
@@ -540,10 +589,13 @@ smtp_mail (CamelSmtpTransport *transport, const char *sender, gboolean has_8bit_
 		d(fprintf (stderr, "received: %s\n", respbuf ? respbuf : "(null)"));
 		
 		if (!respbuf || strncmp (respbuf, "250", 3)) {
+			int error;
+			
+			error = respbuf ? atoi (respbuf) : 0;
 			g_free (respbuf);
 			camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
 					      _("MAIL FROM response error: %s: mail not sent"),
-					      g_strerror (errno));
+					      get_smtp_error_string (error));
 			return FALSE;
 		}
 	} while (*(respbuf+3) == '-'); /* if we got "250-" then loop again */
@@ -581,10 +633,13 @@ smtp_rcpt (CamelSmtpTransport *transport, const char *recipient, CamelException
 		d(fprintf (stderr, "received: %s\n", respbuf ? respbuf : "(null)"));
 		
 		if (!respbuf || strncmp (respbuf, "250", 3)) {
+			int error;
+			
+			error = respbuf ? atoi (respbuf) : 0;
 			g_free (respbuf);
 			camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
 					      _("RCPT TO response error: %s: mail not sent"),
-					      g_strerror (errno));
+					      get_smtp_error_string (error));
 			return FALSE;
 		}
 	} while (*(respbuf+3) == '-'); /* if we got "250-" then loop again */
@@ -628,10 +683,13 @@ smtp_data (CamelSmtpTransport *transport, CamelMedium *message, gboolean has_8bi
 		/* we should have gotten instructions on how to use the DATA command:
 		 * 354 Enter mail, end with "." on a line by itself
 		 */
+		int error;
+			
+		error = respbuf ? atoi (respbuf) : 0;
 		g_free (respbuf);
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
 				      _("DATA response error: %s: mail not sent"),
-				      g_strerror (errno));
+				      get_smtp_error_string (error));
 		return FALSE;
 	}
 
@@ -677,11 +735,14 @@ smtp_data (CamelSmtpTransport *transport, CamelMedium *message, gboolean has_8bi
 		d(fprintf (stderr, "received: %s\n", respbuf ? respbuf : "(null)"));
 		
 		if (!respbuf || strncmp (respbuf, "250", 3)) {
+			int error;
+			
+			error = respbuf ? atoi (respbuf) : 0;
 			g_free (respbuf);
 			camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
 					      _("DATA response error: message termination: "
 						"%s: mail not sent"),
-					      g_strerror (errno));
+					      get_smtp_error_string (error));
 			return FALSE;
 		}
 	} while (*(respbuf+3) == '-'); /* if we got "250-" then loop again */
@@ -717,10 +778,13 @@ smtp_rset (CamelSmtpTransport *transport, CamelException *ex)
 		d(fprintf (stderr, "received: %s\n", respbuf ? respbuf : "(null)"));
 		
 		if (!respbuf || strncmp (respbuf, "250", 3)) {
+			int error;
+			
+			error = respbuf ? atoi (respbuf) : 0;
 			g_free (respbuf);
 			camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
 					      _("RSET response error: %s"),
-					      g_strerror (errno));
+					      get_smtp_error_string (error));
 			return FALSE;
 		}
 	} while (*(respbuf+3) == '-'); /* if we got "250-" then loop again */
@@ -756,10 +820,13 @@ smtp_quit (CamelSmtpTransport *transport, CamelException *ex)
 		d(fprintf (stderr, "received: %s\n", respbuf ? respbuf : "(null)"));
 		
 		if (!respbuf || strncmp (respbuf, "221", 3)) {
+			int error;
+			
+			error = respbuf ? atoi (respbuf) : 0;
 			g_free (respbuf);
 			camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
 					      _("QUIT response error: %s: non-fatal"),
-					      g_strerror (errno));
+					      get_smtp_error_string (error));
 			return FALSE;
 		}
 	} while (*(respbuf+3) == '-'); /* if we got "221-" then loop again */
