@@ -235,23 +235,16 @@ update (EMsgComposerAttachmentBar *bar)
 	for (p = priv->attachments; p != NULL; p = p->next) {
 		EMsgComposerAttachment *attachment;
 		const gchar *desc;
-		gchar *size_string, *label, *mime_type;
-		GMimeContentField *content_type;
+		gchar *size_string, *label;
+		CamelContentType *content_type;
 		GdkPixbuf *pixbuf;
-		gboolean image = FALSE;
+		gboolean image;
 		
 		attachment = p->data;
 		content_type = camel_mime_part_get_content_type (attachment->body);
-
-		mime_type = g_strdup_printf ("%s/%s", content_type->type,
-					     content_type->subtype);
-
 		/* Get the image out of the attachment 
 		   and create a thumbnail for it */
-		if (strcmp (content_type->type, "image") == 0)
-			image = TRUE;
-		else
-			image = FALSE;
+		image = header_content_type_is (content_type, "image", "*");
 		
 		if (image && attachment->pixbuf_cache == NULL) {
 			CamelDataWrapper *wrapper;
@@ -340,14 +333,17 @@ update (EMsgComposerAttachmentBar *bar)
 		if (image) {
 			e_icon_list_append_pixbuf (icon_list, attachment->pixbuf_cache, NULL, label);
 		} else {
+			char *mime_type;
+
+			mime_type = header_content_type_simple (content_type);
 			pixbuf = pixbuf_for_mime_type (mime_type);
+			g_free (mime_type);
 			e_icon_list_append_pixbuf (icon_list, pixbuf, 
 						   NULL, label);
 			if (pixbuf)
 				gdk_pixbuf_unref (pixbuf);
 		}
 
-		g_free (mime_type);
 		g_free (label);
 	}
 
@@ -718,30 +714,32 @@ static void
 attach_to_multipart (CamelMultipart *multipart,
 		     EMsgComposerAttachment *attachment)
 {
-	GMimeContentField *content_type;
+	CamelContentType *content_type;
 	
 	content_type = camel_mime_part_get_content_type (attachment->body);
 	
-	if (!g_strcasecmp (content_type->type, "text")) {
-		CamelStream *stream;
-		GByteArray *array;
-		guchar *text;
-		
-		array = g_byte_array_new ();
-		stream = camel_stream_mem_new_with_byte_array (array);
-		camel_data_wrapper_write_to_stream (CAMEL_DATA_WRAPPER (attachment->body), stream);
-		g_byte_array_append (array, "", 1);
-		text = array->data;
-		
-		if (is_8bit (text))
-			camel_mime_part_set_encoding (attachment->body, best_encoding (text));
-		else
-			camel_mime_part_set_encoding (attachment->body, CAMEL_MIME_PART_ENCODING_7BIT);
-		
-		camel_object_unref (CAMEL_OBJECT (stream));
-	} else if (g_strcasecmp (content_type->type, "message") != 0) {
-		camel_mime_part_set_encoding (attachment->body,
-					      CAMEL_MIME_PART_ENCODING_BASE64);
+	if (!header_content_type_is (content_type, "multipart", "*")) {
+		if (header_content_type_is (content_type, "text", "*")) {
+			CamelStream *stream;
+			GByteArray *array;
+			guchar *text;
+			
+			array = g_byte_array_new ();
+			stream = camel_stream_mem_new_with_byte_array (array);
+			camel_data_wrapper_write_to_stream (CAMEL_DATA_WRAPPER (attachment->body), stream);
+			g_byte_array_append (array, "", 1);
+			text = array->data;
+			
+			if (is_8bit (text))
+				camel_mime_part_set_encoding (attachment->body, best_encoding (text));
+			else
+				camel_mime_part_set_encoding (attachment->body, CAMEL_MIME_PART_ENCODING_7BIT);
+			
+			camel_object_unref (CAMEL_OBJECT (stream));
+		} else if (!header_content_type_is (content_type, "message", "*")) {
+			camel_mime_part_set_encoding (attachment->body,
+						      CAMEL_MIME_PART_ENCODING_BASE64);
+		}
 	}
 	
 	camel_multipart_add_part (multipart, attachment->body);
