@@ -51,7 +51,9 @@ struct _EMsgComposerHdrsPrivate {
 
 	/* The tooltips.  */
 	GtkTooltips *tooltips;
-
+	
+	GSList *from_options;
+	
 	/* Standard headers.  */
 	GtkWidget *from_entry;
 	GtkWidget *to_entry;
@@ -65,12 +67,14 @@ static GtkTableClass *parent_class = NULL;
 
 enum {
 	SHOW_ADDRESS_DIALOG,
+	SUBJECT_CHANGED,
+	HDRS_CHANGED,
 	LAST_SIGNAL
 };
 
 enum {
 	HEADER_ADDRBOOK,
-	HEADER_COMBOBOX,
+	HEADER_OPTIONMENU,
 	HEADER_ENTRYBOX
 };
 
@@ -124,58 +128,93 @@ address_button_clicked_cb (GtkButton *button,
 	CORBA_exception_free (&ev);
 }
 
-static GtkWidget *
-create_dropdown_entry (EMsgComposerHdrs *hdrs,
-		       const char *name)
+static void
+from_changed (GtkWidget *item, gpointer data)
 {
-	GtkWidget *combo;
-	GList *values = NULL;
+	EMsgComposerHdrs *hdrs = E_MSG_COMPOSER_HDRS (data);
+	
+	hdrs->account = gtk_object_get_data (GTK_OBJECT (item), "account");
+}
+
+static GtkWidget *
+create_optionmenu (EMsgComposerHdrs *hdrs,
+		   const char *name)
+{
+	GtkWidget *omenu, *menu, *first = NULL;
+	int i = 0, history = 0;
+	
+	omenu = gtk_option_menu_new ();
+	menu = gtk_menu_new ();
 	
-	combo = gtk_combo_new ();
-	gtk_combo_set_use_arrows (GTK_COMBO (combo), TRUE);
-	gtk_combo_set_case_sensitive (GTK_COMBO (combo), FALSE);
 	if (!strcmp (name, _("From:"))) {
-		GSList *ids, *stmp;
-		GList *tmp;
-		MailConfigIdentity *id;
-		char *val;
+		const GSList *accounts;
+		GtkWidget *item;
 		
-		ids = mail_config_get_identities ();
-		stmp = ids;
-		while (stmp) {
-			char *address;
-			
-			id = stmp->data;
-			g_assert (id);			
-			g_assert (id->name);
-			g_assert (id->address);
-			
-			address = camel_internet_address_format_address(id->name, id->address);
-			values = g_list_append (values, address);
-			stmp = stmp->next;
-		}
-		
-		gtk_combo_set_popdown_strings (GTK_COMBO (combo), values);
+		accounts = mail_config_get_accounts ();
+		while (accounts) {
+			const MailConfigAccount *account;
+			char *label;
+			char *native_label;
 
-		tmp = values;
-		while (tmp) {
-			g_free (tmp->data);
-			tmp = tmp->next;
-		}
-		g_list_free (values);
+			account = accounts->data;
+			
+			/* this should never ever fail */
+			if (!account || !account->name || !account->id) {
+				g_assert_not_reached ();
+				continue;
+			}
+			
+			if (strcmp (account->name, account->id->address))
+				label = g_strdup_printf ("%s <%s> (%s)", account->id->name,
+							 account->id->address, account->name);
+			else
+				label = g_strdup_printf ("%s <%s>", account->id->name, account->id->address);
+			    
 
-		id = mail_config_get_default_identity ();
-		g_assert (id);			
-		g_assert (id->name);
-		g_assert (id->address);
-		
-		val = camel_internet_address_format_address(id->name, id->address);
-		
-		e_utf8_gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (combo)->entry), val);
-		g_free (val);
+			native_label = e_utf8_to_gtk_string (GTK_WIDGET (menu), label);
+			item = gtk_menu_item_new_with_label (native_label);
+			g_free (native_label);
+			g_free (label);
+			
+			gtk_object_set_data (GTK_OBJECT (item), "account", account_copy (account));
+			gtk_signal_connect (GTK_OBJECT (item), "activate",
+					    GTK_SIGNAL_FUNC (from_changed), hdrs);
+			
+			if (account->default_account) {
+				first = item;
+				history = i;
+			}
+			
+			/* this is so we can later set which one we want */
+			hdrs->priv->from_options = g_slist_append (hdrs->priv->from_options, item);
+			
+			gtk_menu_append (GTK_MENU (menu), item);
+			gtk_widget_show (item);
+			
+			accounts = accounts->next;
+			i++;
+		}
 	}
 	
-	return combo;
+	gtk_option_menu_set_menu (GTK_OPTION_MENU (omenu), menu);
+	if (first) {
+		gtk_option_menu_set_history (GTK_OPTION_MENU (omenu), history);
+		gtk_signal_emit_by_name (GTK_OBJECT (first), "activate", hdrs);
+	}
+	
+	return omenu;
+}
+
+static void
+addressbook_entry_changed (BonoboListener    *listener,
+			   char              *event_name,
+			   CORBA_any         *arg,
+			   CORBA_Environment *ev,
+			   gpointer           user_data)
+{
+	EMsgComposerHdrs *hdrs = E_MSG_COMPOSER_HDRS (user_data);
+
+	gtk_signal_emit (GTK_OBJECT (hdrs), signals[HDRS_CHANGED]);
 }
 
 static GtkWidget *
@@ -187,6 +226,8 @@ create_addressbook_entry (EMsgComposerHdrs *hdrs,
 	Bonobo_Control corba_control;
 	GtkWidget *control_widget;
 	CORBA_Environment ev;
+	BonoboControlFrame *cf;
+	Bonobo_PropertyBag pb = CORBA_OBJECT_NIL;
 
 	priv = hdrs->priv;
 	corba_select_names = priv->corba_select_names;
@@ -210,9 +251,31 @@ create_addressbook_entry (EMsgComposerHdrs *hdrs,
 
 	control_widget = bonobo_widget_new_control_from_objref (corba_control, CORBA_OBJECT_NIL);
 
+	cf = bonobo_widget_get_control_frame (BONOBO_WIDGET (control_widget));
+	pb = bonobo_control_frame_get_control_property_bag (cf, NULL);
+
+	bonobo_event_source_client_add_listener (pb, addressbook_entry_changed,
+						 "Bonobo/Property:change:entry_changed",
+						 NULL, hdrs);
+
 	return control_widget;
 }
 
+static void
+entry_changed (GtkWidget *entry, EMsgComposerHdrs *hdrs)
+{
+	gchar *tmp;
+	gchar *subject;
+
+	tmp = e_msg_composer_hdrs_get_subject (hdrs);
+	subject = e_utf8_to_gtk_string (GTK_WIDGET (entry), tmp);
+
+	gtk_signal_emit (GTK_OBJECT (hdrs), signals[SUBJECT_CHANGED], subject);
+	g_free (tmp);
+
+	gtk_signal_emit (GTK_OBJECT (hdrs), signals[HDRS_CHANGED]);
+}
+
 static GtkWidget *
 add_header (EMsgComposerHdrs *hdrs,
 	    const gchar *name,
@@ -229,7 +292,7 @@ add_header (EMsgComposerHdrs *hdrs,
 	
 	if (type == HEADER_ADDRBOOK) {
 		label = gtk_button_new_with_label (name);
-		GTK_OBJECT_UNSET_FLAGS(label, GTK_CAN_FOCUS);
+		GTK_OBJECT_UNSET_FLAGS (label, GTK_CAN_FOCUS);
 		gtk_signal_connect (GTK_OBJECT (label), "clicked",
 				    GTK_SIGNAL_FUNC (address_button_clicked_cb),
 				    hdrs);
@@ -252,16 +315,19 @@ add_header (EMsgComposerHdrs *hdrs,
 	case HEADER_ADDRBOOK:
 		entry = create_addressbook_entry (hdrs, name);
 		break;
-	case HEADER_COMBOBOX:
-		entry = create_dropdown_entry (hdrs, name);
+	case HEADER_OPTIONMENU:
+		entry = create_optionmenu (hdrs, name);
 		break;
+	case HEADER_ENTRYBOX:
 	default:
 		entry = e_entry_new ();
-		gtk_object_set(GTK_OBJECT(entry),
-			       "editable", TRUE,
-			       "use_ellipsis", TRUE,
-			       "allow_newlines", FALSE,
-			       NULL);
+		gtk_object_set (GTK_OBJECT(entry),
+				"editable", TRUE,
+				"use_ellipsis", TRUE,
+				"allow_newlines", FALSE,
+				NULL);
+		gtk_signal_connect (GTK_OBJECT (entry), "changed",
+				    GTK_SIGNAL_FUNC (entry_changed), hdrs);
 	}
 
 	if (entry != NULL) {
@@ -291,7 +357,7 @@ setup_headers (EMsgComposerHdrs *hdrs)
 		(hdrs, _("From:"), 
 		 _("Enter the identity you wish to send this message from"),
 		 NULL,
-		 HEADER_COMBOBOX);
+		 HEADER_OPTIONMENU);
 	priv->to_entry = add_header
 		(hdrs, _("To:"), 
 		 _("Enter the recipients of the message"),
@@ -325,6 +391,7 @@ destroy (GtkObject *object)
 {
 	EMsgComposerHdrs *hdrs;
 	EMsgComposerHdrsPrivate *priv;
+	GSList *l;
 
 	hdrs = E_MSG_COMPOSER_HDRS (object);
 	priv = hdrs->priv;
@@ -338,7 +405,19 @@ destroy (GtkObject *object)
 	}
 
 	gtk_object_destroy (GTK_OBJECT (priv->tooltips));
-
+	
+	l = priv->from_options;
+	while (l) {
+		MailConfigAccount *account;
+		GtkWidget *item = l->data;
+		
+		account = gtk_object_get_data (GTK_OBJECT (item), "account");
+		account_destroy (account);
+		
+		l = l->next;
+	}
+	g_slist_free (priv->from_options);
+	
 	if (GTK_OBJECT_CLASS (parent_class)->destroy != NULL)
 		(* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
 }
@@ -363,6 +442,25 @@ class_init (EMsgComposerHdrsClass *class)
 				gtk_marshal_NONE__NONE,
 				GTK_TYPE_NONE, 0);
 
+	signals[SUBJECT_CHANGED] =
+		gtk_signal_new ("subject_changed",
+				GTK_RUN_LAST,
+				object_class->type,
+				GTK_SIGNAL_OFFSET (EMsgComposerHdrsClass,
+						   subject_changed),
+				gtk_marshal_NONE__STRING,
+				GTK_TYPE_NONE,
+				1, GTK_TYPE_STRING);
+
+	signals[HDRS_CHANGED] =
+		gtk_signal_new ("hdrs_changed",
+				GTK_RUN_LAST,
+				object_class->type,
+				GTK_SIGNAL_OFFSET (EMsgComposerHdrsClass,
+						   hdrs_changed),
+				gtk_marshal_NONE__NONE,
+				GTK_TYPE_NONE, 0);
+
 	gtk_object_class_add_signals (object_class, signals, LAST_SIGNAL);
 }
 
@@ -374,18 +472,22 @@ init (EMsgComposerHdrs *hdrs)
 	priv = g_new (EMsgComposerHdrsPrivate, 1);
 
 	priv->corba_select_names = CORBA_OBJECT_NIL;
-
+	
+	priv->from_options  = NULL;
+	
 	priv->from_entry    = NULL;
 	priv->to_entry      = NULL;
 	priv->cc_entry      = NULL;
 	priv->bcc_entry     = NULL;
 	priv->subject_entry = NULL;
-
+	
 	priv->tooltips = gtk_tooltips_new ();
 
 	priv->num_hdrs = 0;
-
+	
 	hdrs->priv = priv;
+	
+	hdrs->account = NULL;
 }
 
 
@@ -435,25 +537,21 @@ e_msg_composer_hdrs_new (void)
 static void
 set_recipients (CamelMimeMessage *msg, GtkWidget *entry_widget, const gchar *type)
 {
-	char *s, *u;
 	CamelInternetAddress *addr;
-
-	bonobo_widget_get_property (BONOBO_WIDGET (entry_widget), "text", &s, NULL);
-
-	/* FIXME: This is just a temporary workaround, the widget should grok it */
-        u = e_utf8_from_gtk_string(entry_widget, s);
-
-	addr = camel_internet_address_new();
-	camel_address_unformat((CamelAddress *)addr, u);
-
+	char *string;
+	
+	bonobo_widget_get_property (BONOBO_WIDGET (entry_widget), "text", &string, NULL);
+	
+	addr = camel_internet_address_new ();
+	camel_address_unformat (CAMEL_ADDRESS (addr), string);
+	
 	/* TODO: In here, we could cross-reference the names with an alias book
 	   or address book, it should be sufficient for unformat to do the parsing too */
-
-	camel_mime_message_set_recipients(msg, type, addr);
-
-	camel_object_unref((CamelObject *)addr);
-	g_free(s);
-	g_free(u);
+	
+	camel_mime_message_set_recipients (msg, type, addr);
+	
+	camel_object_unref (CAMEL_OBJECT (addr));
+	g_free (string);
 }
 
 void
@@ -461,7 +559,6 @@ e_msg_composer_hdrs_to_message (EMsgComposerHdrs *hdrs,
 				CamelMimeMessage *msg)
 {
 	gchar *subject;
-	char *fromstr;
 	CamelInternetAddress *from;
 	
 	g_return_if_fail (hdrs != NULL);
@@ -472,14 +569,10 @@ e_msg_composer_hdrs_to_message (EMsgComposerHdrs *hdrs,
 	subject = e_msg_composer_hdrs_get_subject (hdrs);
 	camel_mime_message_set_subject (msg, subject);
 	g_free (subject);
-
-	/* get_from() should probably return a CamelInternetAddress anyway */
-	fromstr = e_msg_composer_hdrs_get_from (hdrs);
-	from = camel_internet_address_new();
-	camel_address_unformat((CamelAddress *)from, fromstr);
+	
+	from = e_msg_composer_hdrs_get_from (hdrs);
 	camel_mime_message_set_from (msg, from);
-	camel_object_unref((CamelObject *)from);
-	g_free(fromstr);
+	camel_object_unref (CAMEL_OBJECT (from));
 	
 	set_recipients (msg, hdrs->priv->to_entry, CAMEL_RECIPIENT_TYPE_TO);
 	set_recipients (msg, hdrs->priv->cc_entry, CAMEL_RECIPIENT_TYPE_CC);
@@ -506,17 +599,38 @@ set_entry (BonoboWidget *bonobo_widget,
 	g_string_free (string, TRUE);
 }
 
+
+/* FIXME: yea, this could be better... but it's doubtful it'll be used much */
 void
-e_msg_composer_hdrs_set_from (EMsgComposerHdrs *hdrs,
-			      const char *from)
+e_msg_composer_hdrs_set_from_account (EMsgComposerHdrs *hdrs,
+				      const char *account_name)
 {
-	GtkEntry *entry;
+	GtkOptionMenu *omenu;
+	GtkWidget *item;
+	GSList *l;
+	int i = 0;
 	
 	g_return_if_fail (hdrs != NULL);
 	g_return_if_fail (E_IS_MSG_COMPOSER_HDRS (hdrs));
 	
-	entry = GTK_ENTRY (GTK_COMBO (hdrs->priv->from_entry)->entry);
-	e_utf8_gtk_entry_set_text (entry, from);
+	omenu = GTK_OPTION_MENU (hdrs->priv->from_entry);
+	
+	/* find the item that represents the account and activate it */
+	l = hdrs->priv->from_options;
+	while (l) {
+		MailConfigAccount *account;
+		item = l->data;
+		
+		account = gtk_object_get_data (GTK_OBJECT (item), "account");
+		if (!strcmp (account_name, account->name)) {
+			gtk_option_menu_set_history (omenu, i);
+			gtk_signal_emit_by_name (GTK_OBJECT (item), "activate", hdrs);
+			return;
+		}
+		
+		l = l->next;
+		i++;
+	}
 }
 
 void
@@ -562,14 +676,26 @@ e_msg_composer_hdrs_set_subject (EMsgComposerHdrs *hdrs,
 			NULL);
 }
 
-/* FIXME: This should probably return a CamelInternetAddress */
-char *
+
+CamelInternetAddress *
 e_msg_composer_hdrs_get_from (EMsgComposerHdrs *hdrs)
 {
+	const MailConfigAccount *account;
+	CamelInternetAddress *addr;
+	
 	g_return_val_if_fail (hdrs != NULL, NULL);
 	g_return_val_if_fail (E_IS_MSG_COMPOSER_HDRS (hdrs), NULL);
 	
-	return e_utf8_gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (hdrs->priv->from_entry)->entry));
+	account = hdrs->account;
+	if (!account || !account->id) {
+		/* FIXME: perhaps we should try the default account? */
+		return NULL;
+	}
+	
+	addr = camel_internet_address_new ();
+	camel_internet_address_add (addr, account->id->name, account->id->address);
+	
+	return addr;
 }
 
 /* FIXME this is currently unused and broken.  */
@@ -608,7 +734,6 @@ e_msg_composer_hdrs_get_bcc (EMsgComposerHdrs *hdrs)
 	return NULL;
 }
 
-/* FIXME: This is just changed to return allooc'd mem to be consistant with get_from */
 char *
 e_msg_composer_hdrs_get_subject (EMsgComposerHdrs *hdrs)
 {
@@ -620,7 +745,7 @@ e_msg_composer_hdrs_get_subject (EMsgComposerHdrs *hdrs)
 	gtk_object_get (GTK_OBJECT (hdrs->priv->subject_entry),
 			"text", &subject, NULL);
 	
-	return g_strdup (subject);
+	return subject;
 }
 
 
