@@ -26,6 +26,7 @@
 
 #include <gnome.h>
 #include <camel/camel-mime-message.h>
+#include <mail/mail-config.h>
 
 #ifdef __cplusplus
 extern "C" {
@@ -47,12 +48,20 @@ struct _EMsgComposerHdrs {
 	GtkTable parent;
 
 	EMsgComposerHdrsPrivate *priv;
+	
+	const MailConfigAccount *account;
+	
+	gboolean has_changed;
 };
 
 struct _EMsgComposerHdrsClass {
 	GtkTableClass parent_class;
 
 	void (* show_address_dialog) (EMsgComposerHdrs *hdrs);
+
+	void (* subject_changed) (EMsgComposerHdrs *hdrs, gchar *subject);
+
+	void (* hdrs_changed) (EMsgComposerHdrs *hdrs);
 };
 
 
@@ -62,8 +71,8 @@ GtkWidget  *e_msg_composer_hdrs_new                (void);
 void        e_msg_composer_hdrs_to_message         (EMsgComposerHdrs *hdrs,
 						    CamelMimeMessage *msg);
 
-void        e_msg_composer_hdrs_set_from           (EMsgComposerHdrs *hdrs,
-						    const char       *from);
+void        e_msg_composer_hdrs_set_from_account   (EMsgComposerHdrs *hdrs,
+						    const char *account_name);
 void        e_msg_composer_hdrs_set_to             (EMsgComposerHdrs *hdrs,
 						    const GList      *to_list);
 void        e_msg_composer_hdrs_set_cc             (EMsgComposerHdrs *hdrs,
@@ -73,7 +82,7 @@ void        e_msg_composer_hdrs_set_bcc            (EMsgComposerHdrs *hdrs,
 void        e_msg_composer_hdrs_set_subject        (EMsgComposerHdrs *hdrs,
 						    const char       *subject);
 
-char       *e_msg_composer_hdrs_get_from           (EMsgComposerHdrs *hdrs);
+CamelInternetAddress *e_msg_composer_hdrs_get_from (EMsgComposerHdrs *hdrs);
 GList      *e_msg_composer_hdrs_get_to             (EMsgComposerHdrs *hdrs);
 GList      *e_msg_composer_hdrs_get_cc             (EMsgComposerHdrs *hdrs);
 GList      *e_msg_composer_hdrs_get_bcc            (EMsgComposerHdrs *hdrs);
