@@ -46,6 +46,7 @@
 #include "camel-charset-map.h"
 
 #include "mail/mail.h"
+#include "mail/mail-crypto.h"
 #include "mail/mail-tools.h"
 #include "mail/mail-threads.h"
 
@@ -239,16 +240,16 @@ build_message (EMsgComposer *composer)
 	EMsgComposerAttachmentBar *attachment_bar =
 		E_MSG_COMPOSER_ATTACHMENT_BAR (composer->attachment_bar);
 	MsgFormat type = MSG_FORMAT_ALTERNATIVE;
+	CamelInternetAddress *from;
 	CamelMimeMessage *new;
 	CamelMultipart *body = NULL;
 	CamelMimePart *part;
-	gchar *from = NULL;
 	gboolean plain_e8bit = FALSE, html_e8bit = FALSE;
+	CamelException ex;
 	char *html = NULL, *plain = NULL;
 	char *content_type = NULL;
 	int i;
 	
-
 	if (composer->persist_stream_interface == CORBA_OBJECT_NIL)
 		return NULL;
 	
@@ -260,17 +261,17 @@ build_message (EMsgComposer *composer)
 	/* get and/or set the From field */
 	from = e_msg_composer_hdrs_get_from (E_MSG_COMPOSER_HDRS (composer->hdrs));
 	if (!from) {
-		const MailConfigIdentity *id = NULL;
-		CamelInternetAddress *ciaddr;
+		const MailConfigAccount *account = NULL;
 		
-		id = mail_config_get_default_identity ();
-		ciaddr = camel_internet_address_new ();
-		camel_internet_address_add (ciaddr, id->name, id->address);
-		from = camel_address_encode (CAMEL_ADDRESS (ciaddr));
-		e_msg_composer_hdrs_set_from (E_MSG_COMPOSER_HDRS (composer->hdrs), from);
-		camel_object_unref (CAMEL_OBJECT (ciaddr));
+		account = mail_config_get_default_account ();
+		
+		/* if !account then we have mucho problemos, amigo */
+		if (!account)
+			return NULL;
+		
+		e_msg_composer_hdrs_set_from_account (E_MSG_COMPOSER_HDRS (composer->hdrs), account->name);
 	}
-	g_free (from);
+	camel_object_unref (CAMEL_OBJECT (from));
 	
 	new = camel_mime_message_new ();
 	
@@ -286,10 +287,9 @@ build_message (EMsgComposer *composer)
 	/* the component has probably died */ 
 	if (plain == NULL)
 		return NULL;
-
+	
 	plain_e8bit = is_8bit (plain);
-	content_type = best_content (plain);
-
+	
 	if (type != MSG_FORMAT_PLAIN) {
 		e_msg_composer_clear_inlined_table (composer);
 		html = get_text (composer->persist_stream_interface, "text/html");
@@ -298,7 +298,6 @@ build_message (EMsgComposer *composer)
 		/* the component has probably died */
 		if (html == NULL) {
 			g_free (plain);
-			g_free (content_type);
 			return NULL;
 		}
 	}
@@ -310,14 +309,15 @@ build_message (EMsgComposer *composer)
 		camel_multipart_set_boundary (body, NULL);
 		
 		part = camel_mime_part_new ();
-
+		
+		content_type = best_content (plain);
 		camel_mime_part_set_content (part, plain, strlen (plain), content_type);
-
+		g_free (content_type);
+		
 		if (plain_e8bit)
 			camel_mime_part_set_encoding (part, best_encoding (plain));		
 
 		g_free (plain);
-		g_free (content_type);
 		camel_multipart_add_part (body, part);
 		camel_object_unref (CAMEL_OBJECT (part));
 		
@@ -351,6 +351,7 @@ build_message (EMsgComposer *composer)
 		}
 
 		g_free (html);
+		
 		camel_multipart_add_part (body, part);
 		camel_object_unref (CAMEL_OBJECT (part));
 	}
@@ -369,13 +370,14 @@ build_message (EMsgComposer *composer)
 			camel_object_unref (CAMEL_OBJECT (body));
 			break;
 		case MSG_FORMAT_PLAIN:
-			camel_mime_part_set_content (part, plain, strlen (plain), best_content (plain));
-
+			content_type = best_content (plain);
+			camel_mime_part_set_content (part, plain, strlen (plain), content_type);
+			g_free (content_type);
+			
 			if (plain_e8bit)
 				camel_mime_part_set_encoding (part, best_encoding (plain));
-
+			
 			g_free (plain);
-			g_free (content_type);
 			break;
 		}
 		camel_multipart_add_part (multipart, part);
@@ -383,28 +385,109 @@ build_message (EMsgComposer *composer)
 		
 		e_msg_composer_attachment_bar_to_multipart (attachment_bar, multipart);
 		
-		camel_medium_set_content_object (CAMEL_MEDIUM (new), CAMEL_DATA_WRAPPER (multipart));
+		part = camel_mime_part_new ();
+		camel_medium_set_content_object (CAMEL_MEDIUM (part), CAMEL_DATA_WRAPPER (multipart));
 		camel_object_unref (CAMEL_OBJECT (multipart));
 	} else {
 		switch (type) {
 		case MSG_FORMAT_ALTERNATIVE:
-			camel_medium_set_content_object (CAMEL_MEDIUM (new), CAMEL_DATA_WRAPPER (body));
+			part = camel_mime_part_new ();
+			
+			camel_medium_set_content_object (CAMEL_MEDIUM (part), CAMEL_DATA_WRAPPER (body));
 			camel_object_unref (CAMEL_OBJECT (body));
 			break;
-
 		case MSG_FORMAT_PLAIN:
-			camel_mime_part_set_content (CAMEL_MIME_PART (new), plain, strlen (plain), best_content (plain));
-
-			if (plain_e8bit)
-				camel_mime_part_set_encoding (CAMEL_MIME_PART (new), best_encoding (plain));
-
-			g_free (plain);
+			part = camel_mime_part_new ();
+			
+			content_type = best_content (plain);
+			camel_mime_part_set_content (CAMEL_MIME_PART (part), plain, strlen (plain), content_type);
 			g_free (content_type);
+			
+			if (plain_e8bit)
+				camel_mime_part_set_encoding (part, best_encoding (plain));
+			
+			g_free (plain);
+			
 			break;
 		}
 	}
 	
+	camel_exception_init (&ex);
+	
+	if (composer->pgp_sign) {
+		/* FIXME: should use the PGP key id rather than email address */
+		const char *pgpid;
+		
+		camel_exception_init (&ex);
+		from = e_msg_composer_hdrs_get_from (E_MSG_COMPOSER_HDRS (composer->hdrs));
+		camel_internet_address_get (from, 0, NULL, &pgpid);
+		pgp_mime_part_sign (&part, pgpid, PGP_HASH_TYPE_SHA1,
+				    &ex);
+		camel_object_unref (CAMEL_OBJECT (from));
+		if (camel_exception_is_set (&ex))
+			goto exception;
+	}
+	
+	if (composer->pgp_encrypt) {
+		/* FIXME: recipients should be an array of key ids rather than email addresses */
+		const CamelInternetAddress *addr;
+		const char *address;
+		GPtrArray *recipients;
+		int i, len;
+		
+		camel_exception_init (&ex);
+		recipients = g_ptr_array_new ();
+		
+		addr = camel_mime_message_get_recipients (new, CAMEL_RECIPIENT_TYPE_TO);
+		len = camel_address_length (CAMEL_ADDRESS (addr));
+		for (i = 0; i < len; i++) {
+			camel_internet_address_get (addr, i, NULL, &address);
+			g_ptr_array_add (recipients, g_strdup (address));
+		}
+		
+		addr = camel_mime_message_get_recipients (new, CAMEL_RECIPIENT_TYPE_CC);
+		len = camel_address_length (CAMEL_ADDRESS (addr));
+		for (i = 0; i < len; i++) {
+			camel_internet_address_get (addr, i, NULL, &address);
+			g_ptr_array_add (recipients, g_strdup (address));
+		}
+		
+		addr = camel_mime_message_get_recipients (new, CAMEL_RECIPIENT_TYPE_BCC);
+		len = camel_address_length (CAMEL_ADDRESS (addr));
+		for (i = 0; i < len; i++) {
+			camel_internet_address_get (addr, i, NULL, &address);
+			g_ptr_array_add (recipients, g_strdup (address));
+		}
+		
+		pgp_mime_part_encrypt (&part, recipients, &ex);
+		for (i = 0; i < recipients->len; i++)
+			g_free (recipients->pdata[i]);
+		g_ptr_array_free (recipients, TRUE);
+		if (camel_exception_is_set (&ex))
+			goto exception;
+	}
+	
+	camel_medium_set_content_object (CAMEL_MEDIUM (new),
+					 camel_medium_get_content_object (CAMEL_MEDIUM (part)));
+	camel_object_unref (CAMEL_OBJECT (part));
+	
 	return new;
+	
+ exception:
+	
+	camel_object_unref (CAMEL_OBJECT (part));
+	camel_object_unref (CAMEL_OBJECT (new));
+	
+	if (camel_exception_is_set (&ex)) {
+		GtkWidget *dialog;
+		
+		dialog = gnome_error_dialog_parented (camel_exception_get_description (&ex),
+						      GTK_WINDOW (composer));
+		gnome_dialog_run_and_close (GNOME_DIALOG (dialog));
+		camel_exception_clear (&ex);
+	}
+	
+	return NULL;
 }
 
 static char *
@@ -478,8 +561,8 @@ prepare_engine (EMsgComposer *composer)
 	CORBA_exception_free (&ev);
 }
 
-static void
-mark_orig_text (EMsgComposer *composer)
+void
+e_msg_composer_mark_text_orig (EMsgComposer *composer)
 {
 	g_assert (composer);
 	g_assert (E_IS_MSG_COMPOSER (composer));
@@ -551,8 +634,6 @@ set_editor_text (EMsgComposer *composer, const char *sig_file, const char *text)
 	Bonobo_Unknown_unref (persist, &ev);
 	CORBA_exception_free (&ev);
 	bonobo_object_unref (BONOBO_OBJECT(stream));
-
-	mark_orig_text (composer);
 }
 
 
@@ -710,6 +791,19 @@ static const mail_operation_spec op_save_draft = {
 	cleanup_save_draft
 };
 
+static void
+menu_file_save_draft_cb (BonoboUIComponent *uic, void *data, const char *path)
+{
+	EMsgComposer *composer;
+	save_draft_input_t *input;
+	
+	composer = E_MSG_COMPOSER (data);
+	
+	input = g_new0 (save_draft_input_t, 1);
+	input->composer = composer;
+	mail_operation_queue (&op_save_draft, input, TRUE);
+}
+
 static void
 exit_dialog_cb (int reply, EMsgComposer *composer)
 {
@@ -728,7 +822,7 @@ exit_dialog_cb (int reply, EMsgComposer *composer)
 	default:
 	}
 }
- 
+
 static void
 do_exit (EMsgComposer *composer)
 {
@@ -736,20 +830,24 @@ do_exit (EMsgComposer *composer)
 	GtkWidget *label;
 	gint button;
 	
-	dialog = gnome_dialog_new (_("Evolution"),
-				   GNOME_STOCK_BUTTON_YES,      /* Save */
-				   GNOME_STOCK_BUTTON_NO,       /* Don't save */
-				   GNOME_STOCK_BUTTON_CANCEL,   /* Cancel */
-				   NULL);
-	
-	label = gtk_label_new (_("This message has not been sent.\n\nDo you wish to save your changes?"));
-	gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (dialog)->vbox), label, TRUE, TRUE, 0);
-	gtk_widget_show (label);
-	gnome_dialog_set_parent (GNOME_DIALOG (dialog), GTK_WINDOW (composer));
-	gnome_dialog_set_default (GNOME_DIALOG (dialog), 0);
-	button = gnome_dialog_run_and_close (GNOME_DIALOG (dialog));
-	
-	exit_dialog_cb (button, composer);
+	if (composer->has_changed) {
+		dialog = gnome_dialog_new (_("Evolution"),
+					   GNOME_STOCK_BUTTON_YES,      /* Save */
+					   GNOME_STOCK_BUTTON_NO,       /* Don't save */
+					   GNOME_STOCK_BUTTON_CANCEL,   /* Cancel */
+					   NULL);
+		
+		label = gtk_label_new (_("This message has not been sent.\n\nDo you wish to save your changes?"));
+		gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (dialog)->vbox), label, TRUE, TRUE, 0);
+		gtk_widget_show (label);
+		gnome_dialog_set_parent (GNOME_DIALOG (dialog), GTK_WINDOW (composer));
+		gnome_dialog_set_default (GNOME_DIALOG (dialog), 0);
+		button = gnome_dialog_run_and_close (GNOME_DIALOG (dialog));
+		
+		exit_dialog_cb (button, composer);
+	} else {
+		gtk_widget_destroy (GTK_WIDGET (composer));
+	}
 }
 
 /* Menu callbacks.  */
@@ -1060,19 +1158,70 @@ menu_format_html_cb (BonoboUIComponent           *component,
 	e_msg_composer_set_send_html (composer, new_state);
 }
 
+static void
+menu_security_pgp_sign_cb (BonoboUIComponent           *component,
+			   const char                  *path,
+			   Bonobo_UIComponent_EventType type,
+			   const char                  *state,
+			   gpointer                     user_data)
+
+{
+	EMsgComposer *composer;
+	gboolean new_state;
+	
+	if (type != Bonobo_UIComponent_STATE_CHANGED)
+		return;
+	
+	composer = E_MSG_COMPOSER (user_data);
+	
+	new_state = atoi (state);
+	
+	if ((new_state && composer->pgp_sign) ||
+	    (!new_state && ! composer->pgp_sign))
+		return;
+	
+	e_msg_composer_set_pgp_sign (composer, new_state);
+}
+
+static void
+menu_security_pgp_encrypt_cb (BonoboUIComponent           *component,
+			      const char                  *path,
+			      Bonobo_UIComponent_EventType type,
+			      const char                  *state,
+			      gpointer                     user_data)
+
+{
+	EMsgComposer *composer;
+	gboolean new_state;
+	
+	if (type != Bonobo_UIComponent_STATE_CHANGED)
+		return;
+	
+	composer = E_MSG_COMPOSER (user_data);
+	
+	new_state = atoi (state);
+	
+	if ((new_state && composer->pgp_encrypt) ||
+	    (!new_state && ! composer->pgp_encrypt))
+		return;
+	
+	e_msg_composer_set_pgp_encrypt (composer, new_state);
+}
+
 
 static BonoboUIVerb verbs [] = {
 
-	BONOBO_UI_UNSAFE_VERB ("FileOpen",   menu_file_open_cb),
-	BONOBO_UI_UNSAFE_VERB ("FileSave",   menu_file_save_cb),
-	BONOBO_UI_UNSAFE_VERB ("FileSaveAs", menu_file_save_as_cb),
-	BONOBO_UI_UNSAFE_VERB ("FileClose",  menu_file_close_cb),
-	
-	BONOBO_UI_UNSAFE_VERB ("FileInsertFile", menu_file_insert_file_cb),
-	BONOBO_UI_UNSAFE_VERB ("FileAttach",     menu_file_add_attachment_cb),
-	
-	BONOBO_UI_UNSAFE_VERB ("FileSend",       menu_file_send_cb),
-	BONOBO_UI_UNSAFE_VERB ("FileSendLater",  menu_file_send_later_cb),
+	BONOBO_UI_VERB ("FileOpen",   menu_file_open_cb),
+	BONOBO_UI_VERB ("FileSave",   menu_file_save_cb),
+	BONOBO_UI_VERB ("FileSaveAs", menu_file_save_as_cb),
+	BONOBO_UI_VERB ("FileSaveDraft", menu_file_save_draft_cb),
+	BONOBO_UI_VERB ("FileClose",  menu_file_close_cb),
+		  
+	BONOBO_UI_VERB ("FileInsertFile", menu_file_insert_file_cb),
+	BONOBO_UI_VERB ("FileAttach",     menu_file_add_attachment_cb),
+		  
+	BONOBO_UI_VERB ("FileSend",       menu_file_send_cb),
+	BONOBO_UI_VERB ("FileSendLater",  menu_file_send_later_cb),
 	
 	BONOBO_UI_VERB_END
 };	
@@ -1096,16 +1245,30 @@ setup_ui (EMsgComposer *composer)
 			       "evolution-message-composer.xml",
 			       "evolution-message-composer");
 	
+	/* Format -> HTML */
 	bonobo_ui_component_set_prop (composer->uic, "/commands/FormatHtml",
 				      "state", composer->send_html ? "1" : "0", NULL);
 	
-	bonobo_ui_component_add_listener (
-		composer->uic, "FormatHtml",
-		menu_format_html_cb, composer);
+	bonobo_ui_component_add_listener (composer->uic, "FormatHtml",
+					  menu_format_html_cb, composer);
 	
-	bonobo_ui_component_add_listener (
-		composer->uic, "ViewAttach",
-		menu_view_attachments_activate_cb, composer);
+	/* Security -> PGP Sign */
+	bonobo_ui_component_set_prop (composer->uic, "/commands/SecurityPGPSign",
+				      "state", composer->pgp_sign ? "1" : "0", NULL);
+	
+	bonobo_ui_component_add_listener (composer->uic, "SecurityPGPSign",
+					  menu_security_pgp_sign_cb, composer);
+	
+	/* Security -> PGP Encrypt */
+	bonobo_ui_component_set_prop (composer->uic, "/commands/SecurityPGPEncrypt",
+				      "state", composer->pgp_encrypt ? "1" : "0", NULL);
+	
+	bonobo_ui_component_add_listener (composer->uic, "SecurityPGPEncrypt",
+					  menu_security_pgp_encrypt_cb, composer);
+	
+	/* View -> Attachments */
+	bonobo_ui_component_add_listener (composer->uic, "ViewAttach",
+					  menu_view_attachments_activate_cb, composer);
 }
 
 
@@ -1123,6 +1286,38 @@ attachment_bar_changed_cb (EMsgComposerAttachmentBar *bar,
 		e_msg_composer_show_attachments (composer, TRUE);
 	else
 		e_msg_composer_show_attachments (composer, FALSE);
+
+	/* Mark the composer as changed so it prompts about unsaved changes on close */
+	e_msg_composer_set_changed (composer);
+}
+
+static void
+subject_changed_cb (EMsgComposerHdrs *hdrs,
+		    gchar *subject,
+		    void *data)
+{
+	EMsgComposer *composer;
+
+	composer = E_MSG_COMPOSER (data);
+
+	if (strlen (subject))
+		gtk_window_set_title (GTK_WINDOW (composer), subject);
+	else
+		gtk_window_set_title (GTK_WINDOW (composer),
+				      _("Compose a message"));
+	g_free (subject);
+}
+
+static void
+hdrs_changed_cb (EMsgComposerHdrs *hdrs,
+		 void *data)
+{
+	EMsgComposer *composer;
+
+	composer = E_MSG_COMPOSER (data);
+
+	/* Mark the composer as changed so it prompts about unsaved changes on close */
+	e_msg_composer_set_changed (composer);
 }
 
 
@@ -1292,6 +1487,10 @@ init (EMsgComposer *composer)
 	
 	composer->attachment_bar_visible   = FALSE;
 	composer->send_html                = FALSE;
+	composer->pgp_sign                 = FALSE;
+	composer->pgp_encrypt              = FALSE;
+
+	composer->has_changed              = FALSE;
 }
 
 
@@ -1354,6 +1553,10 @@ e_msg_composer_construct (EMsgComposer *composer)
 	
 	composer->hdrs = e_msg_composer_hdrs_new ();
 	gtk_box_pack_start (GTK_BOX (vbox), composer->hdrs, FALSE, FALSE, 0);
+	gtk_signal_connect (GTK_OBJECT (composer->hdrs), "subject_changed",
+			    GTK_SIGNAL_FUNC (subject_changed_cb), composer);
+	gtk_signal_connect (GTK_OBJECT (composer->hdrs), "hdrs_changed",
+			    GTK_SIGNAL_FUNC (hdrs_changed_cb), composer);
 	gtk_widget_show (composer->hdrs);
 	
 	/* Editor component.  */
@@ -1399,12 +1602,12 @@ e_msg_composer_construct (EMsgComposer *composer)
 	gtk_widget_show (vbox);
 	
 	e_msg_composer_show_attachments (composer, FALSE);
-	
+
 	/* Set focus on the `To:' field.
 	
 	   gtk_widget_grab_focus (e_msg_composer_hdrs_get_to_entry (E_MSG_COMPOSER_HDRS (composer->hdrs)));
-	GTK_WIDGET_SET_FLAGS (composer->editor, GTK_CAN_FOCUS);
-	gtk_window_set_focus (GTK_WINDOW (composer), composer->editor); */
+	   GTK_WIDGET_SET_FLAGS (composer->editor, GTK_CAN_FOCUS);
+	   gtk_window_set_focus (GTK_WINDOW (composer), composer->editor); */
 	gtk_widget_grab_focus (composer->editor);
 }
 
@@ -1422,7 +1625,7 @@ create_composer (void)
 		return NULL;
 	}
 	prepare_engine (new);
-
+	
 	return new;
 }
 
@@ -1480,13 +1683,13 @@ handle_multipart_alternative (EMsgComposer *composer, CamelMultipart *multipart)
 	nparts = camel_multipart_get_number (multipart);
 	
 	for (i = 0; i < nparts; i++) {
-		GMimeContentField *content_type;
+		CamelContentType *content_type;
 		CamelMimePart *mime_part;
 		
 		mime_part = camel_multipart_get_part (multipart, i);
 		content_type = camel_mime_part_get_content_type (mime_part);
 		
-		if (gmime_content_field_is_type (content_type, "text", "html")) {
+		if (header_content_type_is (content_type, "text", "html")) {
 			CamelDataWrapper *contents;
 			char *text, *final_text;
 			gboolean is_html;
@@ -1517,13 +1720,13 @@ handle_multipart (EMsgComposer *composer, CamelMultipart *multipart, int depth)
 	nparts = camel_multipart_get_number (multipart);
 	
 	for (i = 0; i < nparts; i++) {
-		GMimeContentField *content_type;
+		CamelContentType *content_type;
 		CamelMimePart *mime_part;
 		
 		mime_part = camel_multipart_get_part (multipart, i);
 		content_type = camel_mime_part_get_content_type (mime_part);
 		
-		if (gmime_content_field_is_type (content_type, "multipart", "alternative")) {
+		if (header_content_type_is (content_type, "multipart", "alternative")) {
 			/* this structure contains the body */
 			CamelDataWrapper *wrapper;
 			CamelMultipart *mpart;
@@ -1532,7 +1735,7 @@ handle_multipart (EMsgComposer *composer, CamelMultipart *multipart, int depth)
 			mpart = CAMEL_MULTIPART (wrapper);
 			
 			handle_multipart_alternative (composer, mpart);
-		} else if (gmime_content_field_is_type (content_type, "multipart", "*")) {
+		} else if (header_content_type_is (content_type, "multipart", "*")) {
 			/* another layer of multipartness... */
 			CamelDataWrapper *wrapper;
 			CamelMultipart *mpart;
@@ -1578,7 +1781,7 @@ e_msg_composer_new_with_message (CamelMimeMessage *msg)
 {
 	const CamelInternetAddress *to, *cc, *bcc;
 	GList *To = NULL, *Cc = NULL, *Bcc = NULL;
-	GMimeContentField *content_type;
+	CamelContentType *content_type;
 	const gchar *subject;
 	EMsgComposer *new;
 	guint len, i;
@@ -1613,7 +1816,7 @@ e_msg_composer_new_with_message (CamelMimeMessage *msg)
 	for (i = 0; i < len; i++) {
 		const char *name, *addr;
 		
-		if (camel_internet_address_get (to, i, &name, &addr)) {
+		if (camel_internet_address_get (cc, i, &name, &addr)) {
 			CamelInternetAddress *cia;
 			
 			cia = camel_internet_address_new ();
@@ -1627,7 +1830,7 @@ e_msg_composer_new_with_message (CamelMimeMessage *msg)
 	for (i = 0; i < len; i++) {
 		const char *name, *addr;
 		
-		if (camel_internet_address_get (to, i, &name, &addr)) {
+		if (camel_internet_address_get (bcc, i, &name, &addr)) {
 			CamelInternetAddress *cia;
 			
 			cia = camel_internet_address_new ();
@@ -1644,7 +1847,7 @@ e_msg_composer_new_with_message (CamelMimeMessage *msg)
 	free_recipients (Bcc);
 	
 	content_type = camel_mime_part_get_content_type (CAMEL_MIME_PART (msg));
-	if (gmime_content_field_is_type (content_type, "multipart", "alternative")) {
+	if (header_content_type_is (content_type, "multipart", "alternative")) {
 		/* multipart/alternative contains the text/plain and text/html versions of the message body */
 		CamelDataWrapper *wrapper;
 		CamelMultipart *multipart;
@@ -1653,7 +1856,7 @@ e_msg_composer_new_with_message (CamelMimeMessage *msg)
 		multipart = CAMEL_MULTIPART (wrapper);
 		
 		handle_multipart_alternative (new, multipart);
-	} else if (gmime_content_field_is_type (content_type, "multipart", "*")) {
+	} else if (header_content_type_is (content_type, "multipart", "*")) {
 		/* there must be attachments... */
 		CamelDataWrapper *wrapper;
 		CamelMultipart *multipart;
@@ -2039,6 +2242,118 @@ e_msg_composer_get_send_html (EMsgComposer *composer)
 }
 
 
+/**
+ * e_msg_composer_get_preferred_account:
+ * @composer: composer
+ *
+ * Returns the user-specified account (from field).
+ */
+const MailConfigAccount *
+e_msg_composer_get_preferred_account (EMsgComposer *composer)
+{
+	EMsgComposerHdrs *hdrs;
+	
+	g_return_val_if_fail (composer != NULL, NULL);
+	g_return_val_if_fail (E_IS_MSG_COMPOSER (composer), NULL);
+	
+	hdrs = E_MSG_COMPOSER_HDRS (composer->hdrs);
+	
+	return hdrs->account;
+}
+
+
+/**
+ * e_msg_composer_set_pgp_sign:
+ * @composer: A message composer widget
+ * @send_html: Whether the composer should have the "PGP Sign" flag set
+ * 
+ * Set the status of the "PGP Sign" toggle item.  The user can override it.
+ **/
+void
+e_msg_composer_set_pgp_sign (EMsgComposer *composer, gboolean pgp_sign)
+{
+	g_return_if_fail (composer != NULL);
+	g_return_if_fail (E_IS_MSG_COMPOSER (composer));
+	
+	if (composer->pgp_sign && pgp_sign)
+		return;
+	if (!composer->pgp_sign && !pgp_sign)
+		return;
+	
+	composer->pgp_sign = pgp_sign;
+	
+	bonobo_ui_component_set_prop (composer->uic, "/commands/SecurityPGPSign",
+				      "state", composer->pgp_sign ? "1" : "0", NULL);
+}
+
+
+/**
+ * e_msg_composer_get_pgp_sign:
+ * @composer: A message composer widget
+ * 
+ * Get the status of the "PGP Sign" flag.
+ * 
+ * Return value: The status of the "PGP Sign" flag.
+ **/
+gboolean
+e_msg_composer_get_pgp_sign (EMsgComposer *composer)
+{
+	g_return_val_if_fail (composer != NULL, FALSE);
+	g_return_val_if_fail (E_IS_MSG_COMPOSER (composer), FALSE);
+
+	return composer->pgp_sign;
+}
+
+
+/**
+ * e_msg_composer_set_pgp_encrypt:
+ * @composer: A message composer widget
+ * @send_html: Whether the composer should have the "PGP Encrypt" flag set
+ * 
+ * Set the status of the "PGP Encrypt" toggle item.  The user can override it.
+ **/
+void
+e_msg_composer_set_pgp_encrypt (EMsgComposer *composer, gboolean pgp_encrypt)
+{
+	g_return_if_fail (composer != NULL);
+	g_return_if_fail (E_IS_MSG_COMPOSER (composer));
+	
+	if (composer->pgp_encrypt && pgp_encrypt)
+		return;
+	if (!composer->pgp_encrypt && !pgp_encrypt)
+		return;
+	
+	composer->pgp_encrypt = pgp_encrypt;
+	
+	bonobo_ui_component_set_prop (composer->uic, "/commands/SecurityPGPEncrypt",
+				      "state", composer->pgp_encrypt ? "1" : "0", NULL);
+}
+
+
+/**
+ * e_msg_composer_get_pgp_encrypt:
+ * @composer: A message composer widget
+ * 
+ * Get the status of the "PGP Encrypt" flag.
+ * 
+ * Return value: The status of the "PGP Encrypt" flag.
+ **/
+gboolean
+e_msg_composer_get_pgp_encrypt (EMsgComposer *composer)
+{
+	g_return_val_if_fail (composer != NULL, FALSE);
+	g_return_val_if_fail (E_IS_MSG_COMPOSER (composer), FALSE);
+
+	return composer->pgp_encrypt;
+}
+
+
+/**
+ * e_msg_composer_guess_mime_type:
+ * @file_name: filename
+ *
+ * Returns the guessed mime type of the file given by #file_name.
+ **/
 gchar *
 e_msg_composer_guess_mime_type (const gchar *file_name)
 {
@@ -2057,3 +2372,35 @@ e_msg_composer_guess_mime_type (const gchar *file_name)
 	} else
 		return NULL;
 }
+
+/**
+ * e_msg_composer_set_changed:
+ * @composer: An EMsgComposer object.
+ *
+ * Mark the composer as changed, so before the composer gets destroyed
+ * the user will be prompted about unsaved changes.
+ **/
+void
+e_msg_composer_set_changed (EMsgComposer *composer)
+{
+	g_return_if_fail (composer != NULL);
+	g_return_if_fail (E_IS_MSG_COMPOSER (composer));
+
+	composer->has_changed = TRUE;
+}
+
+/**
+ * e_msg_composer_unset_changed:
+ * @composer: An EMsgComposer object.
+ *
+ * Mark the composer as unchanged, so no prompt about unsaved changes
+ * will appear before destroying the composer.
+ **/
+void
+e_msg_composer_unset_changed (EMsgComposer *composer)
+{
+	g_return_if_fail (composer != NULL);
+	g_return_if_fail (E_IS_MSG_COMPOSER (composer));
+
+	composer->has_changed = FALSE;
+}
