@@ -34,6 +34,8 @@ typedef struct _EMsgComposerClass  EMsgComposerClass;
 #include "e-msg-composer-hdrs.h"
 #include "Editor.h"
 
+#include <mail/mail-config.h>
+
 #ifdef __cplusplus
 extern "C" {
 #pragma }
@@ -68,11 +70,15 @@ struct _EMsgComposer {
 	GNOME_GtkHTML_Editor_Engine  editor_engine;
 	BonoboObject            *editor_listener;
 	GHashTable              *inline_images;
-
+	
 	char *sig_file;
 	
 	gboolean attachment_bar_visible : 1;
 	gboolean send_html : 1;
+	gboolean pgp_sign : 1;
+	gboolean pgp_encrypt : 1;
+
+	gboolean has_changed : 1;
 };
 
 struct _EMsgComposerClass {
@@ -99,6 +105,7 @@ void              e_msg_composer_set_headers          (EMsgComposer     *compose
 						       const char       *subject);
 void              e_msg_composer_set_body_text        (EMsgComposer     *composer,
 						       const char       *text);
+void              e_msg_composer_mark_text_orig       (EMsgComposer     *composer);
 void              e_msg_composer_add_header           (EMsgComposer     *composer,
 						       const char       *name,
 						       const char       *value);
@@ -111,9 +118,21 @@ const char       *e_msg_composer_get_sig_file         (EMsgComposer     *compose
 void              e_msg_composer_set_send_html        (EMsgComposer     *composer,
 						       gboolean          send_html);
 gboolean          e_msg_composer_get_send_html        (EMsgComposer     *composer);
+void              e_msg_composer_set_pgp_sign         (EMsgComposer     *composer,
+						       gboolean          pgp_sign);
+
+const MailConfigAccount *e_msg_composer_get_preferred_account (EMsgComposer *composer);
+
+gboolean          e_msg_composer_get_pgp_sign         (EMsgComposer     *composer);
+void              e_msg_composer_set_pgp_encrypt      (EMsgComposer     *composer,
+						       gboolean          pgp_encrypt);
+gboolean          e_msg_composer_get_pgp_encrypt      (EMsgComposer     *composer);
 void              e_msg_composer_clear_inlined_table  (EMsgComposer     *composer);
 gchar *           e_msg_composer_guess_mime_type      (const gchar *file_name);
 
+void              e_msg_composer_set_changed          (EMsgComposer *composer);
+void              e_msg_composer_unset_changed        (EMsgComposer *composer);
+
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
