@@ -73,6 +73,7 @@
 #include <time.h>
 #include <string.h>
 
+#include "e-memory.h"
 #include "e-sexp.h"
 
 #define p(x)			/* parse debug */
@@ -85,9 +86,9 @@ static struct _ESExpTerm * parse_value(ESExp *f);
 
 static void parse_dump_term(struct _ESExpTerm *t, int depth);
 
+#ifdef E_SEXP_IS_GTK_OBJECT
 static GtkObjectClass *parent_class;
-
-
+#endif
 
 static GScannerConfig scanner_config =
 {
@@ -126,16 +127,38 @@ static GScannerConfig scanner_config =
 	FALSE			/* scope_0_fallback */,
 };
 
-struct _ESExpResult *
-e_sexp_result_new(int type)
+/* jumps back to the caller of f->failenv, only to be called from inside a callback */
+void
+e_sexp_fatal_error(struct _ESExp *f, char *why, ...)
 {
-	struct _ESExpResult *r = g_malloc0(sizeof(*r));
+	va_list args;
+
+	if (f->error)
+		g_free(f->error);
+	
+	va_start(args, why);
+	f->error = g_strdup_vprintf(why, args);
+	va_end(args);
+
+	longjmp(f->failenv, 1);
+}
+
+const char *
+e_sexp_error(struct _ESExp *f)
+{
+	return f->error;
+}
+
+struct _ESExpResult *
+e_sexp_result_new(struct _ESExp *f, int type)
+{
+	struct _ESExpResult *r = e_memchunk_alloc0(f->result_chunks);
 	r->type = type;
 	return r;
 }
 
 void
-e_sexp_result_free(struct _ESExpResult *t)
+e_sexp_result_free(struct _ESExp *f, struct _ESExpResult *t)
 {
 	if (t == NULL)
 		return;
@@ -153,7 +176,18 @@ e_sexp_result_free(struct _ESExpResult *t)
 	case ESEXP_RES_UNDEFINED:
 		break;
 	}
-	g_free(t);
+	e_memchunk_free(f->result_chunks, t);
+}
+
+/* used in normal functions if they have to abort, and free their arguments */
+void
+e_sexp_resultv_free(struct _ESExp *f, int argc, struct _ESExpResult **argv)
+{
+	int i;
+
+	for (i=0;i<argc;i++) {
+		e_sexp_result_free(f, argv[i]);
+	}
 }
 
 /* implementations for the builtin functions */
@@ -193,14 +227,17 @@ term_eval_and(struct _ESExp *f, int argc, struct _ESExpTerm **argv, void *data)
 	
 	r(printf("( and\n"));
 
-	r = e_sexp_result_new(ESEXP_RES_UNDEFINED);
+	r = e_sexp_result_new(f, ESEXP_RES_UNDEFINED);
 	
 	for (i=0;bool && i<argc;i++) {
 		r1 = e_sexp_term_eval(f, argv[i]);
 		if (type == -1)
 			type = r1->type;
 		if (type != r1->type) {
-			printf("invalid types in and operation, all types must be the same\n");
+			e_sexp_result_free(f, r);
+			e_sexp_result_free(f, r1);
+			g_hash_table_destroy(ht);
+			e_sexp_fatal_error(f, "Invalid types in AND");
 		} else if ( r1->type == ESEXP_RES_ARRAY_PTR ) {
 			char **a1;
 			int l1, j;
@@ -215,7 +252,7 @@ term_eval_and(struct _ESExp *f, int argc, struct _ESExpTerm **argv, void *data)
 		} else if ( r1->type == ESEXP_RES_BOOL ) {
 			bool &= r1->value.bool;
 		}
-		e_sexp_result_free(r1);
+		e_sexp_result_free(f, r1);
 	}
 	
 	if (type == ESEXP_RES_ARRAY_PTR) {
@@ -246,14 +283,17 @@ term_eval_or(struct _ESExp *f, int argc, struct _ESExpTerm **argv, void *data)
 	
 	r(printf("(or \n"));
 
-	r = e_sexp_result_new(ESEXP_RES_UNDEFINED);
+	r = e_sexp_result_new(f, ESEXP_RES_UNDEFINED);
 	
 	for (i=0;!bool && i<argc;i++) {
 		r1 = e_sexp_term_eval(f, argv[i]);
 		if (type == -1)
 			type = r1->type;
 		if (r1->type != type) {
-			printf("wrong types in or operation\n");
+			e_sexp_result_free(f, r);
+			e_sexp_result_free(f, r1);
+			g_hash_table_destroy(ht);
+			e_sexp_fatal_error(f, "Invalid types in OR");
 		} else if (r1->type == ESEXP_RES_ARRAY_PTR) {
 			char **a1;
 			int l1, j;
@@ -266,7 +306,7 @@ term_eval_or(struct _ESExp *f, int argc, struct _ESExpTerm **argv, void *data)
 		} else if (r1->type == ESEXP_RES_BOOL) {
 			bool |= r1->value.bool;				
 		}
-		e_sexp_result_free(r1);
+		e_sexp_result_free(f, r1);
 	}
 	
 	if (type == ESEXP_RES_ARRAY_PTR) {
@@ -295,7 +335,7 @@ term_eval_not(struct _ESExp *f, int argc, struct _ESExpResult **argv, void *data
 		    && argv[0]->value.bool)
 			res = FALSE;
 	}
-	r = e_sexp_result_new(ESEXP_RES_BOOL);
+	r = e_sexp_result_new(f, ESEXP_RES_BOOL);
 	r->value.bool = res;
 	return r;
 }
@@ -306,13 +346,16 @@ term_eval_lt(struct _ESExp *f, int argc, struct _ESExpTerm **argv, void *data)
 {
 	struct _ESExpResult *r, *r1, *r2;
 
-	r = e_sexp_result_new(ESEXP_RES_UNDEFINED);
+	r = e_sexp_result_new(f, ESEXP_RES_UNDEFINED);
 	
 	if (argc == 2) {
 		r1 = e_sexp_term_eval(f, argv[0]);
 		r2 = e_sexp_term_eval(f, argv[1]);
 		if (r1->type != r2->type) {
-			printf("error, invalid types in compare\n");
+			e_sexp_result_free(f, r1);
+			e_sexp_result_free(f, r2);
+			e_sexp_result_free(f, r);
+			e_sexp_fatal_error(f, "Incompatible types in compare <");
 		} else if (r1->type == ESEXP_RES_INT) {
 			r->type = ESEXP_RES_BOOL;
 			r->value.bool = r1->value.number < r2->value.number;
@@ -320,8 +363,8 @@ term_eval_lt(struct _ESExp *f, int argc, struct _ESExpTerm **argv, void *data)
 			r->type = ESEXP_RES_BOOL;
 			r->value.bool = strcmp(r1->value.string, r2->value.string) < 0;
 		}
-		e_sexp_result_free(r1);
-		e_sexp_result_free(r2);
+		e_sexp_result_free(f, r1);
+		e_sexp_result_free(f, r2);
 	}
 	return r;
 }
@@ -332,13 +375,16 @@ term_eval_gt(struct _ESExp *f, int argc, struct _ESExpTerm **argv, void *data)
 {
 	struct _ESExpResult *r, *r1, *r2;
 
-	r = e_sexp_result_new(ESEXP_RES_UNDEFINED);
+	r = e_sexp_result_new(f, ESEXP_RES_UNDEFINED);
 	
 	if (argc == 2) {
 		r1 = e_sexp_term_eval(f, argv[0]);
 		r2 = e_sexp_term_eval(f, argv[1]);
 		if (r1->type != r2->type) {
-			printf("error, invalid types in compare\n");
+			e_sexp_result_free(f, r1);
+			e_sexp_result_free(f, r2);
+			e_sexp_result_free(f, r);
+			e_sexp_fatal_error(f, "Incompatible types in compare >");
 		} else if (r1->type == ESEXP_RES_INT) {
 			r->type = ESEXP_RES_BOOL;
 			r->value.bool = r1->value.number > r2->value.number;
@@ -346,8 +392,8 @@ term_eval_gt(struct _ESExp *f, int argc, struct _ESExpTerm **argv, void *data)
 			r->type = ESEXP_RES_BOOL;
 			r->value.bool = strcmp(r1->value.string, r2->value.string) > 0;
 		}
-		e_sexp_result_free(r1);
-		e_sexp_result_free(r2);
+		e_sexp_result_free(f, r1);
+		e_sexp_result_free(f, r2);
 	}
 	return r;
 }
@@ -358,7 +404,7 @@ term_eval_eq(struct _ESExp *f, int argc, struct _ESExpTerm **argv, void *data)
 {
 	struct _ESExpResult *r, *r1, *r2;
 
-	r = e_sexp_result_new(ESEXP_RES_BOOL);
+	r = e_sexp_result_new(f, ESEXP_RES_BOOL);
 	
 	if (argc == 2) {
 		r1 = e_sexp_term_eval(f, argv[0]);
@@ -372,8 +418,8 @@ term_eval_eq(struct _ESExp *f, int argc, struct _ESExpTerm **argv, void *data)
 		} else if (r1->type == ESEXP_RES_STRING) {
 			r->value.bool = strcmp(r1->value.string, r2->value.string) == 0;
 		}
-		e_sexp_result_free(r1);
-		e_sexp_result_free(r2);
+		e_sexp_result_free(f, r1);
+		e_sexp_result_free(f, r2);
 	}
 	return r;
 }
@@ -394,9 +440,10 @@ term_eval_plus(struct _ESExp *f, int argc, struct _ESExpResult **argv, void *dat
 				total += argv[i]->value.number;
 			}
 			if (i<argc) {
-				g_warning("Wrong type trying to add integers: ignored");
+				e_sexp_resultv_free(f, argc, argv);
+				e_sexp_fatal_error(f, "Invalid types in (+ ints)");
 			}
-			r = e_sexp_result_new(ESEXP_RES_INT);
+			r = e_sexp_result_new(f, ESEXP_RES_INT);
 			r->value.number = total;
 			break; }
 		case ESEXP_RES_STRING: {
@@ -405,9 +452,10 @@ term_eval_plus(struct _ESExp *f, int argc, struct _ESExpResult **argv, void *dat
 				g_string_append(s, argv[i]->value.string);
 			}
 			if (i<argc) {
-				g_warning("Wrong type trying to concat strings: ignored");
+				e_sexp_resultv_free(f, argc, argv);
+				e_sexp_fatal_error(f, "Invalid types in (+ strings)");
 			}
-			r = e_sexp_result_new(ESEXP_RES_STRING);
+			r = e_sexp_result_new(f, ESEXP_RES_STRING);
 			r->value.string = s->str;
 			g_string_free(s, FALSE);
 			break; }
@@ -416,7 +464,7 @@ term_eval_plus(struct _ESExp *f, int argc, struct _ESExpResult **argv, void *dat
 	}
 
 	if (!r) {
-		r = e_sexp_result_new(ESEXP_RES_INT);
+		r = e_sexp_result_new(f, ESEXP_RES_INT);
 		r->value.number = 0;
 	}
 	return r;
@@ -438,16 +486,17 @@ term_eval_sub(struct _ESExp *f, int argc, struct _ESExpResult **argv, void *data
 				total -= argv[i]->value.number;
 			}
 			if (i<argc) {
-				g_warning("Wrong type trying to subtract numbers: ignored");
+				e_sexp_resultv_free(f, argc, argv);
+				e_sexp_fatal_error(f, "Invalid types in -");
 			}
-			r = e_sexp_result_new(ESEXP_RES_INT);
+			r = e_sexp_result_new(f, ESEXP_RES_INT);
 			r->value.number = total;
 			break; }
 		}
 	}
 
 	if (!r) {
-		r = e_sexp_result_new(ESEXP_RES_INT);
+		r = e_sexp_result_new(f, ESEXP_RES_INT);
 		r->value.number = 0;
 	}
 	return r;
@@ -463,14 +512,14 @@ term_eval_if(struct _ESExp *f, int argc, struct _ESExpTerm **argv, void *data)
 	if (argc >=2 && argc<=3) {
 		r = e_sexp_term_eval(f, argv[0]);
 		doit = (r->type == ESEXP_RES_BOOL && r->value.bool);
-		e_sexp_result_free(r);
+		e_sexp_result_free(f, r);
 		if (doit) {
 			return e_sexp_term_eval(f, argv[1]);
 		} else if (argc>2) {
 			return e_sexp_term_eval(f, argv[2]);
 		}
 	}
-	return e_sexp_result_new(ESEXP_RES_UNDEFINED);
+	return e_sexp_result_new(f, ESEXP_RES_UNDEFINED);
 }
 
 /* implements 'begin' statement */
@@ -482,81 +531,69 @@ term_eval_begin(struct _ESExp *f, int argc, struct _ESExpTerm **argv, void *data
 
 	for (i=0;i<argc;i++) {
 		if (r)
-			e_sexp_result_free(r);
+			e_sexp_result_free(f, r);
 		r = e_sexp_term_eval(f, argv[i]);
 	}
 	if (r)
 		return r;
 	else
-		return e_sexp_result_new(ESEXP_RES_UNDEFINED);
+		return e_sexp_result_new(f, ESEXP_RES_UNDEFINED);
 }
 
 
+/* this must only be called from inside term evaluation callbacks! */
 struct _ESExpResult *
 e_sexp_term_eval(struct _ESExp *f, struct _ESExpTerm *t)
 {
-	struct _ESExpResult *r, *r1;
+	struct _ESExpResult *r = NULL;
 	int i;
+	struct _ESExpResult **argv;
 
 	g_return_val_if_fail(t != NULL, NULL);
 
 	r(printf("eval term :\n"));
 	r(parse_dump_term(t, 0));
 
-	r = g_malloc0(sizeof(*r));
-	r->type = ESEXP_RES_UNDEFINED;
-
 	switch (t->type) {
 	case ESEXP_TERM_STRING:
 		r(printf(" (string \"%s\")\n", t->value.string));
-		r->type = ESEXP_RES_STRING;
+		r = e_sexp_result_new(f, ESEXP_RES_STRING);
 		/* erk, this shoul;dn't need to strdup this ... */
 		r->value.string = g_strdup(t->value.string);
 		break;
 	case ESEXP_TERM_INT:
 		r(printf(" (int %d)\n", t->value.number));
-		r->type = ESEXP_RES_INT;
+		r = e_sexp_result_new(f, ESEXP_RES_INT);
 		r->value.number = t->value.number;
 		break;
 	case ESEXP_TERM_BOOL:
 		r(printf(" (int %d)\n", t->value.number));
-		r->type = ESEXP_RES_BOOL;
+		r = e_sexp_result_new(f, ESEXP_RES_BOOL);
 		r->value.bool = t->value.bool;
 		break;
-	case ESEXP_TERM_IFUNC: {
-		if (t->value.func.sym->f.ifunc) {
-			r1 = t->value.func.sym->f.ifunc(f, t->value.func.termcount, t->value.func.terms, t->value.func.sym->data);
-			if (r1) {
-				e_sexp_result_free(r);
-				r = r1;
-			}
-		}
-		break; }
-	case ESEXP_TERM_FUNC: {
-		struct _ESExpResult **argv;
-
+	case ESEXP_TERM_IFUNC:
+		if (t->value.func.sym->f.ifunc)
+			r = t->value.func.sym->f.ifunc(f, t->value.func.termcount, t->value.func.terms, t->value.func.sym->data);
+		break;
+	case ESEXP_TERM_FUNC:
 		/* first evaluate all arguments to result types */
 		argv = alloca(sizeof(argv[0]) * t->value.func.termcount);
 		for (i=0;i<t->value.func.termcount;i++) {
 			argv[i] = e_sexp_term_eval(f, t->value.func.terms[i]);
 		}
 		/* call the function */
-		if (t->value.func.sym->f.func) {
-			r1 = t->value.func.sym->f.func(f, t->value.func.termcount, argv, t->value.func.sym->data);
-			if (r1) {
-				e_sexp_result_free(r);
-				r = r1;
-			}
-		}
-		for (i=0;i<t->value.func.termcount;i++) {
-			e_sexp_result_free(argv[i]);
-		}
-		break; }
+		if (t->value.func.sym->f.func)
+			r = t->value.func.sym->f.func(f, t->value.func.termcount, argv, t->value.func.sym->data);
+
+		e_sexp_resultv_free(f, t->value.func.termcount, argv);
+		break;
 	default:
-		printf("Warning: Unknown type encountered in parse tree: %d\n", t->type);
-		r->type = ESEXP_RES_UNDEFINED;
+		e_sexp_fatal_error(f, "Unknown type in parse tree: %d", t->type);
 	}
 
+	if (r==NULL)
+		r = e_sexp_result_new(f, ESEXP_RES_UNDEFINED);
+
 	return r;
 }
 
@@ -644,15 +681,15 @@ parse_dump_term(struct _ESExpTerm *t, int depth)
 */
 
 static struct _ESExpTerm *
-parse_new_term(int type)
+parse_term_new(struct _ESExp *f, int type)
 {
-	struct _ESExpTerm *s = g_malloc0(sizeof(*s));
+	struct _ESExpTerm *s = e_memchunk_alloc0(f->term_chunks);
 	s->type = type;
 	return s;
 }
 
 static void
-parse_term_free(struct _ESExpTerm *t)
+parse_term_free(struct _ESExp *f, struct _ESExpTerm *t)
 {
 	int i;
 
@@ -664,7 +701,7 @@ parse_term_free(struct _ESExpTerm *t)
 	case ESEXP_TERM_FUNC:
 	case ESEXP_TERM_IFUNC:
 		for (i=0;i<t->value.func.termcount;i++) {
-			parse_term_free(t->value.func.terms[i]);
+			parse_term_free(f, t->value.func.terms[i]);
 		}
 		g_free(t->value.func.terms);
 		break;
@@ -678,7 +715,7 @@ parse_term_free(struct _ESExpTerm *t)
 	default:
 		printf("parse_term_free: unknown type: %d\n", t->type);
 	}
-	g_free(t);
+	e_memchunk_free(f->term_chunks, t);
 }
 
 static struct _ESExpTerm **
@@ -692,7 +729,7 @@ parse_values(ESExp *f, int *len)
 	p(printf("parsing values\n"));
 
 	/* FIXME: This hardcoded nonsense!!! :) */
-	terms = g_malloc0(20*sizeof(*terms));
+	terms = g_malloc0(30*sizeof(*terms));
 
 	while ( (token = g_scanner_peek_next_token(gs)) != G_TOKEN_EOF
 		&& token != ')') {
@@ -724,18 +761,18 @@ parse_value(ESExp *f)
 		return parse_list(f, TRUE);
 	case G_TOKEN_STRING:
 		p(printf("got string\n"));
-		t = parse_new_term(ESEXP_TERM_STRING);
+		t = parse_term_new(f, ESEXP_TERM_STRING);
 		t->value.string = g_strdup(g_scanner_cur_value(gs).v_string);
 		break;
 	case G_TOKEN_INT:
-		t = parse_new_term(ESEXP_TERM_INT);
+		t = parse_term_new(f, ESEXP_TERM_INT);
 		t->value.number = g_scanner_cur_value(gs).v_int;
 		p(printf("got int\n"));
 		break;
 	case '#':
-		printf("got bool?\n");
+		p(printf("got bool?\n"));
 		token = g_scanner_get_next_token(gs);
-		t = parse_new_term(ESEXP_TERM_BOOL);
+		t = parse_term_new(f, ESEXP_TERM_BOOL);
 		t->value.bool = token=='t';
 		break;
 	case G_TOKEN_SYMBOL:
@@ -745,23 +782,23 @@ parse_value(ESExp *f)
 		case ESEXP_TERM_IFUNC:
 				/* this is basically invalid, since we can't use function
 				   pointers, but let the runtime catch it ... */
-			t = parse_new_term(s->type);
+			t = parse_term_new(f, s->type);
 			t->value.func.sym = s;
 			t->value.func.terms = parse_values(f, &t->value.func.termcount);
 			break;
 		case ESEXP_TERM_VAR:
-			t = parse_new_term(s->type);
+			t = parse_term_new(f, s->type);
 			t->value.var = s;
 			break;
 		default:
-			printf("Invalid symbol type: %d\n", s->type);
+			e_sexp_fatal_error(f, "Invalid symbol type: %s: %d", s->name, s->type);
 		}
 		break;
 	case G_TOKEN_IDENTIFIER:
-		printf("Unknown identifier encountered: %s\n", g_scanner_cur_value(gs).v_identifier);
+		e_sexp_fatal_error(f, "Unknown identifier: %s", g_scanner_cur_value(gs).v_identifier);
 		break;
 	default:
-		printf("Innvalid token trying to parse a list of values\n");
+		e_sexp_fatal_error(f, "Unexpected token encountered: %d", token);
 	}
 	p(printf("done parsing value\n"));
 	return t;
@@ -788,7 +825,7 @@ parse_list(ESExp *f, int gotbrace)
 
 			s = g_scanner_cur_value(gs).v_symbol;
 			p(printf("got funciton: %s\n", s->name));
-			t = parse_new_term(s->type);
+			t = parse_term_new(f, s->type);
 			p(printf("created new list %p\n", t));
 			/* if we have a variable, find out its base type */
 			while (s->type == ESEXP_TERM_VAR) {
@@ -799,88 +836,31 @@ parse_list(ESExp *f, int gotbrace)
 				t->value.func.sym = s;
 				t->value.func.terms = parse_values(f, &t->value.func.termcount);
 			} else {
-				printf("Error, trying to call variable as function\n");
+				parse_term_free(f, t);
+				e_sexp_fatal_error(f, "Trying to call variable as function: %s", s->name);
 			}
 			break; }
 		case G_TOKEN_IDENTIFIER:
-			printf("Unknown identifier: %s\n", g_scanner_cur_value(gs).v_identifier);
+			e_sexp_fatal_error(f, "Unknown identifier: %s", g_scanner_cur_value(gs).v_identifier);
 			break;
 		default:
-			printf("unknown sequence encountered, type = %d\n", token);
+			e_sexp_fatal_error(f, "Unexpected token encountered: %d", token);
 		}
 		token = g_scanner_get_next_token(gs);
 		if (token != ')') {
-			printf("Error, expected ')' not found\n");
+			e_sexp_fatal_error(f, "Missing ')'");
 		}
 	} else {
-		printf("Error, list term without opening (\n");
+		e_sexp_fatal_error(f, "Missing '('");
 	}
 
 	p(printf("returning list %p\n", t));
 	return t;
 }
 
-#if 0
-GList *
-camel_mbox_folder_search_by_expression(CamelFolder *folder, char *expression, CamelException *ex)
-{
-	GScanner *gs;
-	int i;
-	struct _ESExpTerm *t;
-	struct _searchcontext *ctx;
-	struct _ESExpResult *r;
-	GList *matches = NULL;
-
-	gs = g_scanner_new(&scanner_config);
-	for(i=0;i<sizeof(symbols)/sizeof(symbols[0]);i++)
-		g_scanner_scope_add_symbol(gs, 0, symbols[i].name, &symbols[i]);
-
-	g_scanner_input_text(gs, expression, strlen(expression));
-	t = parse_list(gs, 0);
-
-	if (t) {
-		ctx = g_malloc0(sizeof(*ctx));
-		ctx->folder = folder;
-		ctx->summary = camel_folder_get_summary(folder, ex);
-		ctx->message_info = camel_folder_summary_get_message_info_list(ctx->summary);
-#ifdef HAVE_IBEX
-		ctx->index = ibex_open(CAMEL_MBOX_FOLDER(folder)->index_file_path, FALSE);
-		if (!ctx->index) {
-			perror("Cannot open index file, body searches will be ignored\n");
-		}
-#endif
-		r = term_eval(ctx, t);
-
-		/* now create a folder summary to return?? */
-		if (r
-		    && r->type == ESEXP_RES_ARRAY_PTR) {
-			d(printf("got result ...\n"));
-			for (i=0;i<r->value.ptrarray->len;i++) {
-				d(printf("adding match: %s\n", (char *)g_ptr_array_index(r->value.ptrarray, i)));
-				matches = g_list_prepend(matches, g_strdup(g_ptr_array_index(r->value.ptrarray, i)));
-			}
-			e_sexp_result_free(r);
-		}
-
-		if (ctx->index)
-			ibex_close(ctx->index);
-
-		gtk_object_unref((GtkObject *)ctx->summary);
-		g_free(ctx);
-		parse_term_free(t);
-	} else {
-		printf("Warning, Could not parse expression!\n %s\n", expression);
-	}
-
-	g_scanner_destroy(gs);
-
-	return matches;
-}
-#endif
-
-
-static void e_sexp_finalise(GtkObject *);
+static void e_sexp_finalise(void *);
 
+#ifdef E_SEXP_IS_GTK_OBJECT
 static void
 e_sexp_class_init (ESExpClass *class)
 {
@@ -892,6 +872,7 @@ e_sexp_class_init (ESExpClass *class)
 
 	parent_class = gtk_type_class (gtk_object_get_type ());
 }
+#endif
 
 /* 'builtin' functions */
 static struct {
@@ -922,19 +903,24 @@ free_symbol(void *key, void *value, void *data)
 }
 
 static void
-e_sexp_finalise(GtkObject *o)
+e_sexp_finalise(void *o)
 {
 	ESExp *s = (ESExp *)o;
 
 	if (s->tree) {
-		parse_term_free(s->tree);
+		parse_term_free(s, s->tree);
 		s->tree = NULL;
 	}
 
+	e_memchunk_destroy(s->term_chunks);
+	e_memchunk_destroy(s->result_chunks);
+
 	g_scanner_scope_foreach_symbol(s->scanner, 0, free_symbol, 0);
 	g_scanner_destroy(s->scanner);
 
+#ifdef E_SEXP_IS_GTK_OBJECT
 	((GtkObjectClass *)(parent_class))->finalize((GtkObject *)o);
+#endif
 }
 
 static void
@@ -943,6 +929,8 @@ e_sexp_init (ESExp *s)
 	int i;
 
 	s->scanner = g_scanner_new(&scanner_config);
+	s->term_chunks = e_memchunk_new(16, sizeof(struct _ESExpTerm));
+	s->result_chunks = e_memchunk_new(16, sizeof(struct _ESExpResult));
 
 	/* load in builtin symbols? */
 	for(i=0;i<sizeof(symbols)/sizeof(symbols[0]);i++) {
@@ -952,8 +940,13 @@ e_sexp_init (ESExp *s)
 			e_sexp_add_function(s, 0, symbols[i].name, symbols[i].func, &symbols[i]);
 		}
 	}
+
+#ifndef E_SEXP_IS_GTK_OBJECT
+	s->refcount = 1;
+#endif
 }
 
+#ifdef E_SEXP_IS_GTK_OBJECT
 guint
 e_sexp_get_type (void)
 {
@@ -975,15 +968,37 @@ e_sexp_get_type (void)
 	
 	return type;
 }
+#endif
 
 ESExp *
 e_sexp_new (void)
 {
+#ifdef E_SEXP_IS_GTK_OBJECT
 	ESExp *f = E_SEXP ( gtk_type_new (e_sexp_get_type ()));
+#else
+	ESExp *f = g_malloc0(sizeof(*f));
+	e_sexp_init(f);
+#endif
 
 	return f;
 }
 
+#ifndef E_SEXP_IS_GTK_OBJECT
+void		e_sexp_ref		(ESExp *f)
+{
+	f->refcount++;
+}
+
+void		e_sexp_unref		(ESExp *f)
+{
+	f->refcount--;
+	if (f->refcount == 0) {
+		e_sexp_finalise(f);
+		g_free(f);
+	}
+}
+#endif
+
 void
 e_sexp_add_function(ESExp *f, int scope, char *name, ESExpFunc *func, void *data)
 {
@@ -1075,24 +1090,37 @@ e_sexp_input_file (ESExp *f, int fd)
 	g_scanner_input_file(f->scanner, fd);
 }
 
-/* needs some error return? */
-void
+/* returns -1 on error */
+int
 e_sexp_parse(ESExp *f)
 {
-	g_return_if_fail(FILTER_IS_SEXP(f));
+	g_return_val_if_fail(FILTER_IS_SEXP(f), -1);
+
+	if (setjmp(f->failenv)) {
+		g_warning("Error in parsing: %s", f->error);
+		return -1;
+	}
 
 	if (f->tree)
-		parse_term_free(f->tree);
+		parse_term_free(f, f->tree);
 
 	f->tree = parse_list(f, FALSE);
+
+	return 0;
 }
 
+/* returns NULL on error */
 struct _ESExpResult *
 e_sexp_eval(ESExp *f)
 {
 	g_return_val_if_fail(FILTER_IS_SEXP(f), NULL);
 	g_return_val_if_fail(f->tree != NULL, NULL);
 
+	if (setjmp(f->failenv)) {
+		g_warning("Error in execution: %s", f->error);
+		return NULL;
+	}
+
 	return e_sexp_term_eval(f, f->tree);
 }
 
