@@ -75,25 +75,98 @@ static char *descriptions[] = {
 
 typedef struct _PropertyDialog {
 	BonoboListener *listener;
-	Bonobo_Listener corba_listener;
+	int listener_id;
+
 	Bonobo_EventSource eventsource;
 	GtkWidget *dialog;
 } PropertyDialog;
 #define COMPOSER_IID "OAFIID:GNOME_Evolution_Mail_Composer"
 
+#if HAVECACHE
+static ESummaryCache *image_cache = NULL;
+#endif
+
 gboolean e_summary_url_mail_compose (ESummary *esummary,
 				     const char *url);
 gboolean e_summary_url_exec (const char *exec);
 
+struct _DownloadInfo {
+	GtkHTMLStream *stream;
+	char *uri;
+	char *buffer;
+
+	gboolean error;
+};
+typedef struct _DownloadInfo DownloadInfo;
+
+static void
+close_callback (GnomeVFSAsyncHandle *handle,
+		GnomeVFSResult result,
+		gpointer data)
+{
+	DownloadInfo *info = data;
+	if (info->error) {
+		gtk_html_stream_close (info->stream, GTK_HTML_STREAM_ERROR);
+	} else {
+		gtk_html_stream_close (info->stream, GTK_HTML_STREAM_OK);
+	}
+
+	g_free (info->uri);
+	g_free (info->buffer);
+	g_free (info);
+}
+
+static void
+read_callback (GnomeVFSAsyncHandle *handle,
+	       GnomeVFSResult result,
+	       gpointer buffer,
+	       GnomeVFSFileSize bytes_requested,
+	       GnomeVFSFileSize bytes_read,
+	       gpointer data)
+{
+	DownloadInfo *info = data;
+	if (result != GNOME_VFS_OK && result != GNOME_VFS_ERROR_EOF) {
+		g_warning ("Read error");
+		info->error = TRUE;
+		gnome_vfs_async_close (handle, close_callback, info);
+	}
+	
+	if (bytes_read == 0) {
+		info->error = FALSE;
+		gnome_vfs_async_close (handle, close_callback, info);
+	} else {
+		gtk_html_stream_write (info->stream, buffer, bytes_read);
+		gnome_vfs_async_read (handle, buffer, 4095, read_callback,
+				      info);
+	}
+}
+
+static void
+open_callback (GnomeVFSAsyncHandle *handle,
+	       GnomeVFSResult result,
+	       DownloadInfo *info)
+{
+	if (result != GNOME_VFS_OK) {
+		gtk_html_stream_close (info->stream, GTK_HTML_STREAM_ERROR);
+		g_free (info->uri);
+		g_free (info);
+		return;
+	} 
+
+	info->buffer = g_new (char, 4096);
+	gnome_vfs_async_read (handle, info->buffer, 4095, read_callback, info);
+}
+
 void
 e_summary_url_request (GtkHTML *html,
 		       const gchar *url,
 		       GtkHTMLStream *stream)
 {
 	char *filename;
-	GnomeVFSHandle *handle = NULL;
-	GnomeVFSResult result;
+	GnomeVFSAsyncHandle *handle;
+	DownloadInfo *info;
 
+	g_print ("url: %s\n", url);
 	if (strncasecmp (url, "file:", 5) == 0) {
 		url += 5;
 		filename = e_pixmap_file (url);
@@ -108,40 +181,14 @@ e_summary_url_request (GtkHTML *html,
 	}
 
 	g_print ("Filename: %s\n", filename);
-	result = gnome_vfs_open (&handle, filename, GNOME_VFS_OPEN_READ);
-	
-	if (result != GNOME_VFS_OK) {
-		g_warning ("%s: %s", filename, 
-			   gnome_vfs_result_to_string (result));
-		g_free (filename);
-		gtk_html_stream_close (stream, GTK_HTML_STREAM_ERROR);
-		return;
-	}
 
-	g_free (filename);
-	while (1) {
-		char buffer[4096];
-		GnomeVFSFileSize size;
+	info = g_new (DownloadInfo, 1);
+	info->stream = stream;
+	info->uri = filename;
+	info->error = FALSE;
 
-		/* Clear buffer */
-		memset (buffer, 0x00, 4096);
-
-		result = gnome_vfs_read (handle, buffer, 4096, &size);
-		if (result != GNOME_VFS_OK && result != GNOME_VFS_ERROR_EOF) {
-			g_warning ("Error reading data: %s", 
-				   gnome_vfs_result_to_string (result));
-			gnome_vfs_close (handle);
-			gtk_html_stream_close (stream, GTK_HTML_STREAM_ERROR);
-		}
-
-		if (size == 0)
-			break; /* EOF */
-
-		gtk_html_stream_write (stream, buffer, size);
-	}
-	
-	gtk_html_stream_close (stream, GTK_HTML_STREAM_OK);
-	gnome_vfs_close (handle);
+	gnome_vfs_async_open (&handle, filename, GNOME_VFS_OPEN_READ,
+			      (GnomeVFSAsyncOpenCallback) open_callback, info);
 }
 
 static char *
@@ -373,17 +420,33 @@ dialog_destroyed (GtkObject *object,
 
 	CORBA_exception_init (&ev);
 	Bonobo_EventSource_removeListener (dialog->eventsource,
-					   dialog->corba_listener, &ev);
+					   dialog->listener_id, &ev);
 	if (ev._major != CORBA_NO_EXCEPTION) {
 		g_warning ("Error: %s", CORBA_exception_id (&ev));
 	}
 
 	bonobo_object_unref (BONOBO_OBJECT (dialog->listener));
-	bonobo_object_release_unref (dialog->eventsource, &ev);
 	CORBA_exception_free (&ev);
 	g_free (dialog);
 }
 
+struct _idle_data {
+	ESummary *esummary;
+	ESummaryWindow *window;
+};
+
+static gboolean
+idle_remove_window (gpointer data)
+{
+	struct _idle_data *id = data;
+
+	e_summary_remove_window (id->esummary, id->window);
+	e_summary_queue_rebuild (id->esummary);
+	g_free (id);
+
+	return FALSE;
+}
+
 void
 e_summary_url_click (GtkWidget *widget,
 		     const char *url,
@@ -393,7 +456,9 @@ e_summary_url_click (GtkWidget *widget,
 	char *parsed;
 	int address;
 	ESummaryWindow *window;
+	struct _idle_data *id;
 	Bonobo_Control control;
+	Bonobo_Listener corba_listener;
 	GtkWidget *prefsbox, *control_widget;
 	CORBA_Environment ev;
 	PropertyDialog *data;
@@ -426,8 +491,15 @@ e_summary_url_click (GtkWidget *widget,
 		if (window->iid == NULL)
 			break;
 
-		e_summary_remove_window (esummary, window);
-		e_summary_queue_rebuild (esummary);
+		id = g_new (struct _idle_data, 1);
+		id->window = window;
+		id->esummary = esummary;
+
+		/* Close the window on an idle to work around a bug in gnome-vfs
+		   which locaks the e_summary_remove_window function and as
+		   gtkhtml has a pointer grab on, this locks the whole display.
+		   GAH! */
+		g_idle_add (idle_remove_window, id);
 		break;
 
 	case PROTOCOL_CONFIGURE:
@@ -443,13 +515,11 @@ e_summary_url_click (GtkWidget *widget,
 		data->dialog = prefsbox;
 
 		CORBA_exception_init (&ev);
-		data->eventsource = Bonobo_Unknown_queryInterface (window->propertycontrol,
-								   "IDL:Bonobo/EventSource:1.0", 
-								   &ev);
+		data->eventsource = window->event_source;
 		data->listener = bonobo_listener_new (property_event, data);
-		data->corba_listener = bonobo_object_corba_objref (BONOBO_OBJECT (data->listener));
-		Bonobo_EventSource_addListener (data->eventsource,
-						data->corba_listener, &ev);
+		corba_listener = bonobo_object_corba_objref (BONOBO_OBJECT (data->listener));
+		data->listener_id = Bonobo_EventSource_addListener (data->eventsource,
+								    corba_listener, &ev);
 
 		gtk_signal_connect (GTK_OBJECT (prefsbox), "apply",
 				    GTK_SIGNAL_FUNC (property_apply), 
@@ -746,3 +816,26 @@ e_summary_url_over (GtkHTML *html,
 		e_summary_unset_message (esummary);
 	}
 }
+
+/* Cache stuff */
+#if HAVECACHE
+void
+e_summary_url_init_cache (void)
+{
+	if (image_cache != NULL)
+		return;
+
+	image_cache = e_summary_cache_new ();
+}
+
+void
+e_summary_url_cache_destroy (void)
+{
+	if (image_cache == NULL)
+		return;
+
+	gtk_object_unref (GTK_OBJECT (image_cache));
+
+	image_cache = NULL;
+}
+#endif
