@@ -104,6 +104,7 @@ e_summary_destroy (GtkObject *object)
 	e_summary_save_state (esummary, prefix);
 	g_free (prefix);
 
+	e_summary_prefs_free (esummary->prefs);
 	for (l = priv->window_list; l; l = l->next)
 		e_summary_window_free (l->data);
 	g_list_free (priv->window_list);
@@ -337,25 +338,13 @@ make_control_html (ESummaryWindow *window,
 {
 	char *html, *tmp;
 	int id = GPOINTER_TO_INT (window);
-	gboolean u, d, l, r, config;
+	gboolean config;
 
-	u = d = l = r = config = TRUE;
+	config = TRUE;
 
 	if (window->propertycontrol == CORBA_OBJECT_NIL)
 		config = FALSE;
 
-	if (row == 0) /* Top row */
-		u = FALSE;
-
-	if (row >= numwindows - 3) /* Bottom row */
-		d = FALSE;
-
-	if (col == 0) /* Leftmost column */
-		l = FALSE;
-	
-	if (col == 2 || ((row * 3) + col) == numwindows - 1) /* Rightmost column */
-		r = FALSE;
-
 	html = g_strdup_printf ("<table><tr><td><a href=\"close://%d\">"
 				"<img src=\"service-close.png\" border=\"0\">"
 				"</a></td></tr><tr><td>", id);
@@ -371,53 +360,6 @@ make_control_html (ESummaryWindow *window,
 	}
 	g_free (tmp);
 
-#if 0
-	tmp = html;
-	if (!l) {
-		html = g_strdup_printf ("%s<img src=\"service-left-disabled.png\">"
-					"</td><td>", tmp);
-	} else {
-		html = g_strdup_printf ("%s<a href=\"left://%d\">"
-					"<img src=\"service-left.png\" border=\"0\">"
-					"</a></td><td>", tmp, id);
-	}
-	g_free (tmp);
-	
-	tmp = html;
-	if (!r) {
-		html = g_strdup_printf ("%s<img src=\"service-right-disabled.png\">"
-					"</td></tr></table>", tmp);
-	} else {
-		html = g_strdup_printf ("%s<a href=\"right://%d\">"
-					"<img src=\"service-right.png\" border=\"0\">"
-					"</a></td></tr></table>", tmp, id);
-	}
-	g_free (tmp);
-
-
-	tmp = html;
-	if (!d) {
-		html = g_strdup_printf ("%s<img src=\"service-down-disabled.png\">"
-					"</td><td>", tmp);
-	} else {
-		html = g_strdup_printf ("%s<a href=\"down://%d\">"
-					"<img src=\"service-down.png\" border=\"0\">"
-					"</a></td><td>", tmp, id);
-	}
-	g_free (tmp);
-
-	tmp = html;
-	if (!u) {
-		html = g_strdup_printf ("%s<img src=\"service-up-disabled.png\">"
-					"</td></tr></table>", tmp);
-	} else {
-		html = g_strdup_printf ("%s<a href=\"up://%d\">"
-					"<img src=\"service-up.png\" border=\"0\">"
-					"</a></td></tr></table>", tmp, id);
-	}
-	g_free (tmp);
-
-#endif
 	return html;
 }
 
@@ -468,9 +410,10 @@ e_summary_display_window (ESummary *esummary,
 
 	header = g_strdup_printf ("<td bgcolor=\"%s\" valign=\"top\">", colour[col % 2]);
 	gtk_html_write (GTK_HTML (priv->html), priv->stream, header, strlen (header));
+	g_free (header);
 
 	if (window->html != CORBA_OBJECT_NIL) {
-		char *html;
+		char *html = NULL;
 		CORBA_Environment ev;
 
 		CORBA_exception_init (&ev);
@@ -479,11 +422,14 @@ e_summary_display_window (ESummary *esummary,
 		if (ev._major != CORBA_NO_EXCEPTION) {
 			CORBA_exception_free (&ev);
 			g_warning ("Cannot get HTML.");
+			if (html)
+				CORBA_free (html);
 		} else {
 			CORBA_exception_free (&ev);
-
+			
 			gtk_html_write (GTK_HTML (priv->html), priv->stream,
 					html, strlen (html));
+			CORBA_free (html);
 		}
 	} else {
 #if 0
@@ -505,9 +451,11 @@ e_summary_rebuild_page (ESummary *esummary)
 {
 	ESummaryPrivate *priv;
 	GList *windows;
-	char *service_table = "<table numcols=\"3\" cellspacing=\"0\" cellpadding=\"0\" border=\"0\" height=\"100%\">";
+	char *service_table = "<table numcols=\"%d\" cellspacing=\"0\" cellpadding=\"0\" border=\"0\" height=\"100%%\" width=\"100%%\">";
+	char *tmp;
 	int numwindows, numrows;
 	int i, j, k;
+	int columns;
 
 	g_return_val_if_fail (esummary != NULL, FALSE);
 	g_return_val_if_fail (IS_E_SUMMARY (esummary), FALSE);
@@ -531,7 +479,7 @@ e_summary_rebuild_page (ESummary *esummary)
 	gtk_layout_freeze (GTK_LAYOUT (priv->html));
 	e_summary_start_load (esummary);
 	
-	if (priv->header == NULL) {
+	if (priv->header == NULL || *priv->header == '\0') {
 		load_default_header (esummary);
 	} else {
 		gtk_html_write (GTK_HTML (priv->html), priv->stream,
@@ -539,32 +487,33 @@ e_summary_rebuild_page (ESummary *esummary)
 	}
 
 	/* Load the start of the services */
-	gtk_html_write (GTK_HTML (priv->html), priv->stream, service_table,
-			strlen (service_table));
+	tmp = g_strdup_printf (service_table, esummary->prefs->columns);
+	gtk_html_write (GTK_HTML (priv->html), priv->stream, tmp, strlen (tmp));
+	g_free (tmp);
 	/* Load each of the services */
 	numwindows = g_list_length (priv->window_list);
 
+	columns = esummary->prefs->columns;
+
 	windows = priv->window_list;
-	if (numwindows % 3 == 0)
-		numrows = numwindows / 3;
+	if (numwindows % columns == 0)
+		numrows = numwindows / columns;
 	else
-		numrows = numwindows / 3 + 1;
+		numrows = numwindows / columns + 1;
 
 	for (i = 0; i < numrows; i++) {
 		GList *window = windows;
 		
-		g_print ("i: %d/%d\n", i, numrows);
 		/* Do the same row twice: 
 		   Once for the title, once for the contents */
 		for (j = 0; j < 2; j++) {
+			int limit;
 			
 			gtk_html_write (GTK_HTML (priv->html), priv->stream,
 					"<tr>", 4);
 				/* For each window on row i */
-			for (k = 0; k < MIN (3, (numwindows - (i * 3))); k++) {
-				
-				g_print ("%d of %d\n", k, 
-					 MIN (3, (numwindows - (i * 3))));
+			limit = MIN (columns, (numwindows - (i * columns)));
+			for (k = 0; k < limit; k++) {
 				if (window == NULL)
 					break;
 
@@ -582,7 +531,7 @@ e_summary_rebuild_page (ESummary *esummary)
 				
 				if (window != NULL)
 					window = window->next;
-
+				
 				if (window == NULL)
 					break;
 			}
@@ -601,7 +550,7 @@ e_summary_rebuild_page (ESummary *esummary)
 	}
 	gtk_html_write (GTK_HTML (priv->html), priv->stream, "</tr></table>", 13);
 	
-	if (priv->footer == NULL) {
+	if (priv->footer == NULL || *priv->footer == '\0') {
 		load_default_footer (esummary);
 	} else {
 		gtk_html_write (GTK_HTML (priv->html), priv->stream,
@@ -631,43 +580,31 @@ e_summary_queue_rebuild (ESummary *esummary)
 }
 
 static void
-html_event (BonoboListener *listener,
-	    char *event_name,
-	    CORBA_any *any,
-	    CORBA_Environment *ev,
-	    gpointer user_data)
+listener_event (BonoboListener *listener,
+		char *event_name,
+		BonoboArg *event_data,
+		CORBA_Environment *ev,
+		gpointer user_data)
 {
 	ESummaryWindow *window = (ESummaryWindow *) user_data;
 
-	g_print ("Event: %s\n", event_name);
-	if (strcmp (event_name, EXECUTIVE_SUMMARY_HTML_VIEW_HTML_CHANGED) != 0) {
-		return;
-	}
-
-	e_summary_queue_rebuild (window->esummary);
-}
-		
-static void
-prop_changed_cb (BonoboPropertyListener *listener,
-		 char *name,
-		 BonoboArg *arg,
-		 ESummaryWindow *window)
-{
-	if (strcmp (name, "window_title") == 0) {
+	if (strcmp (event_name, "Bonobo/Property:change:window_title") == 0) {
 		if (window->title != NULL)
 			g_free (window->title);
-		window->title = g_strdup (BONOBO_ARG_GET_STRING (arg));
-		e_summary_queue_rebuild (window->esummary);
-		return;
-	}
 
-	if (strcmp (name, "window_icon") == 0) {
+		window->title = g_strdup (BONOBO_ARG_GET_STRING (event_data));
+		e_summary_queue_rebuild (window->esummary);
+	} else if (strcmp (event_name, "Bonobo/Property:change:window_icon") == 0) {
 		if (window->icon != NULL)
 			g_free (window->icon);
-		window->icon = g_strdup (BONOBO_ARG_GET_STRING (arg));
+
+		window->icon = g_strdup (BONOBO_ARG_GET_STRING (event_data));
+		e_summary_queue_rebuild (window->esummary);
+	} else if (strcmp (event_name, EXECUTIVE_SUMMARY_HTML_VIEW_HTML_CHANGED) == 0) {
 		e_summary_queue_rebuild (window->esummary);
-		return;
 	}
+
+	return;
 }
 		
 ESummaryWindow *
@@ -678,7 +615,7 @@ e_summary_add_service (ESummary *esummary,
 	ESummaryWindow *window;
 	ESummaryPrivate *priv;
 	Bonobo_Unknown unknown = CORBA_OBJECT_NIL;
-	Bonobo_PropertyListener corba_listener;
+	Bonobo_Listener listener;
 	CORBA_Environment ev;
 
 	g_return_val_if_fail (esummary != NULL, NULL);
@@ -718,26 +655,22 @@ e_summary_add_service (ESummary *esummary,
 		return NULL;
 	}
 
-	if (window->html != CORBA_OBJECT_NIL) {
-		Bonobo_Listener listener;
-		CORBA_Environment ev2;
+	window->event_source = Bonobo_Unknown_queryInterface(window->component,
+							     "IDL:Bonobo/EventSource:1.0", &ev);
+	if (window->event_source == CORBA_OBJECT_NIL) {
+		g_warning ("There is no Bonobo::EventSource interface");
 
-		/* If HTML view, then set up the listeners. */
-		window->event_source = Bonobo_Unknown_queryInterface (window->html,
-								      "IDL:Bonobo/EventSource:1.0",
-								      &ev);
-		window->html_listener = bonobo_listener_new (html_event,
-							     window);
-		listener = bonobo_object_corba_objref (BONOBO_OBJECT (window->html_listener));
-		window->html_corba_listener = listener;
-
-		CORBA_exception_init (&ev2);
-		Bonobo_EventSource_addListener (window->event_source,
-						listener, &ev2);
-		/* Catch error? FIXME */
-		CORBA_exception_free (&ev2);
+		/* FIXME: Free whatever objects exist */
+		g_free (window);
+		return NULL;
 	}
 
+	window->listener = bonobo_listener_new (NULL, NULL);
+	gtk_signal_connect (GTK_OBJECT (window->listener), "event_notify",
+			    GTK_SIGNAL_FUNC (listener_event), window);
+	listener = bonobo_object_corba_objref (BONOBO_OBJECT (window->listener));
+	window->listener_id = Bonobo_EventSource_addListener (window->event_source, listener, &ev);
+
 	unknown = Bonobo_Unknown_queryInterface (component,
 						 "IDL:Bonobo/PropertyBag:1.0",
 						 &ev);
@@ -747,34 +680,20 @@ e_summary_add_service (ESummary *esummary,
 						 "IDL:Bonobo/PersistStream:1.0",
 						 &ev);
 	window->persiststream = (Bonobo_PersistStream) unknown;
-
+	
 	unknown = Bonobo_Unknown_queryInterface (component,
 						 "IDL:Bonobo/PropertyControl:1.0",
 						 &ev);
 	window->propertycontrol = (Bonobo_PropertyControl) unknown;
 
 	/* Cache the title and icon */
-	window->title = g_strdup (bonobo_property_bag_client_get_value_string (
-				          window->propertybag,
-					  "window_title", 
-					  NULL));
-	window->icon = g_strdup (bonobo_property_bag_client_get_value_string (
-                                          window->propertybag,
-					  "window_icon", NULL));
-	/* Listen to changes */
-	window->listener = bonobo_property_listener_new ();
-	gtk_signal_connect (GTK_OBJECT (window->listener), "prop_changed",
-			    GTK_SIGNAL_FUNC (prop_changed_cb), window);
-	corba_listener = bonobo_object_corba_objref (BONOBO_OBJECT (window->listener));
-	Bonobo_PropertyBag_addChangeListener (window->propertybag, 
-					      "window_title", 
-					      corba_listener, &ev);
+	window->title = bonobo_property_bag_client_get_value_string (window->propertybag,
+								     "window_title", 
+								     NULL);
+	window->icon = bonobo_property_bag_client_get_value_string (window->propertybag,
+								    "window_icon", NULL);
 
-	Bonobo_PropertyBag_addChangeListener (window->propertybag, 
-					      "window_icon", 
-					      corba_listener, &ev);
 	CORBA_exception_free (&ev);
-
 	priv->window_list = g_list_append (priv->window_list, window);
 
 	return window;
@@ -821,38 +740,20 @@ e_summary_window_free (ESummaryWindow *window)
 
 	if (window->event_source != CORBA_OBJECT_NIL) {
 		Bonobo_EventSource_removeListener (window->event_source,
-						   window->html_corba_listener,
+						   window->listener_id,
 						   &ev);
 		if (ev._major != CORBA_NO_EXCEPTION) {
 			g_warning ("CORBA ERROR: %s", CORBA_exception_id (&ev));
 		}
-		bonobo_object_release_unref (window->event_source, &ev);
-	}
-
-	if (window->html != CORBA_OBJECT_NIL) {
-		bonobo_object_release_unref (window->html, &ev);
-	}
-
-	if (window->propertybag != CORBA_OBJECT_NIL) {
-		bonobo_object_release_unref (window->propertybag, &ev);
-	}
-
-	if (window->persiststream != CORBA_OBJECT_NIL) {
-		bonobo_object_release_unref (window->persiststream, &ev);
-	}
-
-	if (window->propertycontrol != CORBA_OBJECT_NIL) {
-		bonobo_object_release_unref (window->propertycontrol, &ev);
-	}
-
-	if (window->listener) {
-		bonobo_object_unref (BONOBO_OBJECT (window->listener));
-	}
-
-	if (window->html_listener) {
-		bonobo_object_unref (BONOBO_OBJECT (window->html_listener));
+    		bonobo_object_release_unref (window->event_source, &ev);
 	}
 
+	bonobo_object_release_unref (window->propertybag, &ev);
+	bonobo_object_release_unref (window->persiststream, &ev);
+	bonobo_object_release_unref (window->propertycontrol, &ev);
+	bonobo_object_unref (BONOBO_OBJECT (window->listener));
+	bonobo_object_release_unref (window->html, &ev);
+	
 	bonobo_object_release_unref (window->component, &ev);
 	CORBA_exception_free (&ev);
 
@@ -996,10 +897,12 @@ e_summary_load_page (ESummary *esummary)
 	
 	priv = esummary->private;
 
-	filename = g_strdup (esummary->prefs->page);
 	/* Pass NULL to reset the page to the default */
-	if (filename == NULL || *filename == '\0') {
+	if ((esummary->prefs->page) == NULL || 
+	    *(esummary->prefs->page) == '\0') {
 		filename = g_concat_dir_and_file (EVOLUTION_DATADIR, "/evolution/summary.html");
+	} else {
+		filename = g_strdup (esummary->prefs->page);
 	}
 
 	toplevel = gtk_widget_get_toplevel (GTK_WIDGET (esummary));
@@ -1044,8 +947,13 @@ e_summary_load_page (ESummary *esummary)
 		return;
 	}
 
+	if (priv->header != NULL)
+		g_free (priv->header);
 	priv->header = g_strndup (str, comment - str);
 	priv->header_len = strlen (priv->header);
+
+	if (priv->footer != NULL)
+		g_free (priv->footer);
 	priv->footer = g_strdup (comment);
 	priv->footer_len = strlen (priv->footer);
 	g_free (str);
@@ -1170,10 +1078,6 @@ load_component (ESummary *esummary,
 void
 e_summary_reconfigure (ESummary *esummary)
 {
-	ESummaryPrefs *prefs;
-
-	prefs = esummary->prefs;
-
 	e_summary_load_page (esummary);
 	e_summary_queue_rebuild (esummary);
 }
@@ -1216,6 +1120,9 @@ e_summary_load_state (ESummary *esummary,
 	g_free (fullpath);
 
 	/* Load the preferences */
+	if (esummary->prefs != NULL)
+		e_summary_prefs_free (esummary->prefs);
+
 	esummary->prefs = e_summary_prefs_load (path);
 	e_summary_reconfigure (esummary);
 }
