@@ -2,6 +2,7 @@
  *  Copyright (C) 2000 Helix Code Inc.
  *
  *  Authors: Not Zed <notzed@lostzed.mmc.com.au>
+ *           Jeffrey Stedfast <fejj@helixcode.com>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public License
@@ -18,7 +19,7 @@
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#include <gtk/gtk.h>
+#include <config.h>
 #include <gnome.h>
 #include <glade/glade.h>
 
@@ -63,7 +64,7 @@ filter_editor_get_type (void)
 			(GtkArgGetFunc)NULL
 		};
 		
-		type = gtk_type_unique(gnome_dialog_get_type (), &type_info);
+		type = gtk_type_unique (gnome_dialog_get_type (), &type_info);
 	}
 	
 	return type;
@@ -75,20 +76,20 @@ filter_editor_class_init (FilterEditorClass *class)
 	GtkObjectClass *object_class;
 	
 	object_class = (GtkObjectClass *)class;
-	parent_class = gtk_type_class(gnome_dialog_get_type ());
-
+	parent_class = gtk_type_class (gnome_dialog_get_type ());
+	
 	object_class->finalize = filter_editor_finalise;
 	/* override methods */
-
+	
 	/* signals */
-
-	gtk_object_class_add_signals(object_class, signals, LAST_SIGNAL);
+	
+	gtk_object_class_add_signals (object_class, signals, LAST_SIGNAL);
 }
 
 static void
 filter_editor_init (FilterEditor *o)
 {
-	o->priv = g_malloc0(sizeof(*o->priv));
+	o->priv = g_malloc0 (sizeof (*o->priv));
 }
 
 static void
@@ -109,7 +110,7 @@ filter_editor_finalise (GtkObject *obj)
 FilterEditor *
 filter_editor_new (void)
 {
-	FilterEditor *o = (FilterEditor *)gtk_type_new(filter_editor_get_type ());
+	FilterEditor *o = (FilterEditor *)gtk_type_new (filter_editor_get_type ());
 	return o;
 }
 
@@ -143,6 +144,7 @@ rule_add (GtkWidget *widget, struct _editor_data *data)
 	GtkWidget *gd;
 	GtkWidget *w;
 	FilterPart *part;
+	gboolean done = FALSE;
 	
 	d(printf ("add rule\n"));
 	/* create a new rule with 1 match and 1 action */
@@ -155,33 +157,63 @@ rule_add (GtkWidget *widget, struct _editor_data *data)
 	filter_filter_add_action (rule, filter_part_clone (part));
 	
 	w = filter_rule_get_widget ((FilterRule *)rule, data->f);
-	gd = gnome_dialog_new (_("Add Rule"), GNOME_STOCK_BUTTON_OK, GNOME_STOCK_BUTTON_CANCEL, NULL);
-	gtk_window_set_policy(GTK_WINDOW(gd), FALSE, TRUE, FALSE);
+	gd = gnome_dialog_new (_("Add Filter Rule"),
+			       GNOME_STOCK_BUTTON_OK,
+			       GNOME_STOCK_BUTTON_CANCEL,
+			       NULL);
+	gtk_window_set_policy (GTK_WINDOW (gd), FALSE, TRUE, FALSE);
 	gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (gd)->vbox), w, TRUE, TRUE, 0);
+	
 	gtk_widget_show (gd);
 	
-	result = gnome_dialog_run_and_close (GNOME_DIALOG (gd));
+	while (!done) {
+		result = gnome_dialog_run (GNOME_DIALOG (gd));
 	
-	if (result == 0) {
-		GtkListItem *item;
-		GList *l = NULL;
-		gchar *s = e_utf8_to_gtk_string (GTK_WIDGET (data->list), ((FilterRule *)rule)->name);
-		
-		item = (GtkListItem *) gtk_list_item_new_with_label (s);
-		g_free (s);
-		
-		gtk_object_set_data (GTK_OBJECT (item), "rule", rule);
-		gtk_widget_show (GTK_WIDGET (item));
-		
-		l = g_list_append (l, item);
-		gtk_list_append_items (data->list, l);
-		gtk_list_select_child (data->list, GTK_WIDGET (item));
-		
-		data->current = (FilterRule *)rule;
-		rule_context_add_rule (data->f, (FilterRule *)rule);
-		set_sensitive (data);
-	} else {
-		gtk_object_unref (GTK_OBJECT (rule));
+		if (result == 0) {
+			GtkWidget *item;
+			GList *parts, *l = NULL;
+			gchar *s;
+			
+			done = TRUE;
+			
+			/* validate rule parts */
+			parts = ((FilterRule *)rule)->parts;
+			while (parts && done) {
+				done = filter_part_validate ((FilterPart *) parts->data);
+				parts = parts->next;
+			}
+			
+			/* validate action parts */
+			parts = rule->actions;
+			while (parts && done) {
+				done = filter_part_validate ((FilterPart *) parts->data);
+				parts = parts->next;
+			}
+			
+			if (done) {
+				s = e_utf8_to_gtk_string (GTK_WIDGET (data->list), ((FilterRule *)rule)->name);
+				item = gtk_list_item_new_with_label (s);
+				g_free (s);
+				
+				gtk_object_set_data (GTK_OBJECT (item), "rule", rule);
+				gtk_widget_show (item);
+				
+				l = g_list_append (l, GTK_LIST_ITEM (item));
+				
+				gtk_list_append_items (data->list, l);
+				gtk_list_select_child (data->list, item);
+				
+				data->current = (FilterRule *)rule;
+				rule_context_add_rule (data->f, (FilterRule *)rule);
+				
+				set_sensitive (data);
+				gtk_widget_destroy (gd);
+			}
+		} else {
+			gtk_widget_destroy (gd);
+			gtk_object_unref (GTK_OBJECT (rule));
+			done = TRUE;
+		}
 	}
 }
 
@@ -192,26 +224,58 @@ rule_edit (GtkWidget *widget, struct _editor_data *data)
 	int result;
 	GtkWidget *gd;
 	FilterRule *rule;
+	gboolean done = FALSE;
 	int pos;
 	
 	d(printf ("edit rule\n"));
 	rule = data->current;
 	w = filter_rule_get_widget (rule, data->f);
-	gd = gnome_dialog_new(_("Edit Rule"), GNOME_STOCK_BUTTON_OK, GNOME_STOCK_BUTTON_CANCEL, NULL);
-	gtk_window_set_policy(GTK_WINDOW(gd), FALSE, TRUE, FALSE);
+	gd = gnome_dialog_new (_("Edit Filter Rule"),
+			       GNOME_STOCK_BUTTON_OK,
+			       GNOME_STOCK_BUTTON_CANCEL,
+			       NULL);
+	gtk_window_set_policy (GTK_WINDOW (gd), FALSE, TRUE, FALSE);
 	gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (gd)->vbox), w, TRUE, TRUE, 0);
-	gtk_widget_show (gd);
-	result = gnome_dialog_run_and_close (GNOME_DIALOG (gd));
 	
-	if (result == 0) {
-		pos = rule_context_get_rank_rule(data->f, data->current, data->current_source);
-		if (pos != -1) {
-			GtkListItem *item = g_list_nth_data (data->list->children, pos);
-			gchar *s = e_utf8_to_gtk_string ((GtkWidget *) item, data->current->name);
-			gtk_label_set_text((GtkLabel *)(((GtkBin *)item)->child), s);
-			g_free (s);
+	gtk_widget_show (gd);
+	
+	while (!done) {
+		result = gnome_dialog_run (GNOME_DIALOG (gd));
+		
+		if (result == 0) {
+			GList *parts;
+			
+			done = TRUE;
+			
+			/* validate rule parts */
+			parts = rule->parts;
+			while (parts && done) {
+				done = filter_part_validate ((FilterPart *) parts->data);
+				parts = parts->next;
+			}
+			
+			/* validate action parts */
+			parts = ((FilterFilter *)rule)->actions;
+			while (parts && done) {
+				done = filter_part_validate ((FilterPart *) parts->data);
+				parts = parts->next;
+			}
+			
+			if (done) {
+				pos = rule_context_get_rank_rule (data->f, data->current, data->current_source);
+				if (pos != -1) {
+					GtkListItem *item = g_list_nth_data (data->list->children, pos);
+					gchar *s = e_utf8_to_gtk_string (GTK_WIDGET (item), data->current->name);
+					gtk_label_set_text (GTK_LABEL (GTK_BIN (item)->child), s);
+					g_free (s);
+				}
+				gtk_widget_destroy (gd);
+			}
+		} else {
+			gtk_widget_destroy (gd);
+			done = TRUE;
 		}
-	 }
+	}
 }
 
 static void
@@ -220,8 +284,8 @@ rule_delete (GtkWidget *widget, struct _editor_data *data)
 	int pos;
 	GList *l;
 	GtkListItem *item;
-
-	d(printf("ddelete rule\n"));
+	
+	d(printf("delete rule\n"));
 	pos = rule_context_get_rank_rule(data->f, data->current, data->current_source);
 	if (pos != -1) {
 		rule_context_remove_rule (data->f, data->current);
@@ -234,6 +298,7 @@ rule_delete (GtkWidget *widget, struct _editor_data *data)
 		gtk_object_unref (GTK_OBJECT (data->current));
 		data->current = NULL;
 	}
+	
 	set_sensitive (data);
 }
 
@@ -260,9 +325,9 @@ rule_up (GtkWidget *widget, struct _editor_data *data)
 	int pos;
 	
 	d(printf("up rule\n"));
-	pos = rule_context_get_rank_rule(data->f, data->current, data->current_source);
+	pos = rule_context_get_rank_rule (data->f, data->current, data->current_source);
 	if (pos > 0) {
-		rule_move (data, pos, pos-1);
+		rule_move (data, pos, pos - 1);
 	}
 }
 
@@ -272,8 +337,8 @@ rule_down (GtkWidget *widget, struct _editor_data *data)
 	int pos;
 	
 	d(printf ("down rule\n"));
-	pos = rule_context_get_rank_rule(data->f, data->current, data->current_source);
-	rule_move (data, pos, pos+1);
+	pos = rule_context_get_rank_rule (data->f, data->current, data->current_source);
+	rule_move (data, pos, pos + 1);
 }
 
 static struct {
@@ -295,11 +360,14 @@ set_sensitive (struct _editor_data *data)
 	
 	while ((rule = rule_context_next_rule (data->f, rule, data->current_source))) {
 		if (rule == data->current)
-			index=count;
+			index = count;
 		count++;
 	}
+	
 	d(printf ("index = %d count=%d\n", index, count));
+	
 	count--;
+	
 	gtk_widget_set_sensitive (GTK_WIDGET (data->buttons[BUTTON_EDIT]), index != -1);
 	gtk_widget_set_sensitive (GTK_WIDGET (data->buttons[BUTTON_DELETE]), index != -1);
 	gtk_widget_set_sensitive (GTK_WIDGET (data->buttons[BUTTON_UP]), index > 0);
@@ -312,7 +380,7 @@ select_rule (GtkWidget *w, GtkWidget *child, struct _editor_data *data)
 	data->current = gtk_object_get_data (GTK_OBJECT (child), "rule");
 	
 	if (data->current)
-		d(printf ("seledct rule: %s\n", data->current->name));
+		d(printf ("selected rule: %s\n", data->current->name));
 	else
 		d(printf ("bad data?\n"));
 	
@@ -362,9 +430,9 @@ select_source (GtkMenuItem *mi, struct _editor_data *data)
 }
 
 static char *source_names[] = {
-	"incoming",
+	N_("incoming"),
 	/*"demand",*/
-	"outgoing"
+	N_("outgoing")
 };
 
 GtkWidget *
@@ -385,7 +453,7 @@ filter_editor_construct (struct _FilterContext *f)
         d = glade_xml_get_widget (gui, "edit_filter");
 	gtk_object_set_data_full (GTK_OBJECT (d), "data", data, g_free);
 	
-	gtk_window_set_title (GTK_WINDOW (d), "Edit Filters");
+	gtk_window_set_title (GTK_WINDOW (d), _("Edit Filters"));
 	for (i = 0; i < BUTTON_LAST; i++) {
 		data->buttons[i] = (GtkButton *)w = glade_xml_get_widget (gui, edit_buttons[i].name);
 		gtk_signal_connect (GTK_OBJECT (w), "clicked", edit_buttons[i].func, data);
@@ -418,6 +486,8 @@ filter_editor_construct (struct _FilterContext *f)
 	gtk_signal_connect (GTK_OBJECT (w), "button_press_event", double_click, data);
 	select_source (GTK_MENU_ITEM (firstitem), data);
 	
+	set_sensitive (data);
+	
 	gtk_object_unref (GTK_OBJECT (gui));
 	
 	return d;
