@@ -3,7 +3,7 @@
   FILE: icalderivedparameters.{c,h}
   CREATOR: eric 09 May 1999
   
-  $Id: icalparameter.c.in,v 1.2 2000/12/14 02:13:54 federico Exp $
+  $Id: icalparameter.c.in,v 1.3.2.1 2001/02/09 08:52:52 jpr Exp $
   $Locker:  $
     
 
@@ -154,7 +154,7 @@ struct param_string_map {
     {ICAL_FBTYPE_PARAMETER,ICAL_FBTYPE_BUSY,"BUSY"},
     {ICAL_FBTYPE_PARAMETER,ICAL_FBTYPE_BUSYUNAVAILABLE,"BUSYUNAVAILABLE"},
     {ICAL_FBTYPE_PARAMETER,ICAL_FBTYPE_BUSYTENTATIVE,"BUSYTENTATIVE"},
-    {ICAL_PARTSTAT_PARAMETER,ICAL_PARTSTAT_NEEDSACTION,"NEEDSACTION"},
+    {ICAL_PARTSTAT_PARAMETER,ICAL_PARTSTAT_NEEDSACTION,"NEEDS-ACTION"},
     {ICAL_PARTSTAT_PARAMETER,ICAL_PARTSTAT_ACCEPTED,"ACCEPTED"},
     {ICAL_PARTSTAT_PARAMETER,ICAL_PARTSTAT_DECLINED,"DECLINED"},
     {ICAL_PARTSTAT_PARAMETER,ICAL_PARTSTAT_TENTATIVE,"TENTATIVE"},
@@ -233,7 +233,8 @@ void icalparameter_set_impl_data(icalparameter_kind kind,
 	   data->v_xlicerrortype=value; break;
 	case ICAL_XLICCOMPARETYPE_PARAMETER:
 	   data->v_xliccomparetype=value; break;
-	default:
+	default: 
+	    break;
     }
 }
 
@@ -428,7 +429,7 @@ icalparameter* icalparameter_new_from_string(icalparameter_kind kind, char* val)
 	}
 	case ICAL_PARTSTAT_PARAMETER:
 	{
-	    if(strcmp(val,"NEEDSACTION") == 0){ 
+	    if(strcmp(val,"NEEDS-ACTION") == 0){ 
 		param = icalparameter_new_partstat(ICAL_PARTSTAT_NEEDSACTION);
 	    }
 	    else if(strcmp(val,"ACCEPTED") == 0){ 
@@ -750,7 +751,7 @@ icalparameter_as_ical_string (icalparameter* parameter)
 
     }
 
-    icalmemory_append_string(&buf, &buf_ptr, &buf_size, "=");
+	icalmemory_append_string(&buf, &buf_ptr, &buf_size, "=");
     
     switch (impl->kind) {
 	case ICAL_CUTYPE_PARAMETER:
@@ -828,7 +829,7 @@ icalparameter_as_ical_string (icalparameter* parameter)
 	{
 	    switch (impl->data.v_partstat) {
 		case ICAL_PARTSTAT_NEEDSACTION: {
-		    strcpy(tend,"NEEDSACTION");break;
+		    strcpy(tend,"NEEDS-ACTION");break;
 		}
 		case ICAL_PARTSTAT_ACCEPTED: {
 		    strcpy(tend,"ACCEPTED");break;
@@ -1236,7 +1237,7 @@ icalproperty* icalparameter_get_parent(icalparameter* param)
 {
     struct icalparameter_impl *impl = (struct icalparameter_impl*)param;
 
-    icalerror_check_arg_rv( (param!=0),"param");
+    icalerror_check_arg_rz( (param!=0),"param");
 
     return impl->parent;
 }
