@@ -3,7 +3,7 @@
   FILE: icalvalue.c
   CREATOR: eric 02 May 1999
   
-  $Id: icalvalue.c.in,v 1.1.1.1 2000/12/11 22:06:02 federico Exp $
+  $Id: icalvalue.c.in,v 1.4 2001/02/05 19:43:57 jpr Exp $
 
 
  (C) COPYRIGHT 2000, Eric Busboom, http://www.softwarestudio.org
@@ -57,45 +57,46 @@ void print_time_to_string(char* str,  struct icaltimetype *data);
 void print_recur_to_string(char* str,  struct icaltimetype *data);
 
 struct icalvalue_impl {
-	icalvalue_kind kind;
-	char id[5];
-	int size;
-	icalproperty* parent;
+    icalvalue_kind kind; /*this is the kind that is visible from the outside*/
 
-	union data {
-		struct icalattachtype v_attach;		
-		/* void *v_binary; */ /* use v_attach */
-		const char *v_string;
-		/*char *v_text;*/
-		/*char *v_caladdress;*/
-		/*char *v_uri;*/
-		float v_float;
-		int v_int;
-		/*int v_boolean;*/
-		/*int v_integer;*/
-		struct icaldurationtype v_duration;
-		/*int v_utcoffset;*/
-
-		struct icalperiodtype v_period;
-		/*struct icalperiodtype v_datetimeperiod;*/
-		struct icalgeotype v_geo;
-		/*time_t v_time;*/
-		struct icaltimetype v_time;
-		/*struct icaltimetype v_date;*/
-		/*struct icaltimetype v_datetime;*/
-		/*struct icaltimetype v_datetimedate;*/
-
-		/* struct icalrecurrencetype was once included
-                   directly ( not referenced ) in this union, but it
-                   contributes 2000 bytes to every value, so now it is
-                   a reference*/
-
-		struct icalrecurrencetype *v_recur;
-		union icaltriggertype v_trigger;
-		icalproperty_method v_method;
-		icalproperty_status v_status;
-
-	} data;
+    char id[5];
+    int size;
+    icalproperty* parent;
+    
+    union data {
+	struct icalattachtype v_attach;		
+	/* void *v_binary; */ /* use v_attach */
+	const char *v_string;
+	/*char *v_text;*/
+	/*char *v_caladdress;*/
+	/*char *v_uri;*/
+	float v_float;
+	int v_int;
+	/*int v_boolean;*/
+	/*int v_integer;*/
+	struct icaldurationtype v_duration;
+	/*int v_utcoffset;*/
+	
+	struct icalperiodtype v_period;
+	/*struct icalperiodtype v_datetimeperiod;*/
+	struct icalgeotype v_geo;
+	/*time_t v_time;*/
+	struct icaltimetype v_time;
+	/*struct icaltimetype v_date;*/
+	/*struct icaltimetype v_datetime;*/
+	/*struct icaltimetype v_datetimedate;*/
+	
+	/* struct icalrecurrencetype was once included
+	   directly ( not referenced ) in this union, but it
+	   contributes 2000 bytes to every value, so now it is
+	   a reference*/
+	
+	struct icalrecurrencetype *v_recur;
+	struct icaltriggertype v_trigger;
+	icalproperty_method v_method;
+	icalproperty_status v_status;
+	
+    } data;
 };
 
 struct icalvalue_impl*  icalvalue_new_impl(icalvalue_kind kind){
@@ -197,7 +198,7 @@ icalvalue* icalvalue_new_clone(icalvalue* value){
 
 char* icalmemory_strdup_and_dequote(const char* str)
 {
-    char* p;
+    const char* p;
     char* out = (char*)malloc(sizeof(char) * strlen(str) +1);
     char* pout;
 
@@ -215,8 +216,9 @@ char* icalmemory_strdup_and_dequote(const char* str)
 	    switch(*p){
 		case 0:
 		{
-		    break;
 		    *pout = '\0';
+		    break;
+
 		}
 		case 'n':
 		{
@@ -417,18 +419,56 @@ icalvalue* icalvalue_new_from_string_with_error(icalvalue_kind kind,const char*
 	}
 
 	case ICAL_RECUR_VALUE:
-	case ICAL_DATE_VALUE:
-	case ICAL_DATETIME_VALUE:
-	case ICAL_DATETIMEDATE_VALUE:
-	case ICAL_DATETIMEPERIOD_VALUE:
+	{
+	    struct icalrecurrencetype rt;
+	    rt = icalrecurrencetype_from_string(str);
+	    value = icalvalue_new_recur(rt);
+	    break;
+	}
+
 	case ICAL_TIME_VALUE:
-	case ICAL_DURATION_VALUE:
-	case ICAL_PERIOD_VALUE:
-	case ICAL_TRIGGER_VALUE:
+	{
+	    struct icaltimetype tt;
+	    tt = icaltime_from_string(str);
+	    value = icalvalue_new_time(tt);
+	    break;
+	}
+	case ICAL_DATE_VALUE:
+	{
+	    struct icaltimetype tt;
+	    tt = icaltime_from_string(str);
+	    value = icalvalue_new_date(tt);
+	    break;
+	}
+	case ICAL_DATETIME_VALUE:
+	{
+	    struct icaltimetype tt;
+	    tt = icaltime_from_string(str);
+	    value = icalvalue_new_datetime(tt);
+	    break;
+	}
+	case ICAL_DATETIMEDATE_VALUE:
+	{
+	    struct icaltimetype tt;
+	    tt = icaltime_from_string(str);
+	    value = icalvalue_new_datetimedate(tt);
+	    break;
+	}
+
+	case ICAL_DATETIMEPERIOD_VALUE:
+        case ICAL_DURATION_VALUE:
+         case ICAL_PERIOD_VALUE:
 	{
 	    value = icalparser_parse_value(kind,str,error);
 	    break;
 	}
+	
+        case ICAL_TRIGGER_VALUE:
+	{
+	    struct icaltriggertype tr = icaltriggertype_from_string(str);
+	    value = icalvalue_new_trigger(tr);
+	    break;
+	}
 
 	default:
 	{
@@ -547,6 +587,7 @@ char* icalvalue_binary_as_ical_string(icalvalue* value) {
     const char* data;
     char* str;
     icalerror_check_arg_rz( (value!=0),"value");
+
     data = icalvalue_get_binary(value);
 
     str = (char*)icalmemory_tmp_buffer(60);
@@ -556,14 +597,17 @@ char* icalvalue_binary_as_ical_string(icalvalue* value) {
 }
 
 
+#define MAX_INT_DIGITS 12 /* Enough for 2^32 + sign*/ 
 char* icalvalue_int_as_ical_string(icalvalue* value) {
     
     int data;
-    char* str = (char*)icalmemory_tmp_buffer(2);
+    char* str = (char*)icalmemory_tmp_buffer(MAX_INT_DIGITS); 
+
     icalerror_check_arg_rz( (value!=0),"value");
+
     data = icalvalue_get_integer(value);
 	
-    sprintf(str,"%d",data);
+    snprintf(str,MAX_INT_DIGITS,"%d",data);
 
     return str;
 }
@@ -573,7 +617,9 @@ char* icalvalue_utcoffset_as_ical_string(icalvalue* value)
     int data,h,m,s;
     char sign;
     char* str = (char*)icalmemory_tmp_buffer(9);
+
     icalerror_check_arg_rz( (value!=0),"value");
+
     data = icalvalue_get_utcoffset(value);
 
     if (abs(data) == data){
@@ -608,101 +654,10 @@ char* icalvalue_string_as_ical_string(icalvalue* value) {
 
 char* icalvalue_recur_as_ical_string(icalvalue* value) 
 {
-    char* str;
-    char *str_p;
-    size_t buf_sz = 200;
-    char temp[20];
-    int i,j;
     struct icalvalue_impl *impl = (struct icalvalue_impl*)value;
     struct icalrecurrencetype *recur = impl->data.v_recur;
 
-    struct { char* str;size_t offset; short limit;  } recurmap[] = 
-      {
-        {";BYSECOND=",offsetof(struct icalrecurrencetype,by_second),60},
-        {";BYMINUTE=",offsetof(struct icalrecurrencetype,by_minute),60},
-        {";BYHOUR=",offsetof(struct icalrecurrencetype,by_hour),24},
-        {";BYDAY=",offsetof(struct icalrecurrencetype,by_day),7},
-        {";BYMONTHDAY=",offsetof(struct icalrecurrencetype,by_month_day),31},
-        {";BYYEARDAY=",offsetof(struct icalrecurrencetype,by_year_day),366},
-        {";BYWEEKNO=",offsetof(struct icalrecurrencetype,by_week_no),52},
-        {";BYMONTH=",offsetof(struct icalrecurrencetype,by_month),12},
-        {";BYSETPOS=",offsetof(struct icalrecurrencetype,by_set_pos),366},
-        {0,0,0},
-      };
-
-
-
-    icalerror_check_arg_rz((value != 0),"value");
-
-    if(recur->freq == ICAL_NO_RECURRENCE){
-	return 0;
-    }
-
-    str = (char*)icalmemory_tmp_buffer(buf_sz);
-    str_p = str;
-
-    icalmemory_append_string(&str,&str_p,&buf_sz,"FREQ=");
-    icalmemory_append_string(&str,&str_p,&buf_sz,
-			     icalrecur_recurrence_to_string(recur->freq));
-
-    if(recur->until.year != 0){
-	
-	temp[0] = 0;
-	print_datetime_to_string(temp,&(recur->until));
-	
-	icalmemory_append_string(&str,&str_p,&buf_sz,";UNTIL=");
-	icalmemory_append_string(&str,&str_p,&buf_sz, temp);
-    }
-
-    if(recur->count != 0){
-	sprintf(temp,"%d",recur->count);
-	icalmemory_append_string(&str,&str_p,&buf_sz,";COUNT=");
-	icalmemory_append_string(&str,&str_p,&buf_sz, temp);
-    }
-
-    if(recur->interval != 0){
-	sprintf(temp,"%d",recur->interval);
-	icalmemory_append_string(&str,&str_p,&buf_sz,";INTERVAL=");
-	icalmemory_append_string(&str,&str_p,&buf_sz, temp);
-    }
-    
-    for(j =0; recurmap[j].str != 0; j++){
-	short* array = (short*)(recurmap[j].offset+ (size_t)recur);
-	short limit = recurmap[j].limit;
-
-	/* Skip unused arrays */
-	if( array[0] != ICAL_RECURRENCE_ARRAY_MAX ) {
-
-	    icalmemory_append_string(&str,&str_p,&buf_sz,recurmap[j].str);
-	    
-	    for(i=0; i< limit  && array[i] != ICAL_RECURRENCE_ARRAY_MAX;
-		i++){
-		if (j == 3) { /* BYDAY */
-		    short dow = icalrecurrencetype_day_day_of_week(array[i]);
-		    const char *daystr = icalrecur_weekday_to_string(dow);
-		    short pos = icalrecurrencetype_day_position(array[i]);  
-		    
-		    if (pos == 0)
-			icalmemory_append_string(&str,&str_p,&buf_sz,daystr);
-		    else {
-			sprintf(temp,"%d%s",pos,daystr);
-			icalmemory_append_string(&str,&str_p,&buf_sz,temp);
-		    }                  
-		    
-		} else {
-		    sprintf(temp,"%d",array[i]);
-		    icalmemory_append_string(&str,&str_p,&buf_sz, temp);
-		}
-		
-		if( (i+1)<limit &&array[i+1] 
-		    != ICAL_RECURRENCE_ARRAY_MAX){
-		    icalmemory_append_char(&str,&str_p,&buf_sz,',');
-		}
-	    }	 
-	}   
-    }
-
-    return  str;
+    return icalrecurrencetype_as_string(recur);
 }
 
 char* icalvalue_text_as_ical_string(icalvalue* value) {
@@ -826,64 +781,17 @@ char* icalvalue_attach_as_ical_string(icalvalue* value) {
     }
 }
 
-void append_duration_segment(char** buf, char** buf_ptr, size_t* buf_size, 
-			     char* sep, unsigned int value) {
 
-    char temp[TMP_BUF_SIZE];
-
-    sprintf(temp,"%d",value);
-
-    icalmemory_append_string(buf, buf_ptr, buf_size, temp);
-    icalmemory_append_string(buf, buf_ptr, buf_size, sep);
-    
-}
-
-char* icalvalue_duration_as_ical_string(icalvalue* value) {
+char* icalvalue_duration_as_ical_string(icalvalue* value) 
+{
 
     struct icaldurationtype data;
-    char *buf, *output_line;
-    size_t buf_size = 256;
-    char* buf_ptr = 0;
 
     icalerror_check_arg_rz( (value!=0),"value");
     data = icalvalue_get_duration(value);
 
-    buf = (char*)icalmemory_new_buffer(buf_size);
-    buf_ptr = buf;
-    
-    icalmemory_append_string(&buf, &buf_ptr, &buf_size, "P");
-    
-    
-    if (data.weeks != 0 ) {
-	append_duration_segment(&buf, &buf_ptr, &buf_size, "W", data.weeks);
-    }
+    return icaldurationtype_as_ical_string(data);
 
-    if (data.days != 0 ) {
-	append_duration_segment(&buf, &buf_ptr, &buf_size, "D", data.days);
-    }
-
-    if (data.hours != 0 || data.minutes != 0 || data.seconds != 0) {
-
-	icalmemory_append_string(&buf, &buf_ptr, &buf_size, "T");
-
-	if (data.hours != 0 ) {
-	    append_duration_segment(&buf, &buf_ptr, &buf_size, "H", data.hours);
-	}
-	if (data.minutes != 0 ) {
-	    append_duration_segment(&buf, &buf_ptr, &buf_size, "M", data.minutes);
-	}
-	if (data.seconds != 0 ) {
-	    append_duration_segment(&buf, &buf_ptr, &buf_size, "S", data.seconds);
-	}
-
-    }
- 
-    output_line = icalmemory_tmp_copy(buf);
-    icalmemory_free_buffer(buf);
-
-    return output_line;
-
-    
 }
 
 void print_time_to_string(char* str,  struct icaltimetype *data)
@@ -951,8 +859,21 @@ char* icalvalue_datetime_as_ical_string(icalvalue* value) {
     
     struct icaltimetype data;
     char* str;
+    icalvalue_kind kind = icalvalue_isa(value);    
+
     icalerror_check_arg_rz( (value!=0),"value");
-    data = icalvalue_get_date(value);
+
+
+    if( !(kind == ICAL_DATETIMEDATE_VALUE ||
+	  kind == ICAL_DATE_VALUE ||
+	  kind == ICAL_DATETIME_VALUE ||
+	  kind == ICAL_TIME_VALUE))
+	{
+	    icalerror_set_errno(ICAL_BADARG_ERROR);
+	    return 0;
+	}
+
+    data = icalvalue_get_datetime(value);
 
     str = (char*)icalmemory_tmp_buffer(20);
  
@@ -1009,89 +930,43 @@ char* icalvalue_geo_as_ical_string(icalvalue* value) {
 }
 
 char* icalvalue_datetimeperiod_as_ical_string(icalvalue* value) {
+    struct icalvalue_impl* impl = (struct icalvalue_impl*)value;
+    struct icaldatetimeperiodtype dtp = icalvalue_get_datetimeperiod(value);
 
-    struct icalperiodtype data;
-    char* str;
     icalerror_check_arg_rz( (value!=0),"value");
-    data = icalvalue_get_datetimeperiod(value);
-
-    str = (char*)icalmemory_tmp_buffer(60);
-
-    if( data.end.second == -1){
-	/* This is a DATE-TIME value, since there is no end value */
-	icalvalue *v= icalvalue_new_datetime(data.start);
-
-	strcpy(str,icalvalue_datetime_as_ical_string(v));
-
-	free(v);
 
+    if(!icaltime_is_null_time(dtp.time)){
+	return icaltime_as_ical_string(dtp.time);
     } else {
-	icalvalue *v1 = icalvalue_new_datetime(data.start);
-	icalvalue *v2 = icalvalue_new_datetime(data.end);
-
-	sprintf(str,"%s/%s",
-		icalvalue_datetime_as_ical_string(v1),
-		icalvalue_datetime_as_ical_string(v2)
-	    );
-
-	free(v1);
-	free(v2);
-
+	return icalperiodtype_as_ical_string(dtp.period);
     }
-
-    return str;
 }
 
 char* icalvalue_period_as_ical_string(icalvalue* value) {
-
     struct icalperiodtype data;
-    char* str;
-    icalvalue *s,*e;
-    
     icalerror_check_arg_rz( (value!=0),"value");
     data = icalvalue_get_period(value);
 
-    str = (char*)icalmemory_tmp_buffer(60);
-
-    s = icalvalue_new_datetime(data.start);
-
-    if (data.end.second != -1){
-	/* use the end date */
-	e = icalvalue_new_datetime(data.end);
-	
-	sprintf(str,"%s/%s",
-		icalvalue_datetime_as_ical_string(s),
-		icalvalue_datetime_as_ical_string(e)
-	    );
-	
-
-    } else {
-	/* use the duration */
-	e = icalvalue_new_duration(data.duration);
-	
-	sprintf(str,"%s/%s",
-		icalvalue_datetime_as_ical_string(s),
-		icalvalue_duration_as_ical_string(e)
-	    );
-	
-    }
-
-    icalvalue_free(e);
-    icalvalue_free(s);
-    return str;
+    return icalperiodtype_as_ical_string(data);
 }
 
-char* icalvalue_trigger_as_ical_string(icalvalue* value) {
+char* icalvalue_trigger_as_ical_string(icalvalue* value) 
+{
 
-    union icaltriggertype data;
+    struct icaltriggertype data;
     char* str;
+
     icalerror_check_arg_rz( (value!=0),"value");
     data = icalvalue_get_trigger(value);
-
+    
     str = (char*)icalmemory_tmp_buffer(60);
-    sprintf(str,"icalvalue_trigger_as_ical_string is not implemented yet");
 
-    return str;
+    if(!icaltime_is_null_time(data.time)){
+	return icaltime_as_ical_string(data.time);
+    } else {
+	return icaldurationtype_as_ical_string(data.duration);
+    }   
+
 }
 
 const char*
@@ -1200,6 +1075,20 @@ icalvalue_isa_value (void* value)
 }
 
 
+int icalvalue_is_time(icalvalue* a) {
+    icalvalue_kind kind = icalvalue_isa(a);
+
+    if(kind == ICAL_DATETIMEDATE_VALUE ||
+       kind == ICAL_DATETIME_VALUE ||
+       kind == ICAL_DATE_VALUE ||
+       kind == ICAL_TIME_VALUE ){
+	return 1;
+    }
+
+    return 0;
+
+}
+
 icalparameter_xliccomparetype
 icalvalue_compare(icalvalue* a, icalvalue *b)
 {
@@ -1210,7 +1099,8 @@ icalvalue_compare(icalvalue* a, icalvalue *b)
     icalerror_check_arg_rz( (b!=0), "b");
 
     /* Not the same type; they can only be unequal */
-    if (icalvalue_isa(a) != icalvalue_isa(b)){
+    if( ! (icalvalue_is_time(a) && icalvalue_is_time(b)) &&
+	icalvalue_isa(a) != icalvalue_isa(b)){
 	return ICAL_XLICCOMPARETYPE_NOTEQUAL;
     }
 
@@ -1251,6 +1141,21 @@ icalvalue_compare(icalvalue* a, icalvalue *b)
 	    }
 	}
 
+	case ICAL_DURATION_VALUE: 
+	{
+	    int a = icaldurationtype_as_int(impla->data.v_duration);
+	    int b = icaldurationtype_as_int(implb->data.v_duration);
+
+	    if (a > b){
+		return ICAL_XLICCOMPARETYPE_GREATER;
+	    } else if (a < b){
+		return ICAL_XLICCOMPARETYPE_LESS;
+	    } else {
+		return ICAL_XLICCOMPARETYPE_EQUAL;
+	    }
+	}	    
+
+
 	case ICAL_TEXT_VALUE:
 	case ICAL_URI_VALUE:
 	case ICAL_CALADDRESS_VALUE:
@@ -1258,7 +1163,6 @@ icalvalue_compare(icalvalue* a, icalvalue *b)
 	case ICAL_DATE_VALUE:
 	case ICAL_DATETIME_VALUE:
 	case ICAL_DATETIMEDATE_VALUE:
-	case ICAL_DURATION_VALUE: /* HACK. Not correct for DURATION */
 	case ICAL_TIME_VALUE:
 	case ICAL_DATETIMEPERIOD_VALUE:
 	{
@@ -1272,7 +1176,7 @@ icalvalue_compare(icalvalue* a, icalvalue *b)
 	    } else if (r < 0){
 		return ICAL_XLICCOMPARETYPE_LESS;
 	    } else {
-		return 0;
+		return ICAL_XLICCOMPARETYPE_EQUAL;
 	    }
 
 		
@@ -1330,9 +1234,7 @@ icalproperty* icalvalue_get_parent(icalvalue* value)
 
 
 
-/* Recur is a special case, so it is not auto generated. Well,
-   actually, it is auto-generated, but you will have to manually
-   remove the auto-generated version after each generation.  */
+/* Recur is a special case, so it is not auto generated. */
 icalvalue*
 icalvalue_new_recur (struct icalrecurrencetype v)
 {
@@ -1381,6 +1283,127 @@ icalvalue_get_recur(icalvalue* value)
 
 
 
+icalvalue*
+icalvalue_new_trigger (struct icaltriggertype v)
+{
+   struct icalvalue_impl* impl = icalvalue_new_impl(ICAL_TRIGGER_VALUE);
+ 
+   icalvalue_set_trigger((icalvalue*)impl,v);
+
+   return (icalvalue*)impl;
+}
+
+void
+icalvalue_set_trigger(icalvalue* value, struct icaltriggertype v)
+{
+    struct icalvalue_impl* impl; 
+    
+    icalerror_check_arg_rv( (value!=0),"value");
+    
+    impl = (struct icalvalue_impl*)value;
+
+   if(!icaltime_is_null_time(v.time)){
+       icalvalue_set_datetime((icalvalue*)impl,v.time);
+       impl->kind = ICAL_DATETIME_VALUE;
+   } else {
+       icalvalue_set_duration((icalvalue*)impl,v.duration);
+       impl->kind = ICAL_DURATION_VALUE;
+   }
+
+}
+
+struct icaltriggertype
+icalvalue_get_trigger(icalvalue* value)
+{
+    struct icalvalue_impl *impl = (struct icalvalue_impl*)value;
+    struct icaltriggertype tr;
+
+    icalerror_check_arg( (value!=0),"value");
+    icalerror_check_arg( (value!=0),"value");
+
+    if(impl->kind == ICAL_DATETIME_VALUE){
+	 tr.duration = icaldurationtype_from_int(0);
+	 tr.time = impl->data.v_time;
+    } else if(impl->kind == ICAL_DURATION_VALUE){
+	tr.time = icaltime_null_time();
+	tr.duration = impl->data.v_duration;
+    } else {
+	tr.duration = icaldurationtype_from_int(0);
+	tr.time = icaltime_null_time();
+	icalerror_set_errno(ICAL_BADARG_ERROR);
+    }
+
+    return tr;
+}
+
+/* DATE-TIME-PERIOD is a special case, and is not auto generated */
+
+icalvalue*
+icalvalue_new_datetimeperiod (struct icaldatetimeperiodtype v)
+{
+   struct icalvalue_impl* impl = icalvalue_new_impl(ICAL_DATETIMEPERIOD_VALUE);
+
+   icalvalue_set_datetimeperiod((icalvalue*)impl,v);
+
+   return (icalvalue*)impl;
+}
+
+void
+icalvalue_set_datetimeperiod(icalvalue* value, struct icaldatetimeperiodtype v)
+{
+    struct icalvalue_impl* impl = (struct icalvalue_impl*)value;
+    
+    icalerror_check_arg_rv( (value!=0),"value");
+    
+    icalerror_check_value_type(value, ICAL_DATETIMEPERIOD_VALUE);
+
+    if(!icaltime_is_null_time(v.time)){
+	if(!icaltime_is_valid_time(v.time)){
+	    icalerror_set_errno(ICAL_BADARG_ERROR);
+	    return;
+	}
+	impl->kind = ICAL_DATETIME_VALUE;
+	icalvalue_set_datetime(impl,v.time);
+    } else if (!icalperiodtype_is_null_period(v.period)) {
+	if(!icalperiodtype_is_valid_period(v.period)){
+	    icalerror_set_errno(ICAL_BADARG_ERROR);
+	    return;
+	}
+	impl->kind = ICAL_PERIOD_VALUE;
+	icalvalue_set_period(impl,v.period);
+    } else {
+	icalerror_set_errno(ICAL_BADARG_ERROR);
+    }
+}
+
+struct icaldatetimeperiodtype
+icalvalue_get_datetimeperiod(icalvalue* value)
+{
+  struct icaldatetimeperiodtype dtp;
+  
+  struct icalvalue_impl* impl = (struct icalvalue_impl*)value;
+  icalerror_check_arg( (value!=0),"value");
+  icalerror_check_value_type(value, ICAL_DATETIMEPERIOD_VALUE);
+  
+  if(impl->kind == ICAL_DATETIME_VALUE){
+      dtp.period = icalperiodtype_null_period();
+      dtp.time = impl->data.v_time;
+  } else if(impl->kind == ICAL_PERIOD_VALUE) {
+      dtp.period = impl->data.v_period;
+      dtp.time = icaltime_null_time();
+  } else {
+      dtp.period = icalperiodtype_null_period();
+      dtp.time = icaltime_null_time();
+      icalerror_set_errno(ICAL_BADARG_ERROR);
+  }	
+
+  return dtp;
+}
+
+
+
+
+
 /* The remaining interfaces are 'new', 'set' and 'get' for each of the value
    types */
 
@@ -1648,41 +1671,6 @@ icalvalue_get_datetimedate(icalvalue* value)
 }
 
 
-icalvalue*
-icalvalue_new_datetimeperiod (struct icalperiodtype v)
-{
-   struct icalvalue_impl* impl = icalvalue_new_impl(ICAL_DATETIMEPERIOD_VALUE);
- 
-   
-   icalvalue_set_datetimeperiod((icalvalue*)impl,v);
-
-   return (icalvalue*)impl;
-}
-
-void
-icalvalue_set_datetimeperiod(icalvalue* value, struct icalperiodtype v)
-{
-    struct icalvalue_impl* impl; 
-    
-    icalerror_check_arg_rv( (value!=0),"value");
-    
-    icalerror_check_value_type(value, ICAL_DATETIMEPERIOD_VALUE);
-
-    impl = (struct icalvalue_impl*)value;
-
-    impl->data.v_period = v;
-}
-
-struct icalperiodtype
-icalvalue_get_datetimeperiod(icalvalue* value)
-{
-    icalerror_check_arg( (value!=0),"value");
-    icalerror_check_value_type(value, ICAL_DATETIMEPERIOD_VALUE);
-  
-    return ((struct icalvalue_impl*)value)->data.v_period;
-}
-
-
 icalvalue*
 icalvalue_new_duration (struct icaldurationtype v)
 {
@@ -2014,41 +2002,6 @@ icalvalue_get_time(icalvalue* value)
 }
 
 
-icalvalue*
-icalvalue_new_trigger (union icaltriggertype v)
-{
-   struct icalvalue_impl* impl = icalvalue_new_impl(ICAL_TRIGGER_VALUE);
- 
-   
-   icalvalue_set_trigger((icalvalue*)impl,v);
-
-   return (icalvalue*)impl;
-}
-
-void
-icalvalue_set_trigger(icalvalue* value, union icaltriggertype v)
-{
-    struct icalvalue_impl* impl; 
-    
-    icalerror_check_arg_rv( (value!=0),"value");
-    
-    icalerror_check_value_type(value, ICAL_TRIGGER_VALUE);
-
-    impl = (struct icalvalue_impl*)value;
-
-    impl->data.v_trigger = v;
-}
-
-union icaltriggertype
-icalvalue_get_trigger(icalvalue* value)
-{
-    icalerror_check_arg( (value!=0),"value");
-    icalerror_check_value_type(value, ICAL_TRIGGER_VALUE);
-  
-    return ((struct icalvalue_impl*)value)->data.v_trigger;
-}
-
-
 icalvalue*
 icalvalue_new_uri (const char* v)
 {
