@@ -80,14 +80,22 @@ BonoboUIVerb verbs [] = {
 	BONOBO_UI_UNSAFE_VERB ("MessageFilterSubj", filter_subject),
 	BONOBO_UI_UNSAFE_VERB ("MessageFilterSndr", filter_sender),
 	BONOBO_UI_UNSAFE_VERB ("MessageFilterRecip", filter_recipient),
+
+	BONOBO_UI_UNSAFE_VERB ("MessageHideClear", hide_none),
+	BONOBO_UI_UNSAFE_VERB ("MessageHideRead", hide_read),
+	BONOBO_UI_UNSAFE_VERB ("MessageHideDeleted", hide_deleted),
+	BONOBO_UI_UNSAFE_VERB ("MessageHideSelected", hide_selected),
 	
 	/* Folder Menu */
 	BONOBO_UI_UNSAFE_VERB ("FolderExpunge", expunge_folder),
 	BONOBO_UI_UNSAFE_VERB ("FolderConfig", configure_folder),
 	
 	/* Toolbar specific */
-	BONOBO_UI_UNSAFE_VERB ("MailGet", send_receieve_mail),
+	BONOBO_UI_UNSAFE_VERB ("MailGet", send_receive_mail),
 	BONOBO_UI_UNSAFE_VERB ("MailCompose", compose_msg),
+	BONOBO_UI_UNSAFE_VERB ("MailStop", stop_threads),
+	BONOBO_UI_UNSAFE_VERB ("MailPrevious", previous_msg),
+	BONOBO_UI_UNSAFE_VERB ("MailNext", next_msg),
 	
 	BONOBO_UI_VERB_END
 };
@@ -100,10 +108,14 @@ set_pixmap (BonoboUIComponent *uic,
 	char *path;
 	GdkPixbuf *pixbuf;
 
-	path = g_concat_dir_and_file (EVOLUTION_DATADIR "/images/evolution/buttons", icon);
+	path = g_concat_dir_and_file (EVOLUTION_DATADIR "/images/evolution", icon);
 
 	pixbuf = gdk_pixbuf_new_from_file (path);
-	g_return_if_fail (pixbuf != NULL);
+	if (pixbuf == NULL) {
+		g_warning ("Cannot load image -- %s", path);
+		g_free (path);
+		return;
+	}
 
 	bonobo_ui_util_set_pixbuf (uic, xml_path, pixbuf);
 
@@ -115,13 +127,27 @@ set_pixmap (BonoboUIComponent *uic,
 static void
 update_pixmaps (BonoboUIComponent *uic)
 {
-	set_pixmap (uic, "/Toolbar/MailGet", "fetch-mail.png");
-	set_pixmap (uic, "/Toolbar/MailCompose", "compose-message.png");
-	set_pixmap (uic, "/Toolbar/Reply", "reply.png");
-	set_pixmap (uic, "/Toolbar/ReplyAll", "reply-to-all.png");
-	set_pixmap (uic, "/Toolbar/Forward", "forward.png");
-	set_pixmap (uic, "/Toolbar/Move", "move-message.png");
-	set_pixmap (uic, "/Toolbar/Copy", "copy-message.png");
+	set_pixmap (uic, "/menu/File/Print/Print", "16_print.xpm");
+	set_pixmap (uic, "/menu/File/Print/Print Preview", "16_print.xpm");
+
+	set_pixmap (uic, "/menu/Component/Message/MessageEdit", "16_edit.xpm");
+	set_pixmap (uic, "/menu/Component/Message/MessageSaveAs", "16_save.xpm");
+	set_pixmap (uic, "/menu/Component/Message/MessagePrint", "16_print.xpm");
+	set_pixmap (uic, "/menu/Component/Message/MessageMove", "16_move_message.xpm");
+	set_pixmap (uic, "/menu/Component/Message/MessageReplyAll", "16_reply_to_all.xpm");
+	set_pixmap (uic, "/menu/Component/Message/MessageReplySndr", "16_reply.xpm");
+
+	set_pixmap (uic, "/menu/Component/Folder/FolderConfig", "16_configure_folder.xpm");
+
+	set_pixmap (uic, "/menu/Settings/SetMailConfig", "16_configure_mail.xpm");
+
+	set_pixmap (uic, "/Toolbar/MailGet", "buttons/fetch-mail.png");
+	set_pixmap (uic, "/Toolbar/MailCompose", "buttons/compose-message.png");
+	set_pixmap (uic, "/Toolbar/Reply", "buttons/reply.png");
+	set_pixmap (uic, "/Toolbar/ReplyAll", "buttons/reply-to-all.png");
+	set_pixmap (uic, "/Toolbar/Forward", "buttons/forward.png");
+	set_pixmap (uic, "/Toolbar/Move", "buttons/move-message.png");
+	set_pixmap (uic, "/Toolbar/Copy", "buttons/copy-message.png");
 }
 
 static void
@@ -150,7 +176,7 @@ control_activate (BonoboControl     *control,
 		uic, EVOLUTION_DATADIR,
 		"evolution-mail.xml", "evolution-mail");
 
-	if (mail_config_thread_list ())
+	if (mail_config_get_thread_list ())
 		bonobo_ui_component_set_prop (
 			uic, "/commands/ViewThreaded", "state", "1", NULL);
 	else
@@ -161,7 +187,7 @@ control_activate (BonoboControl     *control,
 		uic, "ViewThreaded",
 		folder_browser_toggle_threads, folder_browser);
 	
-	if (mail_config_view_source ())
+	if (mail_config_get_view_source ())
 		bonobo_ui_component_set_prop (uic, "/commands/ViewSource",
 					      "state", "1", NULL);
 	else
@@ -174,6 +200,9 @@ control_activate (BonoboControl     *control,
 	
 	update_pixmaps (uic);
 
+	/* this doesn't actually appear to work ? */
+        /*bonobo_ui_component_set_prop(uic, "/Toolbar/Stop", "sensitive", "0", NULL);*/
+
 	bonobo_ui_component_thaw (uic, NULL);
 }
 
@@ -186,7 +215,7 @@ control_deactivate (BonoboControl     *control,
  	bonobo_ui_component_unset_container (uic);
 
 	if (fb->folder)
-		mail_do_sync_folder (fb->folder);
+		mail_sync_folder (fb->folder, NULL, NULL);
 }
 
 static void
