@@ -5,7 +5,7 @@
  * Author:
  *   Miguel de Icaza (miguel@kernel.org)
  *
- * (C) 2000 Helix Code, Inc.
+ * (C) 2000, 2001 Ximian, Inc.
  */
 #include <config.h>
 #include <ctype.h>
@@ -16,11 +16,11 @@
 #include "mail-callbacks.h"
 #include "mail-tools.h"
 #include "message-list.h"
-#include "mail-threads.h"
 #include "mail-ops.h"
 #include "mail-vfolder.h"
 #include "mail-autofilter.h"
 #include "mail-mlist-magic.h"
+#include "mail-mt.h"
 
 #include <gal/util/e-util.h>
 #include <gal/widgets/e-unicode.h>
@@ -65,7 +65,7 @@ folder_browser_destroy (GtkObject *object)
 	g_free (folder_browser->uri);
 	
 	if (folder_browser->folder) {
-		mail_do_sync_folder (folder_browser->folder);
+		mail_sync_folder (folder_browser->folder, NULL, NULL);
 		camel_object_unref (CAMEL_OBJECT (folder_browser->folder));
 	}
 	
@@ -107,13 +107,98 @@ folder_browser_class_init (GtkObjectClass *object_class)
  * }
  */
 
-#define EQUAL(a,b) (strcmp (a,b) == 0)
+static void
+update_unread_count_main(CamelObject *object, gpointer event_data, gpointer user_data)
+{
+	CamelFolder *folder = (CamelFolder *)object;
+	FolderBrowser *fb = user_data;
+	EvolutionStorage *storage;
+	char *name;
+
+	storage = mail_lookup_storage (folder->parent_store);
+
+	if (fb->unread_count == 0)
+		name = g_strdup (camel_folder_get_name (folder));
+	else
+		name = g_strdup_printf ("%s (%d)", camel_folder_get_name (folder), fb->unread_count);
+
+	evolution_storage_update_folder_by_uri (storage, fb->uri, name, fb->unread_count != 0);
+	g_free (name);
+	gtk_object_unref (GTK_OBJECT (storage));
+}
+
+static void
+update_unread_count(CamelObject *object, gpointer event_data, gpointer user_data)
+{
+	CamelFolder *folder = (CamelFolder *)object;
+	FolderBrowser *fb = user_data;
+	int unread;
+
+	unread = camel_folder_get_unread_message_count (folder);
+	if (unread == fb->unread_count)
+		return;
+	fb->unread_count = unread;
+	mail_proxy_event (update_unread_count_main, object, event_data, user_data);
+}
+
+static void
+got_folder(char *uri, CamelFolder *folder, void *data)
+{
+	FolderBrowser *fb = data;
+	EvolutionStorage *storage;
+
+	printf("got folder '%s' = %p\n", uri, folder);
+
+	if (fb->folder == folder)
+		goto done;
+
+	if (fb->folder)
+		camel_object_unref((CamelObject *)fb->folder);
+	g_free(fb->uri);
+	fb->uri = g_strdup(uri);
+	fb->folder = folder;
+
+	if (folder == NULL)
+		goto done;
+
+	camel_object_ref((CamelObject *)folder);
+
+	if ((storage = mail_lookup_storage (folder->parent_store))) {
+		gtk_object_unref (GTK_OBJECT (storage));
+		fb->unread_count = camel_folder_get_unread_message_count (folder);
+		update_unread_count_main ((CamelObject *)folder, NULL, fb);
+		camel_object_hook_event ((CamelObject *)folder, "message_changed",
+					 update_unread_count, fb);
+		camel_object_hook_event ((CamelObject *)folder, "folder_changed",
+					 update_unread_count, fb);
+	}
+
+	gtk_widget_set_sensitive (GTK_WIDGET (fb->search->entry),
+				  camel_folder_has_search_capability (folder));
+	gtk_widget_set_sensitive (GTK_WIDGET (fb->search->option),
+				  camel_folder_has_search_capability (folder));
+	message_list_set_threaded(fb->message_list, mail_config_get_thread_list());
+	message_list_set_folder(fb->message_list, folder);
+done:
+	gtk_object_unref((GtkObject *)fb);
+
+	/* Sigh, i dont like this (it can be set in reconfigure folder),
+	   but its just easier right now to do it this way */
+	fb->reconfigure = FALSE;
+}
 
 gboolean
 folder_browser_set_uri (FolderBrowser *folder_browser, const char *uri)
 {
-	if (*uri)
-		mail_do_load_folder (folder_browser, uri);
+	if (uri && *uri) {
+		gtk_object_ref((GtkObject *)folder_browser);
+		mail_get_folder(uri, got_folder, folder_browser);
+	} else {
+		/* Sigh, i dont like this (it can be set in reconfigure folder),
+		   but its just easier right now to do it this way */
+		folder_browser->reconfigure = FALSE;
+	}
+
 	return TRUE;
 }
 
@@ -133,12 +218,12 @@ enum {
 };
 
 static ESearchBarItem folder_browser_search_menu_items[] = {
-	{ N_("Show All"),    ESB_SHOW_ALL },
-	{ NULL,              0            },
-	{ N_("Advanced..."), ESB_ADVANCED },
-	{ NULL,              0            },
-	{ N_("Save"),        ESB_SAVE     },
-	{ NULL,              -1           }
+	{ N_("Show All"),         	  ESB_SHOW_ALL },
+	{ NULL,                   	  0            },
+	{ N_("Advanced..."),      	  ESB_ADVANCED },
+	{ NULL,                   	  0            },
+	{ N_("Store search as vFolder"),  ESB_SAVE     },
+	{ NULL,                           -1           }
 };
 
 enum {
@@ -147,6 +232,7 @@ enum {
 	ESB_SUBJECT_CONTAINS,
 	ESB_BODY_DOES_NOT_CONTAIN,
 	ESB_SUBJECT_DOES_NOT_CONTAIN,
+	ESB_SENDER_CONTAINS,
 };
 
 static ESearchBarItem folder_browser_search_option_items[] = {
@@ -155,6 +241,7 @@ static ESearchBarItem folder_browser_search_option_items[] = {
         { N_("Subject contains"),         ESB_SUBJECT_CONTAINS         },
 	{ N_("Body does not contain"),    ESB_BODY_DOES_NOT_CONTAIN    },
 	{ N_("Subject does not contain"), ESB_SUBJECT_DOES_NOT_CONTAIN },
+	{ N_("Sender contains"),          ESB_SENDER_CONTAINS          },
 	{ NULL,                           -1                           }
 };
 
@@ -164,9 +251,10 @@ static ESearchBarItem folder_browser_search_option_items[] = {
 static char *search_string[] = {
 	"(or (body-contains %s) (match-all (header-contains \"Subject\" %s)))",
 	"(body-contains %s)",
-	"(match-all (header-contains \"Subject\" %s)",
+	"(match-all (header-contains \"Subject\" %s))",
 	"(match-all (not (body-contains %s)))",
 	"(match-all (not (header-contains \"Subject\" %s)))",
+	"(match-all (header-contains \"from\" %s))",
 };
 
 static void
@@ -278,6 +366,14 @@ search_save (GtkWidget *w, FolderBrowser *fb)
 		element = filter_part_find_element (part, "subject");
 		filter_input_set_value ((FilterInput *)element, text);
 		break;
+	case ESB_SENDER_CONTAINS:
+		part = vfolder_create_part ("from");
+		filter_rule_add_part ((FilterRule *)rule, part);
+		element = filter_part_find_element (part, "from-type");
+		filter_option_set_current ((FilterOption *)element, "contains");
+		element = filter_part_find_element (part, "from");
+		filter_input_set_value ((FilterInput *)element, text);
+		break;
 	}
 	
 	vfolder_gui_add_rule (rule);
@@ -384,22 +480,19 @@ folder_browser_toggle_view_source (BonoboUIComponent           *component,
 void
 vfolder_subject (GtkWidget *w, FolderBrowser *fb)
 {
-	vfolder_gui_add_from_message (fb->mail_display->current_message, AUTO_SUBJECT,
-				      fb->uri);
+	vfolder_gui_add_from_message (fb->mail_display->current_message, AUTO_SUBJECT, fb->uri);
 }
 
 void
 vfolder_sender (GtkWidget *w, FolderBrowser *fb)
 {
-	vfolder_gui_add_from_message (fb->mail_display->current_message, AUTO_FROM,
-				      fb->uri);
+	vfolder_gui_add_from_message (fb->mail_display->current_message, AUTO_FROM, fb->uri);
 }
 
 void
 vfolder_recipient (GtkWidget *w, FolderBrowser *fb)
 {
-	vfolder_gui_add_from_message (fb->mail_display->current_message, AUTO_TO,
-				      fb->uri);
+	vfolder_gui_add_from_message (fb->mail_display->current_message, AUTO_TO, fb->uri);
 }
 
 void
@@ -439,16 +532,126 @@ filter_mlist (GtkWidget *w, FolderBrowser *fb)
 	g_free (header_value);
 }
 
+void
+hide_none(GtkWidget *w, FolderBrowser *fb)
+{
+	message_list_hide_clear(fb->message_list);
+}
+
+void
+hide_selected(GtkWidget *w, FolderBrowser *fb)
+{
+	GPtrArray *uids;
+	int i;
+
+	uids = g_ptr_array_new();
+	message_list_foreach(fb->message_list, enumerate_msg, uids);
+	message_list_hide_uids(fb->message_list, uids);
+	for (i=0; i<uids->len; i++)
+		g_free(uids->pdata[i]);
+	g_ptr_array_free(uids, TRUE);
+}
+
+void
+hide_deleted(GtkWidget *w, FolderBrowser *fb)
+{
+	MessageList *ml = fb->message_list;
+
+	message_list_hide_add(ml, "(match-all (system-flag \"deleted\"))", ML_HIDE_SAME, ML_HIDE_SAME);
+}
+
+void
+hide_read(GtkWidget *w, FolderBrowser *fb)
+{
+	MessageList *ml = fb->message_list;
+
+	message_list_hide_add(ml, "(match-all (system-flag \"seen\"))", ML_HIDE_SAME, ML_HIDE_SAME);
+}
+
+/* dum de dum, about the 3rd copy of this function throughout the mailer/camel */
+static const char *
+strip_re(const char *subject)
+{
+	const unsigned char *s, *p;
+	
+	s = (unsigned char *) subject;
+	
+	while (*s) {
+		while(isspace (*s))
+			s++;
+		if (s[0] == 0)
+			break;
+		if ((s[0] == 'r' || s[0] == 'R')
+		    && (s[1] == 'e' || s[1] == 'E')) {
+			p = s+2;
+			while (isdigit(*p) || (ispunct(*p) && (*p != ':')))
+				p++;
+			if (*p == ':') {
+				s = p + 1;
+			} else
+				break;
+		} else
+			break;
+	}
+	return (char *) s;
+}
+
+void
+hide_subject(GtkWidget *w, FolderBrowser *fb)
+{
+	const char *subject;
+	GString *expr;
+
+	if (fb->mail_display->current_message) {
+		subject = camel_mime_message_get_subject(fb->mail_display->current_message);
+		if (subject) {
+			subject = strip_re(subject);
+			if (subject && subject[0]) {
+				expr = g_string_new("(match-all (header-contains \"subject\" ");
+				e_sexp_encode_string(expr, subject);
+				mail_tool_camel_lock_down();
+				g_string_append(expr, "))");
+				message_list_hide_add(fb->message_list, expr->str, ML_HIDE_SAME, ML_HIDE_SAME);
+				g_string_free(expr, TRUE);
+				return;
+			}
+		}
+	}
+}
+
+void
+hide_sender(GtkWidget *w, FolderBrowser *fb)
+{
+	const CamelInternetAddress *from;
+	const char *real, *addr;
+	GString *expr;
+
+	if (fb->mail_display->current_message) {
+		from = camel_mime_message_get_from(fb->mail_display->current_message);
+		if (camel_internet_address_get(from, 0, &real, &addr)) {
+			expr = g_string_new("(match-all (header-contains \"from\" ");
+			e_sexp_encode_string(expr, addr);
+			mail_tool_camel_lock_down();
+			g_string_append(expr, "))");
+			message_list_hide_add(fb->message_list, expr->str, ML_HIDE_SAME, ML_HIDE_SAME);
+			g_string_free(expr, TRUE);
+			return;
+		}
+	}
+}
+
 /* handle context menu over message-list */
 static gint
 on_right_click (ETable *table, gint row, gint col, GdkEvent *event, FolderBrowser *fb)
 {
 	extern CamelFolder *drafts_folder;
-	const CamelMessageInfo *info;
+	CamelMessageInfo *info;
 	GPtrArray *uids;
 	int enable_mask = 0;
 	int last_item, i;
-	char *mailing_list_name;
+	char *mailing_list_name = NULL;
+	char *subject_match = NULL, *from_match = NULL;
+
 	EPopupMenu filter_menu[] = {
 		{ _("VFolder on Subject"),         NULL, GTK_SIGNAL_FUNC (vfolder_subject),   NULL,  2 },
 		{ _("VFolder on Sender"),          NULL, GTK_SIGNAL_FUNC (vfolder_sender),    NULL,  2 },
@@ -460,6 +663,8 @@ on_right_click (ETable *table, gint row, gint col, GdkEvent *event, FolderBrowse
 		{ _("Filter on Mailing List"),     NULL, GTK_SIGNAL_FUNC (filter_mlist),      NULL, 66 },
 		{ NULL,                            NULL, NULL,                                NULL,  0 }
 	};
+
+
 	EPopupMenu menu[] = {
 		{ _("Open"),                       NULL, GTK_SIGNAL_FUNC (view_msg),          NULL,  0 },
 		{ _("Edit"),                       NULL, GTK_SIGNAL_FUNC (edit_msg),          NULL,  1 },
@@ -487,8 +692,12 @@ on_right_click (ETable *table, gint row, gint col, GdkEvent *event, FolderBrowse
 		{ NULL,                            NULL, NULL,                                NULL,  0 }
 	};
 	
-	/* Evil Hack.  */
 	last_item = (sizeof (filter_menu) / sizeof (*filter_menu)) - 2;
+
+	if (fb->reconfigure) {
+		enable_mask = 0;
+		goto display_menu;
+	}
 	
 	if (fb->folder != drafts_folder)
 		enable_mask |= 1;
@@ -497,8 +706,20 @@ on_right_click (ETable *table, gint row, gint col, GdkEvent *event, FolderBrowse
 		enable_mask |= 2;
 		mailing_list_name = NULL;
 	} else {
-		mailing_list_name = mail_mlist_magic_detect_list (fb->mail_display->current_message,
-								  NULL, NULL);
+		const char *subject, *real, *addr;
+		const CamelInternetAddress *from;
+
+		mailing_list_name = mail_mlist_magic_detect_list (fb->mail_display->current_message, NULL, NULL);
+
+		if ((subject = camel_mime_message_get_subject(fb->mail_display->current_message))
+		    && (subject = strip_re(subject))
+		    && subject[0])
+			subject_match = g_strdup(subject);
+
+		if ((from = camel_mime_message_get_from(fb->mail_display->current_message))
+		    && camel_internet_address_get(from, 0, &real, &addr)
+		    && addr && addr[0])
+			from_match = g_strdup(addr);
 	}
 	
 	/* get a list of uids */
@@ -510,9 +731,12 @@ on_right_click (ETable *table, gint row, gint col, GdkEvent *event, FolderBrowse
 		gboolean have_undeleted = FALSE;
 		gboolean have_seen = FALSE;
 		gboolean have_unseen = FALSE;
-		
+
 		for (i = 0; i < uids->len; i++) {
 			info = camel_folder_get_message_info (fb->folder, uids->pdata[i]);
+			if (info == NULL)
+				continue;
+
 			if (info->flags & CAMEL_MESSAGE_SEEN)
 				have_seen = TRUE;
 			else
@@ -523,10 +747,12 @@ on_right_click (ETable *table, gint row, gint col, GdkEvent *event, FolderBrowse
 			else
 				have_undeleted = TRUE;
 			
+			camel_folder_free_message_info(fb->folder, info);
+
 			if (have_seen && have_unseen && have_deleted && have_undeleted)
 				break;
 		}
-		
+
 		if (!have_unseen)
 			enable_mask |= 4;
 		if (!have_seen)
@@ -537,6 +763,13 @@ on_right_click (ETable *table, gint row, gint col, GdkEvent *event, FolderBrowse
 		if (!have_deleted)
 			enable_mask |= 32;
 	}
+
+	/* free uids */
+	for (i = 0; i < uids->len; i++)
+		g_free (uids->pdata[i]);
+	g_ptr_array_free (uids, TRUE);
+
+display_menu:
 	
 	/* generate the "Filter on Mailing List menu item name */
 	if (mailing_list_name == NULL) {
@@ -545,17 +778,13 @@ on_right_click (ETable *table, gint row, gint col, GdkEvent *event, FolderBrowse
 	} else {
 		filter_menu[last_item].name = g_strdup_printf (_("Filter on Mailing List (%s)"),
 							       mailing_list_name);
+		g_free(mailing_list_name);
 	}
+
+	e_popup_menu_run (menu, event, enable_mask, 0, fb);
 	
-	/* free uids */
-	for (i = 0; i < uids->len; i++)
-		g_free (uids->pdata[i]);
-	g_ptr_array_free (uids, TRUE);
-	
-	e_popup_menu_run (menu, (GdkEventButton *)event, enable_mask, 0, fb);
-	
-	g_free (filter_menu[last_item].name);
-	
+	g_free(filter_menu[last_item].name);
+
 	return TRUE;
 }
 
@@ -599,16 +828,6 @@ etable_key (ETable *table, int row, int col, GdkEvent *ev, FolderBrowser *fb)
 				     0, CAMEL_MESSAGE_DELETED);
 		return TRUE;
 
-	case GDK_Home:
-	case GDK_KP_Home:
-		message_list_select(fb->message_list, 0, MESSAGE_LIST_SELECT_NEXT, 0, 0);
-		return TRUE;
-		
-	case GDK_End:
-	case GDK_KP_End:
-		message_list_select(fb->message_list, -1, MESSAGE_LIST_SELECT_PREVIOUS, 0, 0);
-		return TRUE;
-		
 	case 'n':
 	case 'N':
 		message_list_select (fb->message_list, row,
@@ -623,6 +842,10 @@ etable_key (ETable *table, int row, int col, GdkEvent *ev, FolderBrowser *fb)
 				     0, CAMEL_MESSAGE_SEEN);
 		return TRUE;
 
+	case GDK_Menu:
+		on_right_click (table, row, col, ev, fb);
+		return TRUE;
+
 	default:
 		return FALSE;
 	}
@@ -678,32 +901,104 @@ folder_browser_gui_init (FolderBrowser *fb)
 	                    GTK_SIGNAL_FUNC (fb_resize_cb), NULL);
 	
 	e_paned_add2 (E_PANED (fb->vpaned), GTK_WIDGET (fb->mail_display));
-	e_paned_set_position (E_PANED (fb->vpaned), mail_config_paned_size ());
+	e_paned_set_position (E_PANED (fb->vpaned), mail_config_get_paned_size ());
 	gtk_widget_show (GTK_WIDGET (fb->mail_display));
 	gtk_widget_show (GTK_WIDGET (fb));
 }
 
+/* mark the message seen if the current message still matches */
 static gint 
-mark_msg_seen (gpointer data)
+do_mark_seen (gpointer data)
 {
-	MessageList *ml = data;
-	GPtrArray *uids;
+	FolderBrowser *fb = data;
 
-	if (!ml->cursor_uid) 
-		return FALSE;
+	if (fb->new_uid && fb->loaded_uid
+	    && strcmp(fb->new_uid, fb->loaded_uid) == 0) {
+		camel_folder_set_message_flags(fb->folder, fb->new_uid, CAMEL_MESSAGE_SEEN, CAMEL_MESSAGE_SEEN);
+	}
 
-	uids = g_ptr_array_new ();
-	g_ptr_array_add (uids, g_strdup (ml->cursor_uid));
-	mail_do_flag_messages (ml->folder, uids, FALSE,
-			       CAMEL_MESSAGE_SEEN, CAMEL_MESSAGE_SEEN);
 	return FALSE;
 }
 
+/* callback when we have the message to display, after async loading it (see below) */
+/* if we have pending uid's, it means another was selected before we finished displaying
+   the last one - so we cycle through and start loading the pending one immediately now */
+static void done_message_selected(CamelFolder *folder, char *uid, CamelMimeMessage *msg, void *data)
+{
+	FolderBrowser *fb = data;
+	int timeout = mail_config_get_mark_as_seen_timeout ();
+
+	if (folder != fb->folder)
+		return;
+
+	mail_display_set_message(fb->mail_display, (CamelMedium *)msg);
+
+	/* pain, if we have pending stuff, re-run */
+	if (fb->pending_uid) {	
+		g_free(fb->loading_uid);
+		fb->loading_uid = fb->pending_uid;
+		fb->pending_uid = NULL;
+
+		mail_get_message(fb->folder, fb->loading_uid, done_message_selected, fb, mail_thread_new);
+		return;
+	}
+
+	g_free(fb->loaded_uid);
+	fb->loaded_uid = fb->loading_uid;
+	fb->loading_uid = NULL;
+
+	/* if we are still on the same message, do the 'idle read' thing */
+	if (fb->seen_id)
+		gtk_timeout_remove(fb->seen_id);
+
+	if (msg) {
+		if (timeout > 0)
+			fb->seen_id = gtk_timeout_add(timeout, do_mark_seen, fb);
+		else
+			do_mark_seen(fb);
+	}
+}
+
+/* ok we waited enough, display it anyway (see below) */
+static gboolean
+do_message_selected(FolderBrowser *fb)
+{
+	d(printf ("selecting uid %s (delayed)\n", fb->new_uid));
+
+	/* keep polling if we are busy */
+	if (fb->reconfigure)
+		return TRUE;
+
+	fb->loading_id = 0;
+
+	/* if we are loading, then set a pending, but leave the loading, coudl cancel here (?) */
+	if (fb->loading_uid) {
+		g_free(fb->pending_uid);
+		fb->pending_uid = g_strdup(fb->new_uid);
+	} else {
+		if (fb->new_uid) {
+			fb->loading_uid = g_strdup(fb->new_uid);
+			mail_get_message(fb->folder, fb->loading_uid, done_message_selected, fb, mail_thread_new);
+		} else {
+			mail_display_set_message(fb->mail_display, NULL);
+		}
+	}
+
+	return FALSE;
+}
+
+/* when a message is selected, wait a while before trying to display it */
 static void
 on_message_selected (MessageList *ml, const char *uid, FolderBrowser *fb)
 {
-	d(printf ("selecting uid %s\n", uid));
-	mail_do_display_message (ml, fb->mail_display, uid, mark_msg_seen);
+	d(printf ("selecting uid %s (direct)\n", uid));
+
+	if (fb->loading_id != 0)
+		gtk_timeout_remove(fb->loading_id);
+
+	g_free(fb->new_uid);
+	fb->new_uid = g_strdup(uid);
+	fb->loading_id = gtk_timeout_add(100, (GtkFunction)do_message_selected, fb);
 }
 
 static void
