@@ -9,7 +9,7 @@
 #include "camel/camel-stream.h"
 #include <bonobo/bonobo-property-bag.h>
 #include <bonobo/bonobo-ui-component.h>
-#include "filter/e-search-bar.h"
+#include <widgets/misc/e-search-bar.h>
 #include "filter/filter-rule.h"
 #include "filter/filter-context.h" /*eek*/
 #include "message-list.h"
@@ -35,7 +35,21 @@ struct  _FolderBrowser {
 	 */
 	char        *uri;
 	CamelFolder *folder;
-	
+	int          unread_count; /* last known unread message count */
+
+	/* async loading stuff */
+	char	    *loading_uid;/* what uid am i loading now */
+	char	    *pending_uid; /* what uid should i load next */
+	char	    *new_uid;	/* place to save the next uid during idle timeout */
+	char	    *loaded_uid; /* what we have loaded */
+	guint	     loading_id, seen_id;
+
+	/* a folder we are expunging, dont use other than to compare the pointer value */
+	CamelFolder *expunging;
+
+	/* set to true when we are reconfiguring stuff == can't do much else */
+	int	     reconfigure;
+
 	MessageList *message_list;
 	MailDisplay *mail_display;
 	GtkWidget   *vpaned;
@@ -77,6 +91,13 @@ void filter_sender     (GtkWidget *w, FolderBrowser *fb);
 void filter_recipient  (GtkWidget *w, FolderBrowser *fb);
 void filter_mlist      (GtkWidget *w, FolderBrowser *fb);
 
+void hide_read(GtkWidget *w, FolderBrowser *fb);
+void hide_deleted(GtkWidget *w, FolderBrowser *fb);
+void hide_selected(GtkWidget *w, FolderBrowser *fb);
+void hide_none(GtkWidget *w, FolderBrowser *fb);
+void hide_subject(GtkWidget *w, FolderBrowser *fb);
+void hide_sender(GtkWidget *w, FolderBrowser *fb);
+
 void folder_browser_toggle_threads (BonoboUIComponent           *component,
 				    const char                  *path,
 				    Bonobo_UIComponent_EventType type,
