@@ -33,17 +33,20 @@
 #include "mail.h"
 #include "mail-callbacks.h"
 #include "mail-config.h"
-#include "mail-threads.h"
+#include "mail-accounts.h"
+#include "mail-config-druid.h"
+#include "mail-mt.h"
 #include "mail-tools.h"
 #include "mail-ops.h"
 #include "mail-local.h"
+#include "mail-send-recv.h"
 #include "mail-vfolder.h"
 #include "folder-browser.h"
 #include "subscribe-dialog.h"
 #include "filter/filter-editor.h"
-#include "filter/filter-driver.h"
 #include <gal/e-table/e-table.h>
 #include <gal/widgets/e-gui-utils.h>
+#include "e-messagebox.h"
 
 /* FIXME: is there another way to do this? */
 #include "Evolution.h"
@@ -62,53 +65,72 @@ struct post_send_data {
 	guint32 flags;
 };
 
-static gboolean
-check_configured (FolderBrowser *fb)
+static void
+druid_destroyed (void)
 {
-	if (mail_config_is_configured ())
-		return TRUE;
+	gtk_main_quit ();
+}
+
+static void
+configure_mail (FolderBrowser *fb)
+{
+	MailConfigDruid *druid;
 	
 	if (fb) {
 		GtkWidget *dialog;
 		
-		dialog = gnome_message_box_new (_("You have not configured the mail client.\n"
-						  "You need to do this before you can send,\n"
-						  "receive or compose mail.\n"
-						  "Would you like to configure it now?"),
-						GNOME_MESSAGE_BOX_QUESTION,
-						GNOME_STOCK_BUTTON_YES,
-						GNOME_STOCK_BUTTON_NO, NULL);
-		gnome_dialog_set_parent (GNOME_DIALOG (dialog),
-					 GTK_WINDOW (gtk_widget_get_ancestor (GTK_WIDGET(fb), GTK_TYPE_WINDOW)));
+		dialog = gnome_message_box_new (
+			_("You have not configured the mail client.\n"
+			  "You need to do this before you can send,\n"
+			  "receive or compose mail.\n"
+			  "Would you like to configure it now?"),
+			GNOME_MESSAGE_BOX_QUESTION,
+			GNOME_STOCK_BUTTON_YES,
+			GNOME_STOCK_BUTTON_NO, NULL);
+
+		/*
+		 * Focus YES
+		 */
+		gnome_dialog_set_default (GNOME_DIALOG (dialog), 0);
+		gtk_widget_grab_focus (GTK_WIDGET (GNOME_DIALOG (dialog)->buttons->data));
+		
+		gnome_dialog_set_parent (
+			GNOME_DIALOG (dialog),
+			GTK_WINDOW (gtk_widget_get_ancestor (
+				GTK_WIDGET (fb), GTK_TYPE_WINDOW)));
 		
 		switch (gnome_dialog_run_and_close (GNOME_DIALOG (dialog))) {
 		case 0:
-			mail_config_druid (fb->shell);
+			druid = mail_config_druid_new (fb->shell);
+			gtk_signal_connect (GTK_OBJECT (druid), "destroy",
+					    GTK_SIGNAL_FUNC (druid_destroyed), NULL);
+			gtk_widget_show (GTK_WIDGET (druid));
+			gtk_grab_add (GTK_WIDGET (druid));
+			gtk_main ();
 			break;
 		case 1:
 		default:
 			break;
 		}
-		
-		return mail_config_is_configured ();
-	} else
-		return FALSE;
+	}
 }
 
 static gboolean
 check_send_configuration (FolderBrowser *fb)
 {
-	MailConfigService *xport = NULL;
+	const MailConfigAccount *account;
 	
 	/* Check general */
-	
-	if (!check_configured (fb)) {
+	if (!mail_config_is_configured ()) {
+		configure_mail (fb);
 		return FALSE;
 	}
 	
-	/* Check for an identity */
+	/* Get the default account */
+	account = mail_config_get_default_account ();
 	
-	if (!mail_config_get_default_identity ()) {
+	/* Check for an identity */
+	if (!account) {
 		GtkWidget *message;
 		
 		message = gnome_warning_dialog_parented (_("You need to configure an identity\n"
@@ -120,9 +142,7 @@ check_send_configuration (FolderBrowser *fb)
 	}
 	
 	/* Check for a transport */
-	
-	xport = mail_config_get_transport ();
-	if (!xport || !xport->url) {
+	if (!account->transport || !account->transport->url) {
 		GtkWidget *message;
 		
 		message = gnome_warning_dialog_parented (_("You need to configure a mail transport\n"
@@ -136,6 +156,8 @@ check_send_configuration (FolderBrowser *fb)
 	return TRUE;
 }
 
+#if 0
+/* FIXME: is this still required when we send & receive email ?  I am not so sure ... */
 static void
 main_select_first_unread (CamelObject *object, gpointer event_data, gpointer data)
 {
@@ -151,104 +173,67 @@ select_first_unread (CamelObject *object, gpointer event_data, gpointer data)
 {
 	mail_op_forward_event (main_select_first_unread, object, event_data, data);
 }
+#endif
 
 void
-fetch_mail (GtkWidget *widget, gpointer user_data)
+send_receive_mail (GtkWidget *widget, gpointer user_data)
 {
-	GSList *sources;
-	
-	if (!check_configured (FOLDER_BROWSER (user_data))) {
-		GtkWidget *win = gtk_widget_get_ancestor (GTK_WIDGET (user_data),
-							  GTK_TYPE_WINDOW);
-		
-		gnome_error_dialog_parented (_("You have no mail sources "
-					       "configured"),
-					     GTK_WINDOW (win));
-		return;
-	}
-	
-	sources = mail_config_get_sources ();
-	
-	if (!sources || !sources->data) {
-		GtkWidget *win = gtk_widget_get_ancestor (GTK_WIDGET (user_data),
-							  GTK_TYPE_WINDOW);
-		
-		gnome_error_dialog_parented (_("You have no mail sources "
-					       "configured"),
-					     GTK_WINDOW (win));
-		return;
-	}
-	
-	while (sources) {
-		MailConfigService *source;
-		
-		source = (MailConfigService *) sources->data;
-		sources = sources->next;
-		
-		if (!source || !source->url) {
-			g_warning ("Bad source in fetch_mail??");
-			continue;
-		}
-		
-		mail_do_fetch_mail (source->url, source->keep_on_server, 
-				    NULL, select_first_unread, user_data);
-	}
-}
+	const MailConfigAccount *account;
 
-void
-send_queued_mail (GtkWidget *widget, gpointer user_data)
-{
-	extern CamelFolder *outbox_folder;
-	MailConfigService *transport;
-	
-	if (!mail_config_is_configured ()) {
-		return;
-	}
-	
-	transport = mail_config_get_transport ();
-	if (!transport) {
-		GtkWidget *win = gtk_widget_get_ancestor (GTK_WIDGET (user_data),
-							  GTK_TYPE_WINDOW);
-		
-		gnome_error_dialog_parented (_("You have not set a mail transport method"),
-					     GTK_WINDOW (win));
-		return;
-	}
-	
-	if (!outbox_folder) {
-		GtkWidget *win = gtk_widget_get_ancestor (GTK_WIDGET (user_data),
-							  GTK_TYPE_WINDOW);
-		
-		gnome_error_dialog_parented (_("You have no Outbox configured"),
-					     GTK_WINDOW (win));
-		return;
-	}
-	
-	mail_do_send_queue (outbox_folder, transport->url);
-	
-	mail_do_expunge_folder (outbox_folder);
-}
-
-void
-send_receieve_mail (GtkWidget *widget, gpointer user_data)
-{
 	/* receive first then send, this is a temp fix for POP-before-SMTP */
-	fetch_mail (widget, user_data);
-	send_queued_mail (widget, user_data);
+	if (!mail_config_is_configured ()) {
+		configure_mail (FOLDER_BROWSER (user_data));
+		return;
+	}
+	
+	account = mail_config_get_default_account ();
+	if (!account || !account->transport) {
+		GtkWidget *win = gtk_widget_get_ancestor(GTK_WIDGET (user_data), GTK_TYPE_WINDOW);
+		gnome_error_dialog_parented (_("You have not set a mail transport method"), GTK_WINDOW(win));
+		return;
+	}
+
+	mail_send_receive();
+}
+
+static void
+empty_subject_destroyed (GtkWidget *widget, gpointer data)
+{
+	gboolean *show_again = data;
+	GtkWidget *checkbox;
+	
+	checkbox = e_message_box_get_checkbox (E_MESSAGE_BOX (widget));
+	*show_again = !GTK_TOGGLE_BUTTON (checkbox)->active;
 }
 
 static gboolean
 ask_confirm_for_empty_subject (EMsgComposer *composer)
 {
-	GtkWidget *message_box;
+	/* FIXME: EMessageBox should really handle this stuff
+           automagically. What Miguel thinks would be nice is to pass
+           in a message-id which could be used as a key in the config
+           file and the value would be an int. -1 for always show or
+           the button pressed otherwise. This probably means we'd have
+           to write e_messagebox_run () */
+	gboolean show_again = TRUE;
+	GtkWidget *mbox;
 	int button;
 	
-	message_box = gnome_message_box_new (_("This message has no subject.\nReally send?"),
-					     GNOME_MESSAGE_BOX_QUESTION,
-					     GNOME_STOCK_BUTTON_YES, GNOME_STOCK_BUTTON_NO,
-					     NULL);
+	if (!mail_config_get_prompt_empty_subject ())
+		return TRUE;
 	
-	button = gnome_dialog_run_and_close (GNOME_DIALOG (message_box));
+	mbox = e_message_box_new (_("This message has no subject.\nReally send?"),
+				  E_MESSAGE_BOX_QUESTION,
+				  GNOME_STOCK_BUTTON_YES,
+				  GNOME_STOCK_BUTTON_NO,
+				  NULL);
+	
+	gtk_signal_connect (GTK_OBJECT (mbox), "destroy",
+			    empty_subject_destroyed, &show_again);
+	
+	button = gnome_dialog_run_and_close (GNOME_DIALOG (mbox));
+	
+	mail_config_set_prompt_empty_subject (show_again);
 	
 	if (button == 0)
 		return TRUE;
@@ -268,17 +253,52 @@ free_psd (GtkWidget *composer, gpointer user_data)
 	g_free (psd);
 }
 
+struct _send_data {
+	EMsgComposer *composer;
+	struct post_send_data *psd;
+};
+
+static void
+composer_sent_cb(char *uri, CamelMimeMessage *message, gboolean sent, void *data)
+{
+	struct _send_data *send = data;
+
+	if (sent) {
+		if (send->psd) {
+			camel_folder_set_message_flags(send->psd->folder, send->psd->uid, send->psd->flags, send->psd->flags);
+		}
+		gtk_widget_destroy((GtkWidget *)send->composer);
+	} else {
+		gtk_widget_show((GtkWidget *)send->composer);
+		gtk_object_unref((GtkObject *)send->composer);
+	}
+	g_free(send);
+}
+
 void
 composer_send_cb (EMsgComposer *composer, gpointer data)
 {
-	MailConfigService *xport = NULL;
+	const MailConfigAccount *account = NULL;
 	CamelMimeMessage *message;
 	const CamelInternetAddress *iaddr;
 	const char *subject;
 	struct post_send_data *psd = data;
+	struct _send_data *send;
 	
-	/* Config info */
-	xport = mail_config_get_transport ();
+	if (!mail_config_is_configured ()) {
+		GtkWidget *dialog;
+		
+		dialog = gnome_ok_dialog_parented (_("You must configure an account before you "
+						     "can send this email."),
+						   GTK_WINDOW (composer));
+		gnome_dialog_run_and_close (GNOME_DIALOG (dialog));
+		return;
+	}
+	
+	/* Use the preferred account */
+        account = e_msg_composer_get_preferred_account (composer);
+	if (!account)
+		account = mail_config_get_default_account ();
 	
 	/* Get the message */
 	message = e_msg_composer_get_message (composer);
@@ -288,7 +308,8 @@ composer_send_cb (EMsgComposer *composer, gpointer data)
 	if (!iaddr || CAMEL_ADDRESS (iaddr)->addresses->len == 0) {
 		GtkWidget *message_box;
 		
-		message_box = gnome_message_box_new (_("You must specify recipients in order to send this message."),
+		message_box = gnome_message_box_new (_("You must specify recipients in order to "
+						       "send this message."),
 						     GNOME_MESSAGE_BOX_WARNING,
 						     GNOME_STOCK_BUTTON_OK,
 						     NULL);
@@ -308,19 +329,18 @@ composer_send_cb (EMsgComposer *composer, gpointer data)
 		}
 	}
 	
-	if (psd) {
-		mail_do_send_mail (xport->url, message, 
-				   psd->folder, psd->uid, psd->flags, 
-				   GTK_WIDGET (composer));
-	} else {
-		mail_do_send_mail (xport->url, message, NULL, NULL, 0,
-				   GTK_WIDGET (composer));
-	}
+	send = g_malloc(sizeof(*send));
+	send->psd = psd;
+	send->composer = composer;
+	gtk_object_ref((GtkObject *)composer);
+	gtk_widget_hide((GtkWidget *)composer);
+	mail_send_mail (account->transport->url, message, composer_sent_cb, send);
 }
 
 void
 composer_postpone_cb (EMsgComposer *composer, gpointer data)
 {
+	const MailConfigAccount *account = NULL;
 	extern CamelFolder *outbox_folder;
 	CamelMimeMessage *message;
 	struct post_send_data *psd = data;
@@ -338,16 +358,18 @@ composer_postpone_cb (EMsgComposer *composer, gpointer data)
 		}
 	}
 	
+	/* Attach a X-Evolution-Transport header so we know which account
+	   to use when it gets sent later. */
+	account = e_msg_composer_get_preferred_account (composer);
+	if (!account)
+		account = mail_config_get_default_account ();
+	camel_medium_add_header (CAMEL_MEDIUM (message), "X-Evolution-Transport", account->transport->url);
+	
 	/* Save the message in Outbox */
 	mail_do_append_mail (outbox_folder, message, NULL);
 	
-	if (psd) {
-		guint32 set;
-		
-		set = camel_folder_get_message_flags (psd->folder, psd->uid);
-		camel_folder_set_message_flags (psd->folder, psd->uid,
-						psd->flags, psd->flags);
-	}
+	if (psd)
+		camel_folder_set_message_flags(psd->folder, psd->uid, psd->flags, psd->flags);
 	
 	gtk_widget_destroy (GTK_WIDGET (composer));
 }
@@ -355,25 +377,25 @@ composer_postpone_cb (EMsgComposer *composer, gpointer data)
 static GtkWidget *
 create_msg_composer (const char *url)
 {
-       MailConfigIdentity *id;
-       gboolean send_html;
-       gchar *sig_file = NULL;
-       EMsgComposer *composer;
-       
-       id = mail_config_get_default_identity ();
-       send_html = mail_config_send_html ();
-       
-       if (id)
-               sig_file = id->sig;
-       
-       if (url != NULL) {
-               composer = e_msg_composer_new_from_url (url);
-	       if (composer)
-		       e_msg_composer_set_send_html (composer, send_html);
-       } else
-               composer = e_msg_composer_new_with_sig_file (sig_file, send_html);
-
-       return (GtkWidget *)composer;
+	const MailConfigAccount *account;
+	gboolean send_html;
+	gchar *sig_file = NULL;
+	EMsgComposer *composer;
+	
+	account = mail_config_get_default_account ();
+	send_html = mail_config_get_send_html ();
+	
+	if (account->id)
+		sig_file = account->id->signature;
+	
+	if (url != NULL) {
+		composer = e_msg_composer_new_from_url (url);
+		if (composer)
+			e_msg_composer_set_send_html (composer, send_html);
+	} else
+		composer = e_msg_composer_new_with_sig_file (sig_file, send_html);
+	
+	return (GtkWidget *)composer;
 }
 
 void
@@ -445,8 +467,9 @@ mail_reply (CamelFolder *folder, CamelMimeMessage *msg, const char *uid, gboolea
 			    GTK_SIGNAL_FUNC (composer_postpone_cb), psd);
 	gtk_signal_connect (GTK_OBJECT (composer), "destroy",
 			    GTK_SIGNAL_FUNC (free_psd), psd);
-	
+
 	gtk_widget_show (GTK_WIDGET (composer));	
+	e_msg_composer_unset_changed (composer);
 }
 
 void
@@ -479,58 +502,102 @@ enumerate_msg (MessageList *ml, const char *uid, gpointer data)
 	g_ptr_array_add ((GPtrArray *) data, g_strdup (uid));
 }
 
-static void
-forward_message (FolderBrowser *fb, gboolean attach)
+
+static EMsgComposer *
+forward_get_composer (const char *subject)
 {
+	const MailConfigAccount *account;
 	EMsgComposer *composer;
-	CamelMimeMessage *cursor_msg;
-	MailConfigIdentity *id;
-	GPtrArray *uids;
-	gchar *sig_file = NULL;
-	gboolean send_html;
 	
-	cursor_msg = fb->mail_display->current_message;
-	g_return_if_fail (cursor_msg != NULL);
-	
-	if (!check_send_configuration (fb))
-		return;
-	
-	id = mail_config_get_default_identity ();
-	send_html = mail_config_send_html ();
-	
-	if (id)
-		sig_file = id->sig;
-	
-	composer = e_msg_composer_new_with_sig_file (sig_file, send_html);
-	if (!composer)
-		return;
-	
-	uids = g_ptr_array_new ();
-	if (attach)
-		message_list_foreach (fb->message_list, enumerate_msg, uids);
-	else
-		g_ptr_array_add (uids, g_strdup (fb->message_list->cursor_uid));
-	
-	gtk_signal_connect (GTK_OBJECT (composer), "send",
-			    GTK_SIGNAL_FUNC (composer_send_cb), NULL);
-	gtk_signal_connect (GTK_OBJECT (composer), "postpone",
-			    GTK_SIGNAL_FUNC (composer_postpone_cb), NULL);
-	
-	mail_do_forward_message (cursor_msg,
-				 fb->message_list->folder,
-				 uids, composer, attach);
+	account = mail_config_get_default_account ();
+	composer = e_msg_composer_new_with_sig_file (account && account->id ? account->id->signature : NULL,
+						     mail_config_get_send_html ());
+	if (composer) {
+		gtk_signal_connect (GTK_OBJECT (composer), "send",
+				    GTK_SIGNAL_FUNC (composer_send_cb), NULL);
+		gtk_signal_connect (GTK_OBJECT (composer), "postpone",
+				    GTK_SIGNAL_FUNC (composer_postpone_cb), NULL);
+		e_msg_composer_set_headers(composer, NULL, NULL, NULL, subject);
+	} else {
+		g_warning("Could not create composer");
+	}
+
+	return composer;
+}
+
+static void
+do_forward_inline(CamelFolder *folder, char *uid, CamelMimeMessage *message, void *data)
+{
+	char *subject;
+	char *text;
+
+	if (message) {
+		subject = mail_tool_generate_forward_subject(message);
+		text = mail_tool_quote_message (message, _("Forwarded message:\n"));
+
+		if (text) {
+			EMsgComposer *composer = forward_get_composer(subject);
+			if (composer) {
+				e_msg_composer_set_body_text(composer, text);
+				gtk_widget_show(GTK_WIDGET(composer));
+				e_msg_composer_unset_changed(composer);
+			}
+			g_free(text);
+		}
+
+		g_free(subject);
+	}
+}
+
+static void
+do_forward_attach(CamelFolder *folder, GPtrArray *messages, CamelMimePart *part, char *subject, void *data)
+{
+	if (part) {
+		EMsgComposer *composer = forward_get_composer(subject);
+		if (composer) {
+			e_msg_composer_attach(composer, part);
+			gtk_widget_show(GTK_WIDGET(composer));
+			e_msg_composer_unset_changed(composer);
+		}
+	}
+}
+
+void
+forward_messages(CamelFolder *folder, GPtrArray *uids, int doinline)
+{
+	if (doinline && uids->len == 1) {
+		mail_get_message(folder, uids->pdata[0], do_forward_inline, NULL, mail_thread_new);
+	} else {
+		mail_build_attachment(folder, uids, do_forward_attach, NULL);
+	}
 }
 
 void
 forward_inlined (GtkWidget *widget, gpointer user_data)
 {
-	forward_message (FOLDER_BROWSER (user_data), FALSE);
+	GPtrArray *uids;
+	FolderBrowser *fb = (FolderBrowser *)user_data;
+
+	if (!check_send_configuration (fb))
+		return;
+
+	uids = g_ptr_array_new();
+	g_ptr_array_add(uids, g_strdup (fb->message_list->cursor_uid));
+	forward_messages(fb->message_list->folder, uids, TRUE);
 }
 
 void
 forward_attached (GtkWidget *widget, gpointer user_data)
 {
-	forward_message (FOLDER_BROWSER (user_data), TRUE);
+	GPtrArray *uids;
+	FolderBrowser *fb = (FolderBrowser *)user_data;
+
+	if (!check_send_configuration (fb))
+		return;
+
+	uids = g_ptr_array_new();
+	message_list_foreach(fb->message_list, enumerate_msg, uids);
+	forward_messages(fb->message_list->folder, uids, FALSE);
 }
 
 static void
@@ -592,8 +659,8 @@ apply_filters (GtkWidget *widget, gpointer user_data)
 	
 	uids = g_ptr_array_new ();
 	message_list_foreach (ml, enumerate_msg, uids);
-	
-	mail_do_filter_ondemand (fb->folder, uids);
+
+	mail_filter_on_demand(fb->folder, uids);
 }
 
 void
@@ -620,36 +687,60 @@ invert_selection (BonoboUIComponent *uih, void *user_data, const char *path)
 	e_table_invert_selection (ml->table);
 }
 
+/* flag all selected messages */
+static void
+flag_messages(FolderBrowser *fb, guint32 mask, guint32 set)
+{
+        MessageList *ml = fb->message_list;
+	GPtrArray *uids;
+	int i;
+
+	if (ml->folder == NULL)
+		return;
+
+	/* could just use specific callback but i'm lazy */
+	uids = g_ptr_array_new ();
+	message_list_foreach (ml, enumerate_msg, uids);
+	camel_folder_freeze(ml->folder);
+	for (i=0;i<uids->len;i++) {
+		camel_folder_set_message_flags(ml->folder, uids->pdata[i], mask, set);
+		g_free(uids->pdata[i]);
+	}
+	camel_folder_thaw(ml->folder);
+
+	g_ptr_array_free(uids, TRUE);
+}
+
 void
 mark_as_seen (BonoboUIComponent *uih, void *user_data, const char *path)
 {
-        FolderBrowser *fb = FOLDER_BROWSER(user_data);
-        MessageList *ml = fb->message_list;
-	GPtrArray *uids;
-	
-	if (ml->folder == NULL)
-		return;
-	
-	uids = g_ptr_array_new ();
-	message_list_foreach (ml, enumerate_msg, uids);
-	mail_do_flag_messages (ml->folder, uids, FALSE,
-			       CAMEL_MESSAGE_SEEN, CAMEL_MESSAGE_SEEN);
+	flag_messages(FOLDER_BROWSER(user_data), CAMEL_MESSAGE_SEEN, CAMEL_MESSAGE_SEEN);
 }
 
 void
 mark_as_unseen (BonoboUIComponent *uih, void *user_data, const char *path)
 {
-        FolderBrowser *fb = FOLDER_BROWSER(user_data);
-        MessageList *ml = fb->message_list;
-	GPtrArray *uids;
-	
-	if (ml->folder == NULL)
-		return;
-	
-	uids = g_ptr_array_new ();
-	message_list_foreach (ml, enumerate_msg, uids);
-	mail_do_flag_messages (ml->folder, uids, FALSE,
-			       CAMEL_MESSAGE_SEEN, 0);
+	flag_messages(FOLDER_BROWSER(user_data), CAMEL_MESSAGE_SEEN, 0);
+}
+
+static void
+do_edit_messages(CamelFolder *folder, GPtrArray *uids, GPtrArray *messages, void *data)
+{
+	/*FolderBrowser *fb = data;*/
+	int i;
+
+	for (i=0; i<messages->len; i++) {
+		EMsgComposer *composer;
+
+		composer = e_msg_composer_new_with_message(messages->pdata[i]);
+		if (composer) {
+			gtk_signal_connect (GTK_OBJECT (composer), "send",
+					    composer_send_cb, NULL);
+			gtk_signal_connect (GTK_OBJECT (composer), "postpone",
+					    composer_postpone_cb, NULL);
+			gtk_widget_show (GTK_WIDGET (composer));
+		}
+	}
 }
 				   
 void
@@ -674,8 +765,7 @@ edit_msg (GtkWidget *widget, gpointer user_data)
 	uids = g_ptr_array_new ();
 	message_list_foreach (fb->message_list, enumerate_msg, uids);
 	
-	/* FIXME: do we need to pass the postpone callback too? */
-	mail_do_edit_messages (fb->folder, uids, (GtkSignalFunc) composer_send_cb);
+	mail_get_messages(fb->folder, uids, do_edit_messages, fb);
 }
 
 static void
@@ -712,10 +802,9 @@ save_msg_ok (GtkWidget *widget, gpointer user_data)
 		folder = gtk_object_get_data (GTK_OBJECT (user_data), "folder");
 		uids = gtk_object_get_data (GTK_OBJECT (user_data), "uids");
 		gtk_object_remove_no_notify (GTK_OBJECT (user_data), "uids");
-		mail_do_save_messages (folder, uids, path);
+		mail_save_messages (folder, uids, path, NULL, NULL);
+		gtk_widget_destroy (GTK_WIDGET (user_data));
 	}
-	
-	gtk_widget_destroy (GTK_WIDGET (user_data));
 }
 
 static void
@@ -764,76 +853,57 @@ save_msg (GtkWidget *widget, gpointer user_data)
 void
 delete_msg (GtkWidget *button, gpointer user_data)
 {
-	FolderBrowser *fb = user_data;
-	MessageList *ml = fb->message_list;
-	GPtrArray *uids;
-	
-	uids = g_ptr_array_new ();
-	message_list_foreach (ml, enumerate_msg, uids);
-	
-	/*
-	 * Toggling a flag is an "instantaneous" operation, so if
-	 * we're only doing one, just do it and return, rather than
-	 * queueing it for the other thread. This makes the "Delete"
-	 * key work correctly (move to the next message) again.
-	 * - Dan
-	 */
-	if (uids->len == 1) {
-		char *uid = uids->pdata[0];
-		
-		mail_tool_camel_lock_up ();
-		camel_folder_set_message_flags (ml->folder, uid,
-						CAMEL_MESSAGE_DELETED,
-						CAMEL_MESSAGE_DELETED);
-		mail_tool_camel_lock_down ();
-	} else {
-		mail_do_flag_messages (ml->folder, uids, FALSE,
-				       CAMEL_MESSAGE_DELETED,
-				       CAMEL_MESSAGE_DELETED);
-	}
+	flag_messages(FOLDER_BROWSER(user_data), CAMEL_MESSAGE_DELETED, CAMEL_MESSAGE_DELETED);
 }
 
 void
 undelete_msg (GtkWidget *button, gpointer user_data)
 {
-	FolderBrowser *fb = user_data;
-	MessageList *ml = fb->message_list;
-	GPtrArray *uids;
+	flag_messages(FOLDER_BROWSER(user_data), CAMEL_MESSAGE_DELETED, 0);
+}
+
+void
+next_msg (GtkWidget *button, gpointer user_data)
+{
+	FolderBrowser *fb = FOLDER_BROWSER (user_data);
+	int row;
 	
-	uids = g_ptr_array_new ();
-	message_list_foreach (ml, enumerate_msg, uids);
+	row = e_table_get_cursor_row (fb->message_list->table);
+	message_list_select (fb->message_list, row,
+			     MESSAGE_LIST_SELECT_NEXT,
+			     0, CAMEL_MESSAGE_SEEN);
+}
+
+void
+previous_msg (GtkWidget *button, gpointer user_data)
+{
+	FolderBrowser *fb = FOLDER_BROWSER (user_data);
+	int row;
 	
-	/*
-	 * Toggling a flag is an "instantaneous" operation, so if
-	 * we're only doing one, just do it and return, rather than
-	 * queueing it for the other thread. This makes the "Delete"
-	 * key work correctly (move to the next message) again.
-	 * - Dan
-	 */
-	if (uids->len == 1) {
-		char *uid = uids->pdata[0];
-		
-		mail_tool_camel_lock_up ();
-		camel_folder_set_message_flags (ml->folder, uid,
-						CAMEL_MESSAGE_DELETED,
-						0);
-		mail_tool_camel_lock_down ();
-	} else {
-		mail_do_flag_messages (ml->folder, uids, FALSE,
-				       CAMEL_MESSAGE_DELETED,
-				       0);
-	}
+	row = e_table_get_cursor_row (fb->message_list->table);
+	message_list_select (fb->message_list, row,
+			     MESSAGE_LIST_SELECT_PREVIOUS,
+			     0, CAMEL_MESSAGE_SEEN);
+}
+
+static void expunged_folder(CamelFolder *f, void *data)
+{
+	FolderBrowser *fb = data;
+
+	fb->expunging = NULL;
 }
 
 void
 expunge_folder (BonoboUIComponent *uih, void *user_data, const char *path)
 {
 	FolderBrowser *fb = FOLDER_BROWSER(user_data);
-	
-	e_table_model_pre_change (fb->message_list->table_model);
-	
-	if (fb->message_list->folder)
-		mail_do_expunge_folder (fb->message_list->folder);
+
+	if (fb->folder
+	    && (fb->expunging == NULL
+		|| fb->folder != fb->expunging)) {
+		fb->expunging = fb->folder;
+		mail_expunge_folder(fb->folder, expunged_folder, fb);
+	}
 }
 
 static void
@@ -898,7 +968,11 @@ vfolder_edit_vfolders (BonoboUIComponent *uih, void *user_data, const char *path
 void
 providers_config (BonoboUIComponent *uih, void *user_data, const char *path)
 {
-	mail_config ((FOLDER_BROWSER (user_data))->shell);
+	/* FIXME: should we block until mail-config is done? */
+	MailAccountsDialog *dialog;
+	
+	dialog = mail_accounts_dialog_new ((FOLDER_BROWSER (user_data))->shell);
+	gnome_dialog_run_and_close (GNOME_DIALOG (dialog));
 }
 
 /*
@@ -1017,18 +1091,35 @@ configure_folder (BonoboUIComponent *uih, void *user_data, const char *path)
 	mail_local_reconfigure_folder(fb);
 }
 
+static void
+do_view_message(CamelFolder *folder, char *uid, CamelMimeMessage *message, void *data)
+{
+	/*FolderBrowser *fb = data;*/
+	GtkWidget *view;
+
+	if (message) {
+		view = mail_view_create(folder, uid, message);
+		gtk_widget_show(view);
+	}
+}
+
 void
 view_msg (GtkWidget *widget, gpointer user_data)
 {
 	FolderBrowser *fb = user_data;
 	GPtrArray *uids;
-	
+	int i;
+
 	if (!fb->folder)
 		return;
 	
 	uids = g_ptr_array_new ();
 	message_list_foreach (fb->message_list, enumerate_msg, uids);
-	mail_do_view_messages (fb->folder, uids, fb);
+	for (i=0;i<uids->len;i++) {
+		mail_get_message(fb->folder, uids->pdata[i], do_view_message, fb, mail_thread_queued);
+		g_free(uids->pdata[i]);
+	}
+	g_ptr_array_free(uids, TRUE);
 }
 
 void
@@ -1042,3 +1133,11 @@ edit_message (BonoboUIComponent *uih, void *user_data, const char *path)
 {
         edit_msg (NULL, user_data);
 }
+
+
+void
+stop_threads(BonoboUIComponent *uih, void *user_data, const char *path)
+{
+	camel_cancel_cancel(NULL);
+}
+
