@@ -25,34 +25,31 @@
 
 #include <gnome.h>
 #include <camel/camel.h>
+#include "openpgp-utils.h"
 
 #ifdef __cplusplus
 extern "C" {
 #pragma }
 #endif /* __cplusplus }*/
 
-typedef enum {
-	PGP_HASH_TYPE_NONE,
-	PGP_HASH_TYPE_MD5,
-	PGP_HASH_TYPE_SHA1
-} PgpHashType;
+gboolean mail_crypto_is_rfc2015_signed (CamelMimePart *part);
 
-char *mail_crypto_openpgp_decrypt (const char *ciphertext,
-				   int *outlen,
-				   CamelException *ex);
+gboolean mail_crypto_is_rfc2015_encrypted (CamelMimePart *part);
 
-char *mail_crypto_openpgp_encrypt (const char *plaintext,
-				   int inlen,
-				   const GPtrArray *recipients,
-				   gboolean sign,
-				   const char *userid,
-				   CamelException *ex);
+void pgp_mime_part_sign (CamelMimePart **mime_part,
+			 const gchar *userid,
+			 PgpHashType hash,
+			 CamelException *ex);
 
-char *mail_crypto_openpgp_clearsign (const char *plaintext,
-				     const char *userid,
-				     PgpHashType hash,
-				     gboolean detached,
-				     CamelException *ex);
+gboolean pgp_mime_part_verify (CamelMimePart *mime_part,
+			       CamelException *ex);
+
+void pgp_mime_part_encrypt (CamelMimePart **mime_part,
+			    const GPtrArray *recipients,
+			    CamelException *ex);
+
+CamelMimePart *pgp_mime_part_decrypt (CamelMimePart *mime_part,
+				      CamelException *ex);
 
 #ifdef __cplusplus
 }
