@@ -20,6 +20,10 @@
 #include "mail-display.h"
 #include "mail-config.h"
 #include "mail.h"
+#include "art/empty.xpm"
+
+#include "mail-ops.h"
+#include "mail-mt.h"
 
 #include <bonobo.h>
 #include <libgnorba/gnorba.h>
@@ -31,39 +35,45 @@
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gdk-pixbuf/gdk-pixbuf-loader.h>
 #include <gtkhtml/gtkhtml-embedded.h>
+#include <gtkhtml/htmlengine.h>	/* XXX */
 
 #define PARENT_TYPE (gtk_vbox_get_type ())
 
 static GtkObjectClass *mail_display_parent_class;
 
-static void redisplay (MailDisplay *md, gboolean unscroll);
-
 struct _PixbufLoader {
+	MailDisplay *md;
 	CamelDataWrapper *wrapper; /* The data */
 	CamelStream *mstream;
 	GdkPixbufLoader *loader; 
+	GtkHTMLEmbedded *eb;
 	char *type; /* Type of data, in case the conversion fails */
+	char *cid; /* Strdupped on creation, but not freed until 
+		      the hashtable is destroyed */
 	GtkWidget *pixmap;
+	guint32 destroy_id;
 };
 
 /*----------------------------------------------------------------------*
  *                        Callbacks
  *----------------------------------------------------------------------*/
 
+static void
+write_data_written(CamelMimePart *part, char *name, int done, void *data)
+{
+	int *ret = data;
+
+	/* should we popup a dialogue to say its done too? */
+	*ret = done;
+}
+
 static gboolean
 write_data_to_file (CamelMimePart *part, const char *name, gboolean unique)
 {
-	CamelMimeFilterCharset *charsetfilter;
-	GMimeContentField *content_type;
-	CamelStreamFilter *filtered_stream;
-	CamelStream *stream_fs;
-	CamelDataWrapper *data;
-	const char *charset;
 	int fd;
 	int ret = FALSE;
 	
 	g_return_val_if_fail (CAMEL_IS_MIME_PART (part), FALSE);
-	data = camel_medium_get_content_object (CAMEL_MEDIUM (part));
 	
 	fd = open (name, O_WRONLY | O_CREAT | O_EXCL, S_IRUSR | S_IWUSR);
 	if (fd == -1 && errno == EEXIST && !unique) {
@@ -81,58 +91,12 @@ write_data_to_file (CamelMimePart *part, const char *name, gboolean unique)
 
 		if (gnome_dialog_run_and_close (GNOME_DIALOG (dlg)) != 0)
 			return FALSE;
-
-		fd = open (name, O_WRONLY | O_TRUNC);
 	}
+	if (fd != -1)
+		close(fd);
 
-	if (fd == -1
-	    || (stream_fs = camel_stream_fs_new_with_fd (fd)) == NULL) {
-		char *msg;
-
-		msg = g_strdup_printf (_("Could not open file %s:\n%s"),
-				       name, strerror (errno));
-		gnome_error_dialog (msg);
-		g_free (msg);
-		return FALSE;
-	}
-	
-
-	/* we only convert text/ parts, and we only convert if we have to
-	   null charset param == us-ascii == utf8 always, and utf8 == utf8 obviously */
-	/* this will also let "us-ascii that isn't really" parts pass out in
-	   proper format, without us trying to treat it as what it isn't, which is
-	   the same algorithm camel uses */
-	
-	content_type = camel_mime_part_get_content_type (part);
-	if (gmime_content_field_is_type(content_type, "text", "*")
-	    && (charset = gmime_content_field_get_parameter (content_type, "charset"))
-	    && strcasecmp(charset, "utf-8") != 0) {
-		charsetfilter = camel_mime_filter_charset_new_convert ("utf-8", charset);
-		filtered_stream = camel_stream_filter_new_with_stream (stream_fs);
-		camel_stream_filter_add (filtered_stream, CAMEL_MIME_FILTER (charsetfilter));
-		camel_object_unref (CAMEL_OBJECT (charsetfilter));
-	} else {
-		/* no we can't use a CAMEL_BLAH() cast here, since its not true, HOWEVER
-		   we only treat it as a normal stream from here on, so it is OK */
-		filtered_stream = (CamelStreamFilter *)stream_fs;
-		camel_object_ref (CAMEL_OBJECT (stream_fs));
-	}
-	
-	if (camel_data_wrapper_write_to_stream (data, CAMEL_STREAM (filtered_stream)) == -1
-	    || camel_stream_flush (CAMEL_STREAM (filtered_stream)) == -1) {
-		char *msg;
-		
-		msg = g_strdup_printf (_("Could not write data: %s"),
-				       strerror (errno));
-		gnome_error_dialog (msg);
-		g_free (msg);
-		ret = FALSE;
-	} else {
-		ret = TRUE;
-	}
-
-	camel_object_unref (CAMEL_OBJECT (filtered_stream));
-	camel_object_unref (CAMEL_OBJECT (stream_fs));
+	/* should this have progress of what its doing? */
+	mail_msg_wait(mail_save_part(part, name, write_data_written, &ret));
 
 	return ret;
 }
@@ -168,6 +132,8 @@ save_data_cb (GtkWidget *widget, gpointer user_data)
 	GtkFileSelection *file_select = (GtkFileSelection *)
 		gtk_widget_get_ancestor (widget, GTK_TYPE_FILE_SELECTION);
 
+	/* uh, this doesn't really feel right, but i dont know what to do better */
+	gtk_widget_hide (GTK_WIDGET (file_select));
 	write_data_to_file (user_data,
 			    gtk_file_selection_get_filename (file_select),
 			    FALSE);
@@ -180,12 +146,12 @@ idle_redisplay (gpointer data)
 	MailDisplay *md = data;
 
 	md->idle_id = 0;
-	redisplay (md, FALSE);
+	mail_display_redisplay (md, FALSE);
 	return FALSE;
 }
 
-static void
-queue_redisplay (MailDisplay *md)
+void
+mail_display_queue_redisplay (MailDisplay *md)
 {
 	if (!md->idle_id) {
 		md->idle_id = g_idle_add_full (G_PRIORITY_LOW, idle_redisplay,
@@ -206,7 +172,7 @@ on_link_clicked (GtkHTML *html, const char *url, gpointer user_data)
 	else if (!strcmp (url, "x-evolution-decode-pgp:")) {
 		g_datalist_set_data (md->data, "show_pgp",
 				     GINT_TO_POINTER (1));
-		queue_redisplay (md);
+		mail_display_queue_redisplay (md);
 	} else
 		gnome_url_show (url);
 }
@@ -239,11 +205,11 @@ launch_cb (GtkWidget *widget, gpointer user_data)
 {
 	CamelMimePart *part = gtk_object_get_data (user_data, "CamelMimePart");
 	GnomeVFSMimeApplication *app;
-	GMimeContentField *content_type;
+	CamelContentType *content_type;
 	char *mime_type, *tmpl, *tmpdir, *filename, *argv[2];
 
 	content_type = camel_mime_part_get_content_type (part);
-	mime_type = gmime_content_field_get_mime_type (content_type);
+	mime_type = header_content_type_simple (content_type);
 	app = gnome_vfs_mime_get_default_application (mime_type);
 	g_free (mime_type);
 
@@ -295,11 +261,11 @@ inline_cb (GtkWidget *widget, gpointer user_data)
 	else
 		camel_mime_part_set_disposition (part, "inline");
 
-	queue_redisplay (md);
+	mail_display_queue_redisplay (md);
 }
 
 static gboolean
-pixmap_press (GtkWidget *ebox, GdkEventButton *event, gpointer user_data)
+pixmap_press (GtkWidget *ebox, GdkEventButton *event, EScrollFrame *user_data)
 {
 	EPopupMenu menu[] = {
 		{ N_("Save to Disk..."), NULL,
@@ -314,11 +280,14 @@ pixmap_press (GtkWidget *ebox, GdkEventButton *event, gpointer user_data)
 	MailMimeHandler *handler;
 	int mask = 0;
 
-	if (event->button != 3)
-		return FALSE;
+	if (event->button != 3) {
+		gtk_propagate_event (GTK_WIDGET (user_data),
+				     (GdkEvent *)event);
+		return TRUE;
+	}
 
-	part = gtk_object_get_data (user_data, "CamelMimePart");
-	handler = mail_lookup_handler (gtk_object_get_data (user_data,
+	part = gtk_object_get_data (GTK_OBJECT (ebox), "CamelMimePart");
+	handler = mail_lookup_handler (gtk_object_get_data (GTK_OBJECT (ebox),
 							    "mime_type"));
 
 	/* Save item */
@@ -363,7 +332,7 @@ pixmap_press (GtkWidget *ebox, GdkEventButton *event, gpointer user_data)
 		mask |= 2;
 	}
 
-	e_popup_menu_run (menu, event, mask, 0, user_data);
+	e_popup_menu_run (menu, (GdkEvent *)event, mask, 0, ebox);
 	g_free (menu[1].name);
 	g_free (menu[2].name);
 	return TRUE;
@@ -374,7 +343,7 @@ pixbuf_for_mime_type (const char *mime_type)
 {
 	const char *icon_name;
 	char *filename = NULL;
-	GdkPixbuf *pixbuf;
+	GdkPixbuf *pixbuf = NULL;
 
 	icon_name = gnome_vfs_mime_get_value (mime_type, "icon-filename");
 	if (icon_name) {
@@ -398,11 +367,22 @@ pixbuf_for_mime_type (const char *mime_type)
 		}
 	}
 	
-	if (!filename)
-		filename = gnome_pixmap_file ("gnome-unknown.png");
+	if (filename) {
+		pixbuf = gdk_pixbuf_new_from_file (filename);
+		g_free (filename);
+	}
 
-	pixbuf = gdk_pixbuf_new_from_file (filename);
-	g_free (filename);
+	if (!pixbuf) {
+		filename = gnome_pixmap_file ("gnome-unknown.png");
+		if (filename) {
+			pixbuf = gdk_pixbuf_new_from_file (filename);
+			g_free (filename);
+		} else {
+			g_warning ("Could not get any icon for %s!",mime_type);
+			pixbuf = gdk_pixbuf_new_from_xpm_data (
+				(const char **)empty_xpm);
+		}
+	}
 
 	return pixbuf;
 }
@@ -415,8 +395,29 @@ pixbuf_gen_idle (struct _PixbufLoader *pbl)
 	char tmp[4096];
 	int len, width, height, ratio;
 
-	/* Get a pixbuf from the wrapper */
+	/* Get the pixbuf from the cache */
+	mini = g_hash_table_lookup (pbl->md->thumbnail_cache, pbl->cid);
+	if (mini) {
+		width = gdk_pixbuf_get_width (mini);
+		height = gdk_pixbuf_get_height (mini);
 
+		bonobo_ui_toolbar_icon_set_pixbuf (
+		        BONOBO_UI_TOOLBAR_ICON (pbl->pixmap), mini);
+		gtk_widget_set_usize (pbl->pixmap, width, height);
+		
+		if (pbl->loader) {
+			gdk_pixbuf_loader_close (pbl->loader);
+			gtk_object_destroy (GTK_OBJECT (pbl->loader));
+			camel_object_unref (CAMEL_OBJECT (pbl->mstream));
+		}
+		gtk_signal_disconnect (GTK_OBJECT (pbl->eb), pbl->destroy_id);
+		g_free (pbl->type);
+		g_free (pbl);
+
+		return FALSE;
+	}
+
+	/* Not in cache, so get a pixbuf from the wrapper */
 	if (!GTK_IS_WIDGET (pbl->pixmap)) {
 		/* Widget has died */
 		if (pbl->mstream)
@@ -452,7 +453,7 @@ pixbuf_gen_idle (struct _PixbufLoader *pbl)
 
 	width = gdk_pixbuf_get_width (pixbuf);
 	height = gdk_pixbuf_get_height (pixbuf);
-
+	
 	if (width >= height) {
 		if (width > 24) {
 			ratio = width / 24;
@@ -469,13 +470,17 @@ pixbuf_gen_idle (struct _PixbufLoader *pbl)
 
 	mini = gdk_pixbuf_scale_simple (pixbuf, width, height,
 					GDK_INTERP_BILINEAR);
-	if (error)
+	if (error || !pbl->mstream)
 		gdk_pixbuf_unref (pixbuf);
 	bonobo_ui_toolbar_icon_set_pixbuf (
 		BONOBO_UI_TOOLBAR_ICON (pbl->pixmap), mini);
-	gdk_pixbuf_unref (mini);
+
+	/* Add the pixbuf to the cache */
+
+	g_hash_table_insert (pbl->md->thumbnail_cache, pbl->cid, mini);
 	gtk_widget_set_usize (pbl->pixmap, width, height);
 
+	gtk_signal_disconnect (GTK_OBJECT (pbl->eb), pbl->destroy_id);
 	if (pbl->loader) {
 		gdk_pixbuf_loader_close (pbl->loader);
 		gtk_object_destroy (GTK_OBJECT (pbl->loader));
@@ -486,6 +491,97 @@ pixbuf_gen_idle (struct _PixbufLoader *pbl)
 	return FALSE;
 }
 
+/* Stop the idle function and free the pbl structure
+   as the widget that the pixbuf was to be rendered to
+   has died on us. */
+static void
+embeddable_destroy_cb (GtkObject *embeddable,
+		       struct _PixbufLoader *pbl)
+{
+	g_idle_remove_by_data (pbl);
+	if (pbl->mstream)
+		camel_object_unref (CAMEL_OBJECT (pbl->mstream));
+	
+	if (pbl->loader) {
+		gdk_pixbuf_loader_close (pbl->loader);
+		gtk_object_destroy (GTK_OBJECT (pbl->loader));
+	}
+	
+	g_free (pbl->type);
+	g_free (pbl);
+};
+
+static GtkWidget *
+get_embedded_for_component (const char *iid, MailDisplay *md)
+{
+	GtkWidget *embedded;
+	BonoboControlFrame *control_frame;
+	Bonobo_PropertyBag prop_bag;
+	
+	/*
+	 * First try a control.
+	 */
+	embedded = bonobo_widget_new_control (iid, NULL);
+	if (embedded == NULL) {
+		/*
+		 * No control, try an embeddable instead.
+		 */
+		embedded = bonobo_widget_new_subdoc (iid, NULL);
+		if (embedded != NULL) {
+			/* FIXME: as of bonobo 0.18, there's an extra
+			 * client_site dereference in the BonoboWidget
+			 * destruction path that we have to balance out to
+			 * prevent problems.
+			 */
+			bonobo_object_ref (BONOBO_OBJECT(bonobo_widget_get_client_site (
+				BONOBO_WIDGET (embedded))));
+
+			return embedded;
+		}
+	}
+
+	if (embedded == NULL)
+		return NULL;
+
+	control_frame = bonobo_widget_get_control_frame (BONOBO_WIDGET (embedded));
+		
+	prop_bag = bonobo_control_frame_get_control_property_bag ( control_frame, NULL );
+		
+	if (prop_bag != CORBA_OBJECT_NIL){
+		CORBA_Environment ev;
+		/*
+		 * Now we can take care of business. Currently, the only control
+		 * that needs something passed to it through a property bag is
+		 * the iTip control, and it needs only the From email address,
+		 * but perhaps in the future we can generalize this section of code
+		 * to pass a bunch of useful things to all embedded controls.
+		 */
+		const CamelInternetAddress *from;
+		const MailConfigIdentity *id;
+		
+		id = mail_config_get_default_identity ();
+		CORBA_exception_init (&ev);
+		if (id){
+			char *from_address;
+			
+			
+			from = camel_mime_message_get_from (md->current_message);
+			from_address = camel_address_encode((CamelAddress *)from);
+			bonobo_property_bag_client_set_value_string (
+				prop_bag, "from_address", 
+				from_address, &ev);
+			bonobo_property_bag_client_set_value_string (
+				prop_bag, "my_address", 
+				id ? id->address : "", &ev);
+			g_free(from_address);
+		} 
+		Bonobo_Unknown_unref (prop_bag, &ev);
+		CORBA_exception_free (&ev);
+	}
+
+	return embedded;
+}
+
 static gboolean
 on_object_requested (GtkHTML *html, GtkHTMLEmbedded *eb, gpointer data)
 {
@@ -501,8 +597,6 @@ on_object_requested (GtkHTML *html, GtkHTMLEmbedded *eb, gpointer data)
 	GByteArray *ba;
 	CamelStream *cstream;
 	BonoboStream *bstream;
-	BonoboControlFrame *control_frame;
-	Bonobo_PropertyBag prop_bag;
 	char *cid;
 
 	cid = eb->classid;
@@ -524,14 +618,24 @@ on_object_requested (GtkHTML *html, GtkHTMLEmbedded *eb, gpointer data)
 
 		pbl = g_new0 (struct _PixbufLoader, 1);
 		if (g_strncasecmp (eb->type, "image/", 6) == 0) {
-			pbl->mstream = camel_stream_mem_new ();
-			camel_data_wrapper_write_to_stream (
-				camel_medium_get_content_object (medium),
-				pbl->mstream);
-			camel_stream_reset (pbl->mstream);
+			CamelDataWrapper *content;
+
+			content = camel_medium_get_content_object (medium);
+			if (!camel_data_wrapper_is_offline (content)) {
+				pbl->mstream = camel_stream_mem_new ();
+				camel_data_wrapper_write_to_stream (content, pbl->mstream);
+				camel_stream_reset (pbl->mstream);
+			}
 		}
 		pbl->type = g_strdup (eb->type);
+		pbl->cid = g_strdup (cid);
 		pbl->pixmap = bonobo_ui_toolbar_icon_new ();
+		pbl->eb = eb;
+		pbl->md = md;
+		pbl->destroy_id = gtk_signal_connect (GTK_OBJECT (eb),
+						      "destroy",
+						      embeddable_destroy_cb,
+						      pbl);
 
 		g_idle_add_full (G_PRIORITY_LOW, (GSourceFunc)pixbuf_gen_idle, 
 				 pbl, NULL);
@@ -540,8 +644,6 @@ on_object_requested (GtkHTML *html, GtkHTMLEmbedded *eb, gpointer data)
 		gtk_widget_set_sensitive (GTK_WIDGET (ebox), TRUE);
 		gtk_widget_add_events (GTK_WIDGET (ebox),
 				       GDK_BUTTON_PRESS_MASK);
-		gtk_signal_connect (GTK_OBJECT (ebox), "button_press_event",
-				    GTK_SIGNAL_FUNC (pixmap_press), ebox);
 		gtk_object_set_data (GTK_OBJECT (ebox), "MailDisplay", md);
 		gtk_object_set_data (GTK_OBJECT (ebox), "CamelMimePart",
 				     medium);
@@ -549,6 +651,9 @@ on_object_requested (GtkHTML *html, GtkHTMLEmbedded *eb, gpointer data)
 					  g_strdup (eb->type),
 					  (GDestroyNotify)g_free);
 
+		gtk_signal_connect (GTK_OBJECT (ebox), "button_press_event",
+				    GTK_SIGNAL_FUNC (pixmap_press), md->scroll);
+
 		gtk_container_add (GTK_CONTAINER (ebox), pbl->pixmap);
 		gtk_widget_show_all (ebox);
 		gtk_container_add (GTK_CONTAINER (eb), ebox);
@@ -559,53 +664,7 @@ on_object_requested (GtkHTML *html, GtkHTMLEmbedded *eb, gpointer data)
 	if (!component)
 		return FALSE;
 
-	embedded = bonobo_widget_new_subdoc (component->iid, NULL);
-	if (embedded) {
-		/* FIXME: as of bonobo 0.18, there's an extra
-		 * client_site dereference in the BonoboWidget
-		 * destruction path that we have to balance out to
-		 * prevent problems.
-		 */
-		bonobo_object_ref (BONOBO_OBJECT(bonobo_widget_get_client_site (
-			BONOBO_WIDGET (embedded))));
-	} else {
-		embedded = bonobo_widget_new_control (component->iid, NULL);
-		if (!embedded) {
-			CORBA_free (component);
-			return FALSE;
-		}
-
-		control_frame = bonobo_widget_get_control_frame (BONOBO_WIDGET (embedded));
-		if (control_frame) {
-			prop_bag = bonobo_control_frame_get_control_property_bag ( control_frame, NULL );
-			
-			if (prop_bag != CORBA_OBJECT_NIL) {
-				/* Now we can take care of business. Currently, the only control
-				   that needs something passed to it through a property bag is
-				   the iTip control, and it needs only the From email address,
-				   but perhaps in the future we can generalize this section of code
-				   to pass a bunch of useful things to all embedded controls. */
-				const CamelInternetAddress *from;
-				char *from_address;
-				MailConfigIdentity *id;
-
-				id = mail_config_get_default_identity ();
-				if (!id)
-					g_warning ("No identity configured!");
-
-				CORBA_exception_init (&ev);
-
-				from = camel_mime_message_get_from (md->current_message);
-				from_address = camel_address_encode((CamelAddress *)from);
-				bonobo_property_bag_client_set_value_string (prop_bag, "from_address", 
-									     from_address, &ev);
-				bonobo_property_bag_client_set_value_string (prop_bag, "my_address", 
-									     id ? id->address : "", &ev);
-				g_free(from_address);
-				CORBA_exception_free (&ev);
-			}
-		}
-	}  /* end else (we're going to use a control) */
+	embedded = get_embedded_for_component (component->iid, md);
 	CORBA_free (component);
 	if (!embedded)
 		return FALSE;
@@ -673,6 +732,8 @@ on_url_requested (GtkHTML *html, const char *url, GtkHTMLStream *handle,
 
 		g_return_if_fail (CAMEL_IS_MEDIUM (medium));
 		data = camel_medium_get_content_object (medium);
+		if (!mail_content_loaded (data, md))
+			return;
 
 		ba = g_byte_array_new ();
 		stream_mem = camel_stream_mem_new_with_byte_array (ba);
@@ -750,47 +811,6 @@ clear_data (CamelObject *object, gpointer event_data, gpointer user_data)
 	g_datalist_clear (&data);
 }
 
-static void
-redisplay (MailDisplay *md, gboolean unscroll)
-{
-	GtkAdjustment *adj;
-	gfloat oldv = 0;
-
-	if (!unscroll) {
-		adj = e_scroll_frame_get_vadjustment (md->scroll);
-		oldv = adj->value;
-	}
-	
-	md->stream = gtk_html_begin (md->html);
-	mail_html_write (md->html, md->stream, "%s%s", HTML_HEADER,
-			 "<BODY TEXT=\"#000000\" BGCOLOR=\"#FFFFFF\">\n");
-	
-	if (md->current_message) {
-		camel_object_ref (CAMEL_OBJECT (md->current_message));
-		if (mail_config_view_source ())
-			mail_format_raw_message (md->current_message, md);
-		else
-			mail_format_mime_message (md->current_message, md);
-	}
-	
-	mail_html_write (md->html, md->stream, "</BODY></HTML>\n");
-	gtk_html_end (md->html, md->stream, GTK_HTML_STREAM_OK);
-	md->stream = NULL;
-
-	if (unscroll) {
-		adj = e_scroll_frame_get_hadjustment (md->scroll);
-		gtk_adjustment_set_value (adj, 0);
-		e_scroll_frame_set_hadjustment (md->scroll, adj);
-	} else {
-		adj = e_scroll_frame_get_vadjustment (md->scroll);
-		if (oldv < adj->upper) {
-			gtk_adjustment_set_value (adj, oldv);
-			e_scroll_frame_set_vadjustment (md->scroll, adj);
-		}
-	}
-}
-
-
 /**
  * mail_display_redisplay:
  * @mail_display: the mail display object
@@ -799,11 +819,32 @@ redisplay (MailDisplay *md, gboolean unscroll)
  * Force a redraw of the message display.
  **/
 void
-mail_display_redisplay (MailDisplay *mail_display, gboolean unscroll)
+mail_display_redisplay (MailDisplay *md, gboolean unscroll)
 {
-	redisplay (mail_display, unscroll);
+	printf("redisplaying\n");
+
+	md->stream = gtk_html_begin (GTK_HTML (md->html));
+	if (!unscroll) {
+		/* This is a hack until there's a clean way to do this. */
+		GTK_HTML (md->html)->engine->newPage = FALSE;
+	}
+
+	mail_html_write (md->html, md->stream, "%s%s", HTML_HEADER, "<BODY>\n");
+
+	if (md->current_message) {
+		camel_object_ref (CAMEL_OBJECT (md->current_message));
+		if (mail_config_get_view_source ())
+			mail_format_raw_message (md->current_message, md);
+		else
+			mail_format_mime_message (md->current_message, md);
+	}
+
+	mail_html_write (md->html, md->stream, "</BODY></HTML>\n");
+	gtk_html_end (md->html, md->stream, GTK_HTML_STREAM_OK);
+	md->stream = NULL;
 }
 
+
 /**
  * mail_display_set_message:
  * @mail_display: the mail display object
@@ -828,7 +869,7 @@ mail_display_set_message (MailDisplay *md, CamelMedium *medium)
 	md->current_message = (CamelMimeMessage*)medium;
 
 	g_datalist_init (md->data);
-	redisplay (md, TRUE);
+	mail_display_redisplay (md, TRUE);
 	if (medium) {
 		camel_object_hook_event (CAMEL_OBJECT (medium), "finalize",
 					 clear_data, *(md->data));
@@ -847,6 +888,16 @@ mail_display_init (GtkObject *object)
 
 	/* various other initializations */
 	mail_display->current_message = NULL;
+	mail_display->thumbnail_cache = g_hash_table_new (g_str_hash, g_str_equal);
+}
+
+static void
+thumbnail_cache_free (gpointer key,
+		      gpointer value,
+		      gpointer user_data)
+{
+	g_free (key);
+	gdk_pixbuf_unref (value);
 }
 
 static void
@@ -854,6 +905,9 @@ mail_display_destroy (GtkObject *object)
 {
 	MailDisplay *mail_display = MAIL_DISPLAY (object);
 
+	g_hash_table_foreach (mail_display->thumbnail_cache, 
+			      thumbnail_cache_free, NULL);
+	g_hash_table_destroy (mail_display->thumbnail_cache);
 	g_datalist_clear (mail_display->data);
 	g_free (mail_display->data);
 
@@ -897,6 +951,7 @@ mail_display_new (void)
 	gtk_signal_connect (GTK_OBJECT (html), "link_clicked",
 			    GTK_SIGNAL_FUNC (on_link_clicked),
 			    mail_display);
+
 	gtk_container_add (GTK_CONTAINER (scroll), html);
 	gtk_widget_show (GTK_WIDGET (html));
 
