@@ -28,8 +28,10 @@
 #include "mail-tools.h"
 #include "mail-display.h"
 #include "mail-crypto.h"
+#include "mail-mt.h"
 #include "shell/e-setup.h"
 #include "e-util/e-html-utils.h"
+#include <gal/widgets/e-unicode.h>
 #include <camel/camel-mime-utils.h>
 #include <libgnome/libgnome.h>
 #include <libgnomevfs/gnome-vfs-mime-info.h>
@@ -43,6 +45,7 @@
 static char *get_data_wrapper_text (CamelDataWrapper *data);
 
 static char *try_inline_pgp (char *start, MailDisplay *md);
+static char *try_inline_pgp_sig (char *start, MailDisplay *md);
 static char *try_uudecoding (char *start, MailDisplay *md);
 static char *try_inline_binhex (char *start, MailDisplay *md);
 
@@ -75,6 +78,9 @@ static gboolean handle_multipart_appledouble (CamelMimePart *part,
 static gboolean handle_multipart_encrypted   (CamelMimePart *part,
 					      const char *mime_type,
 					      MailDisplay *md);
+static gboolean handle_multipart_signed      (CamelMimePart *part,
+					      const char *mime_type,
+					      MailDisplay *md);
 static gboolean handle_message_rfc822        (CamelMimePart *part,
 					      const char *mime_type,
 					      MailDisplay *md);
@@ -305,6 +311,8 @@ setup_mime_tables (void)
 			     handle_multipart_appledouble);
 	g_hash_table_insert (mime_function_table, "multipart/encrypted",
 			     handle_multipart_encrypted);
+	g_hash_table_insert (mime_function_table, "multipart/signed",
+			     handle_multipart_signed);
 
 	/* RFC 2046 says unrecognized text subtypes can be treated
 	 * as text/plain (as long as you recognize the character set),
@@ -455,8 +463,7 @@ gboolean
 mail_part_is_inline (CamelMimePart *part)
 {
 	const char *disposition;
-	GMimeContentField *content_type;
-	const char *mime_type;
+	CamelContentType *content_type;
 
 	/* If it has an explicit disposition, return that. */
 	disposition = camel_mime_part_get_disposition (part);
@@ -467,14 +474,13 @@ mail_part_is_inline (CamelMimePart *part)
 	 * be customizable.
 	 */
 	content_type = camel_mime_part_get_content_type (part);
-	mime_type = gmime_content_field_get_mime_type (content_type);
-	if (!g_strncasecmp (mime_type, "message/", 8))
+	if (!header_content_type_is (content_type, "message", "*"))
 		return TRUE;
 
 	/* Otherwise, display it inline if it's "anonymous", and
 	 * as an attachment otherwise.
 	 */
-	return is_anonymous (part, mime_type);
+	return is_anonymous (part, header_content_type_simple (content_type));
 }
 
 static void
@@ -543,7 +549,7 @@ call_handler_function (CamelMimePart *part, MailDisplay *md)
 	if (!handler) {
 		char *id_type;
 
-		id_type = mail_identify_mime_part (part);
+		id_type = mail_identify_mime_part (part, md);
 		if (id_type) {
 			g_free (mime_type);
 			mime_type = id_type;
@@ -553,7 +559,8 @@ call_handler_function (CamelMimePart *part, MailDisplay *md)
 
 	is_inline = mail_part_is_inline (part);
 	attachment_header (part, mime_type, is_inline, md);
-	if (handler && handler->builtin && is_inline)
+	if (handler && handler->builtin && is_inline &&
+	    mail_content_loaded (wrapper, md))
 		output = (*handler->builtin) (part, mime_type, md);
 	else
 		output = TRUE;
@@ -570,25 +577,22 @@ enum {
 static void
 write_field_to_stream(const char *description, const char *value, int flags, GtkHTML *html, GtkHTMLStream *stream)
 {
-	char *encoded_value;
+	char *encoded_desc, *encoded_value;
 	int bold = (flags&WRITE_BOLD) == WRITE_BOLD;
 
-	if (value) {
+	/* The description comes from gettext... */
+	encoded_desc = e_utf8_from_gtk_string (GTK_WIDGET (html), description);
+
+	if (value)
 		encoded_value = e_text_to_html (value, E_TEXT_TO_HTML_CONVERT_NL|E_TEXT_TO_HTML_CONVERT_URLS);
-#if 0				/* I dont think this needs to be here anymore ... e_text_to_html should handle that anyway */
-		char *p;
-		for (p = encoded_value; *p; p++) {
-			if (!isprint (*p))
-				*p = '?';
-		}
-#endif
-	} else
+	else
 		encoded_value = "";
 
 	mail_html_write(html, stream,
 			"<tr valign=top><%s align=right>%s</%s>"
 			"<td>%s</td></tr>", bold ? "th" : "td",
-			description, bold ? "th" : "td", encoded_value);
+			encoded_desc, bold ? "th" : "td", encoded_value);
+	g_free (encoded_desc);
 	if (value)
 		g_free(encoded_value);
 }
@@ -613,8 +617,10 @@ static void
 write_headers (CamelMimeMessage *message, MailDisplay *md)
 {
 	mail_html_write (md->html, md->stream,
-			 "<table bgcolor=\"#EEEEEE\" width=\"100%%\" "
-			 "cellspacing=0 border=1>"
+			 "<font color=\"#000000\">"
+			 "<table bgcolor=\"#000000\" width=\"100%%\" "
+			 "cellspacing=0 cellpadding=1><tr><td>"
+			 "<table bgcolor=\"#EEEEEE\" width=\"100%%\" cellpadding=0 cellspacing=0>"
 			 "<tr><td><table>\n");
 
 	write_address(md, camel_mime_message_get_from(message),
@@ -631,38 +637,121 @@ write_headers (CamelMimeMessage *message, MailDisplay *md)
 			       TRUE, md->html, md->stream);
 	
 	mail_html_write (md->html, md->stream,
-			 "</table></td></tr></table></center><p>");
+			 "</table></td></tr></table></td></tr></table></font>");
 }
 
+struct _load_content_msg {
+	struct _mail_msg msg;
 
-/* Return the contents of a text-based data wrapper, or NULL if it
- * contains only whitespace.
+	MailDisplay *display;
+	CamelMimeMessage *message;
+	CamelDataWrapper *wrapper;
+};
+
+static char *
+load_content_desc (struct _mail_msg *mm, int done)
+{
+	return g_strdup (_("Loading message content"));
+}
+
+static void
+load_content_load (struct _mail_msg *mm)
+{
+	struct _load_content_msg *m = (struct _load_content_msg *)mm;
+	CamelStream *memstream;
+
+	memstream = camel_stream_mem_new ();
+	camel_data_wrapper_write_to_stream (m->wrapper, memstream);
+	camel_object_unref (CAMEL_OBJECT (memstream));
+}
+
+static void
+load_content_loaded (struct _mail_msg *mm)
+{
+	struct _load_content_msg *m = (struct _load_content_msg *)mm;
+
+	if (m->display->current_message == m->message)
+		mail_display_queue_redisplay (m->display);
+}
+
+static void
+load_content_free (struct _mail_msg *mm)
+{
+	struct _load_content_msg *m = (struct _load_content_msg *)mm;
+
+	gtk_object_unref (GTK_OBJECT (m->display));
+	camel_object_unref (CAMEL_OBJECT (m->wrapper));
+	camel_object_unref (CAMEL_OBJECT (m->message));
+}
+
+static struct _mail_msg_op load_content_op = {
+	load_content_desc,
+	load_content_load,
+	load_content_loaded,
+	load_content_free,
+};
+
+gboolean
+mail_content_loaded (CamelDataWrapper *wrapper, MailDisplay *md)
+{
+	struct _load_content_msg *m;
+	GHashTable *loading;
+
+	if (!camel_data_wrapper_is_offline (wrapper))
+		return TRUE;
+
+	loading = g_datalist_get_data (md->data, "loading");
+	if (loading) {
+		if (g_hash_table_lookup (loading, wrapper))
+			return FALSE;
+	} else {
+		loading = g_hash_table_new (NULL, NULL);
+		g_datalist_set_data_full (md->data, "loading", loading,
+					  (GDestroyNotify)g_hash_table_destroy);
+	}
+	g_hash_table_insert (loading, wrapper, GINT_TO_POINTER (1));	
+
+	m = mail_msg_new (&load_content_op, NULL, sizeof (*m));
+	m->display = md;
+	gtk_object_ref (GTK_OBJECT (m->display));
+	m->message = md->current_message;
+	camel_object_ref (CAMEL_OBJECT (m->message));
+	m->wrapper = wrapper;
+	camel_object_ref (CAMEL_OBJECT (m->wrapper));
+
+	e_thread_put (mail_thread_queued, (EMsg *)m);
+	return FALSE;
+}
+
+/* Return the contents of a data wrapper, or %NULL if it contains only
+ * whitespace.
  */
 static char *
-get_data_wrapper_text (CamelDataWrapper *data)
+get_data_wrapper_text (CamelDataWrapper *wrapper)
 {
 	CamelStream *memstream;
 	GByteArray *ba;
 	char *text, *end;
 
+	memstream = camel_stream_mem_new ();
 	ba = g_byte_array_new ();
-	memstream = camel_stream_mem_new_with_byte_array (ba);
+	camel_stream_mem_set_byte_array (CAMEL_STREAM_MEM (memstream), ba);
+	camel_data_wrapper_write_to_stream (wrapper, memstream);
+	camel_object_unref (CAMEL_OBJECT (memstream));
 
-	camel_data_wrapper_write_to_stream (data, memstream);
-
-	for (text = ba->data, end = ba->data + ba->len; text < end; text++) {
+	for (text = ba->data, end = text + ba->len; text < end; text++) {
 		if (!isspace ((unsigned char)*text))
 			break;
 	}
 
-	if (text < end) {
-		text = g_malloc (ba->len + 1);
-		memcpy (text, ba->data, ba->len);
-		text[ba->len] = '\0';
-	} else
-		text = NULL;
+	if (text >= end) {
+		g_byte_array_free (ba, TRUE);
+		return NULL;
+	}
 
-	camel_object_unref (CAMEL_OBJECT (memstream));
+	g_byte_array_append (ba, "", 1);
+	text = ba->data;
+	g_byte_array_free (ba, FALSE);
 	return text;
 }
 
@@ -675,6 +764,7 @@ struct {
 	char * (*handler) (char *start, MailDisplay *md);
 } text_specials[] = {
 	{ "-----BEGIN PGP MESSAGE-----\n", try_inline_pgp },
+	{ "-----BEGIN PGP SIGNED MESSAGE-----\n", try_inline_pgp_sig },
 	{ "begin ", try_uudecoding },
 	{ "(This file must be converted with BinHex 4.0)\n", try_inline_binhex }
 };
@@ -687,7 +777,7 @@ handle_text_plain (CamelMimePart *part, const char *mime_type,
 	CamelDataWrapper *wrapper =
 		camel_medium_get_content_object (CAMEL_MEDIUM (part));
 	char *text, *p, *start;
-	GMimeContentField *type;
+	CamelContentType *type;
 	const char *format;
 	int i;
 
@@ -697,11 +787,11 @@ handle_text_plain (CamelMimePart *part, const char *mime_type,
 
 	/* Check for RFC 2646 flowed text. */
 	type = camel_mime_part_get_content_type (part);
-	format = gmime_content_field_get_parameter (type, "format");
+	format = header_content_type_param (type, "format");
 	if (format && !g_strcasecmp (format, "flowed"))
 		return handle_text_plain_flowed (text, md);
 
-	mail_html_write (md->html, md->stream, "\n<!-- text/plain -->\n");
+	mail_html_write (md->html, md->stream, "\n<!-- text/plain -->\n<font size=\"-3\">&nbsp</font><br>\n");
 
 	p = text;
 	while (p) {
@@ -747,7 +837,7 @@ handle_text_plain_flowed (char *buf, MailDisplay *md)
 	gboolean br_pending = FALSE;
 
 	mail_html_write (md->html, md->stream,
-			 "\n<!-- text/plain, flowed -->\n<tt>\n");
+			 "\n<!-- text/plain, flowed -->\n<font size=\"-3\">&nbsp</font><br>\n<tt>\n");
 
 	for (line = buf; *line; line = eol + 1) {
 		/* Process next line */
@@ -833,19 +923,15 @@ decode_pgp (const char *ciphertext, int *outlen, MailDisplay *md)
 	char *plaintext;
 	
 	camel_exception_init (&ex);
-#ifdef PGP_PROGRAM
+	
 	/* FIXME: multipart parts */
+	/* another FIXME: this doesn't have to return plaintext you realize... */
 	if (g_datalist_get_data (md->data, "show_pgp")) {
-		plaintext = mail_crypto_openpgp_decrypt (ciphertext, outlen, &ex);
+		plaintext = openpgp_decrypt (ciphertext, strlen (ciphertext), outlen, &ex);
 		if (plaintext)
 			return plaintext;
 	}
-#else
-	camel_exception_set (&ex, CAMEL_EXCEPTION_SYSTEM,
-			     _("No GPG/PGP support available in this copy "
-			       "of Evolution."));
-#endif
-
+	
 	mail_html_write (md->html, md->stream,
 			 "<table><tr valign=top><td>"
 			 "<a href=\"x-evolution-decode-pgp:\">"
@@ -873,18 +959,17 @@ try_inline_pgp (char *start, MailDisplay *md)
 {
 	char *end, *ciphertext, *plaintext;
 	int outlen;
-
-	/* FIXME: This should deal with signed data as well. */
-
+	
 	end = strstr (start, "-----END PGP MESSAGE-----");
 	if (!end)
 		return start;
-
-	end += sizeof ("-----END PGP MESSAGE-----") - 1;
-
+	
+	end += strlen ("-----END PGP MESSAGE-----") - 1;
+	
 	mail_html_write (md->html, md->stream, "<hr>");
 	
-	/* FIXME: uhm, pgp decrypted data doesn't have to be plaintext */
+	/* FIXME: uhm, pgp decrypted data doesn't have to be plaintext
+	 * however, I suppose that since it was 'inline', it probably is */
 	ciphertext = g_strndup (start, end - start);
 	plaintext = decode_pgp (ciphertext, &outlen, md);
 	g_free (ciphertext);
@@ -896,7 +981,59 @@ try_inline_pgp (char *start, MailDisplay *md)
 		mail_html_write (md->html, md->stream, "</td></tr></table>");
 		g_free (plaintext);
 	}
+	
+	return end;
+}
 
+static char *
+try_inline_pgp_sig (char *start, MailDisplay *md)
+{
+	char *end, *ciphertext, *plaintext;
+	CamelException *ex;
+	gboolean valid;
+	
+	end = strstr (start, "-----END PGP SIGNATURE-----");
+	if (!end)
+		return start;
+	
+	end += strlen ("-----END PGP SIGNATURE-----");
+	
+	mail_html_write (md->html, md->stream, "<hr>");
+	
+	ciphertext = g_strndup (start, end - start);
+	ex = camel_exception_new ();
+	valid = openpgp_verify (ciphertext, end - start, NULL, 0, ex);
+	g_free (ciphertext);
+	
+	plaintext = g_strndup (start, end - start);
+	mail_text_write (md->html, md->stream, "%s", plaintext);
+	g_free (plaintext);
+	
+	/* Now display the "seal-of-authenticity" or something... */
+	if (valid) {
+		mail_html_write (md->html, md->stream,
+				 "<hr>\n<table><tr valign=top>"
+				 "<td><img src=\"%s\"></td>"
+				 "<td><font size=-1>%s<br><br></font></td></table>",
+				 get_url_for_icon ("wax-seal2.png", md),
+				 _("This message is digitally signed and "
+				   "has been found to be authentic."));
+	} else {
+		mail_html_write (md->html, md->stream,
+				 "<hr>\n<table><tr valign=top>"
+				 "<td><img src=\"%s\"></td>"
+				 "<td><font size=-1>%s<br><br>",
+				 get_url_for_icon ("wax-seal-broken.png", md),
+				 _("This message is digitally signed but can "
+				   "not be proven to be authentic."));
+		mail_error_write (md->html, md->stream,
+				  camel_exception_get_description (ex));
+		mail_html_write (md->html, md->stream,
+				 "<br><br></font></td></table>");
+	}
+	
+	camel_exception_free (ex);
+	
 	return end;
 }
 
@@ -1188,90 +1325,94 @@ handle_multipart_mixed (CamelMimePart *part, const char *mime_type,
 	return TRUE;
 }
 
-static gboolean
-is_rfc2015 (CamelMimePart *part)
-{
-	int nparts;
-	char *text;
-	CamelDataWrapper *wrapper;
-	CamelMultipart *mp;
-	GMimeContentField *type;
-
-	wrapper = camel_medium_get_content_object (CAMEL_MEDIUM (part));
-	mp = CAMEL_MULTIPART (wrapper);
-	nparts = camel_multipart_get_number (mp);
-	if (nparts != 2)
-		return FALSE;
-
-	/* Check for application/pgp-encrypted in the first part. */
-	part = camel_multipart_get_part (mp, 0);
-	type = camel_mime_part_get_content_type (part);
-	if (!gmime_content_field_is_type (type, "application", "pgp-encrypted"))
-		return FALSE;
-
-	/* Check version. */
-	wrapper = camel_medium_get_content_object (CAMEL_MEDIUM (part));
-	text = get_data_wrapper_text (wrapper);
-	if (!text || !strstr(text, "Version: 1")) {
-		g_free(text);
-		return FALSE;
-	}
-	g_free(text);
-
-	/* Check for application/octet-stream in the second part. */
-	part = camel_multipart_get_part(mp, 1);
-	type = camel_mime_part_get_content_type (part);
-	if (!gmime_content_field_is_type (type, "application", "octet-stream"))
-		return FALSE;
-
-	return TRUE;
-}
-
 static gboolean
 handle_multipart_encrypted (CamelMimePart *part, const char *mime_type,
 			    MailDisplay *md)
 {
-	CamelDataWrapper *wrapper =
-		camel_medium_get_content_object (CAMEL_MEDIUM (part));
-	CamelMultipart *mp;
-	char *ciphertext, *plaintext;
-	int outlen;
-
-	g_return_val_if_fail (CAMEL_IS_MULTIPART (wrapper), FALSE);
-	mp = CAMEL_MULTIPART (wrapper);
-
-	/* Currently we only handle RFC2015-style PGP encryption. */
-	if (!is_rfc2015 (part))
-		return handle_multipart_mixed (part, mime_type, md);
-
-	part = camel_multipart_get_part (mp, 1);
-	wrapper = camel_medium_get_content_object (CAMEL_MEDIUM (part));
-	ciphertext = get_data_wrapper_text (wrapper);
-	if (!ciphertext)
-		return FALSE;
+	CamelDataWrapper *wrapper;
+	CamelMimePart *mime_part;
+	CamelException ex;
 	
-	/* FIXME: please note that decrypted data does NOT have to be plaintext */
-	plaintext = decode_pgp (ciphertext, &outlen, md);
-	if (plaintext) {
-		CamelStream *memstream;
-
-		memstream = camel_stream_mem_new_with_buffer (plaintext,
-							      strlen (plaintext));
-		part = camel_mime_part_new ();
-		camel_data_wrapper_construct_from_stream (CAMEL_DATA_WRAPPER (part),
-							  memstream);
-		camel_object_unref (CAMEL_OBJECT (memstream));
-
-		mail_html_write (md->html, md->stream,
-				 "<table width=\"100%%\" border=2 "
-				 "cellpadding=4><tr><td>");
-		call_handler_function (part, md);
-		mail_html_write (md->html, md->stream, "</td></tr></table>");
-		camel_object_hook_event (CAMEL_OBJECT (md->current_message),
-					 "finalize", destroy_part, part);
-		g_free (plaintext);
+	wrapper = camel_medium_get_content_object (CAMEL_MEDIUM (part));
+	g_return_val_if_fail (CAMEL_IS_MULTIPART (wrapper), FALSE);
+	
+	/* Currently we only handle RFC2015-style PGP encryption. */
+	if (!mail_crypto_is_rfc2015_encrypted (part))
+		return handle_multipart_mixed (part, mime_type, md);
+	
+	camel_exception_init (&ex);
+	mime_part = pgp_mime_part_decrypt (part, &ex);
+	if (camel_exception_is_set (&ex)) {
+		/* I guess we just treat this as a multipart/mixed */
+		return handle_multipart_mixed (part, mime_type, md);
+	} else {
+		gboolean retcode;
+		
+		retcode = call_handler_function (mime_part, md);
+		camel_object_unref (CAMEL_OBJECT (mime_part));
+		
+		return retcode;
 	}
+}
 
+static gboolean
+handle_multipart_signed (CamelMimePart *part, const char *mime_type,
+			 MailDisplay *md)
+{
+	CamelDataWrapper *wrapper;
+	CamelMultipart *mp;
+	CamelException *ex;
+	gboolean output = FALSE;
+	gboolean valid;
+	int nparts, i;
+	
+	wrapper = camel_medium_get_content_object (CAMEL_MEDIUM (part));
+	g_return_val_if_fail (CAMEL_IS_MULTIPART (wrapper), FALSE);
+	
+	/* Currently we only handle RFC2015-style PGP signatures. */
+	if (!mail_crypto_is_rfc2015_signed (part))
+		return handle_multipart_mixed (part, mime_type, md);
+	
+	ex = camel_exception_new ();
+	valid = pgp_mime_part_verify (part, ex);
+	
+	/* now display all the subparts *except* the signature */
+	mp = CAMEL_MULTIPART (wrapper);
+	
+	nparts = camel_multipart_get_number (mp);	
+	for (i = 0; i < nparts - 1; i++) {
+		if (i != 0 && output)
+			mail_html_write (md->html, md->stream, "<hr>\n");
+		
+		part = camel_multipart_get_part (mp, i);
+		
+		output = call_handler_function (part, md);
+	}
+	
+	/* Now display the "seal-of-authenticity" or something... */
+	if (valid) {
+		mail_html_write (md->html, md->stream,
+				 "<hr>\n<table><tr valign=top>"
+				 "<td><img src=\"%s\"></td>"
+				 "<td><font size=-1>%s<br><br></font></td></table>",
+				 get_url_for_icon ("wax-seal2.png", md),
+				 _("This message is digitally signed and "
+				   "has been found to be authentic."));
+	} else {
+		mail_html_write (md->html, md->stream,
+				 "<hr>\n<table><tr valign=top>"
+				 "<td><img src=\"%s\"></td>"
+				 "<td><font size=-1>%s<br><br>",
+				 get_url_for_icon ("wax-seal-broken.png", md),
+				 _("This message is digitally signed but can "
+				   "not be proven to be authentic."));
+		mail_error_write (md->html, md->stream,
+				  camel_exception_get_description (ex));
+		mail_html_write (md->html, md->stream,
+				 "<br><br></font></td></table>");
+	}
+	camel_exception_free (ex);
+	
 	return TRUE;
 }
 
@@ -1284,7 +1425,7 @@ handle_multipart_related (CamelMimePart *part, const char *mime_type,
 		camel_medium_get_content_object (CAMEL_MEDIUM (part));
 	CamelMultipart *mp;
 	CamelMimePart *body_part, *display_part = NULL;
-	GMimeContentField *content_type;
+	CamelContentType *content_type;
 	const char *start;
 	int i, nparts;
 
@@ -1293,7 +1434,7 @@ handle_multipart_related (CamelMimePart *part, const char *mime_type,
 	nparts = camel_multipart_get_number (mp);	
 
 	content_type = camel_mime_part_get_content_type (part);
-	start = gmime_content_field_get_parameter (content_type, "start");
+	start = header_content_type_param (content_type, "start");
 	if (start) {
 		int len;
 
@@ -1348,8 +1489,8 @@ find_preferred_alternative (CamelMultipart *multipart, gboolean want_plain)
 	nparts = camel_multipart_get_number (multipart);
 	for (i = 0; i < nparts; i++) {
 		CamelMimePart *part = camel_multipart_get_part (multipart, i);
-		char *mime_type = gmime_content_field_get_mime_type (
-			camel_mime_part_get_content_type (part));
+		CamelContentType *type = camel_mime_part_get_content_type (part);
+		char *mime_type = header_content_type_simple (type);
 
 		g_strdown (mime_type);
 		if (want_plain && !strcmp (mime_type, "text/plain"))
@@ -1422,12 +1563,12 @@ static gboolean
 handle_message_external_body (CamelMimePart *part, const char *mime_type,
 			      MailDisplay *md)
 {
-	GMimeContentField *type;
+	CamelContentType *type;
 	const char *access_type;
 	char *url = NULL, *desc = NULL;
 
 	type = camel_mime_part_get_content_type (part);
-	access_type = gmime_content_field_get_parameter (type, "access-type");
+	access_type = header_content_type_param (type, "access-type");
 	if (!access_type)
 		goto fallback;
 
@@ -1436,12 +1577,12 @@ handle_message_external_body (CamelMimePart *part, const char *mime_type,
 		const char *name, *site, *dir, *mode, *ftype;
 		char *path;
 
-		name = gmime_content_field_get_parameter (type, "name");
-		site = gmime_content_field_get_parameter (type, "site");
+		name = header_content_type_param (type, "name");
+		site = header_content_type_param (type, "site");
 		if (name == NULL || site == NULL)
 			goto fallback;
-		dir = gmime_content_field_get_parameter (type, "directory");
-		mode = gmime_content_field_get_parameter (type, "mode");
+		dir = header_content_type_param (type, "directory");
+		mode = header_content_type_param (type, "mode");
 
 		/* Generate the path. */
 		if (dir) {
@@ -1471,10 +1612,10 @@ handle_message_external_body (CamelMimePart *part, const char *mime_type,
 	} else if (!g_strcasecmp (access_type, "local-file")) {
 		const char *name, *site;
 
-		name = gmime_content_field_get_parameter (type, "name");
+		name = header_content_type_param (type, "name");
 		if (name == NULL)
 			goto fallback;
-		site = gmime_content_field_get_parameter (type, "site");
+		site = header_content_type_param (type, "site");
 
 		url = g_strdup_printf ("file://%s%s", *name == '/' ? "" : "/",
 				       name);
@@ -1492,7 +1633,7 @@ handle_message_external_body (CamelMimePart *part, const char *mime_type,
 
 		/* RFC 2017 */
 
-		urlparam = gmime_content_field_get_parameter (type, "url");
+		urlparam = header_content_type_param (type, "url");
 		if (urlparam == NULL)
 			goto fallback;
 
@@ -1554,7 +1695,7 @@ mail_get_message_body (CamelDataWrapper *data, gboolean want_plain, gboolean *is
 	char *subtext, *old;
 	const char *boundary;
 	char *text = NULL;
-	GMimeContentField *mime_type;
+	CamelContentType *mime_type;
 	
 	/* We only include text, message, and multipart bodies. */
 	mime_type = camel_data_wrapper_get_mime_type_field (data);
@@ -1563,25 +1704,25 @@ mail_get_message_body (CamelDataWrapper *data, gboolean want_plain, gboolean *is
 	 * images. But if we don't do it this way, we don't get
 	 * the headers...
 	 */
-	if (g_strcasecmp (mime_type->type, "message") == 0) {
+	if (header_content_type_is (mime_type, "message", "*")) {
 		*is_html = FALSE;
 		return get_data_wrapper_text (data);
 	}
 	
-	if (g_strcasecmp (mime_type->type, "text") == 0) {
-		*is_html = !g_strcasecmp (mime_type->subtype, "html");
+	if (header_content_type_is (mime_type, "text", "*")) {
+		*is_html = header_content_type_is (mime_type, "text", "html");
 		return get_data_wrapper_text (data);
 	}
 	
 	/* If it's not message and it's not text, and it's not
 	 * multipart, we don't want to deal with it.
 	 */
-	if (g_strcasecmp (mime_type->type, "multipart") != 0)
+	if (!header_content_type_is (mime_type, "multipart", "*"))
 		return NULL;
 	
 	mp = CAMEL_MULTIPART (data);
 	
-	if (g_strcasecmp (mime_type->subtype, "alternative") == 0) {
+	if (header_content_type_is (mime_type, "multipart", "alternative")) {
 		/* Pick our favorite alternative and reply to it. */
 		
 		subpart = find_preferred_alternative (mp, want_plain);
@@ -1674,7 +1815,7 @@ mail_generate_reply (CamelMimeMessage *message, gboolean to_all)
 	const char *message_id, *references;
 	const char *name = NULL, *address = NULL;
 	GList *to = NULL, *cc = NULL;
-	MailConfigIdentity *id;
+	const MailConfigIdentity *id;
 	gchar *sig_file = NULL;
 	const CamelInternetAddress *reply_to, *sender;
 	time_t date;
@@ -1682,22 +1823,24 @@ mail_generate_reply (CamelMimeMessage *message, gboolean to_all)
 	
 	id = mail_config_get_default_identity ();
 	if (id)
-	      sig_file = id->sig;
+	      sig_file = id->signature;
 	
-	composer = e_msg_composer_new_with_sig_file (sig_file, mail_config_send_html ());
+	composer = e_msg_composer_new_with_sig_file (sig_file, mail_config_get_send_html ());
 	if (!composer)
 		return NULL;
 	
+	/* FIXME: should probably use a shorter date string */
 	sender = camel_mime_message_get_from (message);
 	camel_internet_address_get (sender, 0, &name, &address);
 	date = camel_mime_message_get_date (message, &offset);
 	date_str = header_format_date (date, offset);
-	text = mail_tool_quote_message (message, _("On %s, %s wrote:\n"), date_str, name ? name : address);
+	text = mail_tool_quote_message (message, _("On %s, %s wrote:\n"), date_str, name && *name ? name : address);
 	g_free (date_str);
 	
 	if (text) {
 		e_msg_composer_set_body_text (composer, text);
 		g_free (text);
+		e_msg_composer_mark_text_orig (composer);
 	}
 	
 	/* Set the recipients */
