@@ -35,7 +35,6 @@
 #include "filter/vfolder-context.h"
 #include "filter/filter-option.h"
 #include "filter/filter-input.h"
-#include "filter/filter-driver.h"
 #include "mail.h" /*session*/
 #include "mail-tools.h"
 #include "mail-local.h"
@@ -52,6 +51,8 @@ static GPrivate *camel_locklevel = NULL;
 
 void mail_tool_camel_lock_up (void)
 {
+	return;
+
 	G_LOCK (camel_locklevel);
 
 	if (camel_locklevel == NULL)
@@ -70,6 +71,8 @@ void mail_tool_camel_lock_up (void)
 
 void mail_tool_camel_lock_down (void)
 {
+	return;
+
         G_LOCK (camel_locklevel);
 
         if (camel_locklevel == NULL) {
@@ -221,124 +224,6 @@ mail_tool_do_movemail (const gchar *source_url, CamelException *ex)
 	return dest_path;
 }
 
-void
-mail_tool_move_folder_contents (CamelFolder *source, CamelFolder *dest, gboolean use_cache, CamelException *ex)
-{
-	CamelUIDCache *cache;
-	GPtrArray *uids;
-	int i;
-	gboolean summary_capability;
-
-	mail_tool_camel_lock_up();
-
-	camel_object_ref (CAMEL_OBJECT (source));
-	camel_object_ref (CAMEL_OBJECT (dest));
-
-	/* Get all uids of source */
-
-	mail_op_set_message (_("Examining %s"), source->full_name);
-
-	uids = camel_folder_get_uids (source);
-	printf ("mail_tool_move_folder: got %d messages in source\n", uids->len);
-
-	/* If we're using the cache, ... use it */
-
-	if (use_cache) {
-		GPtrArray *new_uids;
-		char *url, *p, *filename;
-
-		url = camel_url_to_string (
-			CAMEL_SERVICE (source->parent_store)->url, FALSE);
-		for (p = url; *p; p++) {
-			if (!isascii ((unsigned char)*p) ||
-			    strchr (" /'\"`&();|<>${}!", *p))
-				*p = '_';
-		}
-		filename = g_strdup_printf ("%s/config/cache-%s",
-					    evolution_dir, url);
-		g_free (url);
-
-		cache = camel_uid_cache_new (filename);
-
-		if (cache) {
-			new_uids = camel_uid_cache_get_new_uids (cache, uids);
-			camel_folder_free_uids (source, uids);
-			uids = new_uids;
-		} else {
-			camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-					      _("Could not read UID "
-					      "cache file \"%s\". You may "
-					      "receive duplicate "
-					      "messages."), filename);
-		}
-
-		g_free (filename);
-	} else
-		cache = NULL;
-
-	printf ("mail_tool_move_folder: %d of those messages are new\n", uids->len);
-
-	summary_capability = camel_folder_has_summary_capability (source);
-
-	/* Copy the messages */
-	for (i = 0; i < uids->len; i++) {
-		CamelMimeMessage *msg;
-		const CamelMessageInfo *info = NULL;
-
-		/* Info */
-
-		mail_op_set_message (_("Retrieving message %d of %d"),
-				     i + 1, uids->len);
-
-		/* Get the message */
-
-		msg = camel_folder_get_message (source, uids->pdata[i], ex);
-		if (camel_exception_is_set (ex)) {
-			camel_object_unref (CAMEL_OBJECT (msg));
-			goto cleanup;
-		}
-		
-		/* Append it to dest */
-
-		mail_op_set_message (_("Writing message %d of %d"),
-				     i + 1, uids->len);
-
-		if (summary_capability)
-			info = camel_folder_get_message_info (source, uids->pdata[i]);
-		camel_folder_append_message (dest, msg, info, ex);
-		if (camel_exception_is_set (ex)) {
-			camel_object_unref (CAMEL_OBJECT (msg));
-			goto cleanup;
-		}
-
-		/* (Maybe) get rid of the message */
-
-		camel_object_unref (CAMEL_OBJECT (msg));
-		if (!use_cache)
-			camel_folder_delete_message (source, uids->pdata[i]);
-	}
-
-	/* All done. Sync n' free. */
-
-	if (cache) {
-		camel_uid_cache_free_uids (uids);
-		
-		if (!camel_exception_is_set (ex))
-			camel_uid_cache_save (cache);
-		camel_uid_cache_destroy (cache);
-	} else
-		camel_folder_free_uids (source, uids);
-
-	mail_op_set_message (_("Saving changes to %s"), source->full_name);
-
-	camel_folder_sync (source, TRUE, ex);
-
- cleanup:
-	camel_object_unref (CAMEL_OBJECT (source));
-	camel_object_unref (CAMEL_OBJECT (dest));
-	mail_tool_camel_lock_down();
-}
-
 void
 mail_tool_set_uid_flags (CamelFolder *folder, const char *uid, guint32 mask, guint32 set)
 {
@@ -354,10 +239,6 @@ mail_tool_generate_forward_subject (CamelMimeMessage *msg)
 	char *fwd_subj, *fromstr;
 	const CamelInternetAddress *from;
 
-	/* we need to lock around the whole function, as we are
-	   only getting references to the message's data */
-	mail_tool_camel_lock_up();
-
 	from = camel_mime_message_get_from(msg);
 	subject = camel_mime_message_get_subject(msg);
 
@@ -379,30 +260,9 @@ mail_tool_generate_forward_subject (CamelMimeMessage *msg)
 			fwd_subj = g_strdup (_("Fwd: (no subject)"));
 	}
 
-	mail_tool_camel_lock_down();
-
 	return fwd_subj;
 }
 
-void
-mail_tool_send_via_transport (CamelTransport *transport, CamelMedium *medium, CamelException *ex)
-{
-	mail_tool_camel_lock_up();
-
-	/*camel_service_connect (CAMEL_SERVICE (transport), ex);*/
-
-	if (camel_exception_is_set (ex))
-		goto cleanup;
-
-	camel_transport_send (transport, medium, ex);
-
-	/*camel_service_disconnect (CAMEL_SERVICE (transport),
-	 *camel_exception_is_set (ex) ? NULL : ex);*/
-
- cleanup:
-	mail_tool_camel_lock_down();
-}
-
 CamelMimePart *
 mail_tool_make_message_attachment (CamelMimeMessage *message)
 {
@@ -410,7 +270,6 @@ mail_tool_make_message_attachment (CamelMimeMessage *message)
 	const char *subject;
 	gchar *desc;
 
-	mail_tool_camel_lock_up();
 	/*camel_object_ref (CAMEL_OBJECT (message));*/
 
 	subject = camel_mime_message_get_subject (message);
@@ -425,13 +284,13 @@ mail_tool_make_message_attachment (CamelMimeMessage *message)
 	camel_medium_set_content_object (CAMEL_MEDIUM (part),
 					 CAMEL_DATA_WRAPPER (message));
 	camel_mime_part_set_content_type (part, "message/rfc822");
+	g_free(desc);
 	/*camel_object_unref (CAMEL_OBJECT (message));*/
-	mail_tool_camel_lock_down();
 	return part;
 }
 
 CamelFolder *
-mail_tool_filter_get_folder_func (FilterDriver *d, const char *uri, void *data)
+mail_tool_filter_get_folder_func (CamelFilterDriver *d, const char *uri, void *data)
 {
 	return mail_tool_uri_to_folder_noex (uri);
 }
@@ -479,7 +338,6 @@ mail_tool_uri_to_folder (const char *uri, CamelException *ex)
 	if (!strcmp (url->protocol, "vfolder")) {
 		folder = vfolder_uri_to_folder (uri, ex);
 	} else {
-		mail_tool_camel_lock_up ();
 		store = camel_session_get_store (session, uri, ex);
 		if (store) {
 			char *name;
@@ -491,7 +349,6 @@ mail_tool_uri_to_folder (const char *uri, CamelException *ex)
 			folder = camel_store_get_folder (
 				store, name, CAMEL_STORE_FOLDER_CREATE, ex);
 		}
-		mail_tool_camel_lock_down ();
 	}
 
 	if (camel_exception_is_set (ex)) {
@@ -530,6 +387,8 @@ mail_tool_uri_to_folder_noex (const char *uri)
 		gtk_widget_destroy (dialog);
 	}
 
+	camel_exception_clear(&ex);
+
 	return result;
 }
 
@@ -548,7 +407,7 @@ mail_tool_quote_message (CamelMimeMessage *message, const char *fmt, ...)
 	gboolean want_plain, is_html;
 	gchar *text;
 	
-	want_plain = !mail_config_send_html ();
+	want_plain = !mail_config_get_send_html ();
 	contents = camel_medium_get_content_object (CAMEL_MEDIUM (message));
 	text = mail_get_message_body (contents, want_plain, &is_html);
 	
