@@ -1,5 +1,5 @@
 /*
-  Copyright 2000 Helix Code Inc.
+  Copyright 2000, 2001 Helix Code Inc.
 
   Author: Michael Zucchi <notzed@helixcode.com>
 
@@ -21,6 +21,7 @@
 #include "mail-vfolder.h"
 #include "mail-tools.h"
 #include "mail-autofilter.h"
+#include "mail.h"
 
 #include "camel/camel.h"
 
@@ -37,7 +38,9 @@ struct _vfolder_info {
 /* list of vfolders available */
 static GList *available_vfolders = NULL;
 static VfolderContext *context;
-static EvolutionStorage *vfolder_storage;
+
+/* Ditto below */
+EvolutionStorage *vfolder_storage;
 
 /* GROSS HACK: for passing to other parts of the program */
 EvolutionShellClient *global_shell_client = NULL;
@@ -192,11 +195,14 @@ vfolder_uri_to_folder(const char *uri, CamelException *ex)
 	rule = (VfolderRule *)rule_context_find_rule((RuleContext *)context, info->name, NULL);
 
 	storeuri = g_strdup_printf("vfolder:%s/vfolder/%s", evolution_dir, info->name);
-	foldername = g_strdup_printf("mbox?%s", info->query);
+	foldername = g_strdup_printf("%s?%s", info->name, info->query);
 
 	/* we dont have indexing on vfolders */
 	folder = mail_tool_get_folder_from_urlname (storeuri, foldername, CAMEL_STORE_FOLDER_CREATE, ex);
 
+	bonobo_object_ref (BONOBO_OBJECT (vfolder_storage));
+	mail_hash_storage ((CamelService *)folder->parent_store, vfolder_storage);
+
 	sourceuri = NULL;
 	sources = 0;
 	while ( (sourceuri = vfolder_rule_next_source(rule, sourceuri)) ) {
@@ -293,11 +299,15 @@ vfolder_gui_add_rule(VfolderRule *rule)
 	GnomeDialog *gd;
 
 	w = filter_rule_get_widget((FilterRule *)rule, (RuleContext *)context);
+
 	gd = (GnomeDialog *)gnome_dialog_new(_("New VFolder"),
 					     GNOME_STOCK_BUTTON_OK,
 					     GNOME_STOCK_BUTTON_CANCEL,
 					     NULL);
+	gnome_dialog_set_default (gd, 0);
+
 	gtk_window_set_policy(GTK_WINDOW(gd), FALSE, TRUE, FALSE);
+	gtk_window_set_default_size (GTK_WINDOW (gd), 500, 500);
 	gtk_box_pack_start((GtkBox *)gd->vbox, w, TRUE, TRUE, 0);
 	gtk_widget_show((GtkWidget *)gd);
 	gtk_object_set_data_full((GtkObject *)gd, "rule", rule, (GtkDestroyNotify)gtk_object_unref);
