@@ -23,9 +23,11 @@
 #include <libgnomevfs/gnome-vfs-mime-handlers.h>
 #include "camel/camel.h"
 #include "composer/e-msg-composer.h"
+#include "mail-accounts.h"
+#include "mail-account-editor.h"
 #include "mail-callbacks.h"
 #include "mail-config.h"
-#include "mail-config-gui.h"
+#include "mail-config-druid.h"
 /*#include "folder-browser.h"*/
 #include "mail-session.h"
 #include "mail-types.h"
@@ -38,6 +40,7 @@ void mail_format_mime_message (CamelMimeMessage *mime_message,
 			       MailDisplay *md);
 void mail_format_raw_message (CamelMimeMessage *mime_message,
 			      MailDisplay *md);
+gboolean mail_content_loaded (CamelDataWrapper *wrapper, MailDisplay *display);
 
 typedef gboolean (*MailMimeHandlerFn) (CamelMimePart *part,
 				       const char *mime_type,
@@ -59,13 +62,14 @@ char *mail_get_message_body (CamelDataWrapper *data, gboolean want_plain,
 			     gboolean *is_html);
 
 /* mail-identify */
-char *mail_identify_mime_part (CamelMimePart *part);
+char *mail_identify_mime_part (CamelMimePart *part, MailDisplay *md);
 
 /* mail view */
 GtkWidget *mail_view_create (CamelFolder *source, const char *uid, CamelMimeMessage *msg);
 
 /* component factory for lack of a better place */
 /*takes a GSList of MailConfigServices */
-void mail_load_storages (GNOME_Evolution_Shell corba_shell, GSList *sources);
-/* used in the subscribe dialog code */
+void mail_load_storages (GNOME_Evolution_Shell shell, const GSList *sources, gboolean is_account_data);
+
+void mail_hash_storage (CamelService *store, EvolutionStorage *storage);
 EvolutionStorage *mail_lookup_storage (CamelStore *store);
