@@ -2,15 +2,13 @@
 #ifndef _MESSAGE_LIST_H_
 #define _MESSAGE_LIST_H_
 
-#include <gnome.h>
+/*#include <gnome.h>*/
+#include <gtk/gtk.h>
+
 #include "mail-types.h"
 #include <gal/e-table/e-table-scrolled.h>
 #include <gal/e-table/e-table-simple.h>
 #include <gal/e-table/e-tree-simple.h>
-#include <gal/e-table/e-cell-text.h>
-#include <gal/e-table/e-cell-toggle.h>
-#include <gal/e-table/e-cell-checkbox.h>
-#include <gal/e-table/e-cell-tree.h>
 
 #define MESSAGE_LIST_TYPE        (message_list_get_type ())
 #define MESSAGE_LIST(o)          (GTK_CHECK_CAST ((o), MESSAGE_LIST_TYPE, MessageList))
@@ -41,6 +39,11 @@ enum {
 #define MESSAGE_LIST_COLUMN_IS_ACTIVE(col) (col == COL_MESSAGE_STATUS || \
 					    col == COL_FLAGGED)
 
+#define ML_HIDE_NONE_START (0)
+#define ML_HIDE_NONE_END (2147483647)
+/* dont change */
+#define ML_HIDE_SAME (2147483646)
+
 struct _MessageList {
 	ETableScrolled parent;
 
@@ -56,6 +59,13 @@ struct _MessageList {
 	GHashTable       *uid_rowmap;
 	struct _EMemPool *uid_pool;
 
+	/* UID's to hide.  Keys in the mempool */
+	/* IMPORTANT: You MUST have obtained the hide lock, to operate on this data */
+	GHashTable	 *hidden;
+	struct _EMemPool *hidden_pool;
+	int hide_unhidden, /* total length, before hiding */
+		hide_before, hide_after; /* hide ranges of messages */
+
 	/* Current search string, or %NULL */
 	char *search;
 
@@ -68,6 +78,9 @@ struct _MessageList {
 
 	/* Row-selection and seen-marking timers */
 	guint idle_id, seen_id;
+
+	/* locks */
+	GMutex *hide_lock;	/* for any 'hide' info above */
 };
 
 typedef struct {
@@ -100,7 +113,18 @@ void           message_list_select     (MessageList *message_list,
 					MessageListSelectDirection direction,
 					guint32 flags, guint32 mask);
 
+/* info */
+unsigned int   message_list_length(MessageList *ml);
+
+/* hide specific messages */
+void	       message_list_hide_add(MessageList *ml, const char *expr, unsigned int lower, unsigned int upper);
+void	       message_list_hide_uids(MessageList *ml, GPtrArray *uids);
+void	       message_list_hide_clear(MessageList *ml);
+
 void	       message_list_set_threaded(MessageList *ml, gboolean threaded);
 void	       message_list_set_search(MessageList *ml, const char *search);
 
+#define MESSAGE_LIST_LOCK(m, l) g_mutex_lock(((MessageList *)m)->l)
+#define MESSAGE_LIST_UNLOCK(m, l) g_mutex_unlock(((MessageList *)m)->l)
+
 #endif /* _MESSAGE_LIST_H_ */
