@@ -25,7 +25,6 @@
 #include <gnome.h>
 #include "subscribe-dialog.h"
 #include "e-util/e-html-utils.h"
-#include "e-title-bar.h"
 #include <gtkhtml/gtkhtml.h>
 #include <gal/util/e-util.h>
 #include <gal/widgets/e-unicode.h>
@@ -58,6 +57,13 @@
 
 #define PARENT_TYPE (gtk_object_get_type ())
 
+#ifdef JUST_FOR_TRANSLATORS
+static char *list [] = {
+	N_("Folder"),
+	N_("Store"),
+};
+#endif
+
 #define FOLDER_ETABLE_SPEC "<ETableSpecification cursor-mode=\"line\"> \
         <ETableColumn model_col=\"0\" pixbuf=\"subscribed-image\" expansion=\"0.0\" minimum_width=\"16\" resizable=\"false\" cell=\"cell_toggle\" compare=\"integer\"/> \
         <ETableColumn model_col=\"1\" _title=\"Folder\" expansion=\"1.0\" minimum_width=\"20\" resizable=\"true\" cell=\"cell_tree\" compare=\"string\"/> \
@@ -218,7 +224,9 @@ static void
 subscribe_do_get_store (SubscribeDialog *sc, const char *url, SubscribeGetStoreCallback cb, gpointer cb_data)
 {
 	get_store_input_t *input;
-
+	
+	g_return_if_fail (url != NULL);
+	
         input = g_new (get_store_input_t, 1);
 	input->sc = sc;
 	input->url = g_strdup (url);
@@ -226,6 +234,7 @@ subscribe_do_get_store (SubscribeDialog *sc, const char *url, SubscribeGetStoreC
 	input->cb_data = cb_data;
 
         mail_operation_queue (&op_get_store, input, TRUE);
+	mail_operation_wait_for_finish ();
 }
 
 /* ** SUBSCRIBE FOLDER ******************************************************* */
@@ -268,6 +277,7 @@ typedef struct subscribe_folder_input_s {
 typedef struct subscribe_folder_data_s {
 	char *path;
 	char *name;
+	char *full_name;
 	char *url;
 } subscribe_folder_data_t;
 
@@ -302,7 +312,8 @@ setup_subscribe_folder (gpointer in_data, gpointer op_data, CamelException *ex)
 	subscribe_folder_data_t *data = (subscribe_folder_data_t *) op_data;
 	
 	data->path = storage_tree_path (input->info);
-	data->name = g_strdup (input->info->full_name);
+	data->name = g_strdup (input->info->name);
+	data->full_name = g_strdup (input->info->full_name);
 	data->url = g_strdup (input->info->url);
 	
 	camel_object_ref (CAMEL_OBJECT (input->store));
@@ -317,12 +328,37 @@ do_subscribe_folder (gpointer in_data, gpointer op_data, CamelException *ex)
 	
 	mail_tool_camel_lock_up ();
 	if (input->subscribe)
-		camel_store_subscribe_folder (input->store, data->name, ex);
+		camel_store_subscribe_folder (input->store, data->full_name, ex);
 	else
-		camel_store_unsubscribe_folder (input->store, data->name, ex);
+		camel_store_unsubscribe_folder (input->store, data->full_name, ex);
 	mail_tool_camel_lock_down ();
 }
 
+static void
+recursive_add_folder (EvolutionStorage *storage, const char *path,
+		      const char *name, const char *url)
+{
+	char *parent, *pname, *p;
+
+	p = strrchr (path, '/');
+	if (p && p != path) {
+		parent = g_strndup (path, p - path);
+		if (!evolution_storage_folder_exists (storage, parent)) {
+			p = strrchr (parent, '/');
+			if (p)
+				pname = g_strdup (p + 1);
+			else
+				pname = g_strdup ("");
+			recursive_add_folder (storage, parent, pname, "");
+			g_free (pname);
+		}
+		g_free (parent);
+	}
+
+	evolution_storage_new_folder (storage, path, name, "mail", url,
+				      _("(No description)"), FALSE);
+}
+
 static void
 cleanup_subscribe_folder (gpointer in_data, gpointer op_data,
 			  CamelException *ex)
@@ -332,12 +368,9 @@ cleanup_subscribe_folder (gpointer in_data, gpointer op_data,
 	
 	if (!camel_exception_is_set (ex)) {
 		if (input->subscribe)
-			evolution_storage_new_folder (input->sc->storage,
-						      data->path,
-						      data->name, "mail",
-						      data->url,
-						      _("(No description)") /* XXX */,
-						      FALSE);
+			recursive_add_folder (input->sc->storage,
+					      data->path, data->name,
+					      data->url);
 		else
 			evolution_storage_removed_folder (input->sc->storage, data->path);
 	}
@@ -347,6 +380,7 @@ cleanup_subscribe_folder (gpointer in_data, gpointer op_data,
 	
 	g_free (data->path);
 	g_free (data->name);
+	g_free (data->full_name);
 	g_free (data->url);
 	
 	camel_object_unref (CAMEL_OBJECT (input->store));
@@ -459,7 +493,7 @@ subscribe_folder_foreach (int model_row, gpointer closure)
 }
 
 static void
-subscribe_folders (GtkWidget *widget, gpointer user_data)
+subscribe_folders (BonoboUIComponent *componet, gpointer user_data, const char *cname)
 {
 	SubscribeDialog *sc = SUBSCRIBE_DIALOG (user_data);
 
@@ -480,7 +514,7 @@ unsubscribe_folder_foreach (int model_row, gpointer closure)
 
 
 static void
-unsubscribe_folders (GtkWidget *widget, gpointer user_data)
+unsubscribe_folders (BonoboUIComponent *component, gpointer user_data, const char *cname)
 {
 	SubscribeDialog *sc = SUBSCRIBE_DIALOG (user_data);
 
@@ -489,7 +523,7 @@ unsubscribe_folders (GtkWidget *widget, gpointer user_data)
 }
 
 static void
-subscribe_refresh_list (GtkWidget *widget, gpointer user_data)
+subscribe_refresh_list (BonoboUIComponent *component, gpointer user_data, const char *cname)
 {
 	SubscribeDialog *sc = SUBSCRIBE_DIALOG (user_data);
 
@@ -780,9 +814,7 @@ build_tree (SubscribeDialog *sc, CamelStore *store)
 }
 
 static void
-storage_selected_cb (ETable *table,
-		     int row,
-		     gpointer data)
+storage_selected_cb (ETable *table, int row, gpointer data)
 {
 	SubscribeDialog *sc = SUBSCRIBE_DIALOG (data);
 	CamelStore *store = (CamelStore*)g_list_nth_data (sc->store_list, row);
@@ -793,9 +825,7 @@ storage_selected_cb (ETable *table,
 
 
 static void
-folder_toggle_cb (ETable *table,
-		  int row,
-		  gpointer data)
+folder_toggle_cb (ETable *table, int row, int col, GdkEvent *event, gpointer data)
 {
 	SubscribeDialog *sc = SUBSCRIBE_DIALOG (data);
 	ETreePath *node = e_tree_model_node_at_row (sc->folder_model, row);
@@ -818,18 +848,18 @@ folder_toggle_cb (ETable *table,
 
 static BonoboUIVerb verbs [] = {
 	/* File Menu */
-	BONOBO_UI_UNSAFE_VERB ("FileCloseWin", subscribe_close),
+	BONOBO_UI_VERB ("FileCloseWin", subscribe_close),
 
 	/* Edit Menu */
-	BONOBO_UI_UNSAFE_VERB ("EditSelectAll", subscribe_select_all),
-	BONOBO_UI_UNSAFE_VERB ("EditInvertSelection", subscribe_invert_selection),
+	BONOBO_UI_VERB ("EditSelectAll", subscribe_select_all),
+	BONOBO_UI_VERB ("EditInvertSelection", subscribe_invert_selection),
 	
 	/* Folder Menu / Toolbar */
-	BONOBO_UI_UNSAFE_VERB ("SubscribeFolder", subscribe_folders),
-	BONOBO_UI_UNSAFE_VERB ("UnsubscribeFolder", unsubscribe_folders),
+	BONOBO_UI_VERB ("SubscribeFolder", subscribe_folders),
+	BONOBO_UI_VERB ("UnsubscribeFolder", unsubscribe_folders),
 
 	/* Toolbar Specific */
-	BONOBO_UI_UNSAFE_VERB ("RefreshList", subscribe_refresh_list),
+	BONOBO_UI_VERB ("RefreshList", subscribe_refresh_list),
 
 	BONOBO_UI_VERB_END
 };
@@ -843,8 +873,7 @@ store_cb (SubscribeDialog *sc, CamelStore *store, gpointer data)
 	if (camel_store_supports_subscriptions (store)) {
 		sc->store_list = g_list_prepend (sc->store_list, store);
 		e_table_model_row_inserted (sc->store_model, 0);
-	}
-	else {
+	} else {
 		camel_object_unref (CAMEL_OBJECT (store));
 	}
 }
@@ -852,19 +881,24 @@ store_cb (SubscribeDialog *sc, CamelStore *store, gpointer data)
 static void
 populate_store_foreach (MailConfigService *service, SubscribeDialog *sc)
 {
+	g_return_if_fail (service->url != NULL);
+	
 	subscribe_do_get_store (sc, service->url, store_cb, NULL);
 }
 
 static void
 populate_store_list (SubscribeDialog *sc)
 {
+	const GSList *news;
 	GSList *sources;
-
+	
 	sources = mail_config_get_sources ();
 	g_slist_foreach (sources, (GFunc)populate_store_foreach, sc);
-	sources = mail_config_get_news ();
-	g_slist_foreach (sources, (GFunc)populate_store_foreach, sc);
-
+	g_slist_free (sources);
+	
+	news = mail_config_get_news ();
+	g_slist_foreach ((GSList *)news, (GFunc)populate_store_foreach, sc);
+	
 	e_table_model_changed (sc->store_model);
 }
 
@@ -915,21 +949,20 @@ subscribe_dialog_gui_init (SubscribeDialog *sc)
 	gtk_widget_show_all (folder_search_widget);
 	search_control = bonobo_control_new (folder_search_widget);
 
-	bonobo_ui_component_object_set (
-		component, "/Toolbar/FolderSearch",
-		bonobo_object_corba_objref (BONOBO_OBJECT (search_control)), NULL);
+	bonobo_ui_component_object_set (component, "/Toolbar/FolderSearch",
+					bonobo_object_corba_objref (BONOBO_OBJECT (search_control)),
+					NULL);
 					
 	/* set our our contents */
 #if 0
 	sc->description = html_new (TRUE);
 	put_html (GTK_HTML (sc->description), EXAMPLE_DESCR);
 
-	gtk_table_attach (
-		GTK_TABLE (sc->table), sc->description->parent->parent,
-		0, 1, 0, 1,
-		GTK_FILL | GTK_EXPAND,
-		0,
-		0, 0);
+	gtk_table_attach (GTK_TABLE (sc->table), sc->description->parent->parent,
+			  0, 1, 0, 1,
+			  GTK_FILL | GTK_EXPAND,
+			  0,
+			  0, 0);
 #endif
 
 	/* set up the store etable */
@@ -1025,9 +1058,8 @@ subscribe_dialog_gui_init (SubscribeDialog *sc)
 	gtk_widget_show (sc->hpaned);
 
 	/* FIXME: Session management and stuff?  */
-	gtk_window_set_default_size (
-		GTK_WINDOW (sc->app),
-		DEFAULT_WIDTH, DEFAULT_HEIGHT);
+	gtk_window_set_default_size (GTK_WINDOW (sc->app),
+				     DEFAULT_WIDTH, DEFAULT_HEIGHT);
 
 	populate_store_list (sc);
 }
