@@ -24,10 +24,6 @@
 #ifndef _E_FOLDER_H_
 #define _E_FOLDER_H_
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #include <gtk/gtkobject.h>
 
 #ifdef __cplusplus
@@ -81,11 +77,12 @@ const char *e_folder_get_description  (EFolder *folder);
 const char *e_folder_get_physical_uri (EFolder *folder);
 gboolean    e_folder_get_highlighted  (EFolder *folder);
 
-void  e_folder_set_name         (EFolder *folder, const char *name);
-void  e_folder_set_type_string  (EFolder *folder, const char *type);
-void  e_folder_set_description  (EFolder *folder, const char *description);
-void  e_folder_set_physical_uri (EFolder *folder, const char *physical_uri);
-void  e_folder_set_highlighted  (EFolder *folder, gboolean highlighted);
+void e_folder_set_name            (EFolder *folder, const char *name);
+void e_folder_set_type_string     (EFolder *folder, const char *type);
+void e_folder_set_description     (EFolder *folder, const char *description);
+void e_folder_set_physical_uri    (EFolder *folder, const char *physical_uri);
+void e_folder_set_highlighted     (EFolder *folder, gboolean highlighted);
+void e_folder_set_child_highlight (EFolder *folder, gboolean highlighted);
 
 #ifdef __cplusplus
 }
