@@ -1,7 +1,7 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
 /* e-setup.c
  *
- * Copyright (C) 2000  Helix Code, Inc.
+ * Copyright (C) 2000, 2001 Ximian, Inc.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -122,6 +122,8 @@ check_evolution_directory (const char *evolution_directory)
 	gtk_widget_show (label1);
 	gtk_widget_show (label2);
 
+	gtk_window_set_policy (GTK_WINDOW (dialog), FALSE, TRUE, FALSE);
+
 	result = gnome_dialog_run_and_close (GNOME_DIALOG (dialog));
 
 	if (result != 0) {
@@ -184,7 +186,7 @@ copy_default_stuff (const char *evolution_directory)
 				   GNOME_STOCK_BUTTON_OK, GNOME_STOCK_BUTTON_CANCEL,
 				   NULL);
 
-	label1 = gtk_label_new (_("This seems to be the first time you run Evolution."));
+	label1 = gtk_label_new (_("This seems to be the first time you are running Evolution."));
 	label2 = gtk_label_new (_("Please click \"OK\" to install the Evolution user files under"));
 	label3 = gtk_label_new (evolution_directory);
 
@@ -196,6 +198,8 @@ copy_default_stuff (const char *evolution_directory)
 	gtk_widget_show (label2);
 	gtk_widget_show (label3);
 
+	gtk_window_set_policy (GTK_WINDOW (dialog), FALSE, TRUE, FALSE);
+
 	result = gnome_dialog_run_and_close (GNOME_DIALOG (dialog));
 	if (result != 0)
 		return FALSE;
