@@ -1,7 +1,7 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
 /* e-shell-view.c
  *
- * Copyright (C) 2000  Helix Code, Inc.
+ * Copyright (C) 2000, 2001  Ximian, Inc.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -32,8 +32,43 @@
 #include "e-shell-constants.h"
 
 #include "e-shell-view-menu.h"
+#include "importer/importer.h"
+
+#include <liboaf/liboaf.h>
 #include <bonobo.h>
 
+
+const char *authors[] = {
+	"Seth Alves",
+	"Anders Carlsson",
+	"Damon Chaplin",
+	"Clifford R. Conover",
+	"Anna Dirks",
+	"Miguel de Icaza",
+	"Radek Doulik",
+	"Arturo Espinoza",
+	"Larry Ewing",
+	"Nat Friedman",
+	"Bertrand Guiheneuf",
+	"Iain Holmes",
+	"Tuomas Kuosmanen",
+	"Christopher J. Lahey",
+	"Matthew Loper",
+	"Federico Mena",
+	"Eskil Heyn Olsen",
+	"Jesse Pavel",
+	"Ettore Perazzoli",
+	"JP Rosevear",
+	"Jeffrey Stedfast",
+	"Russell Steinthal",
+	"Peter Teichman",
+	"Chris Toshok",
+	"Peter Williams",
+	"Dan Winship",
+	"Michael Zucchi",
+	NULL
+};
+
 
 /* EShellView callbacks.  */
 
@@ -79,6 +114,19 @@ folder_bar_mode_changed_cb (EShellView *shell_view,
 
 
 /* Command callbacks.  */
+
+static void
+command_close (BonoboUIComponent *uih,
+	       void *data,
+	       const char *path)
+{
+	EShellView *shell_view;
+
+	shell_view = E_SHELL_VIEW (data);
+
+	gtk_object_destroy (GTK_OBJECT (shell_view));
+}
+
 static void
 command_quit (BonoboUIComponent *uih,
 	      void *data,
@@ -108,15 +156,12 @@ command_run_bugbuddy (BonoboUIComponent *uih,
         };
         args[0] = gnome_is_program_in_path ("bug-buddy");
         if (!args[0]) {
-                /* you might have to call gnome_dialog_run() on the
-                 * dialog returned here, I don't remember...
-                 */
                 gnome_error_dialog (_("Bug buddy was not found in your $PATH."));
+		return;
         }
         pid = gnome_execute_async (NULL, 4, args);
         g_free (args[0]);
         if (pid == -1) {
-                /* same as above */
                 gnome_error_dialog (_("Bug buddy could not be run."));
         }
 }
@@ -134,42 +179,19 @@ command_about_box (BonoboUIComponent *uih,
 {
 	static GtkWidget *about_box = NULL;
 
-	if (about_box)
+	if (about_box) {
 		gdk_window_raise(GTK_WIDGET(about_box)->window);
-	else {
-		const gchar *authors[] = {
-			"Seth Alves",
-			"Anders Carlsson",
-			"Damon Chaplin",
-			"Clifford R. Conover",
-			"Anna Dirks",
-			"Miguel de Icaza",
-			"Radek Doulik",
-			"Arturo Espinoza",
-			"Larry Ewing",
-			"Nat Friedman",
-			"Bertrand Guiheneuf",
-			"Iain Holmes",
-			"Tuomas Kuosmanen",
-			"Christopher J. Lahey",
-			"Matthew Loper",
-			"Federico Mena",
-			"Eskil Heyn Olsen",
-			"Jesse Pavel",
-			"Ettore Perazzoli",
-			"JP Rosevear",
-			"Jeffrey Stedfast",
-			"Russell Steinthal",
-			"Peter Teichman",
-			"Chris Toshok",
-			"Peter Williams",
-			"Dan Winship",
-			"Michael Zucchi",
-			NULL};
+	} else {
+		char *version;
+
+		if (SUB_VERSION[0] == '\0')
+			version = g_strdup (VERSION);
+		else
+			version = g_strdup_printf ("%s [%s]", VERSION, SUB_VERSION);
 
 		about_box = gnome_about_new(_("Evolution"),
-					    VERSION,
-					    _("Copyright 1999, 2000 Helix Code, Inc."),
+					    version,
+					    _("Copyright 1999, 2000, 2001 Ximian, Inc."),
 					    authors,
 					    _("Evolution is a suite of groupware applications\n"
 					      "for mail, calendaring, and contact management\n"
@@ -178,6 +200,8 @@ command_about_box (BonoboUIComponent *uih,
 		gtk_signal_connect(GTK_OBJECT(about_box), "destroy",
 				   GTK_SIGNAL_FUNC(zero_pointer), &about_box);
 		gtk_widget_show(about_box);
+
+		g_free (version);
 	}
 }
 
@@ -224,7 +248,6 @@ command_toggle_shortcut_bar (BonoboUIComponent           *component,
 			     Bonobo_UIComponent_EventType type,
 			     const char                  *state,
 			     gpointer                     user_data)
-
 {
 	EShellView *shell_view;
 	EShellViewSubwindowMode mode;
@@ -266,12 +289,14 @@ command_new_folder (BonoboUIComponent *uih,
 		default_parent_folder = NULL;
 
 	e_shell_show_folder_creation_dialog (shell, GTK_WINDOW (shell_view),
-					     default_parent_folder);
+					     default_parent_folder,
+					     NULL /* result_callback */,
+					     NULL /* result_callback_data */);
 }
 
 static void
 command_new_view (BonoboUIComponent *uih,
-		  void *data,
+		  gpointer          data,
 		  const char *path)
 {
 	EShellView *shell_view;
@@ -318,7 +343,7 @@ folder_selection_dialog_folder_selected_cb (EShellFolderSelectionDialog *folder_
 
 static void
 command_goto_folder (BonoboUIComponent *uih,
-		     void *data,
+		     gpointer    data,
 		     const char *path)
 {
 	GtkWidget *folder_selection_dialog;
@@ -366,7 +391,9 @@ command_create_folder (BonoboUIComponent *uih,
 	else
 		default_folder = NULL;
 
-	e_shell_show_folder_creation_dialog (shell, GTK_WINDOW (shell_view), default_folder);
+	e_shell_show_folder_creation_dialog (shell, GTK_WINDOW (shell_view), default_folder,
+					     NULL /* result_callback */,
+					     NULL /* result_callback_data */);
 }
 
 static void
@@ -383,41 +410,59 @@ command_xml_dump (gpointer    dummy,
 static void								\
 func (BonoboUIComponent *uic, void *data, const char *path)		\
 {									\
-	g_warning ("EShellView: %s: not implemented.", __FUNCTION__);	\
+	g_warning ("e-shell-view-menu.c: %s: not implemented.", #func);	\
 }									\
 
+static void
+command_new_mail_message (BonoboUIComponent *uih,
+			  gpointer          data,
+			  const char *path)
+{
+	CORBA_Environment ev;
+	Bonobo_Unknown object;
+	
+	CORBA_exception_init (&ev);
+	object = bonobo_get_object (
+		"OAFIID:GNOME_Evolution_Mail_Composer!visible=1",
+		"Bonobo/Unknown", &ev);
+
+	CORBA_exception_free (&ev);
+}
+	
 DEFINE_UNIMPLEMENTED (command_new_shortcut)
-DEFINE_UNIMPLEMENTED (command_new_mail_message)
+
 DEFINE_UNIMPLEMENTED (command_new_contact)
 DEFINE_UNIMPLEMENTED (command_new_task_request)
 
 BonoboUIVerb new_verbs [] = {
-	BONOBO_UI_UNSAFE_VERB ("NewView", command_new_view),
-	BONOBO_UI_UNSAFE_VERB ("NewFolder", command_new_folder),
-	BONOBO_UI_UNSAFE_VERB ("NewShortcut", command_new_shortcut),
-	BONOBO_UI_UNSAFE_VERB ("NewMailMessage", command_new_mail_message),
-
-	BONOBO_UI_UNSAFE_VERB ("NewAppointment", command_new_shortcut),
-	BONOBO_UI_UNSAFE_VERB ("NewContact", command_new_contact),
-	BONOBO_UI_UNSAFE_VERB ("NewTask", command_new_task_request),
+	BONOBO_UI_VERB ("NewView", command_new_view),
+	BONOBO_UI_VERB ("NewFolder", command_new_folder),
+	BONOBO_UI_VERB ("NewShortcut", command_new_shortcut),
+	BONOBO_UI_VERB ("NewMailMessage", command_new_mail_message),
+		  
+	BONOBO_UI_VERB ("NewAppointment", command_new_shortcut),
+	BONOBO_UI_VERB ("NewContact", command_new_contact),
+	BONOBO_UI_VERB ("NewTask", command_new_task_request),
 
 	BONOBO_UI_VERB_END
 };
 
 BonoboUIVerb file_verbs [] = {
-	BONOBO_UI_UNSAFE_VERB ("FileGoToFolder", command_goto_folder),
-	BONOBO_UI_UNSAFE_VERB ("FileCreateFolder", command_create_folder),
-	BONOBO_UI_UNSAFE_VERB ("FileExit", command_quit),
+	BONOBO_UI_VERB ("FileImporter", show_import_wizard),
+	BONOBO_UI_VERB ("FileGoToFolder", command_goto_folder),
+	BONOBO_UI_VERB ("FileCreateFolder", command_create_folder),
+	BONOBO_UI_VERB ("FileClose", command_close),
+	BONOBO_UI_VERB ("FileExit", command_quit),
 
 	BONOBO_UI_VERB_END
 };
 
 BonoboUIVerb help_verbs [] = {
-	BONOBO_UI_UNSAFE_VERB_DATA ("HelpIndex", command_help, "index.html"),
-	BONOBO_UI_UNSAFE_VERB_DATA ("HelpGetStarted", command_help, "usage-mainwindow.html"),
-	BONOBO_UI_UNSAFE_VERB_DATA ("HelpUsingMail", command_help, "usage-mail.html"),
-	BONOBO_UI_UNSAFE_VERB_DATA ("HelpUsingCalendar", command_help, "usage-calendar.html"),
-	BONOBO_UI_UNSAFE_VERB_DATA ("HelpUsingContact", command_help, "usage-contact.html"),
+	BONOBO_UI_VERB_DATA ("HelpIndex", command_help, "index.html"),
+	BONOBO_UI_VERB_DATA ("HelpGetStarted", command_help, "usage-mainwindow.html"),
+	BONOBO_UI_VERB_DATA ("HelpUsingMail", command_help, "usage-mail.html"),
+	BONOBO_UI_VERB_DATA ("HelpUsingCalendar", command_help, "usage-calendar.html"),
+	BONOBO_UI_VERB_DATA ("HelpUsingContact", command_help, "usage-contact.html"),
 
 	BONOBO_UI_VERB_END
 };
