@@ -1,7 +1,7 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
 /* e-shell-view.c
  *
- * Copyright (C) 2000  Helix Code, Inc.
+ * Copyright (C) 2000, 2001 Ximian, Inc.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -129,6 +129,9 @@ static guint signals[LAST_SIGNAL] = { 0 };
 
 #define DEFAULT_URI "evolution:/local/Inbox"
 
+static void update_for_current_uri (EShellView *shell_view);
+static const char * get_storage_set_path_from_uri (const char *uri);
+
 
 /* Utility functions.  */
 
@@ -184,18 +187,23 @@ popdown_transient_folder_bar (EShellView *shell_view)
 	e_shell_folder_title_bar_set_toggle_state (E_SHELL_FOLDER_TITLE_BAR (priv->view_title_bar), FALSE);
 }
 
-static void
+static int
 storage_set_view_box_button_release_event_cb (GtkWidget *widget,
 					      GdkEventButton *button_event,
 					      void *data)
 {
 	EShellView *shell_view;
+	EShellViewPrivate *priv;
 
 	shell_view = E_SHELL_VIEW (data);
+	priv = shell_view->priv;
 
-	puts (__FUNCTION__);
+	if (button_event->window == E_PANED (priv->view_hpaned)->handle)
+		return FALSE;
 
 	popdown_transient_folder_bar (shell_view);
+
+	return TRUE;
 }
 
 static void
@@ -234,7 +242,7 @@ storage_set_view_box_map_cb (GtkWidget *widget,
 			       | GDK_LEAVE_NOTIFY_MASK
 			       | GDK_POINTER_MOTION_MASK),
 			      NULL, NULL, GDK_CURRENT_TIME) != 0) {
-		g_warning ("%s -- pointer grab failed.", __FUNCTION__);
+		g_warning ("e-shell-view.c:storage_set_view_box_map_cb() -- pointer grab failed.");
 		e_shell_view_set_folder_bar_mode (shell_view, E_SHELL_VIEW_SUBWINDOW_STICKY);
 		return;
 	}
@@ -280,7 +288,7 @@ pop_up_folder_bar (EShellView *shell_view)
 
 	/* We need to show the storage set view box and do a pointer grab to catch the
            mouse clicks.  But until the box is shown, we cannot grab.  So we connect to
-           the "map" signa; `storage_set_view_box_map_cb' will do the grab.  */
+           the "map" signal; `storage_set_view_box_map_cb()' will do the grab.  */
 
 	gtk_signal_connect (GTK_OBJECT (priv->storage_set_view_box), "map",
 			    GTK_SIGNAL_FUNC (storage_set_view_box_map_cb), shell_view);
@@ -365,6 +373,19 @@ activate_shortcut_cb (EShortcutsView *shortcut_view,
 	e_shell_view_display_uri (shell_view, uri);
 }
 
+/* Callback when user chooses "Hide shortcut bar" via a right click */
+static void
+hide_requested_cb (EShortcutsView *shortcut_view,
+		   void *data)
+{
+	EShellView *shell_view;
+
+	shell_view = E_SHELL_VIEW (data);
+
+	e_shell_view_set_shortcut_bar_mode (shell_view,
+					    E_SHELL_VIEW_SUBWINDOW_HIDDEN);
+}
+
 /* Callback called when a folder on the tree view gets clicked.  */
 static void
 folder_selected_cb (EStorageSetView *storage_set_view,
@@ -392,8 +413,6 @@ storage_selected_cb (EStorageSetView *storage_set_view,
 	path = g_strconcat (G_DIR_SEPARATOR_S, name, NULL);
 	switch_on_folder_tree_click (shell_view, path);
 
-	puts (__FUNCTION__);
-
 	g_free (path);
 }
 
@@ -508,6 +527,9 @@ setup_widgets (EShellView *shell_view)
 	gtk_signal_connect (GTK_OBJECT (priv->shortcut_bar), "activate_shortcut",
 			    GTK_SIGNAL_FUNC (activate_shortcut_cb), shell_view);
 
+	gtk_signal_connect (GTK_OBJECT (priv->shortcut_bar), "hide_requested",
+			    GTK_SIGNAL_FUNC (hide_requested_cb), shell_view);
+
 	/* The storage set view.  */
 
 	setup_storage_set_subwindow (shell_view);
@@ -864,8 +886,29 @@ unmerge_on_error (BonoboObject *object,
 	win = bonobo_ui_container_get_win (BONOBO_UI_CONTAINER (object));
 
 	if (win)
-		bonobo_window_deregister_component_by_ref (
-			win, cobject);
+		bonobo_window_deregister_component_by_ref (win, cobject);
+}
+
+static void
+updated_folder_cb (EStorageSet *storage_set,
+		   const char *path,
+		   void *data)
+{
+	EShellView *shell_view;
+	EShellViewPrivate *priv;
+	char *uri;
+
+	shell_view = E_SHELL_VIEW (data);
+	priv = shell_view->priv;
+
+	uri = g_strconcat (E_SHELL_URI_PREFIX, path, NULL);
+
+	/* Update the shortcut bar */
+	e_shortcuts_update_shortcut_by_uri (e_shell_get_shortcuts (priv->shell), uri);
+	g_free (uri);
+
+	/* Update the folder title bar and the window title bar */
+	update_for_current_uri (shell_view);
 }
 
 
@@ -894,15 +937,16 @@ e_shell_view_construct (EShellView *shell_view,
 
 	window = GTK_OBJECT (view);
 
-	gtk_signal_connect (window, "delete_event",
-			    (GtkSignalFunc) delete_event, NULL);
+	gtk_signal_connect (window, "delete_event", GTK_SIGNAL_FUNC (delete_event), NULL);
 
 	priv->shell = shell;
 
+	gtk_signal_connect_while_alive (GTK_OBJECT (e_shell_get_storage_set (priv->shell)), "updated_folder",
+					updated_folder_cb, shell_view, GTK_OBJECT (shell_view));
+
 	container = bonobo_ui_container_new ();
 	bonobo_ui_container_set_win (container, BONOBO_WINDOW (shell_view));
-	gtk_signal_connect (GTK_OBJECT (container), "system_exception",
-			    (GtkSignalFunc) unmerge_on_error, NULL);
+	gtk_signal_connect (GTK_OBJECT (container), "system_exception", GTK_SIGNAL_FUNC (unmerge_on_error), NULL);
 
 	priv->ui_component = bonobo_ui_component_new ("evolution");
 	bonobo_ui_component_set_container (priv->ui_component,
@@ -910,11 +954,13 @@ e_shell_view_construct (EShellView *shell_view,
 
 	bonobo_ui_component_freeze (priv->ui_component, NULL);
 
-	bonobo_ui_util_set_ui (priv->ui_component, EVOLUTION_DATADIR,
-			       "evolution.xml", "evolution");
+	bonobo_ui_util_set_ui (priv->ui_component, EVOLUTION_DATADIR, "evolution.xml", "evolution");
 
 	setup_widgets (shell_view);
 
+	bonobo_ui_engine_config_set_path (bonobo_window_get_ui_engine (BONOBO_WINDOW (shell_view)),
+					  "/evolution/UIConf/kvps");
+
 	e_shell_view_menu_setup (shell_view);
 
 	e_shell_view_set_folder_bar_mode (shell_view, E_SHELL_VIEW_SUBWINDOW_HIDDEN);
@@ -1074,7 +1120,11 @@ update_for_current_uri (EShellView *shell_view)
 	else
 		folder_name = e_utf8_to_gtk_string ((GtkWidget *) shell_view, e_folder_get_name (folder));
 
-	window_title = g_strdup_printf (_("Evolution - %s"), folder_name);
+	if (SUB_VERSION[0] == '\0')
+		window_title = g_strdup_printf (_("%s - Evolution %s"), folder_name, VERSION);
+	else
+		window_title = g_strdup_printf (_("%s - Evolution %s [%s]"), folder_name, VERSION, SUB_VERSION);
+
 	gtk_window_set_title (GTK_WINDOW (shell_view), window_title);
 	g_free (window_title);
 	g_free (folder_name);
@@ -1372,8 +1422,6 @@ show_existing_view (EShellView *shell_view,
 	EShellViewPrivate *priv;
 	int notebook_page;
 
-	g_print ("Already have view for %s\n", uri);
-
 	priv = shell_view->priv;
 
 	notebook_page = gtk_notebook_page_num (GTK_NOTEBOOK (priv->notebook), control);
@@ -1640,6 +1688,53 @@ e_shell_view_get_current_uri (EShellView *shell_view)
 	return shell_view->priv->uri;
 }
 
+static void
+save_shortcut_bar_icon_modes (EShellView *shell_view)
+{
+	EShellViewPrivate *priv;
+	EShortcutBar *shortcut_bar;
+	int num_groups;
+	int group;
+
+	priv = shell_view->priv;
+	shortcut_bar = E_SHORTCUT_BAR (priv->shortcut_bar);
+
+	num_groups = e_shortcut_model_get_num_groups (shortcut_bar->model);
+
+	for (group = 0; group < num_groups; group++) {
+		char *tmp;
+
+		tmp = g_strdup_printf ("ShortcutBarGroup%dIconMode", group);
+		gnome_config_set_int (tmp, e_shortcut_bar_get_view_type (shortcut_bar, group));
+		g_free (tmp);
+	}
+}
+
+static void
+load_shortcut_bar_icon_modes (EShellView *shell_view)
+{
+	EShellViewPrivate *priv;
+	EShortcutBar *shortcut_bar;
+	int num_groups;
+	int group;
+
+	priv = shell_view->priv;
+	shortcut_bar = E_SHORTCUT_BAR (priv->shortcut_bar);
+
+	num_groups = e_shortcut_model_get_num_groups (shortcut_bar->model);
+
+	for (group = 0; group < num_groups; group++) {
+		char *tmp;
+		int iconmode;
+
+		tmp = g_strdup_printf ("ShortcutBarGroup%dIconMode", group);
+		iconmode = gnome_config_get_int (tmp);
+		g_free (tmp);
+
+		e_shortcut_bar_set_view_type (shortcut_bar, group, iconmode);
+	}
+}
+
 
 /**
  * e_shell_view_save_settings:
@@ -1676,6 +1771,8 @@ e_shell_view_save_settings (EShellView *shell_view,
 	else
 		gnome_config_set_string ("DisplayedURI", DEFAULT_URI);
 
+	save_shortcut_bar_icon_modes (shell_view);
+
 	gnome_config_pop_prefix ();
 	
 	return TRUE;
@@ -1723,6 +1820,8 @@ e_shell_view_load_settings (EShellView *shell_view,
 		e_shell_view_display_uri (shell_view, DEFAULT_URI);
 	g_free (stringval);
 
+	load_shortcut_bar_icon_modes (shell_view);
+
 	gnome_config_pop_prefix ();
 	
 	return TRUE;
