@@ -1,7 +1,7 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
 /* e-shell.c
  *
- * Copyright (C) 2000  Helix Code, Inc.
+ * Copyright (C) 2000, 2001 Ximian, Inc.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -525,12 +525,21 @@ view_destroy_cb (GtkObject *object,
 		 gpointer data)
 {
 	EShell *shell;
+	int nviews;
 
 	g_assert (E_IS_SHELL_VIEW (object));
 
 	shell = E_SHELL (data);
-	shell->priv->views = g_list_remove (
-		shell->priv->views, object);
+
+	nviews = g_list_length (shell->priv->views);
+
+	/* If this is our last view, save settings now because in the
+	   callback for no_views_left shell->priv->views will be NULL
+	   and settings won't be saved because of that */
+	if (nviews - 1 == 0)
+		e_shell_save_settings (shell);
+
+	shell->priv->views = g_list_remove (shell->priv->views, object);
 
 	if (shell->priv->views == NULL) {
 		/* FIXME: This looks like a Bonobo bug to me.  */
@@ -710,6 +719,9 @@ e_shell_construct (EShell *shell,
 	} else {
 		splash = e_splash_new ();
 		gtk_widget_show (splash);
+
+		/* Keep our own reference */
+		gtk_object_ref (GTK_OBJECT (splash));
 	}
 
 	while (gtk_events_pending ())
@@ -753,6 +765,7 @@ e_shell_construct (EShell *shell,
 	g_free (shortcut_path);
 
 	sleep (2);
+	gtk_widget_unref (splash);
 	gtk_widget_destroy (splash);
 
 	return TRUE;
@@ -907,6 +920,23 @@ e_shell_get_folder_type_registry (EShell *shell)
 	return shell->priv->folder_type_registry;
 }
 
+/**
+ * e_shell_get_local_storage:
+ * @shell: An EShell object.
+ *
+ * Get the local storage associated to @shell.
+ *
+ * Return value: A pointer to the ELocalStorage associated to @shell.
+ **/
+ELocalStorage *
+e_shell_get_local_storage (EShell *shell)
+{
+	g_return_val_if_fail (shell != NULL, NULL);
+	g_return_val_if_fail (E_IS_SHELL (shell), NULL);
+
+	return shell->priv->local_storage;
+}
+
 
 static gboolean
 save_settings_for_views (EShell *shell)
