@@ -1,7 +1,7 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
 /* e-splash.c
  *
- * Copyright (C) 2000  Helix Code, Inc.
+ * Copyright (C) 2000, 2001 Ximian, Inc.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -317,10 +317,12 @@ e_splash_construct (ESplash *splash,
 	gtk_signal_connect (GTK_OBJECT (splash), "button-press-event",
 			    GTK_SIGNAL_FUNC (button_press_event), splash);
 	
-	gtk_object_set (GTK_OBJECT (splash), "type", GTK_WINDOW_POPUP, NULL);
+	gtk_object_set (GTK_OBJECT (splash), "type", GTK_WINDOW_TOPLEVEL, NULL);
 	gtk_window_set_position (GTK_WINDOW (splash), GTK_WIN_POS_CENTER);
 	gtk_window_set_policy (GTK_WINDOW (splash), FALSE, FALSE, FALSE);
 	gtk_window_set_default_size (GTK_WINDOW (splash), image_width, image_height);
+	gtk_window_set_title (GTK_WINDOW (splash), "Evolution");
+
 }
 
 /**
@@ -369,6 +371,9 @@ e_splash_add_icon (ESplash *splash,
 	g_return_val_if_fail (E_IS_SPLASH (splash), 0);
 	g_return_val_if_fail (icon_pixbuf != NULL, 0);
 
+	if (GTK_OBJECT_DESTROYED (splash))
+		return 0;
+	
 	priv = splash->priv;
 
 	icon = icon_new (splash, icon_pixbuf);
@@ -400,6 +405,9 @@ e_splash_set_icon_highlight  (ESplash *splash,
 	g_return_if_fail (splash != NULL);
 	g_return_if_fail (E_IS_SPLASH (splash));
 
+	if (GTK_OBJECT_DESTROYED (splash))
+		return;
+	
 	priv = splash->priv;
 
 	icon = (Icon *) g_list_nth (priv->icons, num)->data;
