@@ -24,10 +24,6 @@
 #ifndef __E_STORAGE_SET_VIEW_H__
 #define __E_STORAGE_SET_VIEW_H__
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #include <gal/e-table/e-table.h>
 #include "e-storage-set.h"
 
@@ -60,8 +56,14 @@ struct _EStorageSetViewClass {
 
 	void (* folder_selected)  (EStorageSetView *storage_set_view,
 				   const char *path);
-	void (* storage_selected)  (EStorageSetView *storage_set_view,
-				    const char *name);
+	void (* storage_selected) (EStorageSetView *storage_set_view,
+				   const char *name);
+
+	void (* dnd_action) (EStorageSetView *storage_set_view,
+			     GdkDragContext *context,
+			     const char *source_data,
+			     const char *source_data_type,
+			     const char *target_path);
 };
 
 
