@@ -24,10 +24,6 @@
 #ifndef _E_STORAGE_H_
 #define _E_STORAGE_H_
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #include <gtk/gtkobject.h>
 
 #include "evolution-shell-component-client.h"
@@ -137,7 +133,6 @@ char *e_storage_get_path_for_physical_uri  (EStorage   *storage,
 
 /* Protected.  C++ anyone?  */
 gboolean  e_storage_new_folder      (EStorage *storage, const char *path, EFolder *folder);
-gboolean  e_storage_updated_folder  (EStorage *storage, const char *path);
 gboolean  e_storage_removed_folder  (EStorage *storage, const char *path);
 
 #ifdef __cplusplus
