@@ -1,7 +1,7 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
 /* evolution-shell-component-client.c
  *
- * Copyright (C) 2000  Helix Code, Inc.
+ * Copyright (C) 2000, 2001 Ximian, Inc.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -483,10 +483,10 @@ evolution_shell_component_client_async_create_folder (EvolutionShellComponentCli
 	priv->callback      = callback;
 	priv->callback_data = data;
 
-	GNOME_Evolution_ShellComponent_addFolderAsync (corba_shell_component,
-						       priv->listener_interface,
-						       physical_uri, type,
-						       &ev);
+	GNOME_Evolution_ShellComponent_createFolderAsync (corba_shell_component,
+							  priv->listener_interface,
+							  physical_uri, type,
+							  &ev);
 
 	CORBA_exception_free (&ev);
 }
