@@ -1,7 +1,7 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
 /* evolution-shell-component.h
  *
- * Copyright (C) 2000  Helix Code, Inc.
+ * Copyright (C) 2000, 2001 Ximian, Inc.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -24,10 +24,6 @@
 #ifndef EVOLUTION_SHELL_COMPONENT_H
 #define EVOLUTION_SHELL_COMPONENT_H
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #include <bonobo/bonobo-object.h>
 #include <bonobo/bonobo-control.h>
 
@@ -87,6 +83,12 @@ typedef void (* EvolutionShellComponentRemoveFolderFn) (EvolutionShellComponent
 						        const char *physical_uri,
 						        const GNOME_Evolution_ShellComponentListener listener,
 						        void *closure);
+typedef void (* EvolutionShellComponentCopyFolderFn)   (EvolutionShellComponent *shell_component,
+							const char *source_physical_uri,
+							const char *destination_physical_uri,
+							gboolean remove_source,
+						        const GNOME_Evolution_ShellComponentListener listener,
+						        void *closure);
 typedef void (* EvolutionShellComponentPopulateFolderContextMenu) (EvolutionShellComponent *shell_component,
 								   BonoboUIComponent *uic,
 								   const char *physical_uri,
@@ -124,12 +126,14 @@ void                     evolution_shell_component_construct  (EvolutionShellCom
 							       EvolutionShellComponentCreateViewFn               create_view_fn,
 							       EvolutionShellComponentCreateFolderFn             create_folder_fn,
 							       EvolutionShellComponentRemoveFolderFn             remove_folder_fn,
+							       EvolutionShellComponentCopyFolderFn               copy_folder_fn,
 							       EvolutionShellComponentPopulateFolderContextMenu  populate_folder_context_menu_fn,
 							       void                                             *closure);
 EvolutionShellComponent *evolution_shell_component_new        (const EvolutionShellComponentFolderType           folder_types[],
 							       EvolutionShellComponentCreateViewFn               create_view_fn,
 							       EvolutionShellComponentCreateFolderFn             create_folder_fn,
 							       EvolutionShellComponentRemoveFolderFn             remove_folder_fn,
+							       EvolutionShellComponentCopyFolderFn               copy_folder_fn,
 							       EvolutionShellComponentPopulateFolderContextMenu  populate_folder_context_menu_fn,
 							       void                                             *closure);
 EvolutionShellClient    *evolution_shell_component_get_owner  (EvolutionShellComponent                          *shell_component);
